Package: RRPP
Title: Linear Model Evaluation with Randomized Residuals in a
        Permutation Procedure
Version: 2.1.2
Authors@R: c(
    person("Michael", "Collyer", , "mlcollyer@gmail.com", c("aut", "cre"), 
    comment = c(ORCID = "0000-0003-0238-2201")),
    person("Dean", "Adams", , role = "aut", 
    comment = c(ORCID = "0000-0001-9172-7894"))
    )
Description: Linear model calculations are made for many random versions of data.  
    Using residual randomization in a permutation procedure, sums of squares are 
    calculated over many permutations to generate empirical probability distributions 
    for evaluating model effects.  Additionally, coefficients, statistics, fitted values, and residuals generated over many 
    permutations can be used for various procedures including pairwise tests, prediction, classification, and
    model comparison.  This package should provide most tools one could need for the analysis of
    high-dimensional data, especially in ecology and evolutionary biology, but certainly other fields, as well.
Depends: R (>= 4.4.0)
License: GPL (>= 3)
URL: https://github.com/mlcollyer/RRPP
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.2
Imports: parallel, ape, ggplot2, Matrix
Suggests: knitr, rmarkdown, testthat (>= 3.2.0), dplyr, tibble
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2025-02-18 16:54:13 UTC; m.collyer
Author: Michael Collyer [aut, cre] (<https://orcid.org/0000-0003-0238-2201>),
  Dean Adams [aut] (<https://orcid.org/0000-0001-9172-7894>)
Maintainer: Michael Collyer <mlcollyer@gmail.com>
Repository: CRAN
Date/Publication: 2025-02-18 23:40:12 UTC
Config/testthat/edition: 3
Built: R 4.6.0; ; 2026-04-26 07:20:01 UTC; unix

# Generated by roxygen2: do not edit by hand
S3method(anova,lm.rrpp)
S3method(anova,measurement.error)
S3method(coef,lm.rrpp)
S3method(fitted,lm.rrpp)
S3method(logLik,lm.rrpp)
S3method(model.frame,lm.rrpp)
S3method(model.matrix,lm.rrpp)
S3method(na.omit,rrpp.data.frame)
S3method(plot,interSubVar)
S3method(plot,kcomp)
S3method(plot,lm.rrpp)
S3method(plot,looCV)
S3method(plot,measurement.error)
S3method(plot,model.comparison)
S3method(plot,ordinate)
S3method(plot,predict.lm.rrpp)
S3method(plot,trajectory.analysis)
S3method(predict,lm.rrpp)
S3method(print,ICCstats)
S3method(print,anova.lm.rrpp)
S3method(print,betaTest)
S3method(print,coef.lm.rrpp)
S3method(print,kcomp)
S3method(print,lm.rrpp)
S3method(print,looCV)
S3method(print,lr_test)
S3method(print,measurement.error)
S3method(print,model.comparison)
S3method(print,ordinate)
S3method(print,pairwise)
S3method(print,pairwise.model.Z)
S3method(print,predict.lm.rrpp)
S3method(print,summary.betaTest)
S3method(print,summary.lm.rrpp)
S3method(print,summary.manova.lm.rrpp)
S3method(print,summary.ordinate)
S3method(print,summary.pairwise)
S3method(print,summary.trajectory.analysis)
S3method(print,trajectory.analysis)
S3method(residuals,lm.rrpp)
S3method(summary,ICCstats)
S3method(summary,anova.lm.rrpp)
S3method(summary,betaTest)
S3method(summary,coef.lm.rrpp)
S3method(summary,kcomp)
S3method(summary,lm.rrpp)
S3method(summary,looCV)
S3method(summary,lr_test)
S3method(summary,manova.lm.rrpp)
S3method(summary,measurement.error)
S3method(summary,model.comparison)
S3method(summary,ordinate)
S3method(summary,pairwise)
S3method(summary,pairwise.model.Z)
S3method(summary,predict.lm.rrpp)
S3method(summary,trajectory.analysis)
S3method(terms,lm.rrpp)
export(ICCstats)
export(QRforX)
export(add.trajectories)
export(add.tree)
export(anova.lm.rrpp)
export(anova.measurement.error)
export(betaTest)
export(classify)
export(convert2ggplot)
export(effect.size)
export(focusMEonSubjects)
export(getANOVAStats)
export(getModelCov)
export(getModels)
export(getPermInfo)
export(getResCov)
export(getTerms)
export(interSubVar)
export(kcomp)
export(lm.rrpp)
export(lm.rrpp.ws)
export(looCV)
export(lr_test)
export(mahal_dist)
export(manova.update)
export(measurement.error)
export(model.comparison)
export(ordinate)
export(pairwise)
export(pairwise.model.Z)
export(plot.kcomp)
export(plot.lm.rrpp)
export(plot.ordinate)
export(plot.predict.lm.rrpp)
export(prep.lda)
export(print.ICCstats)
export(print.anova.lm.rrpp)
export(print.coef.lm.rrpp)
export(print.lm.rrpp)
export(print.model.comparison)
export(print.pairwise)
export(print.pairwise.model.Z)
export(print.predict.lm.rrpp)
export(print.summary.lm.rrpp)
export(print.summary.pairwise)
export(print.summary.trajectory.analysis)
export(print.trajectory.analysis)
export(pval)
export(reveal.model.designs)
export(rrpp.data.frame)
export(scaleCov)
export(summary.ICCstats)
export(summary.anova.lm.rrpp)
export(summary.coef.lm.rrpp)
export(summary.lm.rrpp)
export(summary.model.comparison)
export(summary.pairwise)
export(summary.pairwise.model.Z)
export(summary.predict.lm.rrpp)
export(summary.trajectory.analysis)
export(trajectory.analysis)
export(vec.cor.matrix)
import(Matrix)
import(ggplot2)
import(parallel)
importFrom(ape,collapse.singles)
importFrom(ape,multi2di.phylo)
importFrom(ape,root.phylo)
importFrom(graphics,abline)
importFrom(graphics,arrows)
importFrom(graphics,axis)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,par)
importFrom(graphics,plot.default)
importFrom(graphics,points)
importFrom(graphics,text)
importFrom(graphics,title)
importFrom(stats,anova)
importFrom(stats,as.dist)
importFrom(stats,as.formula)
importFrom(stats,cmdscale)
importFrom(stats,coef)
importFrom(stats,delete.response)
importFrom(stats,dist)
importFrom(stats,formula)
importFrom(stats,lm)
importFrom(stats,lm.fit)
importFrom(stats,lm.wfit)
importFrom(stats,loess)
importFrom(stats,logLik)
importFrom(stats,model.frame)
importFrom(stats,model.frame.default)
importFrom(stats,model.matrix)
importFrom(stats,na.omit)
importFrom(stats,optimise)
importFrom(stats,pnorm)
importFrom(stats,prcomp)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,resid)
importFrom(stats,sd)
importFrom(stats,spline)
importFrom(stats,var)
importFrom(utils,combn)
importFrom(utils,object.size)
importFrom(utils,setTxtProgressBar)
importFrom(utils,txtProgressBar)

# RRPP
RRPP is a software package for evaluating linear models with residual randomization in a permutation procedure.  S3 Generic used for the lm function can also be used with lm.rrpp, with the chief difference being that lm coefficients, fitted values, and residuals are estimated many times with random permutations of data.

### To install the current RRPP R-package from CRAN:

<i> Within R:</i>

<code> install.packages("RRPP") </code>

### To install the current version of the RRPP R-package from Github using devtools:

<i> Within R:</i>

<code> install.packages("devtools")</code>

<code> devtools::install_github("mlcollyer/RRPP")</code>

The version on github is updated regularly, especially if errors or programming bugs are discovered.

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(echo = TRUE)

## -----------------------------------------------------------------------------
library(RRPP)
data("PupfishHeads")
PupfishHeads$logHeadSize <- log(PupfishHeads$headSize)
fit <- lm.rrpp(logHeadSize ~ sex + locality/year, 
               SS.type = "I", data = PupfishHeads, 
               print.progress = FALSE,
               turbo = FALSE, verbose = TRUE)
summary(fit)
anova(fit, effect.type = "F") 

## -----------------------------------------------------------------------------
anova(fit, effect.type = "F", 
  error = c("Residuals", "locality:year", "Residuals"))

## -----------------------------------------------------------------------------
fit.sex <- lm.rrpp(logHeadSize ~ sex, 
                   data = PupfishHeads, 
                   print.progress = FALSE)
fit.sex.loc<- lm.rrpp(logHeadSize ~ sex + locality, 
                      data = PupfishHeads, 
                      print.progress = FALSE)
fit.sex.loc.year<- lm.rrpp(logHeadSize ~ sex + locality/year, 
                           data = PupfishHeads, 
                           print.progress = FALSE)
anova(fit.sex, fit.sex.loc, fit.sex.loc.year, print.progress= FALSE)

## -----------------------------------------------------------------------------
coef(fit, test = TRUE)

## ----fig.height=3, fig.width=5------------------------------------------------
sizeDF <- data.frame(sex = c("Female", "Male"))
rownames(sizeDF) <- c("Female", "Male")
sizePreds <- predict(fit, sizeDF)
plot(sizePreds)


## ----fig.height=3, fig.width=5------------------------------------------------
plot(sizePreds, pch = 21, cex = 3, bg = c(2,4), lwd = 2)


## -----------------------------------------------------------------------------
fit2 <- lm.rrpp(logHeadSize ~ sex + locality/year, 
                SS.type = "II", data = PupfishHeads, print.progress = FALSE)
fit3 <- lm.rrpp(logHeadSize ~ sex + locality/year, 
                SS.type = "III", data = PupfishHeads, print.progress = FALSE)

anova(fit)
anova(fit2)
anova(fit3)


## -----------------------------------------------------------------------------
data(Pupfish)
Pupfish$logSize <- log(Pupfish$CS) 
fit <- lm.rrpp(coords ~ logSize + Sex*Pop, SS.type = "I", 
               data = Pupfish, print.progress = FALSE,
               turbo = FALSE, verbose = TRUE) 
summary(fit, formula = FALSE)
anova(fit) 
coef(fit, test = TRUE) 


## ----error = TRUE-------------------------------------------------------------
try({
fit$LM$data$coords <- Pupfish$coords
fit.par <- lm(fit$call$f1, data = fit$LM$data)
all.equal(fit$LM$coefficients, fit.par$coefficients)
summary(manova(fit.par))
})

## ----fig.height=5, fig.width=6------------------------------------------------
shapeDF <- expand.grid(Sex = levels(Pupfish$Sex), Pop = levels(Pupfish$Pop))
rownames(shapeDF) <- paste(shapeDF$Sex, shapeDF$Pop, sep = ".")
shapePreds <- predict(fit, shapeDF, confidence = 0.95)
plot(shapePreds, PC = TRUE, ellipse = TRUE) # generic 
plot(shapePreds, PC = TRUE, ellipse = TRUE, 
     pch = 19, col = 1:NROW(shapeDF)) # with added par arguments

groups <- interaction(Pupfish$Sex, Pupfish$Pop)
plot(fit, type = "PC") # generic
plot(fit, type = "PC", pch = 19, col = groups) # with added par arguments


## ----fig.height=5, fig.width=6------------------------------------------------
plot(fit, type = "regression", reg.type = "PredLine", 
    predictor = Pupfish$logSize, pch=19,
    col = as.numeric(groups))

## -----------------------------------------------------------------------------
PWT <- pairwise(fit, groups = interaction(Pupfish$Sex, Pupfish$Pop))
summary(PWT, confidence = 0.95)


## -----------------------------------------------------------------------------
fit2 <- lm.rrpp(coords ~ logSize * Sex * Pop, SS.type = "I", 
                data = Pupfish, print.progress = FALSE, iter = 999) 
summary(fit2, formula = FALSE)
anova(fit, fit2, print.progress = FALSE)

PW2 <- pairwise(fit2, fit.null = fit, groups = groups, 
                covariate = Pupfish$logSize, print.progress = FALSE) 
PW2
summary(PW2, confidence = 0.95, 
        test.type = "dist") # distances between slope vector lengths
summary(PW2, confidence = 0.95, 
        test.type = "dist", stat.table = FALSE)
summary(PW2, confidence = 0.95, 
        test.type = "VC",
        angle.type = "deg") # correlation between slope vectors (and angles)


## -----------------------------------------------------------------------------
D <- dist(Pupfish$coords) # inter-observation Euclidean distances
Pupfish$D <- D

fitD <- lm.rrpp(D ~ logSize + Sex*Pop, SS.type = "I", 
                data = Pupfish, print.progress = FALSE) 

summary(fitD)
summary(fit)

anova(fitD)
anova(fit)

## -----------------------------------------------------------------------------
data(PlethMorph)
fitOLS <- lm.rrpp(TailLength ~ SVL, 
                  data = PlethMorph,
                  print.progress = FALSE,
                  turbo = FALSE, verbose = TRUE)
fitGLS <- lm.rrpp(TailLength ~ SVL, 
                  data = PlethMorph, 
                  Cov = PlethMorph$PhyCov,
                  print.progress = FALSE,
                  turbo = FALSE, verbose = TRUE)

anova(fitOLS)
anova(fitGLS)

coef(fitOLS, test = TRUE)
coef(fitGLS, test = TRUE)


## ----fig.height=4, fig.width=8------------------------------------------------
Y <- as.matrix(cbind(PlethMorph$TailLength,
PlethMorph$HeadLength,
PlethMorph$TailLength,
PlethMorph$Snout.eye,
PlethMorph$BodyWidth,
PlethMorph$Forelimb,
PlethMorph$Hindlimb))
PlethMorph$Y <- Y

fitOLSm <- lm.rrpp(Y ~ SVL, data = PlethMorph,
                   print.progress = FALSE,
                  turbo = FALSE, verbose = TRUE)
fitGLSm <- lm.rrpp(Y ~ SVL, data = PlethMorph, 
                   Cov = PlethMorph$PhyCov,
                   print.progress = FALSE,
                  turbo = FALSE, verbose = TRUE)

anova(fitOLSm)
anova(fitGLSm)

sizeDF <- data.frame(SVL = sort(PlethMorph$SVL))

plot(predict(fitOLSm, sizeDF), PC= TRUE) # Correlated error
plot(predict(fitGLSm, sizeDF), PC= TRUE) # Independent error


## -----------------------------------------------------------------------------
attributes(fit)

#' Mixed Linear Model Evaluation with RRPP performed within subjects
#'
#' Function performs a mixed linear model fit over many random permutations of 
#' data, using a randomized residual permutation procedure restricted to
#' subjects.  This function is likely to evolve greater flexibility in the future.
#'
#' The function fits a mixed linear model using ordinary least squares (OLS) or 
#' generalized least squares (GLS) estimation of coefficients for fixed effects, and 
#' maximum likelihood (ML) or restricted ML (REML) for random effects, over any 
#' number of random permutations of the data, but the permutations are mostly 
#' restricted to occur with subject blocks for any model terms other than subjects.  
#' Most functionality should resemble that of \code{\link{lm.rrpp}} and \code{\link{lm.rrpp.ws}},  
#' with some caveats.  First, estimation of random effects uses the methodology of the 
#' \code{\link[lme4]{lmer}} function (Bates et al., 2015), 
#' for only one random term based on a subjects designation.  
#' Second, this means that a "random slopes" model involves an interaction between subjects and a numeric 
#' variable (not a categorical variable).  Third, although any number of fixed effects can be used,
#' only one fixed effect can interaction with subjects. Finally, the function is intended to allow
#' analysis of variance (ANOVA), like with \code{\link{lm.rrpp.ws}}, but not treated random 
#' effects as fixed.  
#' There is not a coefficients test associated with this function, as with other \code{\link{lm.rrpp}}
#' that include only fixed effects.
#' 
#' Although this function allows covariance matrices to be used for generalized least squares (GLS)
#' estimation of fixed effects, it does not currently 
#' expand the covariance mnatrix based only on subjects, as does 
#' \code{\link{lm.rrpp.ws}}.  However, that function can be used to obtain a covariance matrix for this 
#' function.  The covariance structure (Lambda matrix) among subjects is estimated via ML or REML (Bates et al., 2015) and 
#' is not impacted by the expected covariances among fixed effects, however, the expected covariances
#' will influence interactions between fixed and random effects.  The Lambda matrix is estimated as a single matrix,
#' even for multivariate data.  This matrix is obtained by projecting data onto the first component of
#' a singular value decomposition of the product between the data and a hat matrix of the the fixed effects 
#' (basically projection following a principal component analysis of model fitted values).  This is a computationally
#' proficient step that is similar to averaging covariance parameters (theta) over all variables.
#' If independent Lambda matrices are desired, then this function can be used on, e.g., each principal
#' component of data, followed by combining the coefficients.  For ANOVA, the precise estimation of Lambda
#' is not as important for tests of fixed effects, as it means a linear transformation of coefficients
#' that would be consistent across random permutations of RRPP.  It might be more important for model 
#' comparison and selection, in which case whether to use LS, ML, REML, and whether to use separate Lambda matrices
#' is a much deeper consideration.
#' 
#' A data frame (preferably an \code{\link{rrpp.data.frame}} object) must be
#' used for this function.  Not all "downstream" functions (e.g., logLik, predict) are guaranteed 
#' to work as expected, as this function is still very much under development.
#' 
#' The \code{\link{lm.rrpp}} arguments not available for this function include: 
#' full.resid, block, and SS.type.  These arguments are fixed because of
#' the within-subject blocking for tests, plus the requirement for type II SS
#' for within-subject effects.
#' 
#' @param fixed A formula for the fixed effects of linear model (e.g., y~x1+x2).  
#' @param subjects A variable that can be found in the data frame indicating the research subjects
#' for the analysis.  This variable must be in the data frame.  Is can be either numeric 
#' (if its slot in the data frame is known) or a character, e.g., "sub_id".  It is imperative that
#' it is ordered the same as the data but that the data do not have row names the same as subjects.
#' For example, the subjects variable in the data frame might be sub_id: sub1, sub1, sub1, sub2,
#' sub2, sub2, ... and the row names of the data might be obs1, obs2, obs3, obs4, obs5, obs6, ...  
#' The data do not need to have row names but the subjects variable has to be provided.
#' @param type An indication whether to use random intercepts ("intercepts") or random intercepts 
#' and slopes ("slopes") for random effects associated with subjects.
#' @param slopeTerm An optional character value (e.g., "mass") indicating which term in the data frame is to be
#' used in the estimation of random slopes.  This term should also be in the fixed formula.
#' @param estimation One of "LS", "ML", or "REML", for guiding how random effects should be estimated.
#' @param iter Number of iterations for significance testing
#' @param data A data frame for the function environment, see 
#' \code{\link{rrpp.data.frame}}.  A data frame is required for this analysis.
#' @param ... Arguments typically used in \code{\link{lm}}, \code{\link{lm.rrpp}},
#' or \code{\link{lm.rrpp.ws}} such as weights or offset, seed for permutations, etc., passed on to
#' \code{lm.rrpp.ws} for estimation of coefficients.  
#' @keywords analysis
#' @export
#' @author Michael Collyer
#' @return An object of class \code{lmm.rrpp} is a list containing the 
#' following
#' \item{call}{The matched call.}
#' \item{LM}{Linear Model objects, including data (Y), coefficients, 
#' design matrix (X), sample size
#' (n), number of dependent variables (p), dimension of data space (p.prime),
#' QR decomposition of the design matrix, fitted values, residuals,
#' weights, offset, model terms, data (model) frame, random coefficients 
#' (through permutations),
#' random vector distances for coefficients (through permutations), 
#' whether OLS or GLS was performed, 
#' and the mean for OLS and/or GLS methods. Note that the data returned 
#' resemble a model frame rather than 
#' a data frame; i.e., it contains the values used in analysis, which 
#' might have been transformed according to 
#' the formula.  The response variables are always labeled Y.1, Y.2, ..., 
#' in this frame.  Most important objects for mixed models are also found here. }
#' \item{ANOVA}{Analysis of variance objects, including the SS type, 
#' random SS outcomes, random MS outcomes,
#' random R-squared outcomes, random F outcomes, random Cohen's f-squared 
#' outcomes, P-values based on random F
#' outcomes, effect sizes for random outcomes, sample size (n), number of 
#' variables (p), and degrees of freedom for
#' model terms (df).  These objects are used to construct ANOVA tables.}
#' \item{PermInfo}{Permutation procedure information, including the number 
#' of permutations (perms), The method
#' of residual randomization (perm.method), and each permutation's sampling 
#' frame (perm.schedule), which
#' is a list of reordered sequences of 1:n, for how residuals were 
#' randomized.}
#' \item{devcomp}{Deviance components, a list of components used
#' in deviance and likelihood calculations.}
#' @seealso \code{\link{lm.rrpp}}; \code{\link{lm.rrpp.ws}}; \code{\link{measurement.error}}
#' @references Douglas Bates, Martin Maechler, Ben Bolker, Steve Walker (2015). Fitting Linear
#' Mixed-Effects Models Using lme4. Journal of Statistical Software, 67(1), 1-48.
#' doi:10.18637/jss.v067.i01.
#' 
#' \subsection{Limitations}{
#' This function currently only works for "Random Intercepts" and "Random slopes" models,
#' where the intercepts or slopes of a single subject term (random effect) are free to vary.  
#' If a Random Slopes model is used, a single covariate (numeric variable) must be specified for slopes.  
#' Generally, this is term that is also among the fixed effects.  These limitations are imposed
#' to allow ML or REML estimation.  More complex models can be entertained with \code{lmm.rrpp.ws},
#' but using only "LS" estimation.  If "LS" estimation is used in this function, results should be the same
#' with a comparable formula, using \code{lmm.rrpp.ws}
#' 
#' If a ML or REML fit is near singular (one or more covariance parameters are 0), the analysis will be arrested
#' rather than continued with some random effects rendered as 0.  This should be an indication that the model
#' is over-fit with the type of estimation used. 
#' 
#' These limitations might be addressed with future versions of this function, as theoretical advances are made.
#'  }

#' @examples 
#' \dontrun{
#' # Random intercepts models
#' 
#' data(ReptAbund)
#' fit_ls <- lmm.rrpp(fixed = reptile_abundance ~ Year, 
#' subjects = "Site",      
#' type = "intercepts", 
#' estimation = "LS", 
#' data = ReptAbund)
#' 
#' fit_ml <- lmm.rrpp(fixed = reptile_abundance ~ Year, 
#' subjects = "Site",      
#' type = "intercepts", 
#' estimation = "ML", 
#' data = ReptAbund)
#'
#' fit_reml <- lmm.rrpp(fixed = reptile_abundance ~ Year, 
#' subjects = "Site",      
#' type = "intercepts", 
#' estimation = "REML", 
#' data = ReptAbund) 
#' 
#' anova(fit_ls, error = c("Site", "Residuals"))
#' anova(fit_ml, error = c("Site", "Residuals"))
#' anova(fit_reml, error = c("Site", "Residuals"))
#' 
#' coef(fit_ls, type = "list")
#' coef(fit_ml, type = "list")
#' coef(fit_reml, type = "list")
#' 
#' # Random slopes models
#' # none should not work!
#' 
#' fit_ls <- lmm.rrpp(fixed = reptile_abundance ~ Year, 
#' subjects = "Site",      
#' type = "slopes",
#' slopeTerm = "Year", 
#' estimation = "LS", 
#' data = ReptAbund)
#' 
#' fit_ml <- lmm.rrpp(fixed = reptile_abundance ~ Year, 
#' subjects = "Site",      
#' type = "slopes",
#' slopeTerm = "Year", 
#' estimation = "ML", 
#' data = ReptAbund)
#'
#' 
#' fit_reml <- lmm.rrpp(fixed = reptile_abundance ~ Year, 
#' subjects = "Site",      
#' type = "slopes",
#' slopeTerm = "Year",  
#' estimation = "REML", 
#' data = ReptAbund) 
#' 
#' anova(fit_ls, error = c("Year:Site", "Year:Site", "Residuals"))
#' anova(fit_ml, error = c("Year:Site", "Year:Site", "Residuals"))
#' 
#' coef(fit_ls, type = "list")
#' coef(fit_ml, type = "list")
#'  }

lmm.rrpp <- function(fixed,
                     subjects,
                     type = c("intercepts", "slopes"),
                     slopeTerm = NULL,
                     estimation = c("REML", "ML", "LS"),
                     iter = 999,
                     data,
                     ...) {
  estimation <- match.arg(estimation)
  dots <- list(...)
  largs <- as.vector(names(formals(lm.rrpp.ws)))
  lm.rrpp.args <- dots[names(dots) %in% largs]
  names(lm.rrpp.args) <- lm.rrpp.args[largs %in% names(dots)]
  
  response <- fixed[[2]]
  trms <- attr(terms(fixed, keep.order = TRUE), "term.labels")
  form <- if(type == "intercepts") 
    reformulate(response = response, 
                termlabels = c(trms, subjects)) else
                  reformulate(response = response, 
                              termlabels = c(trms, subjects,
                                             paste(slopeTerm, subjects, sep = ":")))
  
  useLS <- (estimation == "LS")
  
  form <- formula(terms(form, keep.order = TRUE),
                  keep.order = TRUE)
  lm.rrpp.args$f1 <- form
  lm.rrpp.args$subjects <- subjects
  lm.rrpp.args$verbose <- FALSE
  lm.rrpp.args$data <- data
  if(isTRUE(dots$print.progress)){
    cat("\nInitial lm.rrpp.ws fit:\n ")
  }
  lm.rrpp.args$int.first <- TRUE
  
  if(useLS){
    fit <- suppressWarnings(do.call(lm.rrpp.ws, 
                                    lm.rrpp.args))
  } else {
    lm.rrpp.args$iter <- 0
    fit <- suppressWarnings(do.call(lm.rrpp.ws, 
                                    lm.rrpp.args))
    lm.rrpp.args$iter <- iter
  }
  
  B <- fit$LM$coefficients
  
  lm.rrpp.args$f1 <- fixed
  
  fixed.fit <- suppressWarnings(do.call(lm.rrpp.ws, 
                                        lm.rrpp.args))
  
  fit$call <- match.call()
  dat <- fit$LM$data
  dat$resp <- as.matrix(dat$Y[, 1, drop = FALSE])
  Y <- dat$Y
  dims <- dim(Y)
  n <- dims[1]
  p <- dims[2]
  
  Xs <- suppressWarnings(
    getModels(fixed.fit, "qr"))
  k <- length(Xs$full)
  QR <- Xs$full[[k]]
  
  rands <- if(type == "intercepts")
    paste("(1|", subjects, ")", sep = "") else
      paste("(", slopeTerm, "|", subjects, ")", sep = "")
  
  randform <- reformulate(response = response, 
                          termlabels = c(trms, rands))
  randform <- update(randform, resp ~ .)
  
  init.rand.fit <- suppressMessages(
    suppressWarnings(lmer(randform, 
                          REML = (estimation == "REML"),
                          data = dat)))
  
  Zall <- getME(init.rand.fit, 
                "Z")
  if(type == "slopes"){
    nzc <- ncol(Zall)
    Zslope <- Zall[, seq(2, nzc, 2), drop = FALSE]
    Zsub <- Zall[, seq(1, nzc - 1, 2), drop = FALSE]
  } else {
    Zsub <- Zall
    Zslope <- NULL
  }
  
  nsub <- NCOL(Zsub)
  Xff <- model.matrix(fixed.fit)
  kf <- NCOL(Xff)
  Snames <- rand.coef.names <- dimnames(Zsub)[[2]]
  if(type == "slopes"){
    a <- rep(Snames, each = 2)
    b <- rep(c("(Intercept)", slopeTerm), length(Snames))
    rand.coef.names <- paste(a, b, sep = ":")
  }
  
  sub1 <- paste(subjects, levels(fit$subjects), sep = "")
  sub2 <- if(type == "slopes") 
    paste(slopeTerm, sub1, sep = ":") else NULL
  xnames <- c(colnames(Xff), sub1, sub2)
  
  Bfill <- matrix(0, nrow = length(xnames), 
            ncol = p)
  dimnames(Bfill) <- list(xnames, colnames(dat$resp))
  
  Bfill[rownames(Bfill) %in% rownames(B), ] <- B
  B <- Bfill
  rm(Bfill)
  
  if(type == "slopes"){
    Bff <- as.matrix(B[1:kf, , drop = FALSE])
    Bs <- as.matrix(B[-(1:kf), , drop = FALSE])
    ks <- NROW(Bs)
    sseq <- NULL
    for(i in 1:(ks/2)) sseq <- c(sseq, c(i, i + ks/2))
    Bs <- as.matrix(Bs[sseq, , drop = FALSE])
    B <- as.matrix(rbind(Bff, Bs))
  }
  
  if(p > 1){
    H <- tcrossprod(QR$Q)
    yp <- ordinate(Y, A = H, rank. = 1)$x
    ft <- refit(init.rand.fit, yp)
  } else ft <- init.rand.fit
  
  if(isSingular(ft, tol = 1e-7))
    stop("Model is near-singular.\n",  
    "Try a different estimation method or a simpler model.\n",
         call. = FALSE)
  
  Lambda <- getME(ft, "Lambda")
  Zall <- Zall %*% Lambda
  
  if(type == "slopes"){
    nzc <- ncol(Zall)
    Zslope <- Zall[, seq(2, nzc, 2), drop = FALSE]
    Zsub <- Zall[, seq(1, nzc - 1, 2), drop = FALSE]
  } else {
    Zsub <- Zall
    Zslope <- NULL
  }
  
  if(!useLS){
    
    
    if(fit$LM$gls) {
      if(!is.null(fit$LM$Cov)) {
        Pcov <- getModelCov(fit, "Pcov")
        Y <- Pcov %*% Y
      } else {
        w <- sqrt(fit$LM$weights)
        Y <- Y * w
      }
    }
    
    subjFactor <- unlist(dat[[which(names(dat) == subjects)]])
    ns <- nlevels(subjFactor)
    subjLevels <- levels(subjFactor)
    
    if(type == "slopes") {
      slopeVar <- unlist(dat[[which(names(dat) == slopeTerm)]])
      if(is.factor(slopeVar) && estimation != "LS"){
        stop("\nThe slope term must be numeric for ML or REML fits...",
             "\nThis is a requirement of the lme4::lmer function dependency. ",
             "\nConsider using dummy (binary) variables for two-group ",
             "comparisons. \nSee the lme4::dummy function for details. \n",
             call. = FALSE)
      }
    }
    
    
    for(i in 1:2){
      reduced <- lapply(Xs$reduced, 
                        function(x) as.matrix(x$X))
      full <- lapply(Xs$full, 
                     function(x) as.matrix(x$X))
    }
    Xs$reduced <- reduced
    Xs$full <- full
    rm(reduced, full)
    
    Xs$reduced[[k + 1]] <- Xs$full[[k]]
    Xs$full[[k + 1]] <- Xs$full[[k]]
    names(Xs$reduced)[[k + 1]] <- 
      names(Xs$full)[[k + 1]] <- subjects
    if(type == "slopes"){
      Xs$reduced[[k + 2]] <- Xs$full[[k]]
      Xs$full[[k + 2]] <- Xs$full[[k]]
      names(Xs$reduced)[[k + 2]] <- 
        names(Xs$full)[[k + 2]] <- paste(
          slopeTerm, subjects, sep = ":")
    }
    
    Hbs.reduced <- Hbs.full <- 
      XZs.reduced <- XZs.full <- list()
    
    for(i in 1:k){
      Hbs.reduced[[i]] <- getLMM_Hb(Xs$reduced[[i]], Zsub) 
      Hbs.full[[i]] <- getLMM_Hb(Xs$full[[i]], Zsub)
      XZs.reduced[[i]] <- cbind(Xs$reduced[[i]], Zsub)
      XZs.full[[i]] <- cbind(Xs$full[[i]], Zsub)
      
    }
    
    
    # If Intercepts model
    
    if(type == "intercepts"){
      Hbs.reduced[[k+1]] <- getLMM_Hb(Xs$full[[k]])
      XZs.reduced[[k+1]] <- Xs$full[[k]]
      Hbs.full[[k+1]] <- getLMM_Hb(Xs$full[[k]], Zsub)
      XZs.full[[k+1]] <- cbind(Xs$full[[k]], Zsub)
    }
    
    # If slopes model
    
    if(type == "slopes"){
      Hbs.reduced[[k+1]] <- getLMM_Hb(Xs$full[[k]], Zslope)
      XZs.reduced[[k+1]] <- cbind(Xs$full[[k]], Zslope)
      Hbs.full[[k+1]] <- getLMM_Hb(Xs$full[[k]], Zall)
      XZs.full[[k+1]] <- cbind(Xs$full[[k]], Zall)
      Hbs.reduced[[k+2]] <- getLMM_Hb(Xs$full[[k]], Zsub)
      XZs.reduced[[k+2]] <- cbind(Xs$full[[k]], Zsub)
      Hbs.full[[k+2]] <- getLMM_Hb(Xs$full[[k]], Zall)
      XZs.full[[k+2]] <- cbind(Xs$full[[k]], Zall)
      
    }
    
    
    Hs.full <- Map(function(x, h){
      x %*% h
    }, XZs.full, Hbs.full)
    
    Hs.reduced <- Map(function(x, h){
      x %*% h
    }, XZs.reduced, Hbs.reduced)
    
    Fitted <- lapply(Hs.reduced, function(h){
      as.matrix(h %*% Y)
    })
    
    Resid <- lapply(Fitted, function(f) Y - f)
    
    
    ### start RRPP
    
    ind <- perm.index(n, iter, block = subjFactor,
                      seed = dots$seed)
    
    ind_s <- perm.index(n, iter, block = NULL,
                        seed = dots$seed)
    
    perms <- length(ind)
    kk <- length(XZs.full)
    
    XZ_full <- XZs.full[[kk]]
    XZ_null <- XZ_full[, 1, drop = FALSE] # need to consider no intercept
    Hb_full <- Hbs.full[[kk]]
    QRnull <- QRforX(matrix(1, n, 1))
    Hb_null <- getHb(QRnull)
    Qnull <- QRnull$Q
    
    B <- as.matrix(Hb_full %*% Y)
    
    dimnames(B) <- list(c(colnames(fixed.fit$LM$X),
                          rand.coef.names),
                        colnames(Y))
    U <- as.matrix(B[-seq_len(ncol(fixed.fit$LM$X)), , drop = FALSE])
    UtU_det <- try(det(crossprod(U)), silent = TRUE)
    if(inherits(UtU_det, "try-error"))
      UtU_det <- NA
    
    sub_no <- which(names(Xs$full) == subjects)
    
    Result <- as.list(array(NA, perms))
    names(Result) <- names(ind)
    
    H_residual <- XZ_full %*% Hb_full 
    H_residual <- H_residual - diag(n)
    
    ss <- function(hf, hr, y) {
      ssr <- sum(crossprod(hr, y)^2)
      ssf <- sum(crossprod(hf, y)^2)
      r <- H_residual %*% y
      ssm <- sum(r^2)
      c(ssf - ssr, ssm)
    } 
    
    trmnms <- names(Xs$full)
    
    for(i in 1:perms){
      s <- ind[[i]]
      Y_i <- lapply(1:kk, function(j){
        sj <- if(j == sub_no) ind_s[[i]] else s
        Fitted[[j]] + as.matrix(Resid[[j]][sj, , drop = FALSE])
      })
      yy <- Y[s, , drop = FALSE]
      
      res <- vapply(1:max(1, kk), function(j){
        ss(Hs.full[[j]], Hs.reduced[[j]], Y_i[[j]])
      }, numeric(2))
      
      dimnames(res) <- list(c("SS", "RSS"), trmnms)
      
      Ralls <- Map(function(y){
        H_residual %*% y}, Y_i)
      
      RSS <- sapply(Ralls, function(r) sum(r^2))
      
      Rm <- H_residual %*% yy 
      RSS.model <- sum(Rm^2)
      F0 <- fastFit(Qnull, yy, n, p)
      R0 <- yy - F0
      TSS <- sum(R0^2)
      Result[[i]] <- list(
        SS = res,
        RSS.model = RSS.model,
        TSS = TSS
      )
      
    }
    
    SS <- sapply(Result, function(x) x$SS[1,])
    RSS <- sapply(Result, function(x) x$SS[2,])
    RSS.model <- sapply(Result, function(x) x$RSS.model)
    RSS.model <- matrix(RSS.model, kk, perms, byrow = TRUE)
    TSS <- sapply(Result, function(x) x$TSS)
    TSS <- matrix(TSS, kk, perms, byrow = TRUE)
    dimnames(TSS) <- dimnames(RSS.model) <- dimnames(SS)
    Df <- fit$ANOVA$df
    MS <- SS / Df[1:kk]
    Rsq <- SS / TSS
    
    fit$ANOVA$SS <- SS
    fit$ANOVA$RSS <- RSS
    fit$ANOVA$RSS.model <- RSS.model
    fit$ANOVA$TSS <- TSS
    fit$ANOVA$Rsq <- Rsq
    
    fit$PermInfo$perms <- perms
    
  }
  
  fit$LM$LMM <- TRUE
  fit$LM$X <- fixed.fit$LM$X
  fit$LM$Z <- getME(init.rand.fit, "Z")
  fit$LM$Lambda <- Lambda
  fit$LM$QR <- NULL
  fit$LM$coefficients <- B
  
  fit$LM$coef.fixed <- seq_len(ncol(fixed.fit$LM$X))
  fit$LM$coef.random <- seq_len(NROW(B))[-fit$LM$coef.fixed]

  
  if(!useLS){
    if(fit$LM$gls) {
      fit$L$gls.fitted <- XZ_full %*% fit$LM$coefficients
      fit$L$gls.residuals <- Y - fit$LM$gls.fitted
    } else {
      fit$LM$fitted <- XZ_full %*% fit$LM$coefficients
      fit$LM$residuals <- Y - fit$LM$fitted
    }
  } 

  fit$LM$fixed <- fixed
  fit$LM$estimation <- estimation
  fit$LM$ranef.type <- type
  fit$LM$ranef.slopeTerm <- slopeTerm
  fit$LM$lm_form <- form
  fit$LM$lmer_form <- randform
  fit$LM$cnms <- init.rand.fit@cnms
  
  # deviance component list
  fit$devcomp <- list()
  fit$devcomp$UtU_det <- if(!useLS) 
    UtU_det else NA
  
  X <- QRforX(fit$LM$X)$X
  nX <- NCOL(X)
  if(!is.null(fit$LM$weights)) X <- X / sqrt(fit$LM$weights)
  Pcov <- try(getModelCov(fit, "Pcov"), silent = TRUE)
  if(!inherits(Pcov, "try-error")) X <- Pcov %*% X
  
  ZX_full <- cbind(fit$LM$Z %*% fit$LM$Lambda, X)
  R <- resid(fit)
  if(!inherits(Pcov, "try-error")) R <- Pcov %*% R
  if(!is.null(fit$LM$weights)) R <- R / sqrt(fit$LM$weights)
  
  Omega <- if(!inherits(Pcov, "try-error")) 
    fast.solve(tcrossprod(Pcov)) else 
      if(!is.null(fit$LM$weights)) diag(fit$LM$weights) else
        diag(n)
  ldO <- log(det(Omega))
  
  nZX <- ncol(ZX_full) 
  nZ <- nZX - nX
  
  RtR_det <- try(det(crossprod(R)), silent = TRUE)
  if(inherits(RtR_det, "try-error")) RtR_det <- NA
  fit$devcomp$RtR_det <- RtR_det
  
  cpZX_full <- crossprod(ZX_full)
  cpZX_full[1:nZ, 1:nZ] <- cpZX_full[1:nZ, 1:nZ] +
    diag(nZ)
  Ch <- try(chol(cpZX_full), silent = TRUE)
  if(inherits(Ch, "try-error")) Ch <- NA
  if(!all(is.na(Ch))) {
    fit$devcomp$ldLZ <- 2 * log(det(Ch[1:nZ, 1:nZ]))
    cpX <- crossprod(ZX_full[, -(1:nZ)]) -
      crossprod(Ch[1:nZ, -(1:nZ)])
    fit$devcomp$ldLX <- log(det(cpX))
  } else fit$devcomp$ldLZ <- fit$devcomp$ldLX <- NA
  fit$devcomp$ldO <- ldO
  
  
  # out
  class(fit) <- c("lmm.rrpp", class(fit))
  fit$call <- match.call()
  
  fit
}


getLMMQ <- function(XZ, Hb){
  QR <- QRforX(XZ %*% Hb)
  Q <- QR$Q[, 1:QR$rank, drop = FALSE]
  Q
}

# Not used but could in the future
getSubBlocks <- function(flmer){
  Zt <- getME(flmer, "Ztlist")[[1]]
  block <- as.factor(Zt@i + 1)
  block
}

# Not used but could in the future
indexZtoX <- function(X, Z){
  Z <- Z0 <- as.matrix(Z)
  kx <- ncol(X)
  kz <- ncol(Z)
  nx <- nrow(X)
  nz <- nrow(Z)
  if(kx != kz || nx != nz)
    stop("Dimensions of matrices do not match.\n",
         call. = FALSE)
  res <- array(NA, kx)
  z.index <- 1:kz
  
  for(i in seq_len(kx)){
    x <- X[, i, drop = FALSE]
    r <- sapply(1:kz, function(j){
      identical(x, Z[, j])
    })
    a <- which(r)
    res[i] <- z.index[a]
    Z <- as.matrix(Z[, -a, drop = FALSE])
    z.index <- z.index[-a]
    kz <- kz - 1
  }
  
  res
}

#' Diagnostic cross-validation tool including ordination based on fitted values
#'
#' Function performs a leave-one-out cross-validation estimate (method = "fit")
#' or ordination scores from estimates (method = "PC").  
#' The latter can be helpful for determining if apparent
#' group differences in ordination plots arise merely from data dimensionality.  
#' The choice of method
#' essentially chooses one of two functions.
#' 
#' The fit method is standard, using n - 1 observations as training
#' data, and estimating the response for 1 datum, not included in the training
#' data, based on the model design of an original model fit.  This method
#' provides alternative fitted values than the original fit.
#' 
#' The PC method uses the strategy of Thioulouse et al. (2021) to perform N
#' ordinations for N observations, in which each of the N observations are left
#' out of the estimation of linear model coefficients, but the vector of data for
#' the left-out observation is projected on the eigenvectors of the fitted values 
#' obtained from the leave-one-out cross-validation (jackknife) strategy.
#' The purpose of this diagnostic tool is to determine whether apparent "group differences"
#' in an ordination plot (using the function, \code{\link{ordinate}}) are
#' because of high-dimensional data (number of variables exceed number of observations)
#' rather than real differences.  An apparent group difference is common for high-dimensional
#' data, when variables are far greater in number than observations (Cardini et al., 2019).
#' However, leave-one-out cross-validation can help elucidate whether an observed visual 
#' difference is spurious.
#' 
#' This function differs from the strategy of Thioulouse et al. (2021) in two important 
#' ways.  First, this function uses the linear model design from a \code{\link{lm.rrpp}}
#' fit, and can contain any number of independent variables, rather than a single factor 
#' for groups.  Second, after obtaining leave-one-out cross-validated scores, a Procrustes
#'  alignment between cross-validated scores and "observed" (real) scores is performed, which 
#' minimizes summed squared distances between the alternative ordinations.  This latter
#' step assures comparisons are appropriate.
#' 
#' The type = "PC" plot from \code{\link{plot.lm.rrpp}} has the same scores as obtained
#' from ordinate(Y, A = H), using the \code{\link{ordinate}} function, where H is a hat
#' matrix (that can be calculated from \code{\link{plot.lm.rrpp}} output), and Y is a matrix 
#' of data.  This function updates H for every possible case that one row of Y is left out
#' (meaning the rotation matrix from \code{\link{ordinate}} is updated N times).  If
#' the H matrix is robust in spite of dropped data and design matrix parameters, the result
#' will be similar to the original ordination.  If apparent group differences are spurious,
#' H will tend to change, as will data projections.
#' 
#' The functions \code{\link{summary.looCV}} and \code{\link{plot.looCV}} are essential for 
#' evaluating results.  These support functions compare eigenvalues and 
#' projected scores, between observed and cross-validated cases.  
#' 
#' This function should be viewed as a diagnostic tool and not as a data transformation tool!
#' The cross-validated scores will not retain Euclidean distances among observations.  This
#' could cause problems in analyses that substitute cross-validated scores as data.
#' 
#' 
#' @param fit A \code{\link{lm.rrpp}} fit.
#' @param method Whether to find cross-validated fitted values (fit) or 
#' principal component scores (PC) of fitted values.
#' @param ... Arguments passed to \code{\link{ordinate}}
#' @keywords analysis
#' @export
#' @author Michael Collyer
#' @return An object of class \code{looCV} is a list containing 
#' the following
#' \item{d}{List of eigenvalues, for observed and cross-validated cases.}
#' \item{scores}{List of principal component scores, 
#' for observed and cross-validated cases.}
#' @references Thioulouse, J., Renaud, S., Dufour, A. B., & Dray, S. (2021). 
#' Overcoming the Spurious Groups Problem in Between-Group PCA. 
#' Evolutionary Biology, In press.
#' @references Cardini, A., O’Higgins, P., & Rohlf, F. J. (2019). Seeing distinct groups 
#' where there are none: spurious patterns from between-group PCA. 
#' Evolutionary Biology, 46(4), 303-316.
#' 
#' @seealso \code{\link{summary.looCV}}, 
#' \code{\link{plot.looCV}}, \code{\link{looCV_table}}
#' @examples
#' 
#' # Example with real group differences
#' 
#' data(Pupfish)
#' fit <- lm.rrpp(coords ~ Pop * Sex, 
#' data = Pupfish, iter = 0)
#' CV1f <- looCV(fit)
#' 
#' summary(CV1f)
#' 
#' CV1p <- looCV(fit, method = "PC")
#' 
#' summary(CV1p)
#' 
#' group <- interaction(Pupfish$Pop, Pupfish$Sex)
#' plot(CV1p, flip = 1, pch = 19, col = group)
#' 
#' # Example with apparent but not real group differences
#' 
#' n <- NROW(Pupfish$coords)
#' p <- NCOL(Pupfish$coords)
#' set.seed(1001)
#' Yr <- matrix(rnorm(n * p), n, p) # random noise
#' 
#' fit2 <-lm.rrpp(Yr ~ Pop * Sex, 
#' data = Pupfish, iter = 0)
#' CV2p <- looCV(fit2, method = "PC")
#' summary(CV2p)
#' group <- interaction(Pupfish$Pop, Pupfish$Sex)
#' plot(CV2p, pch = 19, col = group) 
#' 
looCV <- function(fit, method = c("fit", "PC"),
                  ...){
  method <- match.arg(method)
  
  out <- list(fitted = NULL, cv_fitted = NULL,
              residuals = NULL, cv_residuals = NULL,
              RSS = NULL, cv_RSS = NULL,
              MSE = NULL, cv_MSE = NULL,
              d = NULL, scores = NULL,
              df_model = NULL, df_residual = NULL,
              method = method, 
              model = NULL)
       
  out$model <- deparse(substitute(fit))
  Df <- fit$ANOVA$df
  dfl <- length(Df)
  df_residual <- out$df_residual <- Df[dfl - 1]
  df_model <- out$df_model <- sum(Df[-c(dfl - 1, dfl)])
  
  if(method == "PC"){
    res <- looPCAll(fit, ...)
    d <- list(obs = res$raw$d, cv = res$cv$d)
    scores <- list(obs = res$raw$x, cv = res$cv$x)
    out$d <- d
    out$scores <- scores
  }
  
  if(method == "fit"){
    gls <- fit$LM$gls
    Pcov <- try(getModelCov(fit, type = "Pcov"),
                silent = TRUE)
    if(gls && inherits(Pcov, "try-error")){
      w <- sqrt(fit$LM$weights)
      Pcov <- diag(1/w)
    }
    n <- fit$LM$n
    
    yhat <- fitted(fit)
    cv_yhat <- looCVAll(fit)
    res <- resid(fit)
    cv_res <- fit$LM$Y - cv_yhat
    RSS <- if(gls) sum((Pcov %*% res)^2) else
      sum(res^2)
    cv_RSS <- if(gls) sum((Pcov %*% cv_res)^2) else
      sum(cv_res^2)
    out$fitted <- yhat
    out$cv_fitted <- cv_yhat
    out$residuals <- res
    out$cv_residuals <- cv_res
    out$RSS <- RSS
    out$cv_RSS <- cv_RSS
    out$MSE <- RSS / df_residual
    out$cv_MSE <- cv_RSS / df_residual
    
  }

  class(out) <- "looCV"
  out
}

#' Table for looCV comparisons
#'
#' Function simply produces a table of MSE and cross-validated
#' MSE for any number of compared models.  The table is sorted
#' from lowest to highest cross-validated MSE.  The looCV objects
#' must use method = 'fit' or a table cannot be generated.

#' @param ... Any number of \code{\link{looCV}} objects
#' @keywords analysis
#' @export
#' @author Michael Collyer

#' @seealso \code{\link{looCV}}, \code{\link{model.comparison}}
#' @examples
#' 
#' # Example with snakeHS data
#' 
#' data(snakeHS)
#' fitS <- lm.rrpp(HS ~ SVL, data = snakeHS,
#'                 iter = 0)
#' fitSR <- lm.rrpp(HS ~ SVL + Region, data = snakeHS,
#'                  iter = 0)
#' fitSxR <- lm.rrpp(HS ~ SVL * Region, data = snakeHS,
#'                   iter = 0)
#' 
#' CVS <- looCV(fitS)
#' CVSR <- looCV(fitSR)
#' CVSxR <- looCV(fitSxR)
#' 
#' summary(CVS)
#' summary(CVSR)
#' summary(CVSxR)
#' 
#' looCV_table(CVS, CVSR, CVSxR)
#' 
looCV_table <- function(...){
  dots <- list(...)
  isfit <- function(x) is.null(x$d)
  check <- unlist(lapply(dots, inherits, "looCV"))
  if(any(!check)) stop("\nObjects must be looCV of lm.rrpp fits.\n")
  check <- unlist(lapply(dots, isfit))
  if(any(!check)) stop("\nAll looCV objects must use method = 'fit'.\n")
  
  dot.names <- lapply(dots, function(x) x$model)
  
  getMSE <- function(x) {
    res <- c(x$df_model, x$df_residual,
             x$MSE, x$cv_MSE)
    names(res) <- c("Df Model", "Df Residual",
                    "Observed MSE",
                    "Cross-validated MSE")
    res
  }
    
  out <- as.data.frame(
    t(sapply(dots, getMSE)))
  rownames(out) <- dot.names
  
  out <- out[order(out[, 4]), ]
  out
}

#' Deprecated functions in RRPP
#'
#' The following function has been deprecated in RRPP
#'
#' This function has been deprecated. Use \code{\link{prep.lda}} instead.
#' 
#' @export
classify <- function(){
  .Deprecated("classify")
}

#' Pairwise comparisons of model effects 
#'
#' Function generates pairwise statistics for comparing model fits and
#' returns important statistics for hypothesis tests.
#'
#' The function statistically compares the effect sizes of two or more models fit
#' and evaluated using RRPP.  Input for the function is a list of fitted models
#' of the class 'model.comparison', whose options included type = 'Z' and 
#' verbose = TRUE when the models were compared with that function.
#' 
#' A two-sample test is performed on each pair of models, comparing the strength 
#' of model fits to one another (Collyer and Adams 2025). This might be used to 
#' compare the strength of fit of the data to differing statistical models (as in 
#' model selection) or for comparing the fit across differing 
#' datasets for the same model to determine whether the strength of fit in one dataset
#' is greater than that found in another (see Collyer and Adams 2025). In the latter
#' case, one is advised to include a vector containing the sample sizes of each
#' dataset, so that two-sample tests may account for differences in sample size. 
#' 
#' @param ... Either an object of class \code{\link{model.comparison}}, 
#' or several objects of class \code{\link{lm.rrpp}}.  If the former, arguments of
#' type = 'Z' and verbose = TRUE are required.  If the latter, a model comparison analysis
#' will first be performed with these arguments.
#' @param nsamp An optional vector containing the sample sizes for each model fit
#' @param two.tailed A logical value to indicate whether a two-tailed test (typical and default) should be performed.
#' @param predictor An optional vector to be passed to \code{\link{model.comparison}}, if used.
#' @param tol An optional value to be passed to \code{\link{model.comparison}}, if used.
#' @param pc.no An optional value to be passed to \code{\link{model.comparison}}, if used.
#' @param gls.null An optional logical value to be passed to \code{\link{model.comparison}}, if used.
#' @keywords analysis
#' @export
#' @author Dean Adams and Michael Collyer
#' 
#' @return A list containing the following
#' \item{sample.z}{A vector of model effect sizes.}
#' \item{pairwise.z}{A matrix of pairwise test statistics comparing model effect sizes.}
#' \item{pairwise.P}{A matrix of pairwise significance levels.}
#' \item{tails}{Number of tails used for P-value calculation.}
#' @references Collyer, M.L., and D.C. Adams. 2025. Permutational Biometry. 
#'   Volume 1: Univariate Data. Iowa State University Digital Press. (Forthcoming).
#' @examples
#' \dontrun{
#'  data(Pupfish)
#'  Pupfish$logSize <- log(Pupfish$CS)
#'  fit1 <- lm.rrpp(coords ~ logSize, data = Pupfish,
#'  print.progress = FALSE)
#'  fit2 <- lm.rrpp(coords ~ Pop, data = Pupfish,
#'  print.progress = FALSE)
#'  fit3 <- lm.rrpp(coords ~ Sex, data = Pupfish, 
#'  print.progress = FALSE)
#'  fit6 <- lm.rrpp(coords ~ logSize + Sex * Pop, data = Pupfish, 
#'  print.progress = FALSE)
#'  Mod.C <- model.comparison(fit1, fit2, fit3, fit6,
#'  pc.no = 4, type = "Z", verbose = TRUE)
#'  res <- pairwise.model.Z(Mod.C)
#'  summary(res, stats.table = TRUE)
#'  summary(res, stats.table = FALSE)
#'  }

pairwise.model.Z <- function(..., 
                             nsamp = NULL, 
                             two.tailed = TRUE,
                             predictor = NULL,
                             tol = NULL,
                             pc.no = NULL,
                             gls.null = FALSE
                             ){
  dots <- list(...)
  if(length(dots) == 1){
    if(!inherits(dots[[1]], "model.comparison"))
      stop("\nNot a class model.comparison object.\n", call. = FALSE)
    MC <- dots[[1]]
  } else {
    MC <- model.comparison(..., type = "Z", verbose = TRUE,
                           predictor = predictor, pc.no = pc.no,
                           tol = tol, gls.null = gls.null)
  }
  rm(dots)
  if(is.null(MC$random.logL))
    stop("\nThe model.comparison analysis requires type = 'Z' and verbose = TRUE.\n",
         call. = FALSE)
  
  dists <- MC$random.logL
  k <- length(dists)
  list.names <- names(dists)
  if(is.null(list.names)) list.names <- seq(1:k)
  sdn <- function(x) sqrt(sum((x - mean(x))^2) / length(x))
  k.combn <- combn(k, 2, simplify = FALSE)
  bct <- lapply(dists, function(x) powerTrans(x)$transformed)
  tails <- if(two.tailed) 2 else 1
  list.drs <- sapply(1:k, function(j) bct[[j]][1] - mean(bct[[j]])) 
  list.sds <- sapply(1:k, function(j) sdn(bct[[j]]))
  list.zs <- sapply(1:k, function(j) effect.size(dists[[j]], center=TRUE))
  z12 <- sapply(1:length(k.combn), function(j){
    x <- k.combn[[j]]
    a <- x[1]
    b <- x[2]
    r1 <- list.drs[a]
    r2 <- list.drs[b] 
    sd1 <- list.sds[a]
    sd2 <- list.sds[b]
    if (is.null(nsamp)) {n1 <- n2 <- 1}
    if (!is.null(nsamp)) {n1 <- nsamp[a]
      n2 <- nsamp[b]}
    (r1 - r2)/sqrt(sd1^2 * (2 * n1 / ((n1 + n2))) + 
                     sd2^2 * (2 * n2 / ((n1 + n2))))
  }) 
    z12.p <- sapply(1:length(z12), function(j) 
      pnorm(abs(z12[[j]]), lower.tail = FALSE) * tails)
  d <- rep(0, k)
  names(d) <- list.names
  D <-dist(d)
  z12.pw <- p12.pw <- D
  for(i in 1:length(z12)) z12.pw[i] <-z12[i]
  for(i in 1:length(z12)) p12.pw[i] <-z12.p[i]
  names(list.zs) <- names(list.sds) <-list.names
  pairwise.z <- as.matrix(z12.pw)
  pairwise.P <- as.matrix(p12.pw)
  diag(pairwise.P) <- 1
  
  out <- list(sample.z = list.zs,
              pairwise.z = abs(pairwise.z),
              pairwise.P = pairwise.P, 
              tails = tails)
  
  attr(out, "class") <- "pairwise.model.Z"
  out
}

#' Evaluation of measurement error for two or more multivariate measurements, 
#' for common research subjects
#'
#' Function performs analyses concerned with the repeatability (reliability) of multivariate data 
#' (measurements) collected from the same research subjects.  Although there is no 
#' requirement for repeated measurements on all research subjects, the analysis assumes
#' that multiple observations are made. 
#' 
#' This function performs analyses as described in Collyer and Adams (2024)
#'  to assess systematic and random components of 
#' measurement error (ME).  It basically performs ANOVA with RRPP,
#' but with different restricted randomization strategies.  The reliability of research subject variation 
#' can be considered by restricting randomization within replicates; the consistency of replicate measures
#' can be considered by restricting randomization within subjects.
#' Inter-subject variation remains constant across all random permutations within subjects and 
#' inter-replicate variation remains constant across all random permutations within replicates.  Type II
#' sums of squares and cross-products (SSCP) are calculated to assure conditional estimation.
#' 
#' The results include univariate-like statistics based on dispersion of values and
#' eigenanalysis performed on a signal to noise matrix product of SSCP matrices 
#' (sensu Bookstein and Mitteroecker, 2014) 
#' including the inverse of the random component of ME and the systematic
#' component of ME.  The multivariate test is a form of multivariate ANOVA (MANOVA), using
#' RRPP to generate sampling distributions of the major eigenvalue (Roy's maximum root).
#' Likelihood-ratio tests can also be performed using \code{\link{lr_test}}.
#' 
#' Intraclass correlation coefficients (ICC) can also be 
#' calculated (using \code{\link{ICCstats}}), 
#' both based on dispersion of values and 
#' covariance matrices, as descriptive statistics.  
#' Details are provided in \code{\link{ICCstats}}.
#'
#' 
#' @param data A required data frame, either of class \code{\link{data.frame}} or
#' class \code{\link{rrpp.data.frame}}.  This function cannot be used without a
#' data frame.  All arguments for data and variables are names that must exist in the data frame.
#' @param Y A name for a matrix (n x p) of data for n observations and p variables that can be found
#' in the data frame.  For example, Y = "morphData".
#' @param subjects A name for a vector or factor of research subjects, found within the data frame
#'(each subject should occur twice or more).  The length of the vector in the data frame must equal the 
#' number of observations and will be coerced into a factor.  For example, subjects = "ID".
#' @param replicates A name for a vector or factor for replicate measurements for research subjects, found
#' within the data frame.  The length of the vector  in the data frame must equal the number of observations and will 
#' be coerced into a factor.  For example, replicates = "Rep".
#' @param groups An optional name for a vector in the data frame, coercible to factor, to be included 
#' in the linear model (as an interaction with replicates).  This would be of interest if one 
#' were concerned with systematic ME occurring perhaps differently among certain strata within the data.  
#' For example, systematic ME because of an observer bias might only be observed with females or males,
#' in which case the argument might be: groups = "Sex".
#' @param groups.first A logical value for whether to use groups as a term before subjects, so that it can be included
#' in an ANOVA table (if TRUE).  Otherwise, a group effect is likely subsumed by a subject effect, since subjects are 
#' unique to groups.
#' @param iter Number of iterations for significance testing
#' @param seed An optional argument for setting the seed for random 
#' permutations of the resampling procedure.
#' If left NULL (the default), the exact same P-values will be found 
#' for repeated runs of the analysis (with the same number of iterations).
#' If seed = "random", a random seed will be used, and P-values will vary.  
#' One can also specify an integer for specific seed values,
#' which might be of interest for advanced users.
#' @param multivariate Logical value for whether to include multivariate analyses.  Intraclass correlation 
#' matrices and relative eigenanalysis are based on products of sums of squares and cross-products (SSCP)
#' matrices, some of which must be inverted and potentially require
#'  significant computation time.  If FALSE, only statistics based on dispersion of values are calculated.
#' @param use.PCs A logical argument for whether to use the principal components of the data.  
#' This might be helpful for relative eigenanalysis, and if p > n, 
#' in which case inverting singular covariance matrices would not be possible.
#' @param tol A value indicating the magnitude below which 
#' components should be omitted, if use.PCs is TRUE. (Components are omitted if their 
#' standard deviations are less than or equal to tol times the 
#' standard deviation of the first component.)  See \code{\link{ordinate}} for more details.
#' @param Parallel The same argument as in \code{\link{lm.rrpp}} to govern parallel processing (
#' either a logical value -- TRUE or FALSE -- or the number of threaded cores to use).  See \code{\link{lm.rrpp}} 
#' for additional details.
#' @param turbo Logical value for whether to suppress coefficient estimation in RRPP iteration,
#' thus turbo-charging RRPP.
#' @param print.progress A logical value to indicate whether a progress 
#' bar should be printed to the screen.
#' @param verbose A logical value to indicate if all the output from an
#' \code{\link{lm.rrpp}} analysis should be retained.  If FALSE, only the needed
#' output for summaries and plotting is retained.
#' @export
#' @keywords analysis
#' @author Michael Collyer
#' @return Objects of class "measurement.error" return the same objects
#' as a \code{\link{lm.rrpp}} fit, plus a list of the following:
#'  \item{AOV}{Analysis of variance to test for systematic error, based on dispersion of values.}
#'  \item{mAOV}{Multivariate AOV based on product of the inverse of the random component (SSCP) of ME
#'  times the systematic component of ME.}
#'  \item{SSCP}{The sums of squares and cross-products matrices for model effects.}
#'  \item{SSCP.ME.product}{The products of the inverse of the random ME SSCP and the SSCP matrices
#'  for systematic ME,.  These are the same matrix products used for eigenanalysis.  
#'  This is the observed matrix.}
#'  \item{SSCP.ME.product.std}{A list of the symmetric forms of standardized SSCP.ME.products 
#'  that yield orthogonal eigenvectors.}

#' @references Collyer, M.L. and D.C. Adams.  2024. Interrogating Random and Systematic Measurement Error 
#' in Morphometric Data. Evolutionary Biology, 51, 179–20.
#' @references Bookstein, F.L., & Mitteroecker, P. (2014). Comparing covariance matrices by relative eigenanalysis, 
#' with applications to organismal biology. Evolutionary Biology, 41(2), 336-350.

#' @seealso \code{\link{lm.rrpp.ws}}, \code{\link{manova.update}}, 
#' \code{\link{lr_test}}

#' @examples
#' \dontrun{
#' # Measurement error analysis on simulated data of fish shapes
#' 
#' data(fishy)
#' 
#' # Example two digitization replicates of the same research subjects
#' rep1 <- matrix(fishy$coords[1,], 11, 2, byrow = TRUE)
#' rep2 <- matrix(fishy$coords[61,], 11, 2, byrow = TRUE)
#' plot(rep1, pch = 16, col = gray(0.5, alpha = 0.5), cex = 2, asp = 1)
#' points(rep2, pch = 16, col = gray(0.2, alpha = 0.5), cex = 2, asp = 1)
#' 
#' # Analysis unconcerned with groups 
#' 
#' ME1 <- measurement.error(
#'   Y = "coords",
#'   subjects = "subj",
#'   replicates = "reps",
#'   data = fishy)
#' 
#' anova(ME1)
#' ICCstats(ME1, subjects = "Subjects", with_in = "Systematic ME")
#' plot(ME1)
#' 
#' # Analysis concerned with groups 
#' 
#' ME2 <- measurement.error(
#'   Y = "coords",
#'   subjects = "subj",
#'   replicates = "reps",
#'   groups = "groups",
#'   data = fishy)
#' 
#' anova(ME2)
#' ICCstats(ME2, subjects = "Subjects", 
#'   with_in = "Systematic ME", groups = "groups")
#' P <- plot(ME2)
#' focusMEonSubjects(P, subjects = 18:20, shadow = TRUE)
#' }
#' 
measurement.error <- function(data, 
                              Y, 
                              subjects, 
                              replicates, 
                              groups = NULL,
                              groups.first = FALSE,
                              iter = 999, 
                              seed = NULL,
                              multivariate = FALSE,
                              use.PCs = TRUE, 
                              tol = 0.001, 
                              Parallel = FALSE,
                              turbo = TRUE,
                              print.progress = FALSE,
                              verbose = FALSE) {
  if(!inherits(data, "rrpp.data.frame")){
    if(!inherits(data, "data.frame") && 
       !inherits(data, "list"))
      stop(paste("\nThe data argument is neither an rrpp.data.frame",
                 "object, a data.frame object, nor a list.\n",
                 "Please see function details.\n", sep = " "), 
           call. = FALSE)
  }
  
  Y <- as.character(Y)
  Yslot <- which(names(data) %in% Y)
  if(length(Yslot) == 0)
    stop(paste("\nThe Y argument must be a character",
               "value, like 'myData',",
               "which can be found in the RRPP data frame.\n", 
               sep = " "), call. = FALSE)
  
  Y <- data[Yslot][[1]]
  if(use.PCs){
    Y <- ordinate(Y, tol = tol)$x
  }
  
  subjects <- as.character(subjects)
  Sslot <- which(names(data) %in% subjects)
  if(length(Sslot) == 0)
    stop(paste("\nThe subjects argument must be a character",
               "value, like 'mySubjects',",
               "which can be found in the RRPP data frame.\n", 
               sep = " "), call. = FALSE)
  
  subjects <- data[Sslot][[1]]
  
  replicates <- as.character(replicates)
  Rslot <- which(names(data) %in% replicates)
  if(length(Rslot) == 0)
    stop(paste("\nThe replicates argument must be a character",
               "value, like 'myReps',",
               "which can be found in the RRPP data frame.\n", 
               sep = " "), call. = FALSE)
  
  replicates <- data[Rslot][[1]]
  
  useGroups <- !is.null(groups)
  
  if(useGroups){
    
    groups <- as.character(groups)
    Gslot <- which(names(data) %in% groups)
    if(length(Gslot) == 0)
      stop(paste("\nThe groups argument must be a character",
                 "value, like 'myGroups',",
                 "which can be found in the RRPP data frame.\n", 
                 sep = " "), call. = FALSE)
    
    groups <- data[Gslot][[1]]
  }
  
  form <- if(useGroups) Y ~ subjects + groups * replicates else
    Y ~ subjects + replicates
  
  if(useGroups && groups.first)
    form <- update(form, ~ groups + .)
  
  dat <- rrpp.data.frame(
    Y = Y,
    subjects = as.factor(subjects),
    replicates = as.factor(replicates)
  )
  if(useGroups) dat$groups <- as.factor(groups)
  
  lm.rrpp.args <- list(
    f1 = form,
    data = dat,
    seed = seed,
    Parallel = Parallel,
    turbo = turbo,
    print.progress = print.progress,
    verbose = verbose,
    subjects = "subjects",
    iter = iter,
    Cov = NULL
  )
  
  fit <- suppressWarnings(do.call(lm.rrpp.ws, lm.rrpp.args))
  fit$call <- form
  data <- dat <- lm.rrpp.args <- NULL
  
  n <- fit$LM$n
  p <- fit$LM$p.prime
  
  if(multivariate && fit$LM$p > 1)
    fit <- manova.update(fit, tol = tol,
                         print.progress = print.progress,
                         verbose = verbose) 
  
  Df <- fit$ANOVA$df
  ANOVA <- getANOVAStats(fit, stat = "all")
  SNR <- ANOVA$SS / ANOVA$RSS
  AOVo <- AOV <- anova(fit, print.progress = FALSE)$table
  
  rownames(AOV)[which(
    rownames(AOV) == "subjects")] <- "Subjects"
  rownames(AOV)[which(
    rownames(AOV) == "replicates")] <- "Systematic ME"
  rownames(AOV)[which(
    rownames(AOV) == "groups")] <- "Groups"
  rownames(AOV)[which(
    rownames(AOV) == "groups:replicates")] <- 
    "Systematic ME:Groups"
  rownames(AOV)[which(
    rownames(AOV) == "Residuals")] <- "Random ME"
  
  if(multivariate){
    suppressWarnings(mAOV <- summary(fit)$stats.table)
    mAOV <- mAOV[rownames(mAOV) %in% rownames(AOVo), , drop = FALSE]
    mAOV <- mAOV[-NROW(mAOV), , drop = FALSE]
    rownames(mAOV)[which(
      rownames(mAOV) == "subjects")] <- "Subjects / Random ME"
    rownames(mAOV)[which(
      rownames(mAOV) == "replicates")] <- "Systematic ME / Random ME"
    rownames(mAOV)[which(
      rownames(mAOV) == "groups")] <- "Groups / Random ME"
    rownames(mAOV)[which(
      rownames(mAOV) == "groups:replicates")] <- 
      "Systematic ME:Groups / Random ME"
    mAOV <- mAOV[, -(1:2), drop = FALSE]
  } else mAOV <- NULL
  
  rm(AOVo)
  rm(SNR)
  
  Xsub <- model.matrix(~ subjects)
  R <- LM.fit(Xsub, Y)$residuals
  rm(Xsub)
  SS.tot <- sum(R^2)
  rm(R)
  
  SNR <- ANOVA$SS / fit$ANOVA$RSS
  rmove <- which(Df == 0)
  if(length(rmove) > 0)
    SNR <- SNR[-rmove, , drop = FALSE]
  
  AOV$EtaSq.ME <- NA
  AOV$EtaSq.ME[which(rownames(AOV) == "Systematic ME")] <- 
    AOV$SS[which(rownames(AOV) == "Systematic ME")] / SS.tot
  AOV$EtaSq.ME[which(rownames(AOV) == "Systematic ME:Groups")] <- 
    AOV$SS[which(rownames(AOV) == "Systematic ME:Groups")] / SS.tot
  AOV$EtaSq.ME[which(rownames(AOV) == "Random ME")] <- 
    AOV$SS[which(rownames(AOV) == "Random ME")] / SS.tot
  
  indx <- length(na.omit(AOV$F))    
  AOV$F[1:indx] <- AOV$SS[1:indx] / AOV$SS[indx + 1]
  names(AOV)[which(names(AOV) == "F")] <- "SNR"
  AOV <- AOV[c(1:4, 8, 5:7 )]
  AOV[1:indx, 7] <- apply(SNR, 1, effect.size)
  AOV[1:indx, 8] <- apply(SNR, 1, pval)
  names(AOV)[8] <- "Pr(>SNR)"
  
  out <- fit
  out$ANOVA <- ANOVA
  out$ANOVA$SS.type <- "Within-subject II"
  out$AOV <- AOV
  out$mAOV <- mAOV
  
  rm(AOV)
  rm(mAOV)
  rm(ANOVA)
  
  ### SSCP products
  
  if(p > 1){
    
    fitb <- fit
    class(fitb) <- "lm.rrpp"
    SSCP <- summary(fitb, formula = FALSE)$SSCP
    rm(fitb)
    if(length(rmove) > 0)
      SSCP <- SSCP[-rmove]
    
    SSCP.ME.product <- fast.solve(SSCP$Residuals) %*% 
      SSCP$replicates
    sscp.sqrt <- Cov.proj(SSCP$Residuals, symmetric = TRUE)
    SSCP.ME.product.std<- as.matrix(t(sscp.sqrt) %*% 
                                      SSCP$replicates %*% sscp.sqrt)
    
  } else  SSCP.ME.product <- SSCP.ME.product.std <- SSCP <- NULL
  
  out$SSCP <- SSCP
  rm(SSCP)
  out$SSCP.ME.product = SSCP.ME.product
  rm(SSCP.ME.product)
  out$SSCP.ME.product.std = SSCP.ME.product.std
  rm(SSCP.ME.product.std)
  rm(fit)
  
  if(!verbose) {
    out$ANOVA$MS <- out$ANOVA$Rsq <- out$ANOVA$Fs <-
      out$ANOVA$cohenf <- NULL
    out$MANOVA$SSCP <- out$MANOVA$invR.h <- NULL
    out$LM$QR <- NULL
    out$Models <- NULL
    out$PermInfo$perm.schedule <- NULL
  } 
  
  class(out) <- c("measurement.error", class(out))
  out
  
}
    
  
  

#' Landmarks on pupfish
#'
#' @name Pupfish
#' @docType data
#' @author Michael Collyer
#' @keywords datasets
#' @description Landmark data from Cyprinodon pecosensis body shapes, with 
#' indication of Sex and
#' Population from which fish were sampled (Marsh or Sinkhole).
#' @details These data were previously aligned with GPA.  Centroid size (CS) 
#' is also provided.  
#' See the \pkg{geomorph} package for details.
#' 
#' @references Collyer, M.L., D.J. Sekora, and D.C. Adams. 2015. A method for 
#' analysis of phenotypic
#' change for phenotypes described by high-dimensional data. Heredity. 113: 
#' doi:10.1038/hdy.2014.75.
NULL

#' Landmarks on pupfish heads
#'
#' @name PupfishHeads
#' @docType data
#' @author Michael Collyer
#' @description Landmark data from Cyprinodon pecosensis head shapes, with 
#' variables for 
#' sex, month and year sampled, locality, head size, and coordinates of 
#' landmarks for head shape,
#' per specimen.  These data are a subset of a larger data set.
#' @details The variable, "coords", are data that were previously aligned
#' with GPA.  The variable, "headSize", is the Centroid size of each vector 
#' of coordinates.
#' See the \pkg{geomorph} package for details.
#' @references Gilbert, M.C. 2016. Impacts of habitat fragmentation on the 
#' cranial morphology of a 
#' threatened desert fish (Cyprinodon pecosensis). Masters Thesis, 
#' Western Kentucky University.
NULL

#' Experimental metabolic rate data from pupfish
#'
#' @name PupfishMR
#' @docType data
#' @author Michael Collyer
#' @description Simulated metabolic rate data from experimental pupfish
#' @details These data are simulated data meant to mimic a subset of the 
#' data from Collyer & Stockwell (2004).  Variables include experimental treatment 
#' (Control versus artificially infected), mass in grams, standard length in mm,
#' and metabolic rate (oxygen consumption, mg O2 per L per hour).  These data do not have
#' a temporal component like the original data, and do not include aberrant or missing
#' observations.  These data are suitable data for demonstrating linear regression
#' and analysis of covariance 
#' (ANCOVA), used in the book, Permutational Biometry (Collyer & Adams, 2025).
#' 
#' @references Collyer, M. L., & Stockwell, C. A. (2004). Experimental 
#' evidence for costs of parasitism for a threatened species, 
#' White Sands pupfish (Cyprinodon tularosa). Journal of 
#' Animal Ecology, 73(5), 821-830.
#' 
#' @references Collyer, M. L., & Adams D. C. (2025). Permutational Biometry: Volume
#' 1, Univariate Data. Iowa State University Digital Press.  NOT YET PUBLISHED.
#' 
NULL

#' Plethodon comparative morphological data 
#'
#' @name PlethMorph
#' @docType data
#' @author Michael Collyer and Dean Adams
#' @keywords datasets
#' @description Data for 37 species of plethodontid salamanders.  
#' Variables include snout to vent length
#' (SVL) as species size, tail length, head length, snout to eye length, 
#' body width, forelimb length,
#' and hind limb length, all measured in mm.  A grouping variable is also 
#' included for functional guild size.  A variable for species names is also 
#' included.
#' The data set also includes a phylogenetic covariance matrix based on a 
#' Brownian model of evolution, to assist in 
#' generalized least squares (GLS) estimation.
#' @details The covariance matrix was estimated with the vcv.phylo function 
#' of the R package, ape, based on the tree
#' described in Adams and Collyer (2018).
#' @references Adams, D.C and Collyer, M.L. 2018. Multivariate phylogenetic 
#' anova: group-clade aggregation, biological 
#' challenges, and a refined permutation procedure. Evolution, 72: 1204-1215.
NULL


#' Plethodon comparative SVL data 
#'
#' @name PlethCinHoff
#' @docType data
#' @author Dean Adams
#' @keywords datasets
#' @description Snout-to-vent-length (SVL) data for two species of salamanders 
#' @details The data set is a subset of data from two species of salamanders 
#' (Plethodon cinereus and 
#' P. hoffmani) in eastern North America (based on Adams and Rohlf, 2000; 
#' Adams, 2000), collected from the Smithsonian Museum (Washington DC, USA).  Data 
#' include species, locality (whether in sympatry or allopatry), and SVL in mm.
#' These data are suitable data for demonstrating single factor and
#' factorial analysis of variance (ANOVA).
#' (ANCOVA), used in the book, Permutational Biometry (Collyer & Adams, 2025).
#' @references Adams, D. C. (2000). Divergence of trophic morphology and resource use
#' among populations of Plethodon cinereus and P. hoffmani in Pennsylvania.
#' In Bruce, R. C., Jaeger, R. G., and Houck, L. D., editors, The Biology of
#' Plethodontid Salamanders, pages 383–394. Springer US.
#' @references Adams, D. C. and Rohlf, F. J. (2000). Ecological character displacement in
#' Plethodon: Biomechanical differences found from a geometric morphometric
#' study. Proceedings of the National Academy of Sciences, 97:4106–4111.
#' @references Collyer, M. L., & Adams D. C. (2025). Permutational Biometry: Volume
#' 1, Univariate Data. Iowa State University Digital Press.  NOT YET PUBLISHED.
NULL

#' Reptile Abundance Data in Western Australia
#'
#' @name ReptAbund
#' @docType data
#' @keywords datasets
#' @description Reptile abundance data from several sites in Western Australia,
#' originally collected to examine recovery after a large fire. 
#' @details The data set is from a study examining reptile abundance from
#' several sites in Western Australia (Davis and Doherty, 2015). The original 
#' study examined the recovery of reptile communities in ten remnant
#' localities following a large fire in 2009. Half of localities were in the burn zone 
#' and the remaining half were unaffected by the fire. Data were sampled over five years.
#' These data are suitable data for demonstrating mixed linear models, used in the book, 
#' Permutational Biometry (Collyer & Adams, 2025).
#' @references Davis, R. A. and Doherty, T. S. (2015). Rapid recovery of an urban remnant
#' reptile community following summer wildfire. PLOS ONE, 10(5):e0127925.
#' @references Collyer, M. L., & Adams D. C. (2025). Permutational Biometry: Volume
#' 1, Univariate Data. Iowa State University Digital Press.  NOT YET PUBLISHED.
NULL

#' Mammal Home Range Data
#'
#' @name mammalHR
#' @docType data
#' @keywords datasets
#' @description Home range and body mass data for 49 species of mammals.
#' @details The data set is from a study of mammal species home range size 
#' (squared kilometers) and its covariation with body mass or whether mammals
#' are carnivores or ungulates (Garland et al., 1993).  the data set also includes a
#' covariance matrix based on phylogenetic relatedness among species, to account for the
#' non-independence of observations.
#' These data are suitable data for demonstrating generalized least squares (GLS)
#' estimation, used in the book, 
#' Permutational Biometry (Collyer & Adams, 2025).
#' @references Garland, T., Dickerman, A. W., Janis, C. M., and Jones, J. A. (1993). 
#' Phylogenetic analysis of covariance by computer simulation. Systematic Biology,
#' 42:265–292.
#' @references Collyer, M. L., & Adams D. C. (2025). Permutational Biometry: Volume
#' 1, Univariate Data. Iowa State University Digital Press.  NOT YET PUBLISHED.
NULL

#' Phylogenetic Data for the Mammal Home Range Data
#'
#' @name mammalHRphy
#' @docType data
#' @keywords datasets
#' @description Phylogenetic tree data that can be used along with 
#' \code{\link{mammalHR}}.
#' @details A class \code{phylo} object containing data for the phylogenetic tree
#' that pertains to the \code{\link{mammalHR}} data set.  Although these data are
#' not required for the generalized least squares (GLS)
#' estimation example, used in the book, 
#' Permutational Biometry (Collyer & Adams, 2025), they provide additional 
#' graphical support for the example.

#' @references Collyer, M. L., & Adams D. C. (2025). Permutational Biometry: Volume
#' 1, Univariate Data. Iowa State University Digital Press.  NOT YET PUBLISHED.
NULL

#' Simulated motion paths
#'
#' @name motionpaths
#' @docType data
#' @author Dean Adams
#' @references Adams, D. C., and M. L. Collyer. 2009. A general framework for 
#' the analysis of phenotypic
#'   trajectories in evolutionary studies. Evolution 63:1143-1154.
#' @keywords datasets
NULL

#' Simulated Fish Data for Measurement Error Analysis
#'
#' @name fishy
#' @docType data
#' @author Michael Collyer
#' @references Collyer, M.L, and D.C. Adams. 2024. 
#' Interrogating random and systematic measurement 
#' error in morphometric data. Evolutionary Biology. 
#' In press.
#' @details Data as simulated in Collyer and Adams (2024),
#' resembling a fish shape, comprising Procrustes coordinates
#' for 11 anatomical landmarks.  Data represent 120 
#' configurations for 60 subjects, each with two replicates of
#' measurement.  The 60 subjects represent 20 subjects each 
#' from three groups.
#' @keywords datasets
NULL

#' Phytoplankton Biomass and Eutrophication
#'
#' @name phytoplankton
#' @docType data
#' @keywords datasets
#' @description A data set containing variables for phytoplanton biomass,
#' phosphorous levels, trophic status, and locations (coordinates) of 70 temperate lakes.
#' @details The data in this data set are samples from a larger study 
#' that compared phytoplankton biomass
#' from temperate lakes in Alberta, Canada, with different levels of eutrophication
#' (Loewen et al., 2020).  These 
#' data are suitable for considering the spatial non-independence of observations in 
#' generalized least squares (GLS) estimation, for the example in the book,
#' Permutational Biometry (Collyer & Adams, 2025).
#' @references Loewen, C. J. G., Wyatt, F. R., Mortimer, C. A., Vinebrooke, R. D., and
#' Zurawell, R. W. (2020). Multiscale drivers of phytoplankton communities
#' in north‐temperate lakes. Ecological Applications, 30(5):e02102.
#' @references Collyer, M. L., & Adams D. C. (2025). Permutational Biometry: Volume
#' 1, Univariate Data. Iowa State University Digital Press.  NOT YET PUBLISHED.
NULL

#' Flower Production and Temperature
#'
#' @name flower
#' @docType data
#' @keywords datasets
#' @description A data set containing variables for flower production and
#' temperature, averaged yearly for 30 years, from a dry tropical forest.
#' @details The data in this data set are samples from a larger study 
#' that compared flower production between forest types over 30 years, 
#' based on climate variables.  The data in this set are from a dry tropical forest,
#' include variables for annual flower production, temperature, and year. These 
#' data are suitable for considering the temporal non-independence of observations in 
#' generalized least squares (GLS) estimation, for the example in the book,
#' Permutational Biometry (Collyer & Adams, 2025).
#' @references Pau, S., Wolkovich, E. M., Cook, B. I., Nytch, C. J., Regetz, J., Zimmerman,
#' J. K., and Joseph Wright, S. (2013). Clouds and temperature drive dynamic
#' changes in tropical flower production. Nature Climate Change, 3:838–842.
#' @references Collyer, M. L., & Adams D. C. (2025). Permutational Biometry: Volume
#' 1, Univariate Data. Iowa State University Digital Press.  NOT YET PUBLISHED.
NULL

#' Snake Head Size and Regional Variation
#'
#' @name snakeHS
#' @docType data
#' @keywords datasets
#' @description A data set containing variables for snake head size, 
#' snout-to-vent-length, sex, and geographic information.
#' 
#' @details These data were collected from prairie rattlesnakes (Crotalus viridis viridis),
#' from several geographic locations in the upper Midwest of the United States (Smith and 
#' Collyer, 2008).  Variables include a spatial measure of head size (HS), 
#' snout-to-vent-length (SVL, cm), sex, and the site and region where 107
#' snakes were captured.  These 
#' data are suitable for demonstrating model selection  in the book,
#' Permutational Biometry (Collyer & Adams, 2025).
#' @references Smith, M. T. and Collyer, M. L. (2008). The biology of rattlesnakes, chapter
#' Regional variation and sexual dimorphism in head form of the prairie
#' rattlesnake (Crotalus viridid viridis): Comparisons using new analytical techniques
#' and collection methods, pages 79–90. Loma Linda University Press.
#' @references Collyer, M. L., & Adams D. C. (2025). Permutational Biometry: Volume
#' 1, Univariate Data. Iowa State University Digital Press.  NOT YET PUBLISHED.
NULL

#' Podarcis Wall Lizards from the Mediterranean Basin 
#'
#' @name podarcis
#' @docType data
#' @keywords datasets
#' @description A data set of body size for males and females form different lizard 
#' species in different geographic locations.
#' 
#' @details These data are a subset of the data collected originally by 
#' Kaliontzopoulou et al.(2015).  Variables include snout-to-vent-length (SVL),
#' sex, species, and habitat preference.  These 
#' data are suitable for comparing effect sizes in the book,
#' Permutational Biometry (Collyer & Adams, 2025).
#' @references Kaliontzopoulou, A., Carretero, M. A., and Adams, D. C. (2015). Ecomorphological
#' variation in male and female wall lizards and the macroevolution
#' of sexual dimorphism in relation to habitat use. Journal of Evolutionary
#' Biology, 28:80–94.
#' @references Collyer, M. L., & Adams D. C. (2025). Permutational Biometry: Volume
#' 1, Univariate Data. Iowa State University Digital Press.  NOT YET PUBLISHED.
NULL

## logLik.lm.rrpp

#' Calculate the log-likelihood of a lm.rrpp fit
#' 
#' \code{logLik.lm.rrpp} returns the log-likelihood of
#' an \code{lm.rrpp} object or an \code{lmm.rrpp} object.  
#' Ridge regularization will be performed for 
#' ill-conditioned or singular residual covariance matrices, but dimension
#' reduction could be augmented via projection, using the arguments, tol
#' and pc.no.  See \code{\link{ordinate}} for details.
#' 
#' Note that options are limited for \code{lmm.rrpp} objects.  It is currently
#' not possible to use RRPP to find Z scores.
#' 
#'
#' @param object Object from \code{\link{lm.rrpp}}
#' @param tol A value indicating the magnitude below which 
#' components should be omitted, following projection.  See \code{\link{ordinate}} 
#' for details.
#' @param pc.no Optionally, a number specifying the maximal number of 
#' principal components, passed onto \code{\link{ordinate}}, as argument, rank.
#' @param Z A logical value for whether to calculate Z scores based on RRPP.
#' @param gls.null A logical value for if a fit has a GLS estimation, should
#' the null model (intercept) also have a GLS estimation, for estimating Z.  
#' Should be FALSE if the log-likelihood is measured to compare different GLS
#' estimations for a covariance matrices
#' @param verbose A logical value for whether to return random log-likelihood values,
#' if Z-scores are calculated.
#' @param ...	further arguments passed to or from other methods
#' @export
#' @author Michael Collyer
#' @keywords utilities

logLik.lm.rrpp <- function(object, tol = NULL, 
                            pc.no = NULL, Z = TRUE,
                            verbose = FALSE,
                            gls.null = FALSE, ...){
  
  ll <- if(isTRUE(object$LM$LMM))
    .logLik.lmm.rrpp(object, tol = tol,
                     pc.no = pc.no, Z = Z,
                     verbose = verbose,
                     gls.null = gls.null, ...) else
            .logLik.lm.rrpp(object, tol = tol,
                            pc.no = pc.no, Z = Z,
                            verbose = verbose,
                            gls.null = gls.null, ...)
  Z <- ll$Z
  if(is.list(ll)) val <- ll[[1]] else val <- as.numeric(ll)
  k <- attr(ll, "k") 
  p <- attr(ll, "p")
  attr(val, "df") <- k * p
  attr(val, "nall") <- attr(ll, "nall") 
  attr(val, "nobs") <- attr(ll, "nobs") 
  attr(val, "p") <- attr(ll, "p") 
  attr(val, "Z") <- Z
  class(val) <- "logLik"
  val
}

.logLik.lm.rrpp <- function(object, tol = NULL, 
                           pc.no = NULL, Z = TRUE,
                           verbose = FALSE,
                           gls.null = FALSE, ...){
  
  if(is.null(tol)) tol = 0
  
  PermInfo <- getPermInfo(object, attribute = "all")
  ind <- PermInfo$perm.schedule
  gls <- object$LM$gls
  n <- object$LM$n
  p <- object$LM$p.prime
  X <- as.matrix(object$LM$X)
  Y <- as.matrix(object$LM$Y)
  R <- if(gls) object$LM$gls.residuals else object$LM$residuals
  PCA <- ordinate(R, tol = tol, rank. = min(c(pc.no, p)))
  p <- ncol(PCA$x)
  rnk <- length(PCA$d)
  w <- object$LM$weights
  Cov <- object$LM$Cov  
  if(!is.null(object$LM$Cov) && is.null(object$LM$Pcov)) 
    object$LM$Pcov <- Cov.proj(object$LM$Cov)
  Pcov <- NULL
  if(!is.null((object$LM$Pcov)))
    Pcov <- object$LM$Pcov
  
  PY <- if(gls) {
    if(!is.null(Pcov)) Pcov %*% Y else Y * sqrt(w)
    } else NULL
  PX <- if(gls) {
    if(!is.null(Pcov)) Pcov %*% X else X * sqrt(w)
  } else NULL
    
  U <- if(gls) QRforX(PX, reduce = FALSE)$Q else 
    QRforX(X, reduce = FALSE)$Q
  
  TY <- if(gls.null) PY else Y
  
  ll <- logL(object, tol = tol, pc.no = pc.no,
             C = Cov, w = w) 
  
  if(Z) {
    
    perms <- length(ind)
    
    if(!is.null(w)) {
      excl <- w <= 0
      logdetC <- sum(log(w[!excl]))
    } else {
      logdetC <- if(gls) log(det(Cov)) else 0
    }
    
    ll.list <- sapply(1:perms, function(j){
      y <- TY[ind[[j]], ]
      if(gls && !gls.null) {
        ty <- Pcov %*% y
        r <- ty - fastFit(U, ty, n, rnk)
      } else r <- y - fastFit(U, y, n, rnk)
      r <- as.matrix(r)
      logL(object, tol = tol, pc.no = pc.no,
           logdetC = logdetC, R = r, rnk = rnk)$logL
    })
     
    z <- effect.size(ll.list)
    ll <- c(ll, Z = z)
    if(verbose) ll <- list(ll = ll, Z = z, 
                           random.logL = ll.list)
  }
  
  attr(ll, "k") <- ncol(U)
  attr(ll, "p") <- p
  attr(ll, "nall") <- n
  attr(ll, "nobs") <- n
  
  return(ll)
    
}


.logLik.lmm.rrpp <- function(object, tol = NULL, 
                             pc.no = NULL, Z = TRUE,
                             verbose = FALSE,
                             gls.null = FALSE, ...){
  
  if(is.null(tol)) tol = 0
  
  PermInfo <- getPermInfo(object, attribute = "all")
  ind <- PermInfo$perm.schedule
  gls <- object$LM$gls
  n <- object$LM$n
  p <- object$LM$p.prime
  X <- QRforX(object$LM$X)$X
  Y <- as.matrix(object$LM$Y)
  R <- if(gls) object$LM$gls.residuals else object$LM$residuals
  PCA <- ordinate(R, tol = tol, rank. = min(c(pc.no, p)))
  rnk <- length(PCA$d)
  w <- object$LM$weights
  Cov <- object$LM$Cov  
  if(!is.null(object$LM$Cov) && is.null(object$LM$Pcov)) 
    object$LM$Pcov <- Cov.proj(object$LM$Cov)
  Pcov <- NULL
  if(!is.null((object$LM$Pcov)))
    Pcov <- object$LM$Pcov
  
  PY <- if(gls) {
    if(!is.null(Pcov)) Pcov %*% Y else Y * sqrt(w)
  } else NULL
  PX <- if(gls) {
    if(!is.null(Pcov)) Pcov %*% X else X * sqrt(w)
  } else NULL
  R <- if(gls) {
    if(!is.null(Pcov)) Pcov %*% R else R * sqrt(w)
  } else R
  
  RtR_det <- try(det(crossprod(R)), silent = TRUE)
  if(!inherits(RtR_det, "try-error")) 
    object$devcomp$RtR_det <- RtR_det else
      stop("The residuals appear to be singular using the current attributes.\n",
           call. = FALSE)
  # update U
  # get hresid
  XZ <- if(gls) cbind(PX, object$LM$Z %*% object$LM$Lambda) else
    cbind(X, object$LM$Z %*% object$LM$Lambda)
  Hb <- if(gls) getLMM_Hb(PX, object$LM$Z %*% object$LM$Lambda) else
    getLMM_Hb(X, object$LM$Z %*% object$LM$Lambda)
  H <- XZ %*% Hb  - diag(n)
  
  TY <- if(gls.null) PY else Y
    
  k <- NCOL(X)
  
  est <- object$LM$estimation
  devcomp <- object$devcomp
  r <- devcomp$RtR_det 
  if(!is.na(devcomp$UtU_det)) r <- r + devcomp$UtU_det
  lres <- log(2 * pi * r)
  n <- object$LM$n
  p <- object$LM$p.prime
  np <- n * p
  if(est == "REML") npk <- np - 
    length(object$LM$coef.fixed) * object$LM$p.prime else 
      npk <- np
  
  ll <- -0.5 * p * ((devcomp$ldLZ + devcomp$ldLX - devcomp$ldO) +
                      npk * (1 + log((2 * pi * r) / npk)))
  if(Z) {
    
    perms <- length(ind)
    
    ll.list <- sapply(1:perms, function(j){
      y <- TY[ind[[j]], ]
      if(gls && !gls.null) {
        ty <- Pcov %*% y
        r <- H %*% ty
      } else r <- H %*% y
      r <- as.matrix(r)
      if(NCOL(r) > rnk) r <- ordinate(r, rank. = rnk)$x
      r <- det(crossprod(r))
      if(!is.na(devcomp$UtU_det)) r <- r + devcomp$UtU_det
      -0.5 * p * ((devcomp$ldLZ + devcomp$ldLX - devcomp$ldO) +
                    npk * (1 + log((2 * pi * r) / npk)))
    })
    
    z <- effect.size(ll.list)
    ll <- c(ll, Z = z)
    if(verbose) ll <- list(ll = ll, Z = z, 
                           random.logL = ll.list)
  }
  
  
  attr(ll, "k") <- ncol(X) + 
    length(unique(as.vector(object$LM$Lambda))) -1
  attr(ll, "p") <- p
  attr(ll, "nall") <- n
  attr(ll, "nobs") <- n 
  
  ll
}




logL <- function(fit, tol = NULL, pc.no = NULL,
                 logdetC = NULL, R = NULL, rnk = NULL,
                 useRiReg = TRUE, 
                 C = NULL, w = NULL
                 ){
  if(is.null(tol)) tol = 0
  gls <- fit$LM$gls
  n <- fit$LM$n
  p <- fit$LM$p.prime
  
  Pcov <- try(getModelCov(fit, "Pcov"),
              silent = TRUE)
  if(inherits(Pcov, "try-error")) Pcov <- NULL
  
  if(is.null(R)){
    R <- if(gls) fit$LM$gls.residuals else 
      fit$LM$residuals
    R <- if(gls) {
      if(!is.null(Pcov)) Pcov %*% R else R * sqrt(w)
    } else R
  }
  
  if(gls && is.null(logdetC)) {
    if(!is.null(w)) {
      excl <- w <= 0
      logdetC <- sum(log(w[!excl]))
    } else {
      logdetC <- log(det(C))
    }
  } else logdetC <- 0
  
  R <- as.matrix(R)
  if(is.null(rnk)){
    PCA <- ordinate(R, tol = tol, rank. = min(c(pc.no, p)))
    rnk <- length(PCA$d)
  }
  
  if(NCOL(R) > rnk) R <- ordinate(R, tol = tol, 
                                  rank. = rnk)$x
  Sig <- as.matrix(crossprod(R) / n)
  if(kappa(Sig) > 1e10 && useRiReg) Sig <- RiReg(Sig, R)
  
  logdetSig <- log(det(Sig))
  
  ll <- -0.5 * (n * rnk * log(2 * pi) + rnk * logdetC +
                  n * logdetSig + n * rnk) 
  
  list(logL = ll, rank = rnk)
  
}

#' QR decomposition of linear model design matrices
#' 
#' This function performs a QR decomposition (factorization) on a linear 
#' model design matrix (X) and returns useful results for subsequent analysis.
#' This is intended as an internal function but can be used externally.  Because
#' base::qr and Matrix::qr have different options for QR algorithms, this
#' function assures that results are consistent for other RRPP function use, 
#' whether X is a dense or sparse matrix.
#'
#' @param X A linear model design matrix, but can be any object coercible to matrix.
#' @param returnQ A logical value whether to return the Q matrix.  Generating a
#' Q matrix can be computationally intense for large matrices.  If it is not
#' explicitly needed, this argument can be FALSE.
#' @param ... Further arguments passed to base::qr.
#' @return An object of class \code{QR} is a list containing the 
#' following:
#' \item{Q}{The Q matrix, if requested.}
#' \item{R}{The R matrix.}
#' \item{X}{The X matrix, which could be changed from dense to sparse,
#' or vice versa, and redundant columns removed.}
#' \item{dimnames}{The dimnames associated with Q, R, or X}
#' \item{rank}{The rank of the X matrix.}
#' \item{S4}{Logical value for whether Q, R, and X are S4 class objects.}
#' @export
#' @author Michael Collyer
#' @keywords utilities
#' @examples
#' ## Simple Example
#' data(Pupfish)
#' fit <- lm.rrpp(coords ~ Pop, data = Pupfish, print.progress = FALSE)
#' QR <- QRforX(model.matrix(fit))
#' QR$Q
#' QR$R
#' QR$rank
#' QR$S4
#' 
#' ## Not run, but one could get base::qr and Matrix::qr results as
#' 
#' # base::qr(as.matrix(QR$X))
#' # Matrix::qr(QR$X)
#' 
#' ## Complex example
#' 
#' data("PupfishHeads")
#' fit <- suppressWarnings(lm.rrpp(headSize ~ sex + 
#' locality/year, data = PupfishHeads))
#' X <- model.matrix(fit)
#' dim(X) # Already reduced
#' colnames(X)
#' X <- model.matrix(terms(fit), fit$LM$data)
#' dim(X) # Retains redundant parameters
#' colnames(X)
#' QR <- QRforX(X)
#' dim(QR$X)
#' dim(QR$Q)
#' dim(QR$R)
#' QR$rank
#' 
QRforX <- function(X, returnQ = TRUE,
                   ...){
  
  cs <- colSums(as.matrix(X))
  
  if(NCOL(X) > 1 && any(cs == 0))
    X <- X[, cs != 0, drop = FALSE]
  
  if(!isS4(X) && NCOL(X) > 1){
    if(length(which(X != 0)) <= 0.9 * length(X))
      X <- Matrix(X, sparse = TRUE)
  }
  
  if(isS4(X)){
    if(length(which(X != 0)) > 0.9 * length(X))
      X <- as.matrix(X)
  }
  
  dims <- dim(X)
  n <- dims[1]
  k <- dims[2]
  
  use1 <- TRUE
  if(NCOL(X) > 1 && isS4(X)) use1 <- FALSE
  
  QR1 <- function(X){
    QR <- qr(X)
    Q <- qr.Q(QR)
    R <- qr.R(QR)
    X <- X[, QR$pivot[1:QR$rank], drop = FALSE]
    out <- list(Q = Q, R = R, X = X)
    out
  }
  
  QR2 <- function(X){
    QR <- suppressWarnings(qr(X))
    R <- suppressWarnings(qrR(QR))
    Q <- suppressWarnings(qr.Q(QR))
    QR <- suppressWarnings(qr(as.matrix(R)))
    if(QR$rank < NCOL(X)) {
      X <- X[,QR$pivot[1:QR$rank], drop = FALSE]
      QR <- suppressWarnings(qr(X))
      R <- suppressWarnings(qrR(QR))
      Q <- suppressWarnings(qr.Q(QR))
    }
    Q <- drop0(Q, tol = 1e-10)
    R <- drop0(R, tol = 1e-10)
    X <- drop0(X, tol = 1e-10) 
    out <- list(Q = Q, R = R, X = X)
    out
  }
  
  QR <- try(if(use1) QR1(as.matrix(X)) else QR2(X),
            silent = TRUE)
  if(inherits(QR, "try-error")) {
    
    X <- as.matrix(X)
    qrt <- qr(X)
    X <- X[, qrt$pivot[1:qrt$rank], drop = FALSE]
    if(length(which(X != 0)) <= 0.9 * length(X))
      X <- Matrix(X, sparse = TRUE)
    if(NCOL(X) > 1 && isS4(X)) use1 <- FALSE
    QR <- if(use1) QR1(as.matrix(X)) else QR2(X)
  
  }
    
  QR$rank <- NCOL(QR$X)
  if(sum(QR$R) == 0) QR$rank <- 0
  QR$S4 <- isS4(QR$X)
  QR$dimnames <- dimnames(QR$X)
  
  if(!returnQ)
    QR$Q <- NULL
  
  class(QR) <- "QR"
  QR
}

#' Model Comparisons, in terms of the log-likelihood, covariance trace,
#' or Z-score.
#'
#' Function calculates either log-likelihoods or traces of covariance 
#' matrices for comparison with 
#' respect to parameter penalties, or calculates Z-scores from RRPP, which
#' can be profiled across a gradient (predictor).
#' 
#'
#' The function calculates either log-likelihoods or traces of (residual) 
#' covariance matrices, plus parameter 
#' penalties, to assist in comparative model evaluation or selection.  
#' Because high-dimensional data often
#' produce singular or ill-conditioned residual covariance matrices, 
#' this function does one of two things: 1) uses 
#' the trace of a covariance matrix rather than its determinant; or 2) 
#' provides a ridge-regularization (Warton, 2008)
#' of the covariance matrix, only if it is determined that it is 
#' ill-conditioned.  Regardless of implementation,
#' covariance matrices are projected into a principal component (PC) 
#' space of appropriate dimensions.
#' 
#' The parameter penalty is based on that proposed by Bedrick and Tsai 
#' (1994), equal to 2(pk + p(p + 1)/2), where 
#' p is the appropriate dimension (not number of variables) of the 
#' covariance matrix.  The parameter, k,
#'  is the rank of the model design matrix.
#'  
#' In the case that "logLik" is chosen for the argument, type, 
#' AIC scores are calculated.  These scores
#' may not perfectly match other packages or software that 
#' calculate AIC for multivariate data, if ridge regularization
#' was used (and if other packages require p = the number of 
#' data variables).  When choosing logLik as the type of comparison,
#' it might be a good idea to adjust the tolerance or number of data 
#' principal components.  The default (NULL) values will
#' use all data dimensions to calculate log-likelihoods, which 
#' might cause problems if the number of variables exceeds the number 
#' of observations (producing singular residual covariance matrices).  
#' However, one should not reduce data dimensions haphazardly,
#' as this can lead to poor estimates of log-likelihood.  Furthermore, 
#' using the tolerance argument could result in different
#' numbers of principal components used for each model to calculate 
#' log-likelihoods, which might be a concern for comparing models.  
#' If both tol and pc.no arguments are used, the solution will use 
#' the fewest PCs produced by either argument.  Because the trace
#' of a covariance matrix is not sensitive to matrix singularity, 
#' no PC adjustment is used for the cov.trace argument.
#' 
#' This function can also calculate Z-scores from RRPP on model log-likelihoods, 
#' which can be compared directly or profiled along a gradient (predictor).  
#' This might be useful for for comparing generalized least-squares (GLS) models, 
#' for example, along a gradient of a parameter used to scale the covariance 
#' matrix for GLS estimation. See Collyer et al. 2022 for an example of using
#' RRPP on log-likelihoods with different covariance matrices.
#' 
#' Users can construct their own tables 
#' from the results but this function does not attempt to 
#' summarize results, as interpreting results requires 
#' some arbitrary decisions.  The \code{\link{anova}} function 
#' explicitly tests multiple models and can be used for nested 
#' model comparisons.
#' 
#' Results can also be plotted using the generic \code{\link{plot}} 
#' function.
#' 
#' 
#' 
#' 
#' @param ... Any number of lm.rrpp class objects for model fits 
#' to be compared.
#' @param type An argument to choose between log-likelihood, 
#' covariance trace, or Z results.  If Z is chosen, Z-scores are calculated, the same log-likelihoods
#' are calculated as with the log-likelihood type, but also in every RRPP permutation,
#' as describe for the initial mode fits, with choice of null model (below).
#' @param predictor An optional vector that can be used to profile the results based on type
#' across a range of numerical values described by the predictor.  A spline will also be fit, which
#' will reveal estimated values of the predictor that yield maximum and minimum values of model
#' comparison metric.
#' @param tol If type = logLik or Z, tol is a tolerance value between 0 and 1, 
#' indicating the magnitude below which 
#' components should be omitted (if standard deviations of 
#' components are less than the eigenvalue of the first 
#' component times the tolerance), for calculating the log-likelihood.
#' @param pc.no If type = logLik or Z, an optional value to indicate the 
#' number of principal components (maximum rank) to use 
#' for calculating the log-likelihood.
#' @param gls.null A logical value indicating whether GLS estimation should be
#' used with the null (intercept) model, for calculating Z scores via RRPP of 
#' log-likelihoods.  This should be FALSE if comparing different GLS estimations
#' of covariance matrices.  It should be TRUE if comparing different model fits
#' with the same GLS-estimated covariance matrix.
#' @param mod.names A logical value for whether to label models by their object names (TRUE)
#' or formulas (FALSE).
#' @param verbose A logical value for whether to include distributions of
#' random log-likelihoods, if applicable. 
#' 
#' @keywords analysis
#' @export
#' @author Michael Collyer
#' @return An object of class \code{model.comparison} is a data 
#' frame with either log-likelihoods,
#' covariance traces, or, Z-scores, plus parameter penalties.  AIC scores 
#' might be included, if applicable. If verbose results are returned,
#' random log-likelihoods are also included.
#' @references Bedrick, E.J., and C.L. Tsai. 1994. Model selection 
#' for multivariate regression in small samples. 
#' Biometrics, 226-231.
#' @references Warton, D.I., 2008. Penalized normal likelihood and 
#' ridge regularization of correlation and covariance matrices. 
#' Journal of the American Statistical Association. 103: 340-349.
#' @references Collyer,  M.L., E.K. Baken, & D.C. Adams.  A standardized 
#' effect size for evaluating and comparing the strength of phylogenetic 
#' signal. Methods in Ecology and Evolution. 13: 367–382.
#' @examples 
#' \dontrun{
#' data(Pupfish)
#' Pupfish$logSize <- log(Pupfish$CS)
#' fit1 <- lm.rrpp(coords ~ logSize, data = Pupfish, iter = 0, 
#' print.progress = FALSE)
#' fit2 <- lm.rrpp(coords ~ Pop, data = Pupfish, iter = 0, 
#' print.progress = FALSE)
#' fit3 <- lm.rrpp(coords ~ Sex, data = Pupfish, iter = 0, 
#' print.progress = FALSE)
#' fit4 <- lm.rrpp(coords ~ logSize + Sex, data = Pupfish, iter = 0, 
#' print.progress = FALSE)
#' fit5 <- lm.rrpp(coords ~ logSize + Pop, data = Pupfish, iter = 0, 
#' print.progress = FALSE)
#' fit6 <- lm.rrpp(coords ~ logSize + Sex * Pop, data = Pupfish, iter = 0, 
#' print.progress = FALSE)
#' 
#' modComp1 <- model.comparison(fit1, fit2, fit3, fit4, fit5, 
#' fit6, type = "cov.trace")
#' modComp2 <- model.comparison(fit1, fit2, fit3, fit4, fit5, 
#' fit6, type = "logLik", tol = 0.01)
#' 
#' summary(modComp1)
#' summary(modComp2)
#' 
#' par(mfcol = c(1,2))
#' plot(modComp1)
#' plot(modComp2)
#' 
#' # Comparing fits with covariance matrices
#' # an example for scaling a phylogenetic covariance matrix with
#' # the scaling parameter, lambda
#' 
#' data("PlethMorph")
#' Cov <- PlethMorph$PhyCov
#' lambda <- seq(0, 1, 0.1)
#' 
#' Cov1 <- scaleCov(Cov, scale. = lambda[1])
#' Cov2 <- scaleCov(Cov, scale. = lambda[2])
#' Cov3 <- scaleCov(Cov, scale. = lambda[3])
#' Cov4 <- scaleCov(Cov, scale. = lambda[4])
#' Cov5 <- scaleCov(Cov, scale. = lambda[5])
#' Cov6 <- scaleCov(Cov, scale. = lambda[6])
#' Cov7 <- scaleCov(Cov, scale. = lambda[7])
#' Cov8 <- scaleCov(Cov, scale. = lambda[8])
#' Cov9 <- scaleCov(Cov, scale. = lambda[9])
#' Cov10 <- scaleCov(Cov, scale. = lambda[10])
#' Cov11 <- scaleCov(Cov, scale. = lambda[11])
#' 
#' 
#' fit1 <- lm.rrpp(SVL ~ 1, data = PlethMorph, Cov = Cov1)
#' fit2 <- lm.rrpp(SVL ~ 1, data = PlethMorph, Cov = Cov2)
#' fit3 <- lm.rrpp(SVL ~ 1, data = PlethMorph, Cov = Cov3)
#' fit4 <- lm.rrpp(SVL ~ 1, data = PlethMorph, Cov = Cov4)
#' fit5 <- lm.rrpp(SVL ~ 1, data = PlethMorph, Cov = Cov5)
#' fit6 <- lm.rrpp(SVL ~ 1, data = PlethMorph, Cov = Cov6)
#' fit7 <- lm.rrpp(SVL ~ 1, data = PlethMorph, Cov = Cov7)
#' fit8 <- lm.rrpp(SVL ~ 1, data = PlethMorph, Cov = Cov8)
#' fit9 <- lm.rrpp(SVL ~ 1, data = PlethMorph, Cov = Cov9)
#' fit10 <- lm.rrpp(SVL ~ 1, data = PlethMorph, Cov = Cov10)
#' fit11 <- lm.rrpp(SVL ~ 1, data = PlethMorph, Cov = Cov11)
#' 
#' par(mfrow = c(1,1))
#' 
#' MC1 <- model.comparison(fit1, fit2, fit3, fit4, fit5, fit6,
#' fit7, fit8, fit9, fit10, fit11,
#' type = "logLik")
#' MC1
#' plot(MC1)
#' 
#' MC2 <- model.comparison(fit1, fit2, fit3, fit4, fit5, fit6,
#' fit7, fit8, fit9, fit10, fit11,
#' type = "logLik", predictor = lambda)
#' MC2
#' plot(MC2)
#' 
#' 
#' MC3 <- model.comparison(fit1, fit2, fit3, fit4, fit5, fit6,
#' fit7, fit8, fit9, fit10, fit11,
#' type = "Z", predictor = lambda)
#' MC3
#' plot(MC3)
#' }

model.comparison<- function(..., type = c("cov.trace", "logLik", "Z"), 
                            predictor = NULL, tol = NULL, pc.no = NULL,
                            gls.null = FALSE, 
                            mod.names = TRUE,
                              verbose = FALSE) {
  
  dots <- list(...)
  check <- unlist(lapply(dots, inherits, "lm.rrpp"))
  if(any(!check)) stop("\nObjects must be lm.rrpp fits\n.")
  Call <- match.call()
  Call$type <- Call$predictor <- Call$tol <- Call$pc.no <- Call$gls.null <-
    Call$mod.names <- Call$verbose <- NULL
  Mod.names <- as.character(Call)[-1L]
  dot.names <- lapply(dots, function(x) x$LM$Terms[[3]])
  if(length(unique(dot.names)) < length(dot.names)) {
    cat("Models do not have unique term combinations.\n")
    cat("Labelling models based on order presented.\n")
    dot.names <- paste("Mod", 1:length(dots), sep = ".")
  }
  type = match.arg(type)
  
  if(type == "logLik") {
    if(!is.null(tol)) {
      if(tol > 1 || tol < 0) 
        stop("tol must be a value between 0 and 1\n", call. = FALSE)
    }
    if(!is.null(pc.no)) {
      p.list <- sapply(dots, function(x) x$LM$p.prime)
      rank.list <- ifelse(p.list < pc.no, p.list, pc.no)
      rank.list <- as.data.frame(matrix(rank.list, nrow = 1))
      rownames(rank.list) <- "pc.no"
      colnames(rank.list) <- dot.names
      
      if(any(pc.no > p.list)) {
        
        warning(
          paste(
            "\nThis is not an error!  It is a friendly warning.\n",
            "\nThe number of PCs requested exceeds possible dimensions for some model fits.",
            "\nPC number will be dropped initially to", rank.list, 
            "and might be reduced further, if the tolerance was also adjusted.\n",
            "\nUse options(warn = -1) to turn off these warnings. \n\n", sep = " "),
          noBreaks. = TRUE, call. = FALSE, immediate. = TRUE) 
        
        pc.no <- p.list
      }
    }
    
    ll.args <- list(object = dots[[1]], 
                    tol = tol, pc.no = pc.no,
                    gls.null = gls.null, Z = FALSE)
    temp <- lapply(1:length(dots), function(j){
      ll.args$object <- dots[[j]]
      do.call(logLik, ll.args)
    }) 
    res <- sapply(temp, as.numeric)
    rank <- sapply(dots, function(x){
      d <- ordinate(as.matrix(resid(x)), 
                    tol = tol, rank. = pc.no)$d
      length(d)
    })
  } else res <- sapply(dots, cov.trace) 
  
  if(type == "Z") {
    res <- lapply(1:length(dots), function(j){
      f <- dots[[j]]
      LL <- .logLik.lm.rrpp(f, Z = TRUE, tol = tol, 
                   pc.no = pc.no, gls.null = gls.null,
                   verbose = verbose)
      z <- LL$Z
      if(verbose) {
        random.logL <- LL$random.logL
      } else random.logL <- NULL
      list(z = z, random.logL = random.logL)   
    })
    if(verbose) random.logL <- 
        lapply(res, function (x) x$random.logL) else
          NULL
    res <- sapply(res, function (x) x$z)
  }
  
  par.pen <- function(x){
   ll <- logLik(x, tol = tol, pc.no = pc.no, 
                gls.null = gls.null, Z = FALSE)
   p <- attr(ll, "p")
   k <- attr(ll, "df")
   2 * (p * k + 0.5 * p* (p + 1))
   
  }
  
  estim <- function(x){
    if(!is.null(x$LM$LMM) && x$LM$LMM){
      res <- paste("LMM", x$LM$estimation, sep = "-")
    } else {
      res <- if(x$LM$gls) "LM-GLS" else "LM-OLS"
    }
  }
  pp <- sapply(dots, par.pen)
  estimation <- sapply(dots, estim)
  
  if(type == "logLik") {
    out <- data.frame(logLik = as.numeric(res), 
                               estimation = estimation,
                               residual.pc.no = as.numeric(rank), 
                               penalty = as.numeric(pp), 
                               AIC = as.numeric(-2*res + pp))

  } else if(type == "cov.trace") {
    out <- as.data.frame(cbind(res, pp))
    names(out) <- c("cov.trace", "penalty")
  } else {
    out <- data.frame(Z = res)
  }
  
  if(!is.null(predictor)) {
    out$predictor <- predictor
    names(out)[which(names(out) == "predictor")] <- deparse(substitute(predictor))
  }
   
  rownames(out) <- if(mod.names) unlist(Mod.names) else
    unlist(dot.names)
  out <- list(table = out, names = dot.names)
  if(mod.names)
    out$names <- Mod.names
  
  if(verbose) {
    out$random.logL <- random.logL
    names(out$random.logL) <- if(mod.names) Mod.names else dot.names
  }
  
  attr(out, "class") <- "model.comparison"
  out
  
}

#' Reveal the inter-subject variability from a measurement error analysis
#'
#' Function produces both a list of inter-subject Euclidean distance matrices, 
#' based on replicate measurements of the same subjects, and one matrix that 
#' summarizes the variability among the inter-subject distances, across subjects.  
#' This function can be considered a tool for the evaluation of subject 
#' estimate precision.  The function, \code{\link{plot.interSubVar}} can produce a 
#' heat map of the inter-subject variability.
#'
#' 
#' @param ME A measurement error object
#' @param type A value to indicate the type of variability (statistic)
#' to measure, which can be
#' one of range (the maximum value minus the minimum value, not the two values),
#' standard deviation (sd), variance (var), or coefficient of variation (cv).  No 
#' attempt is made to assure the distribution of values is appropriate for the 
#' statistics.  For example, if only two replicates are available, using sd, var, or
#' cv might not be wise.  Or if the replicated values are exact, cv 
#' will be NA (and other stats will be 0).  Choice of statistic should consider
#' the distribution of values.
#' @keywords utilities
#' @export
#' @author Michael Collyer
#' @return An object of class \code{interSubVar} is a list containing the 
#' following
#' \item{var.map}{A distance matrix object with values that map the variability
#' statistic used for inter-subject Euclidean distances.}
#' \item{distance.mats}{The inter-subject distance matrices for every replicate.}
#' \item{subject.order}{A vector of subject levels in the order that was used to
#' guarantee consistent sorting across distance matrices.}
#' \item{var.map}{The variability type (statistic) that was used.}
#' @examples
#' 
#' \dontrun{
#' # Measurement error analysis on simulated data of fish shapes
#' 
#' data(fishy)
#' 
#' # Analysis unconcerned with groups 
#' 
#' ME1 <- measurement.error(
#'   Y = "coords",
#'   subjects = "subj",
#'   replicates = "reps",
#'   data = fishy)
#' 
#' anova(ME1)
#' ICCstats(ME1, subjects = "Subjects", with_in = "Systematic ME")
#' plot(ME1)
#' 
#' # Analysis concerned with groups 
#' 
#' ME2 <- measurement.error(
#'   Y = "coords",
#'   subjects = "subj",
#'   replicates = "reps",
#'   groups = "groups",
#'   data = fishy)
#' 
#' anova(ME2)
#' ICCstats(ME2, subjects = "Subjects", 
#'   with_in = "Systematic ME", groups = "groups")
#' P <- plot(ME2)
#' focusMEonSubjects(P, subjects = 18:20, shadow = TRUE)
#'  }
#'  
interSubVar <- function(ME, type = c("range", "sd", "var", "cv")){
  type <- match.arg(type)
  if(!type %in% c("range", "sd", "var", "cv")) type <- "range"
  
  statfx <- if(type == "cv") function(x) {sd(x) / mean(x) * 100} else
    if(type == "sd") function(x) sd(x) else if(type == "var") function(x)
      var(x) else function(x) {max(x) - min(x)}
  
  Y <- ME$LM$data$Y
  subj <- ME$LM$data$subjects
  reps <- ME$LM$data$replicates
  rep.levels <- levels(reps)
  subj.levels <- levels(subj)
  
  tb <- table(subj, reps)
  if(any(tb == 0))
    stop(paste("The ME design is not balanced and not all inter-subject distances",
               "can be estimated for every replicate.\n", sep = " "), call. = FALSE)
  
  result <- lapply(1:length(rep.levels), function(j){
    keep <- which(reps == rep.levels[j])
    Yj <- Y[keep, ]
    subj.j <- subj[keep]
    rownames(Yj) <- subj.j
    Yj <- Yj[subj.levels, ]
    dist(Yj)
  })
  
  names(result) <- rep.levels
  
  result2 <- sapply(result, as.vector)
  result3 <- apply(result2, 1, statfx)
  
  d <- result[[1]]
  d[1:length(d)] <- result3
  
  out <- list(var.map = d,
              distance.mats = result,
              subject.order = attr(d, "Labels"),
              type = type)
  
  class(out) <- "interSubVar"
  out
  
}

#' MANOVA update for lm.rrpp model fits
#' 
#' @description 
#' Function updates a lm.rrpp fit to add $MANOVA, which like 
#' $ANOVA, provides statistics
#' or matrices typically associated with multivariate analysis 
#' of variance (MANOVA).
#' 
#' MANOVA statistics or sums of squares and cross-products (SSCP) matrices
#' are calculated over the random permutations of a \code{\link{lm.rrpp}} 
#' fit.  SSCP matrices are 
#' computed, as are the inverse of R times H (invR.H), where R is a SSCP 
#' for the residuals or random effects and H is
#' the difference between SSCP matrices of full and reduced models 
#' (see below).   From invR.H,
#' MANOVA statistics are calculated, including Roy's maximum root 
#' (eigenvalue), Pillai trace, Hotelling-Lawley trace,
#' and Wilks lambda (via \code{\link{summary.manova.lm.rrpp}}).
#' 
#' The manova.update to add $MANOVA to \code{\link{lm.rrpp}} fits 
#' requires more computation time than the $ANOVA
#' statistics that are computed automatically in \code{\link{lm.rrpp}}.  
#' Generally, the same inferential conclusions will
#' be found with either approach, when observations outnumber response 
#' variables.  For high-dimensional data (more variables
#' than observations) data are projected into a Euclidean space of 
#' appropriate dimensions (rank of residual covariance matrix).  
#' One can vary the tolerance for eigenvalue decay or specify the number 
#' of PCs, if a smaller set of PCs than the maximum is desired.  
#' This is advised if there is strong correlation among variables 
#' (the data space could be simplified to fewer dimensions), as spurious
#' results are possible.  Because distributions of MANOVA stats 
#' can be generated from the random permutations,
#' there is no need to approximate F-values, like with parametric MANOVA. 
#' By restricting analysis to the real, positive eigenvalues calculated,
#' all statistics can be calculated (but Wilks lambda, as a product but 
#' not a trace, might be less reliable as variable number approaches
#' the number of observations).
#' 
#'  \subsection{ANOVA vs. MANOVA}{ 
#'  Two SSCP matrices are calculated for each linear model effect, for 
#'  every random permutation: R (Residuals or Random effects) and
#'  H, the difference between SSCPs for "full" and "reduced" models. 
#'  (Full models contain and reduced models lack
#'  the effect tested; SSCPs are hypothesized to be the same under 
#'  a null hypothesis, if there is no effect.  The 
#'  difference, H, would have a trace of 0 if the null hypothesis 
#'  were true.)  In RRPP, ANOVA and MANOVA correspond to
#'  two different ways to calculate statistics from R and H matrices.
#'  
#'  ANOVA statistics are those that find the trace of R and H SSCP 
#'  matrices before calculating subsequent statistics,
#'  including sums of squares (SS), mean squares (MS), and F-values.  
#'  These statistics can be calculated with univariate data
#'  and provide univariate-like statistics for multivariate data.  
#'  These statistics are dispersion measures only (covariances
#'  among variables do not contribute) and are the same as "distance-based" 
#'  stats proposed by Goodall (1991) and Anderson (2001).
#'  MANOVA stats require multivariate data and are implicitly 
#'  affected by variable covariances.  For MANOVA, the inverse of R times
#'  H (invR.H) is first calculated for each effect, then eigen-analysis 
#'  is performed on these matrix products.  Multivariate
#'  statistics are calculated from the positive, real eigenvalues.  
#'  In general, inferential
#'  conclusions will be similar with either approach, but effect 
#'  sizes might differ.
#'  
#'  Two important differences between manova.update and 
#'  \code{\link{summary.manova}} (for \code{\link{lm}} objects) 
#'  are that manova.update
#'  does not attempt to normalize residual SSCP matrices (unneeded 
#'  for non-parametric statistical solutions) and (2) uses a generalized
#'  inverse of the residual SSCP, if needed, when the number of 
#'  variables could render eigen-analysis problematic.  This approach 
#'  is consistent
#'  with covariance regularization methods that attempt to make covariance 
#'  matrices positive-definite for calculating model likelihoods or
#'  multivariate statistics.  If the number of observations far exceeds 
#'  the number of response variables, observed statistics from 
#'  manova.update and 
#'  \code{\link{summary.manova}} will be quite similar.  If the 
#'  number of response variables approaches or exceeds the number 
#'  of observations, manova.update
#'  statistics will be much more reliable.
#'  
#'  ANOVA tables are generated by \code{\link{anova.lm.rrpp}} on 
#'  lm.rrpp fits and MANOVA tables are generated
#'  by \code{\link{summary.manova.lm.rrpp}}, after running 
#'  manova.update on lm.rrpp fits.
#'  
#'  Currently, mixed model effects are only possible with 
#'  $ANOVA statistics, not $MANOVA.
#'  
#'  More detail is found in the vignette, ANOVA versus MANOVA.  
#' }
#' 
#' @references Goodall, C.R. 1991. Procrustes methods in the 
#' statistical analysis of shape. Journal of the Royal Statistical 
#' Society B 53:285-339.
#' @references Anderson MJ. 2001. A new method for non-parametric 
#' multivariate analysis of variance.
#'    Austral Ecology 26: 32-46.
#' @param fit Linear model fit from \code{\link{lm.rrpp}}
#' @param error An optional character string to define R matrices 
#' used to calculate invR.H.
#' (Currently only Residuals can be used and this argument defaults 
#' to NULL.  Future versions
#' will update this argument.)
#' @param tol A tolerance value for culling data dimensions to 
#' prevent spurious results.  The distribution
#' of eigenvalues for the data will be examined and if the decay 
#' becomes less than the tolerance,
#' the data will be truncated to principal components ahead of this 
#' point.  This will possibly prevent spurious results
#' calculated from eigenvalues near 0.  If tol = 0, all possible 
#' PC axes are used, which is likely
#' not a problem if observations outnumber variables.  If tol = 0 
#' and the number of variables exceeds the number of observations,
#' the value of tol will be made slightly positive to prevent problems 
#' with eigen-analysis.
#' @param PC.no A value that, if not NULL, can override the tolerance 
#' argument, and forces a desired number of 
#' data PCs to use for analysis.  If a value larger than the possible 
#' number of PCs is chosen, the full compliment of PCs
#' (the full data space) will be used.  If a number larger than tol 
#' would permit is chosen, the minimum number of PCs between the tol
#' argument and PC.no argument is returned.
#' @param print.progress A logical value to indicate whether a 
#' progress bar should be printed to the screen.
#' This is helpful for long-running analyses.
#' @param verbose Either a NULL or logical value for whether to retain
#' all MANOVA result (if TRUE).  If NULL, the verbose argument used for
#' the \code{\link{lm.rrpp}} is retained for MANOVA update.  Essentially,
#' verbose indicates whether to retain all SSCP matrices and all invR.H matrices,
#' for every model effect, in every RRPP iteration.
#' @keywords analysis
#' @export
#' @author Michael Collyer
#' @return An object of class \code{lm.rrpp} is updated to include 
#' class \code{manova.lm.rrpp}, and the object,
#' $MANOVA, which includes
#' \item{SSCP}{Terms and Model SSCP matrices.}
#' \item{invR.H}{The inverse of the residuals SSCP times the H SSCP.}
#' \item{eigs}{The eigenvalues of invR.H.}
#' \item{e.rank}{Rank of the error (residuals) covariance matrix.  
#' Currently NULL only.}
#' \item{PCA}{Principal component analysis of data, using either tol 
#' or PC.no.}
#' \item{manova.pc.dims}{Resulting number of PC vectors in the 
#' analysis.}
#' \item{e.rank}{Rank of the residual (error) covariance matrix, 
#' irrespective of the number of dimensions 
#'  used for analysis.}
#' 
#' @examples 
#' \dontrun{
#' # Body Shape Analysis (Multivariate) ----------------
#' 
#' data(Pupfish)
#' 
#' # Although not recommended as a practice, this example will use only
#' # three principal components of body shape for demonstration.  
#' # A larger number of random permutations should also be used.
#' 
#' Pupfish$shape <- ordinate(Pupfish$coords)$x[, 1:3]
#' 
#' Pupfish$logSize <- log(Pupfish$CS) 
#' 
#' fit <- lm.rrpp(shape ~ logSize + Sex, SS.type = "I", 
#' data = Pupfish, print.progress = FALSE, iter = 499) 
#' summary(fit, formula = FALSE)
#' anova(fit) # ANOVA table
#' 
#' # MANOVA
#' 
#' fit.m <- manova.update(fit, print.progress = FALSE, tol = 0.001)
#' summary(fit.m, test = "Roy")
#' summary(fit.m, test = "Pillai")
#' 
#' fit.m$MANOVA$eigs$obs # observed eigenvalues
#' fit.m$MANOVA$SSCP$obs # observed SSCP
#' fit.m$MANOVA$invR.H$obs # observed invR.H 
#' 
#' # Distributions of test statistics
#' 
#' summ.roy <- summary(fit.m, test = "Roy")
#' dens <- apply(summ.roy$rand.stats, 1, density)
#' par(mfcol = c(1, length(dens)))
#' for(i in 1:length(dens)) {
#'      plot(dens[[i]], xlab = "Roy max root", ylab = "Density",
#'      type = "l", main = names(dens)[[i]])
#'      abline(v = summ.roy$rand.stats[1, i], col = "red")
#' }
#' par(mfcol = c(1,1))
#' }

manova.update <- function(fit, error = NULL, 
                          tol = 1e-7, PC.no = NULL,
                          print.progress = TRUE,
                          verbose = NULL) {
  
  if(inherits(fit, "manova.lm.rrpp")) 
    stop("\nlm.rrpp object has already been updated for MANOVA.\n", 
         call. = FALSE)
  if(!inherits(fit, "lm.rrpp")) 
    stop("\nOnly an lm.rrpp object can be updated for MANOVA.\n", 
         call. = FALSE)
  if(is.null(verbose)) verbose <- fit$verbose else
    verbose <- as.logical(verbose)
  
  p <- fit$LM$p
  if(p < 2) 
    stop("\n Multiple responses are required for this analysis.\n", 
         call. = FALSE)
  p.prime <- fit$LM$p.prime
  n <- fit$LM$n
  gls <- fit$LM$gls
  w <- if(!is.null(fit$LM$weights)) fit$LM$weights else NULL
  if(gls && is.null(fit$LM$weights) && is.null(fit$LM$Pcov))
    fit$LM$Pcov <- Cov.proj(fit$LM$Cov)
  Pcov <- if(!is.null(fit$LM$Pcov)) fit$LM$Pcov else NULL
  Y <- fit$LM$Y
  
  PermInfo <- getPermInfo(fit, attribute = "all")
  
  perm.method <- PermInfo$perm.method
  if(perm.method == "RRPP") RRPP = TRUE else RRPP = FALSE
  ind <- PermInfo$perm.schedule
  perms <- length(ind)
  
  Models <- getModels(fit, attribute = "all")
  
  reduced <- Models$reduced
  full <- Models$full
  Terms <- fit$LM$Terms
  trms <- fit$LM$term.labels
  k <- length(trms)
  
  if(k > length(Models$full)) {
    k <- length(Models$full)
    trms <- names(Models$full)
  }
  
  if(k == 0) stop("\nNo model terms from which to calculate SSCPs.\n",
                  call. = FALSE)
  int <- attr(Terms, "intercept")
  
  E.rank <- QRforX(var(as.matrix(resid(fit))))$rank
  
  PCA <- ordinate(Y, tol = tol, rank. = PC.no)
  d <- PCA$sdev
  if(!is.null(PC.no)) {
    ld <- seq_len(min(PC.no, length(d)))
    d <- PCA$sdev <- PCA$sdev[ld]
    PCA$x <- PCA$x[, ld]
  }
  
  if(length(d) > min(n - 1, p)) {
    mnp <- seq_len(min(n - 1, p))
    d <- PCA$sdev <- PCA$sdev[mnp]
    PCA$x <- PCA$x[, mnp]
  }
  
  kk <- if (!is.null(PC.no)) {
    stopifnot(length(PC.no) == 1, is.finite(PC.no), as.integer(PC.no) > 
                0)
    min(as.integer(PC.no), n, p)
  } else min(n, p)
  
  
  if (tol > 0) {
    rank <- sum(d > (d[1L] * tol))
    if (rank < kk) {
      j <- seq_len(rank)
      PCA$x <- PCA$x[, j, drop = FALSE]
    }
  }
  
  SS.tot <- sum(diag(crossprod(center(Y))))
  if(NCOL(Y) > NCOL(PCA$x)) Y <- PCA$x
  
  if(!is.null(error)) {
    if(!inherits(error, "character")) 
      stop("The error description is illogical.  
           It should be a string of character values matching ANOVA terms.",
           call. = FALSE)
    kk <- length(error)
    if(kk != k) 
      stop("The error description should match in length the number of ANOVA terms 
           (not including Residuals)",
           call. = FALSE)
    Ematch <- match(error, c(trms, "Residuals"))
    if(any(is.na(Ematch))) 
      stop("At least one of the error terms is not an ANOVA term",
           call. = FALSE)
  } else Ematch <- NULL
  
  # Until a better solution is found, this must be forced
  error <- Ematch <- NULL
  
  Qr <- lapply(reduced, function(q) q$qr)
  Qf <- lapply(full, function(q) q$qr)
  
  Ur <- lapply(Qr, function(q) q$Q)
  Uf <- lapply(Qf, function(q) q$Q)
  Ufull <- Uf[[max(1, k)]]
  
  int <- attr(Terms, "intercept")
  Qint <- if(!is.null(Pcov)) 
    QRforX(Pcov %*% rep(int, n), reduce = FALSE) else if(!is.null(w)) 
      QRforX(rep(int, n) * sqrt(w), reduce = FALSE) else
        QRforX(rep(int, n), reduce = FALSE)
  Unull <- Qint$Q
  
  if(!is.null(Pcov)) Y <- Pcov %*% Y
  if(!is.null(w)) Y <- Y * sqrt(w)
  
  yh0 <- fastFit(Unull, Y, n, p.prime)
  r0 <- Y - yh0
  
  
  if(inherits(fit, "lm.rrpp.ws")){
    ind_s <- perm.index(n, iter = (perms - 1), 
                        block = NULL, seed = PermInfo$perm.seed)
    if(fit$subTest){
      STerm <- 
        which(fit$LM$term.labels == fit$subjects.var)
      use_ind_s <- TRUE
    } else {
      STerm <- NULL
      use_ind_s <- FALSE
    }
    
  } else {
    use_ind_s <- FALSE
    STerm <- NULL
    ind_s <- NULL
  }
  
  rrpp <- function(FR, ind.i,
                   use_ind_s, 
                   STerm, ind_s.i) {
    result <- lapply(FR, function(x) as.matrix(x$fitted) + as.matrix(x$residuals)[ind.i, ])
    if(use_ind_s)
      FR[[STerm]] <- as.matrix(FR[[STerm]]$fitted) + 
        as.matrix(FR[[STerm]]$residuals)[ind_s.i, ]
    result
  }
  
  fitted <- lapply(Ur, function(u) fastFit(u, Y, n, p.prime))
  res <- lapply(fitted, function(f) Y - f)
  
  if(!RRPP) {
    fitted <- lapply(fitted, function(.) matrix(0, n, p.prime))
    res <- lapply(res, function(.) as.matrix(Y))
  } 
  
  
  FR <- lapply(1:max(1, k), function(j) list(fitted = fitted[[j]], 
                                             residuals = res[[j]]))
  rrpp.args <- list(FR = FR, ind.i = ind[[1]], 
                    use_ind_s = use_ind_s, 
                    STerm = STerm, ind_s.i = NULL)
  
  sscp.args <- list(Uf = Uf, Ur = Ur, 
                    Ufull = Ufull, Unull = Unull,
                    Y = NULL, 
                    n = n, p = p.prime,
                    Yt = Y)
  
  sscp <- function(Uf, Ur, Ufull, Unull, Y, 
                   n, p, Yt) {
    ss <- Map(function(uf, ur, y) list(SS = crossprod(
      fastFit(uf, y, n, p.prime) - fastFit(ur, y, n, p.prime)),
      Residuals = crossprod(y - fastFit(Ufull, y, n, p.prime))), 
      Uf, Ur, Y)
    
    names(ss) <- trms
    rss <- crossprod(Yt - fastFit(Ufull, Yt, n, p.prime))
    tss <- crossprod(Yt - fastFit(Unull, Yt, n, p.prime))
    
    result <- c(ss, list(Full.Model = list(SS = tss - rss, 
                                           Residuals = rss)))
    
    result
  }
  
  
  invR.H <- function(ss){ # modified from stats::summary.manova version
    tss <- diag(Reduce("+", ss[[length(ss)]]))
    result <- lapply(1:length(ss), function(j){
      H <- ss[[j]]$SS
      R <- ss[[j]]$Residuals
      fast.solve(R) %*% H
    })
    
    names(result) <- c(trms, "Full.Model")
    
    result
  }
  
  getEigs <- function(x) Re(eigen(x, symmetric = FALSE, 
                                  only.values = TRUE)$values)
  
  getEigsL <- function(L) t(sapply(L, getEigs))
  
  
  if(print.progress){
    cat(paste("\nCalculation of SSCP matrix products:", 
              perms, "permutations.\n")) 
    pb <- txtProgressBar(min = 0, max = perms+1, initial = 0, style=3)
  }
  
  result <- lapply(1: perms, function(j){
    step <- j
    if(print.progress) setTxtProgressBar(pb,step)
    x <- ind[[j]]
    rrpp.args$ind.i <- x
    if(rrpp.args$use_ind_s)
      rrpp.args$ind_s.i <- ind_s[[j]]
    sscp.args$Y <- do.call(rrpp, rrpp.args) 
    sscp.args$Yt <- yh0 + r0[x, ]
    ss <- do.call(sscp, sscp.args)
    invRH <- invR.H(ss)
    eigs <- getEigsL(invRH)
    list(SSCP = if(verbose) ss else NULL, 
         invR.H = if(verbose) invRH else NULL, 
         eigs = eigs)
    
  })
  
  
  if(print.progress) {
    step <- perms + 1
    setTxtProgressBar(pb,step)
    close(pb)
  }
  
  
  SSCP <- if(verbose) lapply(result, function(x) x$SSCP) else NULL
  invR.H <- if(verbose) lapply(result, function(x) x$invR.H) else NULL
  eigs <- lapply(result, function(x) x$eigs)
  
  if(verbose){
    names(SSCP) <- names(invR.H) <- c("obs", paste("iter", 1:(perms - 1), sep = "."))
  }
  names(eigs) <- c("obs", paste("iter", 1:(perms - 1), sep = "."))
  
  
  out <- fit
  out$MANOVA <- list(SSCP = SSCP, invR.H = invR.H, eigs = eigs,
                     error = error, PCA = PCA, manova.pc.dims = length(d), 
                     e.rank = E.rank, SS.tot = SS.tot)
  
  class(out) <- c("manova.lm.rrpp", class(fit))
  out
}

#' Linear Model Evaluation with RRPP performed within subjects
#'
#' Function performs a linear model fit over many random permutations of 
#' data, using a randomized residual permutation procedure restricted to
#' subjects.
#'
#' The function fits a linear model using ordinary least squares (OLS) or 
#' generalized least squares (GLS) estimation of coefficients over any 
#' number of random permutations of the data, but the permutations are mostly 
#' restricted to occur with subject blocks for any model terms other than subjects.  
#' All functionality should resemble that of \code{\link{lm.rrpp}}.  However, 
#' an argument for research subjects is also required.  The purpose of this function 
#' is to account for the non-independence among observations of research subjects 
#' (due to sampling within subjects), while also allowing for the non-independence 
#' among subjects to be considered (Adams and Collyer, submitted).  
#' 
#' By comparison, the covariance matrix option in \code{\link{lm.rrpp}} must have a
#' one-to-one match to observations, which can be matched by the row names of the data.
#' In this function, the covariance matrix can be the same one used in \code{\link{lm.rrpp}}
#' but the number of observations can be greater.  For example, if subjects are
#' species or some other level of taxonomic organization, data can comprise measurements
#' on individuals.  Users have the option to expand the covariance matrix for subjects
#' or input one they have generated.
#' 
#' Irrespective of covariance matrix type, the row names of the data matrix must match the
#' subjects.  This step assures that the analysis can proceed in \code{\link{lm.rrpp}}.  It
#' is also best to make sure to use an \code{\link{rrpp.data.frame}}, so that the subjects
#' can be a name in that data frame.  For example, if research subjects are species and
#' data (observations) are collected from individuals within species, then a procedure like 
#' the following should produce results:
#' 
#' rownames(Y) <- species
#' 
#' rdf <- rrpp.data.frame(Y = Y, subjects = species, x = x)
#' 
#' fit <- lm.rrpp.ws(Y ~ species * x, subject = species, data = rdf, Cov = myCov, ...)
#' 
#' where ... means other arguments.  The covariances in the the Covariance matrix can be 
#' sorted by the subjects factor but data will not be sorted.  Therefore, names matching
#' the subjects is essential.  Additionally, subjects must be a factor in the data frame
#' or a factor in the global environment.  It cannot be part of a list.  Something like
#' subjects <- mylist$species will not work.  Assuring that data and subjects are in the 
#' same \code{\link{rrpp.data.frame}} object as data is the best way to avoid errors.
#' 
#' Most attributes for this analysis are explained with \code{\link{lm.rrpp}}.  
#' The notable different attributes for this function are that: (1) a covariance 
#' matrix for the non-independence of subjects can be either a symmetric matrix 
#' that matches in dimensions the number of subjects or the number of observations; 
#' (2) a parameter (delta) that can range between near 0 and 1 to calibrate the 
#' covariances between observations of different subjects; and (3) a 
#' parameter (gamma) that is either 1 (equal) or the square-root of the subject 
#' sample size (sample) to calibrate the covariances among observations 
#' within subjects.  If delta = 0, it is expected that the covariance between
#' individual observations, between subjects, is the same as expected from the
#' covariance matrix, as if observations were the single observations made on subjects.
#' As delta approaches 1, the observations become more independent, as if it is 
#' expected that the many observations would not be expected to be
#' as correlated as if from one observation.  Increasing delta might be useful, if, 
#' for example, many individuals are sampled within species, from different locations,
#' different age groups, etc.  Alternatively, the sample size (n_i) for subject i 
#' can also influence the trend of inter-subject covariances.  If more individual
#' observations are sampled, the correlation between subjects might be favored
#' to be smaller compared to fewer observations.  The covariances can be adjusted
#' to allow for greater independence among observations to be assumed for larger samples.
#'
#' A design matrix, \bold{X}, is constructed with 0s and 1s to indicate subjects association,
#' and it is used to expand the covariance matrix (\bold{C}) by \bold{XCt(X)}, where \bold{t(X)}
#' is the matrix transpose.  The parameters in \bold{X} are multiplied by exp(-delta * gamma)
#' to scale the covariances.  (If delta = 0 and gamma = 1, they are unscaled.)  
#' 
#' These options for scaling covariances could be important for data with 
#' hierarchical organization.
#' For example, data sampled from multiple species with expected covariances 
#' among species based on phylogenetic distances, might be expected to not covary as
#' strongly if sampling encounters other strata like population, sex, and age.
#' An a priori expectation is that covariances among observations would be expected to
#' be smaller than between species, if only one observation per species were made.
#' 
#' If one wishes to have better control over between-subject and within-subject
#' covariances, based on either a model or empirical knowledge, a covariance matrix should 
#' be generated prior to analysis.  One can input a covariance matrix with dimensions 
#' the same as \bold{XCt(X)}, if they prefer to define covariances in an alternative way.
#' A function to generate such matrices based on separate
#' inter-subject and intra-subject covariance matrices is forthcoming.
#' 
#' IMPORTANT.  It is assumed that either the levels of the covariance matrix (if 
#' subject by subject) match the subject levels in the subject argument, or that
#' the order of the covariance matrix (if observation by observation) matches the order
#' of the observations in the data.  
#' No attempt is made to reorder a covariance matrix by observations
#' and row-names of data are not used to re-order the covariance matrix.  If the
#' covariance matrix is small (same in dimension as the number of subject levels), the function
#' will compile a large covariance matrix that is correct in terms of order, but
#' this is based on the subjects argument, only.
#' 
#' The covariance matrix is important for describing the expected covariances
#' among observations, especially knowing observations between and within subjects 
#' are not independent.  However, the randomization of residuals in a permutation
#' procedure (RRPP) is also important for testing inter-subject and 
#' intra-subject effects.  There are two RRPP philosophies used.  If the
#' variable for subjects is part of the formula, the subject effect is evaluated with
#' type III sums of squares and cross-products (estimates SSCPs between a model with all
#' terms and a model lacking subject term), and RRPP performed for all residuals of the
#' reduced model.  Effects for all other terms are evaluated with type II SSCPs and RRPP
#' restricted to randomization of reduced model residuals, within subject blocks.  This
#' assures that subject effects are held constant across permutations, so that intra-subject
#' effects are not confounded by inter-subject effects.
#' 
#' 
#' More details will be made and examples provided after publication of articles introducing 
#' the novel RRPP approach.
#' 
#' 
#' The \code{\link{lm.rrpp}} arguments not available for this function include: 
#' full.resid, block, and SS.type.  These arguments are fixed because of
#' the within-subject blocking for tests, plus the requirement for type II SS
#' for within-subject effects.
#' 
#' 
#' @param f1 A formula for the linear model (e.g., y~x1+x2).  
#' @param subjects A variable that can be found in the data frame indicating the research subjects
#' for the analysis.  This variable must be in the data frame.  Is can be either numeric 
#' (if its slot in the data frame is known) or a character, e.g., "sub_id".  It is imperative that
#' it is ordered the same as the data but that the data do not have row names the same as subjects.
#' For example, the subjects variable in the data frame might be sub_id: sub1, sub1, sub1, sub2,
#' sub2, sub2, ... and the row names of the data might be obs1, obs2, obs3, obs4, obs5, obs6, ...  
#' The data do not need to have row names but the subjects variable has to be provided.
#' @param iter Number of iterations for significance testing
#' @param turbo A logical value that if TRUE, suppresses coefficient estimation 
#' in every random permutation.  This will affect subsequent analyses that 
#' require random coefficients (see \code{\link{coef.lm.rrpp}})
#' but might be useful for large data sets for which only ANOVA is needed.
#' @param seed An optional argument for setting the seed for random 
#' permutations of the resampling procedure.
#' If left NULL (the default), the exact same P-values will be found 
#' for repeated runs of the analysis (with the same number of iterations).
#' If seed = "random", a random seed will be used, and P-values will vary.  
#' One can also specify an integer for specific seed values,
#' which might be of interest for advanced users.
#' @param int.first A logical value to indicate if interactions of first 
#' main effects should precede subsequent main effects
#' @param RRPP A logical value indicating whether residual randomization 
#' should be used for significance testing
#' @param Cov An optional argument for including a covariance matrix 
#' to address the non-independence
#' of error in the estimation of coefficients (via GLS).  If included, 
#' any weights are ignored.  This matrix must match in dimensions either
#' the number of subject levels or the number of observations.
#' @param delta A within-subject scaling parameter for covariances, ranging from 
#' 0 to 1.  If delta = 0, a sight value (0.001) is added to assure variances of the 
#' covariance matrix are 0.1 percent larger than covariances.
#' @param gamma A sample-size scaling parameter that is adjusted to be 1 ("equal")
#' scaling or the square-root of the sample size for subject observations ("sample").
#' @param data A data frame for the function environment, see 
#' \code{\link{rrpp.data.frame}}.  A data frame is required for this analysis.
#' @param print.progress A logical value to indicate whether a progress 
#' bar should be printed to the screen.
#' This is helpful for long-running analyses.
#' @param verbose A logical value to indicate if all possible output from an analysis 
#' should be retained. Generally this should be FALSE, unless one wishes to extract, 
#' e.g., all possible terms, model matrices, QR decomposition, or random permutation 
#' schemes.
#' @param Parallel Either a logical value to indicate whether parallel processing 
#' should be used, a numeric value to indicate the number of cores to use, or a predefined
#' socket cluster.  This argument defines parallel processing via the \code{parallel} library. 
#' If TRUE, this argument invokes forking or socket cluster assignment of all processor cores, 
#' except one.  If FALSE, only one core is used. A numeric value directs the number of cores to 
#' use, but one core will always be spared.  If a predefined socket cluster (Windows) is provided,
#' the cluster information will be passed to \code{parallel}.
#' @param ... Arguments typically used in \code{\link{lm}}, such as 
#' weights or offset, passed on to
#' \code{lm.rrpp} for estimation of coefficients.  If both weights and 
#' a covariance matrix are included,
#' weights are ignored (since inverses of weights are the diagonal elements 
#' of weight matrix, used in lieu
#' of a covariance matrix.)
#' @keywords analysis
#' @export
#' @author Michael Collyer
#' @return An object of class \code{lm.rrpp.ws} is a list containing the 
#' following
#' \item{call}{The matched call.}
#' \item{LM}{Linear Model objects, including data (Y), coefficients, 
#' design matrix (X), sample size
#' (n), number of dependent variables (p), dimension of data space (p.prime),
#' QR decomposition of the design matrix, fitted values, residuals,
#' weights, offset, model terms, data (model) frame, random coefficients 
#' (through permutations),
#' random vector distances for coefficients (through permutations), 
#' whether OLS or GLS was performed, 
#' and the mean for OLS and/or GLS methods. Note that the data returned 
#' resemble a model frame rather than 
#' a data frame; i.e., it contains the values used in analysis, which 
#' might have been transformed according to 
#' the formula.  The response variables are always labeled Y.1, Y.2, ..., 
#' in this frame.}
#' \item{ANOVA}{Analysis of variance objects, including the SS type, 
#' random SS outcomes, random MS outcomes,
#' random R-squared outcomes, random F outcomes, random Cohen's f-squared 
#' outcomes, P-values based on random F
#' outcomes, effect sizes for random outcomes, sample size (n), number of 
#' variables (p), and degrees of freedom for
#' model terms (df).  These objects are used to construct ANOVA tables.}
#' \item{PermInfo}{Permutation procedure information, including the number 
#' of permutations (perms), The method
#' of residual randomization (perm.method), and each permutation's sampling 
#' frame (perm.schedule), which
#' is a list of reordered sequences of 1:n, for how residuals were 
#' randomized.}
#' \item{Models}{Reduced and full model fits for every possible model 
#' combination, based on terms
#' of the entire model, plus the method of SS estimation.}
#' @seealso \code{\link{lm.rrpp}}; \code{\link{measurement.error}}
#' @references Adams, D.C and M.L Collyer. (submitted) Extended phylogenetic regression models for 
#' comparing within-species patterns across the Tree of Life. 
#' Methods in Ecology and Evolution
#' @references ter Braak, C.J.F. 1992. Permutation versus bootstrap significance tests in 
#' multiple regression and ANOVA. pp .79–86 In Bootstrapping and Related Techniques. eds K-H. Jockel, 
#' G. Rothe & W. Sendler.Springer-Verlag, Berlin.
#' \code{\link[stats]{lm}} for more on linear model fits.
#' @examples 
#' \dontrun{
#' data(fishy)
#' 
#' suppressWarnings(fit <- lm.rrpp.ws(coords ~ subj + groups * reps,
#'   subjects = "subj", 
#'   data = fishy))
#' 
#' anova(fit)
#' }

lm.rrpp.ws <- function(f1, subjects, 
                       iter = 999, turbo = FALSE, 
                       seed = NULL, int.first = FALSE,
                       RRPP = TRUE, 
                       data, Cov = NULL,
                       delta = 0.001, 
                       gamma = c("sample", "equal"),
                       print.progress = FALSE, 
                       verbose = FALSE,
                       Parallel = FALSE, ...) {
  
  L <- c(as.list(environment()), list(...))
  if(is.null(L$data))
    stop("\nWithin-subjects analysis requires a data frame, continaing the subjects factor.\n",
         call. = FALSE)
  if(is.numeric(subjects) && length(subjects) > 1)
    stop("\nThe subjects argument should be a single numeric or character values.\n",
         call. = FALSE)
  if(is.numeric(subjects)) subj.no <- subjects else
    subj.no <- which(names(L$data) %in% subjects)
  
  if(length(subj.no) == 0)
    stop("\nSubjects factor not found in data frame.\n",
         call. = FALSE)
  
  if(is.null(L$data))
    stop(paste("\nA data frame is required for this analysis,",
               "and it must contain the subjects factor.\n", sep = " "),
         call. = FALSE)
  L$sub.var.name <- names(L$data)[[subj.no]]
  L$subjects <- as.factor(L$data[[subj.no]])
  L$SS.type <- "IIws"
  L$full.resid <- FALSE
  L$gamma <- match.arg(gamma)
  L$block <- NULL
  if(length(L$gamma) > 1) L$gamma <- "equal"
  out <- do.call(.lm.rrpp, L)
  out$call <- match.call()
  out$ANOVA$SS.type <- "Within-subject II"
  class(out) <- c("lm.rrpp.ws", class(out))
  out

}

#' coef for lm.rrpp model fits
#'
#' @description Computes ordinary or generalized least squares coefficients
#' over the permutations of an \code{\link{lm.rrpp}} model fit with predefined 
#' random permutations.
#' For each coefficient vector, the Euclidean distance is calculated as an 
#' estimate of
#' the amount of change in Y, the n x p matrix of dependent variables; larger 
#' distances mean more change 
#' in location in the data space associated with a one unit change in the model 
#' design, for the parameter
#' described.  Random coefficients are based on either RRPP or FRPP, as defined 
#' by the 
#' \code{\link{lm.rrpp}} model fit.  
#' 
#'  \subsection{The test argument will be deprecated}{ 
#'  
#' The following details will also be removed:
#'  
#' This function can be used to test the specific coefficients of an 
#' lm.rrpp fit.  The test
#' statistics are the distances (d), which are also standardized (Z-scores).  
#' The Z-scores might be easier to compare,
#' as the expected values for random distances can vary among coefficient 
#' vectors.
#' 
#' If RRPP is used, all distributions of coefficient vector distances are 
#' based on appropriate null models, as defined by SS type.  Please be aware that this 
#' can result in two seemingly strange but reasonable phenomena.  First, if type II or
#' type III SS is used, the intercept will not appear in test results (because the function
#' seeks model parameter differences to know for which coefficients to calculate Euclidean 
#' distances).  Even if it appears for type I SS, this is merely an artifact of sequential
#' model building and there really is no meaningful test of intercept = 0.  Second, 
#' Euclidean distances might not always be logical, especially when viewing univariate
#' coefficients, in which case the expected d is |b|.  Coefficients without a test are
#' based on the full model; tests are based on the estimates of coefficients (b), 
#' given a null model.  For example, for a model, y ~ b1 + b2 + b3, with type I SS,
#' b2 will be estimated and tested, using a null model, y ~ b1 and a full model, 
#' y ~ b1 + b2.  The estimate for b2 might not be the same in the test as when estimated 
#' from the model, y ~ b1 + b2 + b3.  Therefore, the d statistic might not reflect what one
#' would expect from the full model (like when using type III SS).  
#'  }
#' 
  
#'  \subsection{Difference between coef.lm.rrpp test and betaTest}{ 
#'  The test for coef.lm.rrpp uses the square-root of inner-products of vectors (d) as a 
#'  test statistic and only tests the null hypothesis that the length of the vector is 0.  
#'  The significance of the test is based on random values produced by RRPP, based on the
#'  matrices of coefficients that are produced in all permutations.  The null models for generating
#'  RRPP distributions are consistent with those used for ANOVA, as specified in the 
#'  \code{\link{lm.rrpp}} fit by choice of SS type.  Therefore, the random coefficients are
#'  consistent with those produced by RRPP for generating random estimates used in ANOVA.
#'  
#'  The betaTest analysis allows different null hypotheses to be used (vector length is not necessarily 0) 
#'  and unless otherwise specified, uses a null model that lacks one vector of parameters and a full
#'  model that contains all vectors of parameters, for the parameter for which coefficients are estimated.
#'  This is closest to a type III SS method of estimation, but each parameter is dropped from the model,
#'  rather than terms potentially comprising several parameters.  Additionally, betaTest calculates
#'  Mahalanobis distance, in addition to Euclidean distance, for vectors of coefficients.  
#'  This statistic is probably 
#'  better for more types of models (like generalized least squares fits).
#' }
#' \subsection{coef.lmm.rrpp}{
#' The coef.lmm.rrpp function computes 
#' combined fixed and random effects by subject for 
#' \code{\link{lmm.rrpp}} model fits. This function does not operate the same as
#' \code{\link{coef.lm.rrpp}}, for which coefficients and fixed effects are the same,
#' and tests can be performed on fixed effects.  It is merely a wrapper to combine 
#' the results of \code{\link{ranef.lmm.rrpp}} and \code{\link{fixef.lmm.rrpp}}.  
#' }
#' @param object Object from \code{\link{lm.rrpp}}
#' @param SE Whether to include standard errors of coefficients.  Standard
#' errors are muted if test = TRUE.
#' @param test Logical argument that if TRUE, performs hypothesis tests 
#' (Null hypothesis is vector distance = 0)
#' for the observed coefficients.  If FALSE, only the observed coefficients 
#' are returned.
#' @param confidence The desired confidence limit to print with a table of 
#' summary statistics,
#' if test = TRUE.  Because distances are directionless, confidence limits 
#' are one-tailed.
#' @param ... Other arguments (currently none)
#' @export
#' @author Michael Collyer
#' @keywords utilities
#' @seealso \code{\link{betaTest}}
#' @examples 
#' \dontrun{
#' # See examples for lm.rrpp to see how anova.lm.rrpp works in conjunction
#' # with other functions
#' 
#' data(Pupfish)
#' names(Pupfish)
#' Pupfish$logSize <- log(Pupfish$CS)
#'
#' fit <- lm.rrpp(coords ~ logSize + Sex*Pop, 
#' SS.type = "I", data = Pupfish, verbose = TRUE) 
#' 
#' coef(fit) # Just coefficients
#' coef(fit, SE = TRUE) # Coefficients with SE
#' coef(fit, test = TRUE, 
#' confidence = 0.99) # Test of coefficients
#' }
coef.lm.rrpp <- function(object, SE = FALSE, test = FALSE, confidence = 0.95, ...) {
  x <- object
  k <- length(x$LM$term.labels)
  rc <- x$LM$random.coef
  rd <- x$LM$random.coef.distances
  gls <- x$LM$gls
  coef.obs <- if(gls) x$LM$gls.coefficients else x$LM$coefficients
  n <- x$LM$n
  p <- x$LM$p
  p.prime = x$LM$p.prime
  model.terms <- x$LM$Terms
  
  PermInfo <- getPermInfo(x, "all")
  
  perms <- PermInfo$perms
  SS.type <- x$ANOVA$SS.type
  RRPP <- PermInfo$perm.method
  
  indb <- boot.index(length(PermInfo$perm.schedule[[1]]), 
                            PermInfo$perms -1, 
                     PermInfo$block, 
                     PermInfo$perm.seed)
  if(SE) {
    betas <- beta.boot.iter(x, indb)
    betas <- sapply(betas, as.vector)
    if(is.vector(betas))
      betas <- matrix(betas, 1, length(betas))
    bd <- betas - rowMeans(betas)
    if(is.vector(bd))
      bd <- matrix(bd, 1, length(bd))
    se <- sqrt(rowSums(bd^2) / perms)
    if(is.matrix(coef.obs)){
      se <- matrix(se, nrow(coef.obs), ncol(coef.obs))
      dimnames(se) <- dimnames(coef.obs)
    } else names(se) <- names(coef.obs)
    
    if(is.matrix(se) && any(rownames(se) == "(Intercept)")){
      rmove <- which(rownames(se) == "(Intercept)")
      Y <- as.matrix(x$LM$Y)
      X <- x$LM$X
      if(gls) {
        Pcov <- x$LM$Pcov
        if(is.null(Pcov) && !is.null(x$LM$Cov))
          Pcov <- Cov.proj(x$LM$Cov)
        if(is.null(Pcov)){
          w <- sqrt(x$LM$weights)
          Y <- Y * w
          X <- X * w
        } else {
          Y <- Pcov %*% Y
          X <- Pcov %*% X
        }
      }
      
      X0 <- X[, -rmove, drop = FALSE]
      Q0 <- QRforX(X0)$Q
      Hb <- as.matrix(getHb(QRforX(X)))
      Ft <- fastFit(Q0, Y, n, p)
      Re <- as.matrix(Y - Ft)

      result <- sapply(indb, function(x)
        as.matrix(Hb %*% (Ft + Re[x, , drop = FALSE]))[rmove, , drop = FALSE])
      
      if(is.vector(result)) result <- matrix(result, 1, perms)
      ri <- as.matrix(result - rowMeans(result))
      se[rmove, ] <- sqrt(rowSums(ri^2) / perms)
    }
    
  } else se <- NULL
  
  test.ok <- (x$verbose && !x$turbo)
  if(test && !test.ok) {
    cat("\nCoefficients test not available because you either turbo-charged\n")
    cat("your model fit or used verbose = FALSE.\n")
    cat("Go back to lm.rrpp and choose turbo = FALSE & verbose = TRUE\n")
    cat("if you wish to test coefficients.\n\n")
    test = FALSE
  }
  if(test){
    if(confidence < 0) stop("Confidence level should be between 0 and 1")
    if(confidence > 1) stop("Confidence level should be between 0 and 1")
    if(k > 0) {
      PV <- apply(rd, 1, pval)
      Z <- apply(rd, 1, effect.size)
      ucl <- apply(rd, 1, function(x) quantile(x, confidence))
      stat.tab <- data.frame(d.obs = rd[,1], ucl=ucl, Zd=Z, P=PV)
      colnames(stat.tab)[2] <- paste("UCL (", confidence*100,"%)", sep = "")
      colnames(stat.tab)[4] <- "Pr(>d)"

    } else {
      PV <- Z <- ucl <- stat.tab <- NULL
    }
    
    out <- list(coef.obs = coef.obs,
                coef.se <- se,
                random.coef = rc,
                random.distances = rd,
                n = n, p=p, p.prime=p.prime, k.terms = k, 
                confidence = confidence,
                model.terms = model.terms, nperms = perms,
                RRPP = RRPP, gls=gls, SS.type = SS.type,
                stat.table = stat.tab, test = test)
    class(out) <- "coef.lm.rrpp"
  } else if(SE){
    out <- list(coef.obs = coef.obs, coef.se = se)
    out$test <- FALSE
    class(out) <- "coef.lm.rrpp"
    } else out <- coef.obs
  
  out
}

#' coef for lmm.rrpp model fits
#'
#' @description The coef.lmm.rrpp function computes 
#' combined fixed and random effects by subject for 
#' \code{\link{lmm.rrpp}} model fits. This function does not operate the same as
#' \code{\link{coef.lm.rrpp}}, for which coefficients and fixed effects are the same,
#' and tests can be performed on fixed effects.  It is merely a wrapper to combine 
#' the results of \code{\link{ranef.lmm.rrpp}} and \code{\link{fixef.lmm.rrpp}}.
#' @param object Object from \code{\link{lmm.rrpp}}
#' @param type Whether to combine coefficients in a matrix or a list by variable.
#' @param ... Other arguments (currently none)
#' @export
#' @author Michael Collyer
#' @keywords utilities
coef.lmm.rrpp <- function(object, 
                          type = c("matrix",
                                   "list"),
                          ...){
  type <- match.arg(type)
  Bf <- as.matrix(object$LM$coefficients[
    object$LM$coef.fixed,
  ])
  resr <- ranef.lmm.rrpp(object, type = "list")
  ns <- NROW(resr[[1]])
  subjLevels <- levels(object$subjects)
  
  fix <- lapply(1:ncol(Bf), function(j){
    r <- t(matrix(Bf[,j], nrow(Bf), ns))
    colnames(r) <- rownames(Bf)
    r
  })
  
  res <- Map(function(r, f){
    addto <- which(colnames(f) %in% colnames(r))
    f[, addto] <- f[, addto] + r
    rownames(f) <- subjLevels
    f
  }, resr, fix)
  
  names(res) <- names(resr)
  if(type == "matrix"){
    rnms <- rep(colnames(res[[1]]), ns)
    subjLevels <- levels(object$subjects)
    rnms[which(rnms == "(Intercept)")] <- subjLevels
    res <- sapply(res, function(m){
      as.vector(t(m))
    })
    
    dimnames(res) <- list(rnms, colnames(Bf))
    
  }
  
  res
}

#' Intraclass correlation statistics from an lm.rrpp model fits
#'
#' Function performs analyses concerned with the repeatability (reliability) of multivariate data 
#' (measurements) collected from the same research subjects.  Although there is no
#' requirement for repeated measurements on all research subjects, the analysis assumes
#' that multiple observations are made. 
#' 
#' Function uses ANOVA statistics or SSCP matrices to find the ratio of among-subject
#' to within-subject variance.  The former is a dispersion-based approach and the latter
#' is a multivariate generalization of the ICC statistic (as a matrix product).  The
#' multivariate generalizations of the statistics
#' described by Liljequist et al. (2019) are used to find matrix products, 
#' from which eigenanalysis is performed, providing ICC statistics by eigenvectors.  
#' 
#' Three statistics describe the ICC for the population,
#' agreement of measurements among subjects, and consistency between measurements.  
#' The last statistic does not necessarily measure the sameness 
#' between measurements but the consistency of change between measurements,
#' which might be indicative of a systematic measurement error.  
#' If groups are used, these three statistics are 
#' repeated, using the SSCP for groups-adjusted data.  
#' This approach accounts for group differences,
#' which would avoid large subject variation compared to measurement error 
#' inflating ICC values.  If there are 
#' inherently disparate groups from which subjects are sampled, 
#' this approach can elucidate better agreement and 
#' consistency in light of group differences.
#' 
#' This function is most useful for analyses performed with 
#' \code{\link{measurement.error}}, but any \code{\link{lm.rrpp}} fit can be used,
#' so long as research subjects can be defined. 
#' 
#' It is essential that all arguments are terms that can be found in the model frame
#' of the model fit, as provoke by ANOVA.  Using anova(fit) will elucidate the row
#' names of the ANOVA that could be used.
#' 
#' @export
#' @keywords analysis
#' @author Michael Collyer
#' @return Objects of class "ICCstats" return the following:
#'  \item{ICC_disp}{The intraclass correlation coefficient (ICC) based on the dispersion of values.}
#'  \item{ICC_mult}{The eigenvalues of ICC matrices} 
#' @param fit The \code{\link{lm.rrpp}}, previously evaluated.
#' @param subjects A single character value indicating which term in an ANOVA table
#' corresponds to research subjects.
#' @param with_in One or more character values indicating which terms in an ANOVA table
#' are measured within subjects (replications, plus maybe interactions).  If NULL,
#' the only replication within-subject will be considered as residuals.
#' @param groups An optional character value to indicate if a factor in the 
#' model frame of the \code{\link{lm.rrpp}} fit that could account for subject variation.
#' Using this argument might minimize the importance of subject variation, if subjects
#' have disparate values that could inflate ICC.  Note that this name could be different
#' than what is shown in the ANOVA table, if \code{\link{measurement.error}}
#' was used.  Use names(fit$LM$data), substituting fit with the name assigned
#' to the \code{\link{measurement.error}} object, to know the groups factor, 
#' if used.
#' @param multivariate Logical value for whether to include to calculate ICC matrix 
#' generalizations and perform eigenanalysis.
#' @param print.AOV Logical value for whether to include ANOVA table as screen 
#' output, when calculating ISS statistics.
#'
#' Note that this function can return ICC statistics, even if they do not make sense.
#' It is possible to generate ICC stats with any ANOVA table, with at least one
#' term.  
#'
#' @references Liljequist, D., Elfving, B., & Skavberg Roaldsen, K. (2019). Intraclass correlation–A discussion 
#' and demonstration of basic features. PloS one, 14(7), e0219854.
#' 
#' @examples
#' \dontrun{
#' # Measurement error analysis on simulated data of fish shapes
#' 
#' data(fishy)
#' 
#' # Analysis unconcerned with groups 
#' 
#' ME1 <- measurement.error(
#'   Y = "coords",
#'   subjects = "subj",
#'   replicates = "reps",
#'   data = fishy)
#' 
#' anova(ME1)
#' ICCstats(ME1, subjects = "Subjects", with_in = "Systematic ME")
#' 
#' # Analysis concerned with groups 
#' 
#' ME2 <- measurement.error(
#'   Y = "coords",
#'   subjects = "subj",
#'   replicates = "reps",
#'   groups = "groups",
#'   data = fishy)
#' 
#' anova(ME2)
#' ICCstats(ME2, subjects = "Subjects", 
#'   with_in = "Systematic ME", groups = "groups")
#' ICCstats(ME2, subjects = "Subjects", 
#'   with_in = c("Systematic ME", "Systematic ME:Groups"), 
#'   groups = "groups")
#'   }
#'   

ICCstats <- function(fit, 
                     subjects = NULL,
                     with_in = NULL,
                     groups = NULL,
                     multivariate = FALSE,
                     print.AOV = TRUE){
  if(!inherits(fit, "lm.rrpp"))
    stop(paste("\nThe fit argument must use an object of",
    " class lm.rrpp.", sep = ""), call. = FALSE)
  
  AOV <- anova(fit)$table
  if(print.AOV) print(AOV)
  
  if(is.null(subjects) || all(!rownames(AOV) %in% as.character(subjects)))
    stop(paste("\n\nThe ANOVA (if printed above) should be able to help you choose the",
               "correct subjects name (row name in table)\n", sep = " "), 
         call. = FALSE)
  
  if(!is.numeric(subjects)) 
    sub_no = which(rownames(AOV) %in% as.character(subjects))
  
  if(is.null(with_in)) with_in <- "Residuals"
  
  if(!is.numeric(with_in)) 
    with_in_no = which(rownames(AOV) %in% as.character(with_in))
  if(length(with_in_no) == 0) {
    with_in <- "Random ME"
    with_in_no = which(rownames(AOV) %in% with_in)
  }
  
  
  if(!is.null(groups)){
    gfac <- unlist(fit$LM$data[names(fit$LM$data) %in% groups])
    if(is.null(gfac))
      stop("\nThe groups variable is not found in the fit model frame.\n",
            call. = FALSE)
    Y <- fit$LM$Y
    df <- rrpp.data.frame(Y = Y, gfac = gfac)
    
    R <- resid(lm.rrpp(Y ~ gfac, 
               Cov = fit$LM$Cov, weights = fit$LM$weights,
               iter = 1, data = df,
               print.progress =  FALSE))
    form <- formula(fit$LM$Terms)
    fitb <- fit
    fitb$LM$Y <- fitb$LM$data$Y <- R
    if(fit$ANOVA$SS.type == "Within-subject II") {
      fit <- suppressWarnings(
        lm.rrpp(form, 
                Cov = fitb$LM$Cov, weights = fitb$LM$weights,
                iter = 1, data = fitb$LM$data, 
                print.progress = FALSE, 
                subjects = fit$LM$data[names(fit$LM$data) == fit$subjects.var]))
    } else {
      fit <- suppressWarnings(lm.rrpp(form,  
                                      Cov = fitb$LM$Cov, 
                                      weights = fitb$LM$weights,
                                      iter = 1, data = fitb$LM$data, 
                                      SS.type = fit$ANOVA$SS.type,
                                        print.progress =  FALSE))
    }
    
    rm(fitb)
    AOV2 <- anova(fit)$table
    rownames(AOV2) <- rownames(AOV)
    AOV <- AOV2
    rm(AOV2)
  }
  
  resid_no<- which(rownames(AOV) == "Residuals")
  if(length(resid_no) == 0)
    resid_no<- which(rownames(AOV) == "Random ME")
  
  type <- 1
  if(!identical(with_in_no, resid_no)) type <- 2
  
  sub.line <- AOV[sub_no, , drop = FALSE]
  w.lines <- AOV[unique(c(with_in_no, resid_no)), , drop = FALSE]
  res.line <- AOV[resid_no, , drop = FALSE]
  
  MSBS <- sub.line[3]
  if(type == 2)
    MSWS <- sum(w.lines[,2]) / sum(w.lines[,1]) else
      MSWS <- res.line[3]
  
  if(type == 2){
    MSBM <- w.lines[1, 3]
    MSE <- res.line[3]
  }
  ns <- sub.line[1] + 1
  nr <- w.lines[1, 1] + 1
  
  icc <- (MSBS - MSWS) / (MSBS + (nr - 1) * MSWS)
  if(type == 2){
    icca <- (MSBS - MSE) / (MSBS + (nr - 1) * MSE + 
                              nr / ns * (MSBM - MSE))
    iccc <- (MSBS - MSE) / (MSBS + (nr - 1) * MSE)
  } else icca <- iccc <- NULL
  
  icc.out <- unlist(c(icc , icca, iccc))
  names(icc.out) <- c("ICC", "ICC_a", "ICC_c")[1:length(icc.out)]
  
  if(multivariate){
    
    fitb <- fit
    class(fitb) <- "lm.rrpp"
    SSCP <- summary(fitb, formula = FALSE)$SSCP
    rm(fitb)
    
    Df <- fit$ANOVA$df
    rmove <- which(Df == 0)
    if(length(rmove) > 0)
      SSCP <- SSCP[-rmove]
    
    if(!identical(names(SSCP), rownames(AOV))){
      if(length(SSCP) != (nrow(AOV) - 1))
        stop("\nIncommensurate ANOVA and MANOVA terms\n",
             call. = FALSE)
      names(SSCP) <- rownames(AOV)[1:length(SSCP)]
    }
    
    sub_no <- which(names(SSCP) %in% subjects)
    with_in_no <- which(names(SSCP) %in% with_in)
    resid_no <- length(SSCP)

    RSSCP <- as.matrix(SSCP[[resid_no]])
    SSCPBS <- as.matrix(SSCP[[sub_no]])
    if(type == 2){
      SSCPBM <- lapply(with_in_no, function(x) SSCP[[x]])
      SSCPBM <- Reduce("+", SSCPBM)
      SSCPWS <- RSSCP + SSCPBM
    } else SSCPWS <- RSSCP
    
    Df <- AOV$Df
    
    MSCPBS <- SSCPBS / Df[sub_no]
    
    if(type == 2) {
      MSCPBM <- SSCPBM / sum(Df[with_in_no])
      MSCPWS <- SSCPWS / sum(Df[c(with_in_no, resid_no)])
      MSCPE <- RSSCP / Df[resid_no]
    } else MSCPWS <- SSCPWS / Df[resid_no]
  
    micc <- fast.solve(MSCPBS + (nr - 1) * MSCPWS) %*% (MSCPBS - MSCPWS) 
    
    if(type == 2){
      micca <- fast.solve(MSCPBS + ((nr - 1) * MSCPE) + 
                            as.numeric(nr / ns) * 
                            (MSCPBM - MSCPE)) %*% (MSCPBS - MSCPE) 
      miccc <- solve(MSCPBS + (nr - 1) * MSCPE) %*% (MSCPBS - MSCPE) 
    } else micca <- miccc <- NULL
    
    
    ev_icc <- eigen(micc)$values
    ev_icc <- Re(ev_icc)
    ev_icc <- ev_icc[ev_icc > 0]
    
    if(type == 2){
      ev_icc_a <-eigen(micca)$values
      ev_icc_a <- Re(ev_icc_a)
      ev_icc_a <- ev_icc_a[ev_icc_a > 0]
      ev_icc_c <-eigen(miccc)$values
      ev_icc_c <- Re(ev_icc_c)
      ev_icc_c <- ev_icc_c[ev_icc_c > 0]
    } else ev_icc_a <- ev_icc_c <- NA
    
    k <- max(c(length(ev_icc), 
               length(ev_icc_a), 
               length(ev_icc_c)), na.rm = TRUE)
    nrm <- if(type == 1) 1 else 3
    iccm.out <- matrix(NA, nrm, k)
    
    iccm.out[1, 1:length(ev_icc)] <- ev_icc
    if(type == 2){
      iccm.out[2, 1:length(ev_icc_a)] <- ev_icc_a
      iccm.out[3, 1:length(ev_icc_c)] <- ev_icc_c
    }
    rownames(iccm.out) <- names(icc.out)
    colnames(iccm.out) <- paste("EV", 1:k, sep = "")
  } else iccm.out <- NULL
  
  out <- list(ICC_disp = icc.out, ICC_mult = iccm.out)
  class(out) <- "ICCstats"
  out
}
  

#' Ordination tool for data aligned to another matrix
#'
#' Function performs a singular value decomposition of ordinary 
#' least squares (OLS) or 
#' generalized least squares (GLS) residuals, aligned to an alternative 
#' matrix, plus projection of 
#' data onto vectors obtained.  
#' 
#' The function performs a singular value decomposition, 
#' \bold{t(A)Z} = \bold{UDt(V)}, where
#' \bold{Z} is a matrix of residuals (obtained from \bold{Y}; 
#' see below) and \bold{A}
#' is an alignment matrix with the same number of rows as \bold{Z}.  
#' (\bold{t} indicates matrix transposition.)  \bold{U}  and \bold{V}
#' are the matrices of left and right singular vectors, and \bold{D} 
#' is a diagonal matrix of singular
#' values. \bold{V} are the vectors that describe maximized 
#' covariation between \bold{Y} and \bold{A}. 
#' If \bold{A} = \bold{I}, an n x n identity matrix, \bold{V} are the
#' eigen vectors (principal components) of \bold{Y}.
#' 
#' \bold{Z} represents a centered and potentially standardized 
#' form of \bold{Y}.  This
#' function can center data via OLS or GLS means (the latter if 
#' a covariance matrix to describe
#' the non-independence among observations is provided).  If 
#' standardizing variables is preferred,
#' then \bold{Z} both centers and scales the vectors of \bold{Y} 
#' by their standard deviations.  
#' 
#' Data are projected onto aligned vectors, \bold{ZV}.  If a 
#' GLS computation is made, the option to transform centered values 
#' (residuals) before projection is 
#' available.  This is required for orthogonal projection, but 
#' from a transformed data space.  Not transforming
#' residuals maintains the Euclidean distances among observations 
#' and the OLS multivariate variance, but
#' the projection is oblique (scores can be correlated).
#'
#' The versatility of using an alignment approach is that 
#' alternative data space rotations are possible.
#' Principal components are thus the vectors that maximize 
#' variance with respect to the data, themselves,
#' but "components" of (co)variation can be described for any 
#' inter-matrix relationship, including
#' phylogenetic signal, ecological signal, ontogenetic signal, 
#' size allometry, etc.
#' More details are provided in Collyer and Adams (2021).
#' 
#' Much of this function is consistent with the \code{\link{prcomp}} 
#' function, except that centering data
#' is not an option (it is required).
#' 
#' SUMMARY STATISTICS: For principal component plots, the traditional 
#' statistics to summarize the analysis include
#' eigenvalues (variance by component), proportion of variance by 
#' component, and cumulative proportion of variance. 
#' When data are aligned to an alternative matrix, the statistics 
#' are less straightforward.  A summary of
#' of such an analysis (performed with \code{\link{summary.ordinate}}) 
#' will produce these additional statistics:
#'
#' \describe{
#' \item{\bold{Singular Value}}{  Rather than eigenvalues, the 
#' singular values from singular value decomposition of the 
#' cross-product of the scaled alignment matrix and the data.}
#' \item{\bold{Proportion of Covariance}}{  Each component's 
#' singular value divided by the sum of singular values.  The cumulative
#' proportion is also returned.  Note that these values do not 
#' explain the amount of covariance between the alignment matrix 
#' and data, but
#' explain the distribution of the covariance.  Large proportions 
#' can be misleading.}
#' \item{\bold{RV by Component}}{  The partial RV statistic by 
#' component.  Cumulative values are also returned.  The sum of partial
#' RVs is Escoffier's RV statistic, which measures the amount of 
#' covariation between the alignment matrix and data.  Caution should
#' be used in interpreting these values, which can vary with the 
#' number of observations and number of variables.  However,
#' the RV is more reliable than proportion of singular value for 
#' interpretation of the strength of linear association for 
#' aligned components.  (It is most analogous to proportion of 
#' variance for principal components.)}
#' }
#' 
#' @param Y An n x p data matrix.
#' @param A An optional n x n symmetric matrix or an n x k data 
#' matrix, where k is the number of variables that could
#' be associated with the p variables of Y.  If NULL, an n x n 
#' identity matrix will be used.
#' @param Cov An optional n x n covariance matrix to describe 
#' the non-independence among
#' observations in Y, and provide a GLS-centering of data.  
#' Note that Cov and A can be the same, if one
#' wishes to align GLS residuals to the same matrix used to 
#' obtain them.  Note also that no explicit GLS-centering
#' is performed on A.  If this is desired, A should be 
#' GLS-centered beforehand.
#' @param transform. An optional argument if a covariance matrix 
#' is provided to transform GLS-centered residuals, if TRUE.  If FALSE, 
#' only GLS-centering is performed.  Only if transform = TRUE 
#' (the default) can one expect the variances of ordinate scores 
#' in a principal component analysis to match eigenvalues.
#' @param scale. a logical value indicating whether the variables 
#' should be scaled to have unit variance before the analysis 
#' takes place. The default is FALSE.
#' @param tol A value indicating the magnitude below which 
#' components should be omitted. (Components are omitted if their 
#' standard deviations are less than or equal to tol times the 
#' standard deviation of the first component.) 
#' With the default null setting, no components are omitted 
#' (unless rank. is provided). 
#' Other settings for tol could be tol = sqrt(.Machine$double.eps), 
#' which would omit essentially constant components, or tol = 0,
#' to retain all components, even if redundant.
#' This argument is exactly the same as in \code{\link{prcomp}}
#' @param rank. Optionally, a number specifying the maximal rank, 
#' i.e., maximal number of aligned components to be used. 
#' This argument can be set as alternative or in addition to tol, 
#' useful notably when the desired rank is considerably 
#' smaller than the dimensions of the matrix.  This argument is 
#' exactly the same as in \code{\link{prcomp}}
#' @param newdata An optional data frame of values for the same 
#' variables of Y to be projected onto 
#' aligned components.  This is only possible with OLS 
#' (transform. = FALSE).
#' @keywords analysis
#' @export
#' @author Michael Collyer
#' @return An object of class \code{ordinate} is a list containing 
#' the following
#' \item{x}{Aligned component scores for all observations}
#' \item{xn}{Optional projection of new data onto components.}
#' \item{d}{The portion of the squared singular values attributed to 
#' the aligned components.}
#' \item{sdev}{Standard deviations of d; i.e., the scale of the components.}
#' \item{rot}{The matrix of variable loadings, i.e. the singular 
#' vectors, \bold{V}.}
#' \item{center}{The OLS or GLS means vector used for centering.}
#' \item{transform}{Whether GLS transformation was used in projection 
#' of residuals 
#' (only possible in conjunction with GLS-centering).}
#' \item{scale}{The scaling used, or FALSE.}
#' \item{alignment}{Whether data were aligned to principal axes or 
#' the name of another matrix.}
#' \item{GLS}{A logical value to indicate if GLS-centering and 
#' projection was used.}
#' @references Collyer, M.L. and D.C. Adams.  2021. 
#' Phylogenetically-aligned Component Analysis. Methods 
#' in Ecology and evolution. In press.
#' @references Revell, L. J.  2009. Size-correction and principal 
#' components for interspecific comparative 
#' studies. Evolution, 63:3258-3268.
#' 
#' @seealso \code{\link{plot.ordinate}}, \code{\link{prcomp}}, 
#' \code{\link{plot.default}},
#' \code{gm.prcomp} within \code{geomorph}
#' @examples
#' 
#' # Examples use residuals from a regression of salamander 
#' # morphological traits against body size (snout to vent length, SVL).
#' # Observations are species means and a phylogenetic covariance matrix
#' # describes the relatedness among observations.
#'
#' data("PlethMorph")
#' Y <- as.data.frame(PlethMorph[c("TailLength", "HeadLength", 
#' "Snout.eye", "BodyWidth", 
#' "Forelimb", "Hindlimb")])
#' Y <- as.matrix(Y)
#' R <- lm.rrpp(Y ~ SVL, data = PlethMorph, 
#' iter = 0, print.progress = FALSE)$LM$residuals
#' 
#' # PCA (on correlation matrix)
#' 
#' PCA.ols <- ordinate(R, scale. = TRUE)
#' PCA.ols$rot
#' prcomp(R, scale. = TRUE)$rotation # should be the same
#' 
#' # phyPCA (sensu Revell, 2009)
#' # with projection of untransformed residuals (Collyer & Adams 2020)
#' 
#' PCA.gls <- ordinate(R, scale. = TRUE, 
#' transform. = FALSE, 
#' Cov = PlethMorph$PhyCov)
#' 
#' # phyPCA with transformed residuals (orthogonal projection, 
#' # Collyer & Adams 2020)
#' 
#' PCA.t.gls <- ordinate(R, scale. = TRUE, 
#' transform. = TRUE, 
#' Cov = PlethMorph$PhyCov)
#'  
#'  # Align to phylogenetic signal (in each case)
#'  
#'  PaCA.ols <- ordinate(R, A = PlethMorph$PhyCov, scale. = TRUE)
#'  
#'  PaCA.gls <- ordinate(R, A = PlethMorph$PhyCov, 
#'  scale. = TRUE,
#'  transform. = FALSE, 
#'  Cov = PlethMorph$PhyCov)
#'  
#'  PaCA.t.gls <- ordinate(R, A = PlethMorph$PhyCov, 
#'  scale. = TRUE,
#'  transform. = TRUE, 
#'  Cov = PlethMorph$PhyCov)
#'  
#'  # Summaries
#'  
#'  summary(PCA.ols)
#'  summary(PCA.gls)
#'  summary(PCA.t.gls)
#'  summary(PaCA.ols)
#'  summary(PaCA.gls)
#'  summary(PaCA.t.gls)
#'  
#'  # Plots
#'  
#'  par(mfrow = c(2,3))
#'  plot(PCA.ols, main = "PCA OLS")
#'  plot(PCA.gls, main = "PCA GLS")
#'  plot(PCA.t.gls, main = "PCA t-GLS")
#'  plot(PaCA.ols, main = "PaCA OLS")
#'  plot(PaCA.gls, main = "PaCA GLS")
#'  plot(PaCA.t.gls, main = "PaCA t-GLS")
#'  par(mfrow = c(1,1))
#'  
#'  # Changing some plot aesthetics (the arguments in plot.ordinate and 
#'  # plot.default are important for changing plot parameters)
#'  
#'  P1 <- plot(PaCA.gls, main = "PaCA GLS", include.axes = TRUE)
#'  
#'  P2 <- plot(PaCA.gls, main = "PaCA GLS", include.axes = TRUE, 
#'  col = 4, pch = 21, bg = PlethMorph$group)
#'  add.tree(P2, PlethMorph$tree, edge.col = 4)
#' 
#'  P3 <- plot(PaCA.gls, main = "PaCA GLS", include.axes = TRUE, 
#'  col = 4, pch = 21, bg = PlethMorph$group,
#'  flip = 1)
#'  add.tree(P3, PlethMorph$tree, edge.col = 4)
#'  
#'  
ordinate <- function(Y, A = NULL, Cov = NULL, transform. = TRUE, 
                     scale. = FALSE, 
                     tol = NULL, rank. = NULL, newdata = NULL) {
  
  Ycheck <- try(as.matrix(Y), silent = TRUE)
  if(inherits(Ycheck, "try-error"))
    stop("Y must be a matrix or data frame.\n", call. = FALSE)
  
  id <- if(is.vector(Y)) names(Y) else rownames(Y)
  Y <- as.matrix(Y)
  
  dims <- dim(Y)
  n <- dims[1]
  p <- dims[2]
  
  if(is.null(id)) id <- 1:n
  rownames(Y) <- id
  
  alignment <- if(!is.null(A)) 
    try(deparse(substitute(A)), silent = TRUE) else 
      "principal"
  if(length(alignment) != 1) alignment <- "A"

  I <- Matrix(diag(n), sparse = TRUE)
  
  if(is.null(A)) A <- I
  if(!inherits(A, "matrix") || !inherits(A, "Matrix")){
    A <- try(as.matrix(A), silent = TRUE)
    if(inherits(A, "try-error"))
      stop("\nUnable to coerce A into a matrix.\n", call. = FALSE)
    As <- Matrix(A, sparse = TRUE)
    As@x <- round(As@x, 12)
    As <- Matrix(A, sparse = TRUE)
    if(length(As@x) < length(A))
      A <- As
    rm(As)
  }
    
  if(NROW(A) != n)
    stop("A must be a matrix with the same number of rows as data\n", 
         call. = FALSE)
  if(is.null(rownames(A))) rownames(A) <- rownames(Y) else {
    nnames <- length(intersect(rownames(Y), rownames(A)))
    if(nnames > n)
      stop("The row names of A are not the same as the row names of Y\n",
           call. = FALSE)
    if(isSymmetric(A)) A <- A[id, id] else
      A <- A[id, , drop = FALSE]
  }
  
  X <- matrix(1, n)
  rownames(X) <- id
  if(!is.null(Cov)) Pcov <- Cov.proj(Cov, rownames(Y))
  cen <- if(is.null(Cov)) colMeans(Y) else 
    lm.fit(as.matrix(Pcov %*% X), as.matrix(Pcov %*% Y))$coefficients
  Z <- as.matrix(scale(Y, center = cen, scale = scale.))
  cen <- attr(Z, "scaled:center")
  sc <- attr(Z, "scaled:scale")
  if (any(sc == 0)) 
    stop("cannot rescale a constant/zero column to unit variance")
  
  if(!is.null(rank.)) {
    stopifnot(length(rank.) == 1, is.finite(rank.), 
              as.integer(rank.) > 0)
    k <- min(as.integer(rank.), n, p)
  } else k <- min(n, p)
  
  tf <- if(!is.null(Cov) && transform.) TRUE else FALSE
  if(tf) Z <- as.matrix(Pcov %*% Z)

  rownames(Z) <- id
  
  Saz <-  if(tf || is.null(Cov)) crossprod(A, Z) else crossprod(A, 
                                                        Pcov %*% Z)
  Saz <- as.matrix(Saz)
  
  s <- svd(Saz, nu = 0, nv = k)
  
  if(alignment != "principal") {
    Sa <- crossprod(A)
    Sz <- crossprod(Z)
    RV <- svd(crossprod(A, Z))$d^2 / sqrt(sum(Sa^2) * sum(Sz^2))
  } else RV <- NULL
  
  j <- seq_len(k)
  s$v <- s$v[,j , drop = FALSE]
  s$d <- s$d[j]
  
  sy <- if(tf || is.null(Cov)) sum(svd(Z)$d^2) else 
    sum(svd(Pcov %*% Z)$d^2)
  s$d <- s$d^2/sum(s$d^2) * sy / max(1, n - 1)
  s$sdev <- sqrt(s$d)

  if (!is.null(tol)) {
    rank <- sum(s$sdev > (s$sdev[1L] * tol))
    if (rank > 0 && k > 1 && rank < k) {
      j <- seq_len(k <- rank)
      s$v <- s$v[, j, drop = FALSE]
      s$d <- s$d[j]
      s$sdev <- s$sdev[j]
    }
  }
  
  s$v <- as.matrix(s$v)
  dimnames(s$v) <- list(colnames(Z), paste0("Comp", j))
  
  if(!is.null(RV))
    RV <- RV[1:min(length(RV), length(s$d))]
  
  r <- list(d = s$d, sdev = s$sdev, 
            rot = s$v, 
            center = cen, 
            scale = if(is.null(sc)) FALSE else sc,
            GLS = if(is.null(Cov)) FALSE else TRUE,
            transform = tf,
            alignment = alignment,
            RV = RV)
  
  r$x <- as.matrix(Z %*% s$v)
  rownames(r$x) <- id
  names(r$d) <- names(r$sdev) <- colnames(r$rot)
  if(!is.null(r$RV)) names(r$RV) <- colnames(r$rot)

  if(!is.null(newdata)){
    if(tf) 
    stop("\nNew data cannot be projected onto vectors that were calculated\n
         with respect to the covariances among original observations\n", 
                call. = FALSE)
    xn <- as.matrix(newdata)
    xn <- scale(xn, center = cen, scale = scale.)
    if(NCOL(Z) != NCOL(xn)) 
      stop("Different number of variables in newdata\n", call. = FALSE) 
    r$xn <- xn %*% s$v
    if(!is.null(rownames(newdata)))
      rownames(r$xn) <- rownames(newdata)
  }

  class(r) <- "ordinate"
  r
  
}

# Important function for performing lm.rrpp or lm.rrpp.ws
# All arguments must be vetted by primary functions.

.lm.rrpp <- function(f1, subjects = NULL, 
                     sub.var.name = NULL,
                     iter, turbo, seed = NULL, 
                     int.first = FALSE,
                     RRPP, full.resid, block = NULL,
                     SS.type, data, Cov,
                     delta, gamma, 
                     print.progress, Parallel, 
                     verbose, ...) {
  
  Parallel.args <- Parallel.setup(Parallel)
  
  ### List of Covariance options
  
  # 0 = No Cov
  # 1 = Cov, no subjects, must match no. obs.
  # 2 = Cov matches subjects in name and length
  # 3 = Cov matches subjects in name, but needs expansion
  
  tTerms <- if(inherits(f1, "lm")) 
    terms.formula(f1$terms, keep.order = int.first) else
      terms.formula(f1, keep.order = int.first)
  
  cov.type <- 0
  if(!is.null(Cov) && is.null(subjects))
    cov.type <- 1
  if(!is.null(Cov) && !is.null(subjects))
    cov.type <- 2
  
  STerm <- sub.lev <- NULL
  useSubjects <- !is.null(subjects)
  
  L <- c(as.list(environment()), list(...))
  names(L)[which(names(L) == "f1")] <- "formula"
  
  L$int.first <- int.first
  SS.type <- L$SS.type
  full.resid <- L$full.resid
  
  if(full.resid && SS.type != "III"){
    SS.type = "III"
    warning(
      paste(
        "\nThis is not an error!  It is a friendly warning.\n",
        "\nBecause a permutation of full model residuals was chosen,",
        "\nSS.type is being forced to be III, as this is the only applicable",
        "\nestimation method when using full model residuals as exchangeable units",
        "\nunder the null hypotheses of model effects.  Additionally, all random",
        "\nANOVA statistics will have the form, |random.stat - observed.stat|.\n",
        "\nUse options(warn = -1) to turn off these warnings. \n\n", sep = " "),
        noBreaks. = TRUE, call. = FALSE, immediate. = TRUE) 
  }
  
  dots <- list(...)
  if(length(dots) > 0) {
    w <- dots$weights
    o <- dots$offset
  } else w <- o <- NULL
  
  if(!is.null(w) && !is.null(Cov)) {
    w <- NULL
    warning(
    paste(
      "\nThis is not an error!  It is a friendly warning.\n",
      "\nIt is not possible to use both a Cov matrix and weights.",
      "\nBoth are inputs to perform generalized least squares estimation of coefficients,",
      "\nbut at present only one covariance matrix can be used.",
      "\nAs a result, weights are set to NULL.",
      "\nYou could consider adjusting your Cov matrix; e.g., Cov <- Cov * 1/weights,",
      "\nmaking sure the Cov matrix and weights are ordered consistently.\n",
      "\nUse options(warn = -1) to turn off these warnings. \n\n", sep = " "),
    noBreaks. = TRUE, call. = FALSE, immediate. = TRUE) 
  }
  
  Terms <- D <- NULL
  
  if(print.progress) {
    cat("\nPlease be aware that printing progress slows down the analysis (perhaps slightly).\n")
    cat("\nPreliminary Model Fit...\n")
  } 
  
  if(inherits(f1, "lm")) {
    exchange.args <- f1[attributes(f1)$names %in% 
                          c("terms", "offset", "weights")]
    exchange.args$model <- f1$model
    exchange.args$Y <- Y <- as.matrix(f1$model[[1]])
    exchange.args$tol <- f1$qr$tol
    exchange.args$SS.type <- SS.type
    names(exchange.args)[which(names(exchange.args) == "terms")] <- "Terms"
    if("weights" %in% names(exchange.args))
      names(exchange.args)[which(names(exchange.args) == "weights")] <- "w"
    Terms <- f1$terms
  } else {
    f1 <- formula(tTerms,
                  keep.order = int.first)
  }
  
  if(inherits(f1, "formula")) {
    L$formula <- f1
    L$keep.order <- int.first
    exchange.args <- lm.args.from.formula(L)
    if(!is.null(exchange.args$D)) D <- exchange.args$D
    exchange.args <- exchange.args[c("Terms", "Y", "model")]
    exchange.args$tol <- 1e-7
    exchange.args$SS.type <- SS.type
    Terms <- exchange.args$Terms 
    Y <- as.matrix(exchange.args$Y)
  }
  
  if(!identical(attr(tTerms, "term.labels"),
                attr(Terms, "term.labels"))) {
    trms <- attr(terms(f1, keep.order = int.first),
                 "term.labels")
    if(length(trms) > 0)
    nform <- reformulate(termlabels = trms,
                         response = "Y") else nform <- update(f1, Y ~ .)
    Terms <- try(terms(nform, data = exchange.args$data,
                               keep.order = int.first),
                 silent = TRUE)
    
    model <- try(model.frame(Terms, data = exchange.args$data),
                 silent = TRUE)
    
    if(inherits(model, "try-error") || inherits(Terms, "try-error"))
      stop("Variables or data might be missing from either the data frame or 
           global environment, or a linear model fit just does not work...\n", 
           call. = FALSE)
    
    exchange.args$Terms <- Terms
    exchange.args$model <- model
  }
  
  id <- get.names(Y)
  dims <- dim(Y)
  n <- dims[1]
  p <- dims[2]
  if(is.null(id)) {
    id <- 1:n
    Y <- add.names(Y, id)
  }
  
  attr(Terms, ".Environment") <- NULL
  term.labels <- attr(Terms, "term.labels")
  k <- length(term.labels)
  
  offst <- (!is.null(o)) 
  weighted <- (!is.null(w))
  exchange.args <- c(exchange.args, list(w = w, offset = o))
  
  if(!inherits(f1, c("lm", "formula")))
    stop("\nf1 must be either a formula or class lm objects.\n",
         call. = FALSE)
  
  gls <- FALSE
  ols <- TRUE
  
  if(!is.null(w)) {
    if(length(w) != n)
      stop("The number of weights does not match the number of observations.  This could be because of missing data.\n",
           call. = FALSE)
    gls <- TRUE
    ols <- FALSE
  }
  
  # Resolve cov.type
  
  if(!is.null(Cov)){
    Cov.name <- deparse(substitute(Cov))
    Cov.match <- match(Cov.name, names(data))
    if(all(is.na(Cov.match))) Cov <- Cov else Cov <- data[[Cov.match]]
    if(is.null(rownames(Cov))) rownames(Cov) <- 1:n
    if(is.null(colnames(Cov))) colnames(Cov) <- 1:n
    if(!inherits(Cov, "Matrix") && !inherits(Cov, "matrix")) 
      stop("The covariance matrix must be a matrix.")
  }
  
  if(cov.type > 0) {
    if(cov.type == 1){
      if(nrow(Cov) != n)
        stop(paste("\nA covariance matrix is used but has a different",
        "number of observations than the data.\n", sep = " "), 
        call. = FALSE)
    }
    if(cov.type == 2){
      if(nrow(Cov) != length(subjects))
        cov.type <- 3
    }
  }
  
  ### Subject options
  
  # 0 = No subjects
  # 1 = Subjects for block but not terms
  # 2 = Subjects as blocks and terms
  
  sub.type <- 0
  useSubjects <- FALSE
  
  if(!is.null(subjects)) {
    sub.type <- 1
    useSubjects <- TRUE
  }
  
  subTest <- FALSE
  STerm <- which(term.labels == sub.var.name)
  if(!(length(STerm) == 0)) {
    sub.type <- 3
    subTest <- TRUE
  } else STerm <- NULL
  
  if(sub.type > 0) {
    subjects <- as.factor(subjects)
    sub.lev <- levels(subjects)
  } 
  
  if(useSubjects){
    if(length(subjects) != NROW(exchange.args$model)){
      msg <- paste("\nThere appears to be missing data in the data frame ",
                   "\nor the number of subjects and the number of observations ",
                   "\nin the independent variables do not match.  There ",
                   "\nis no current option for removing subjects that correspond to ",
                   "\nNA values.  You will have to do this before attempting a ",
                   "\nlinear model fit by reconstructing your data frame.\n\n", sep = "")
      stop(msg, call. = FALSE)
    }

    
    if(!sub.var.name %in% names(data))
      stop("\nVariable for subjects is not found in the RRPP data frame.\n",
           call. = FALSE)
  }
  
  perms <- iter + 1
  ind_s <- NULL
  ind <- perm.index(n, iter = iter, block = block, seed = seed)
  
  if(sub.type == 1){
    ind <- perm.index(n, iter = iter, block = subjects, seed = seed)
  }
  
  if(sub.type == 2){
    ind <- perm.index(n, iter = iter, block = subjects, seed = seed)
    ind_s <- perm.index(n, iter = iter, block = block, seed = seed)
  }
  
  Pcov <- CovEx <- NULL
  
  if(cov.type == 1){
    cov.lev <- rownames(Cov)
    if(length(setdiff(id, cov.lev)) > 0)
      stop("Data names and covariance matrix names do not match.\n", call. = FALSE)
    
    Cov <- Cov[id, id]
  }

  if(cov.type == 2) {
    if(nrow(Cov) != n){
      stop(paste("\nThe dimensions of the covariance matrix does not",
                 "match the number of observations.\n", sep = " "),
                 call. = FALSE)
    } else cat("\n It is assumed that the covariance matrix and", 
               "data are ordered the same...\n")
  }
  
  if(cov.type == 3){
    
    cov.lev <- levels(as.factor(dimnames(Cov)[[1]]))
    if(!all(sub.lev %in% cov.lev) || !all(cov.lev %in% sub.lev))
      stop("\nThere is a mismatch between subject levels and covariance levels\n
         Make sure there is direct correspondence between subjects in the model\n
         and subjects in the covariance matrix.  Check spelling of names.\n", 
           call. = FALSE)
    
    Xsub <- model.matrix(~subjects + 0)
    colnames(Xsub) <- sub.lev
    Xs <- Matrix(Xsub, sparse = TRUE)
    
    Cov <- Cov[colnames(Xsub), colnames(Xsub)]
    Cov.names <- dimnames(Cov)[[1]]
    Cov <- Matrix(Cov, sparse = TRUE)
    if(is.vector(delta) && length(delta) > 1 &&
       length(delta) != length(sub.lev)) {
      stop(paste("Either a single value or vector equal in length to the number",
                 "of subject levels is required.\n", sep = " "), call. = FALSE)
      
    }
    if(!is.numeric(delta))
      stop("delta must be a numeric value or vector.\n", call. = FALSE)
    
    delta[which(delta <= 0)] <- 0.001
    delta[which(delta > 1)] <- 1
    delta[which(zapsmall(delta) == 0)] <- 0.001
    
    n.list <- as.vector(by(subjects, subjects, length))
    gam <- if(gamma == "equal") 1 else 
      sqrt(n.list)
    
    Xr <- t(t(Xs) * exp(-delta * gam))
    CovEx <- Xr %*% Cov %*% t(Xr)
    D <- diag(Cov)
    D <- unlist(lapply(1:length(D), function(j) 
      rep(D[j], n.list[j])))
    diag(CovEx) <- D
    Xs <- Xr <- NULL
  }
  
  if(cov.type > 0){
    Pcov <- if(cov.type == 3) Cov.proj (CovEx) else Cov.proj(Cov)
    gls <- TRUE
    ols <- FALSE
  } else {
    Pcov <- NULL
    if(is.null(w)) {
      ols <- TRUE
      gls <- FALSE
    } else {
      ols <- FALSE
      gls <- TRUE
    }
    
  }

  X.args <- exchange.args[c("Terms", "Y", "SS.type", "tol", "model")]
  X.args$subjects.term <- STerm
  Xs <- do.call(getXs, X.args)
  X <- Xs$Xfs[[length(Xs$Xfs)]]
  
  Qs <- lapply(1:2, function(j){
    X.j <- Xs[[j]]
    kk <- length(X.j)
    res <- lapply(1:kk, function(jj){
      x <- Matrix(round(X.j[[jj]], 12), sparse = TRUE)
      if(!is.null(Pcov)) x <- Pcov %*% x
      if(!is.null(w)) x <- x * sqrt(w)
      QRforX(x, reduce = FALSE)
      
    })
    res
  })
  
  names(Qs) <- c("reduced", "full")

  checkers.args <- list(Y = Y, Qs = Qs, Xs = Xs,
                        turbo = turbo, Terms = Terms, Pcov = Pcov, w = w)
  cks <- do.call(checkers, checkers.args)

  Qs <- checkers.args <- NULL
  
  TY <- if(!is.null(Pcov)) Pcov %*% Y else if(!is.null(w)) Y * sqrt(w) else Y
  
  PCA <- p > (n - 1) 
  
  if(PCA) {
    Yp <- ordinate(Y, tol = 1e-7)$x
    p.prime <- ncol(Yp)
    TYp <- if(!is.null(Pcov)) 
      Pcov %*% Yp else if(!is.null(w)) 
        Yp * sqrt(w) else Yp
    Yp <- NULL
  } else {
    TYp <- TY
    p.prime <- p
  }
  
  Ur <- cks$Ur
  kk <- length(Ur)
  
  if(RRPP) {
    if(full.resid) {
      Uf <- cks$Uf
      FR <- lapply(1:max(1, kk), function(j){
        fitted <- as.matrix(fastFit(Uf[[j]], TY, n , p))
        residuals <- as.matrix(TY - fitted)
        out <- list(fitted = fitted, residuals = residuals)
      })
      
      if(sub.type == 2) {
        FR[[STerm]]$fitted <- as.matrix(fastFit(Uf[[STerm]], TY, n , p))
        FR[[STerm]]$residuals <- as.matrix(TY - FR[[STerm]]$fitted)
      }
      
      Uf <- NULL
      
    } else {
      
      FR <-lapply(1:max(1, kk), function(j){
        fitted <- as.matrix(fastFit(Ur[[j]], TY, n , p))
        residuals <- as.matrix(TY - fitted)
        out <- list(fitted = fitted, residuals = residuals)
      })
      
      if(sub.type == 2) {
        FR[[STerm]]$fitted <- as.matrix(fastFit(Ur[[STerm]], TY, n , p))
        FR[[STerm]]$residuals <- as.matrix(TY - FR[[STerm]]$fitted)
      }
      
    }

  } else {
    FR <- lapply(1:max(1, kk), function(j){
      fitted <-  matrix(0, n, p)
      residuals <- as.matrix(TY)
      list(fitted = fitted, residuals = residuals)
    })
  }
  
  cks$FR <- FR
  
  if(!turbo) {
    cks$offset <- o
    cks$Y <- TY
    cks$trms <- term.labels
    
    beta.args <- list(checkrs = cks, ind = ind,
                      ind_s = ind_s,
                      subTest = subTest,
                      STerm = STerm,
                      Parallel.args = Parallel.args, 
                      print.progress = print.progress)
    
    betas <- do.call(beta.iter, beta.args)
    random.coef <- betas$random.coef
    random.coef.distances <- betas$random.coef.distances
    betas <- beta.args <- NULL
  } else random.coef <- random.coef.distances <- NULL
  
  cks$Y <- TYp
  
  if(PCA){
    
    if(RRPP) {
      if(full.resid) {
        Uf <- cks$Uf
        FR <- lapply(1:max(1, k), function(j){
          fitted <- as.matrix(fastFit(Uf[[j]], TYp, n , p.prime))
          residuals <- as.matrix(TYp - fitted)
          list(fitted = fitted, residuals = residuals)
        })
        
        if(sub.type == 2) {
          FR[[STerm]]$fitted <- as.matrix(fastFit(Uf[[STerm]], TYp, n , p))
          FR[[STerm]]$residuals <- as.matrix(TYp - FR[[STerm]]$fitted)
        }
        
        Uf <- NULL
        
      } else {
        FR <- lapply(1:max(1, k), function(j){
          fitted <- as.matrix(fastFit(Ur[[j]], TYp, n , p.prime))
          residuals <- as.matrix(TYp - fitted)
          list(fitted = fitted, residuals = residuals)
        })
        
        if(sub.type == 2) {
          FR[[STerm]]$fitted <- as.matrix(fastFit(Ur[[STerm]], TYp, n , p))
          FR[[STerm]]$residuals <- as.matrix(TYp - FR[[STerm]]$fitted)
        }
      }
      
      
      } else {
      FR <- lapply(1:max(1, k), function(j){
        fitted <- matrix(0, n, p.prime)
        residuals <- as.matrix(TYp)
        list(fitted = fitted, residuals = residuals)
      })
      }
    
    cks$FR <- FR
  }
  
  SS.args <- list(checkrs = cks, ind = ind, 
                  ind_s = ind_s, 
                  subTest = subTest, STerm = STerm,
                  print.progress = print.progress,
                  Parallel.args = Parallel.args)
  FR <- NULL
  SS <- do.call(SS.iter, SS.args)
  cks$SS.type <- SS.type

  ANOVA <- anova_parts(cks, SS, full.resid)
  
  if(!verbose){
    ANOVA$MS <- ANOVA$Fs <- ANOVA$cohenf <- NULL
  }
  
  SS.args <- NULL
  
  obs.fit <- lm.rrpp.fit(X, Y, Pcov = Pcov, w = w, offset = o, 
                         tol = exchange.args$tol)
  
  TX <- if(!is.null(Pcov)) Pcov %*% X else
    if(!is.null(w)) X / sqrt(w) else X
  QR <- QRforX(TX)
  
  Hb <- as.matrix(tcrossprod(fast.solve(QR$R), QR$Q))
  if(!identical(colnames(X), colnames(QR$R)))
    Hb <- Hb[colnames(X),]
  coefficients <- as.matrix(Hb %*% TY)
  if(is.null(rownames(coefficients)))  
    rownames(coefficients) <- rownames(Hb)
  R <- U <- QR <- NULL
  
  LM <- list(form = formula(Terms, keep.order = int.first), 
             coefficients = coefficients,
             ols = ols,
             gls = gls,
             Y = Y,  
             X = X, 
             n = n, p = p, p.prime = p.prime,
             QR = QR,
             Terms = Terms, term.labels = term.labels,
             data = exchange.args$model,
             random.coef = if(verbose) random.coef else NULL,
             random.coef.distances = if(verbose) 
               random.coef.distances else NULL
  )
  
  attr(LM$Terms,".Environment") <- NULL
  attr(LM$form,".Environment") <- NULL
  if(verbose && !turbo) {
    environment(LM$random.coef) <- 
      environment(LM$random.coef.distances) <- NULL
  }
  
  LM$weights <- w
  LM$offset <- o
  
  if(gls) {
    names(LM)[[2]] <- "gls.coefficients"
    LM$gls.fitted <- as.matrix(obs.fit$fitted.values)
    LM$gls.residuals <- as.matrix(obs.fit$residuals)
    rownames(LM$gls.fitted) <- rownames(LM$gls.residuals) <- id
    LM$gls.mean <- if(NCOL(LM$gls.fitted) > 1) colMeans(LM$gls.fitted) else
      mean(LM$gls.fitted)
  } else {
    LM$fitted <- as.matrix(obs.fit$fitted.values)
    LM$residuals <- as.matrix(obs.fit$residuals)
    rownames(LM$fitted) <- rownames(LM$residuals) <- id
    LM$mean <- if(NCOL(LM$fitted) > 1) colMeans(LM$fitted) else
      mean(LM$fitted)
  }
  
  if(!is.null(Cov)) {
    LM$Cov <- if(!is.null(CovEx)) CovEx else Cov
    LM$Pcov <- if(verbose) Pcov else NULL
    rm(Cov, Pcov, CovEx)
  }
  
  
  PermInfo <- list(perms = perms,
                   perm.method = ifelse(RRPP==TRUE,"RRPP", "FRPP"), 
                   full.resid = full.resid, block = block,
                   perm.schedule = ind, perm.seed = seed)
  if(!verbose) PermInfo$perm.schedule <- NULL
  rm(ind)
  rm(ind_s)
  
  out <- list(call = match.call(), 
              LM = LM, ANOVA = ANOVA, PermInfo = PermInfo, turbo = turbo)
  environment(out$PermInfo$perm.seed) <- 
    environment(out$PermInfo$perm.sschedule) <- NULL
  
  if(k == 0 && print.progress)
    cat("\nNo terms for ANOVA; only RSS calculated in each permutation\n")
  
  if(!is.null(exchange.args$D)) {
    qrf <- qr(as.matrix(LM$QR$X))
    D.coef <- qr.coef(qrf, D)
    out$LM$dist.coefficients <- D.coef
  }
  
  if(verbose) {
    
    Models <-lapply(1:2, function(j){
      res <- lapply(1:max(1, kk), function(jj){
        X <- Xs[[j]][[jj]]
        qr <- cks$QR[[j]][[jj]]
        list(X = X, qr = qr)
      })
      names(res) <- cks$realized.trms
      res
    })
    names(Models) <- c("reduced", "full")
    
    Model.Terms <- .getTerms(fit = NULL, Terms = Terms, SS.type = SS.type)
    
    for(i in 1:2){
      for(j in 1:max(1, kk)){
        Models[[i]][[j]]$terms <- Model.Terms[[i]][[j]]
      }
    }
    
  } else Models <- NULL

  out$Models <- Models
  out$verbose <- verbose
  
  out$subjects <- subjects
  out$subjects.var <- sub.var.name
  out$subTest <- subTest
  
  class(out) = "lm.rrpp"
  
  out
  
}

#' Plot tool to add phylogenetic trees to ordination plots
#'
#' Function adds a tree based on a description of edges from a class phylo object 
#' to an existing plot made from an ordinate object.
#' 
#' With some \code{\link{ordinate}} plots, it might be desirable to add a tree 
#' connecting points in a prescribed way, which would be tedious using 
#' \code{\link{points}} or \code{\link{lines}}.  This function will project a 
#' tree from an object of class phylo into a plot with class, 
#' \code{\link{plot.ordinate}}.  Using an edges matrix from a phylo object, 
#' this function will systematically connect plot points with lines that pass 
#' through estimated ancestral character points in the same plot space.  
#' Ancestral states are estimated assuming a Brownian motion model 
#' of evolutionary divergence.

#' @param OP An object with class \code{\link{plot.ordinate}}.
#' @param tree An object of class phylo.
#' @param edge.col A single value or vector equal to the number of edges for edge colors.
#' @param edge.lty A single value or vector equal to the number of edges for edge line type
#' @param edge.lwd A single value or vector equal to the number of edges for edge line weight.
#' @param anc.pts A logical value for whether to add points for ancestral values.
#' @param return.ancs A logical value for whether ancestral values should be printed.
#' @param ... Arguments passed onto \code{\link{points}}, used only for ancestral points.
#' @keywords graphics
#' @export
#' @author Michael Collyer
#' @seealso \code{\link{lines}} and \code{\link{points}}
#' @examples
#' 
#' # Examples use residuals from a regression of salamander morphological 
#' # traits against body size (snout to vent length, SVL).
#' # Observations are species means and a phylogenetic covariance matrix
#' # describes the relatedness among observations.
#'
#' data("PlethMorph")
#' Y <- as.data.frame(PlethMorph[c("TailLength", "HeadLength", 
#' "Snout.eye", "BodyWidth", 
#' "Forelimb", "Hindlimb")])
#' Y <- as.matrix(Y)
#' R <- lm.rrpp(Y ~ SVL, data = PlethMorph, 
#' iter = 0, print.progress = FALSE)$LM$residuals
#' 
#' PCA <- ordinate(R, scale. = TRUE)
#' pc.plot <- plot(PCA, pch = 19, col = "blue")
#' 
#' add.tree(pc.plot, tree = PlethMorph$tree, anc.pts = TRUE, 
#' pch = 19, cex = 0.5, col = "red")
#' 
add.tree <- function(OP, tree, 
                     edge.col = 1, edge.lty = 1, edge.lwd = 1,
                     anc.pts = FALSE, return.ancs = FALSE, ...) {
  
  if(!inherits(OP, c("plot.ordinate", "plot.kcomp")))
    stop("\nThe plot must be a plot object obtained from ordinate or kcomp.\n",
         call. = FALSE)
  
  if(!inherits(tree, c("phylo")))
    stop("\ntree is not a class phylo object.\n",
         call. = FALSE)

  pts <- as.matrix(OP$points)
  ancs <- anc.BM(tree, pts)
  
  if(is.null(rownames(pts)))
    stop("Plot points do not have associated taxa names.\n", 
         call. = FALSE)
  
  ind <- match(tree$tip.label, rownames(pts))
  
  if(any(is.na(ind)))
    stop("\nRow names of data and tree tip names do not match.\n",
         call. = FALSE)
  
  pts <- pts[ind, ]
  z <- rbind(pts, ancs)
  
  edges <- as.matrix(tree$edge)
  
  for (i in 1:NROW(edges)) {
    pts <- z[edges[i,], ]
    points(pts, type = "l", col = edge.col, 
            lwd = edge.lwd, lty = edge.lty)
  }
  
  if(anc.pts) points(ancs, ...)
  
  do.call(points, OP$plot_args)
  
  if(return.ancs) return(ancs)
  
}
  

#' ANOVA for lm.rrpp model fits
#'
#' @description Computes an analysis of variance (ANOVA) table using 
#' distributions of random statistics from \code{\link{lm.rrpp}}.  
#' ANOVA can be performed on one model or multiple models.  
#' If the latter, the first model is considered a null model for 
#' comparison to other models.  The ANOVA is functionally similar to a 
#' non-parametric likelihood ratio test for all null-full model comparisons
#' Residuals from the null model will be used to generate random pseudo-values 
#' via RRPP for evaluation of subsequent models. The permutation schedule from 
#' the null model will be used for random permutations.
#' This function does not correct for improper null models.  One must assure 
#' that the null model is nested within the other models.  Illogical results 
#' can be generated if this is not the case.
#'
#' @param object Object from \code{\link{lm.rrpp}}
#' @param ... Additional lm.rrpp model fits or other arguments passed to anova. 
#' @param effect.type One of "F", "cohenf", "SS", "MS", "Rsq" to choose from 
#' which distribution of statistics to calculate effect sizes (Z).  "LR" can
#' be chosen to include a log-likelihood ratio, for multi-model comparisons.  If
#' "LR" is used, the effect type is automatically "F".
#' See \code{\link{lm.rrpp}}.
#' @param error An optional character string to define MS error term for 
#' calculation of F values. See \code{\link{lm.rrpp}} for examples.
#' @param print.progress A logical argument if multiple models are used and 
#' one wishes to view progress for sums of squares (SS) calculations.
#' @export anova.lm.rrpp
#' @export
#' @author Michael Collyer
#' @keywords utilities
#' @examples 
#' \dontrun{
#' # See examples for lm.rrpp to see how anova.lm.rrpp works in conjunction
#' # with other functions
#' 
#' data(Pupfish)
#' names(Pupfish)
#' Pupfish$logSize <- log(Pupfish$CS) # better to not have functions in formulas
#'
#'# Single-Model ANOVA
#' 
#' fit <- lm.rrpp(coords ~ logSize + Sex*Pop, SS.type = "I", 
#' data = Pupfish, print.progress = FALSE, iter = 999) 
#' anova(fit)
#' anova(fit, effect.type = "MS")
#' anova(fit, effect.type = "Rsq")
#' anova(fit, effect.type = "cohenf")
#' 
#' # Multi-Model ANOVA (like a Likelihood Ratio Test)
#' fit.size <- lm.rrpp(coords ~ logSize, SS.type = "I", data = Pupfish, 
#' print.progress = FALSE, iter = 999) 
#' fit.sex <- lm.rrpp(coords ~ logSize + Sex, SS.type = "I", data = Pupfish, 
#' print.progress = FALSE, iter = 999) 
#' fit.pop <- lm.rrpp(coords ~ logSize + Pop, SS.type = "I", data = Pupfish, 
#' print.progress = FALSE, iter = 999) 
#' anova(fit.size, fit.sex, fit.pop, 
#' print.progress = FALSE) # compares two models to the first
#' 
#' # see lm.rrpp examples for mixed model ANOVA example and how to vary SS type
#' }
#' 
anova.lm.rrpp <- function(object, ...,
                          effect.type = c("F", "cohenf", 
                                          "SS", "MS", 
                                          "Rsq", "LR"),
                          error = NULL, print.progress = TRUE) {
  
  effect.type <- match.arg(effect.type)
  if(object$PermInfo$full.resid && effect.type != "F") {
    effect.type = "F"
    warning(
      paste(
        "\nThis is not an error!  It is a friendly warning.\n",
        "\nBecause permutation of full model residuals was chosen,",
        "\nthe effect size type must be focrced to be F to have appropriate effect sizes.\n",
        "\nUse options(warn = -1) to turn off these warnings. \n\n", sep = " "),
      noBreaks. = TRUE, call. = FALSE, immediate. = TRUE) 
  }
  
  doLR <- effect.type == "LR"
  
  dots <- list(...)
  lm.check <- sapply(dots, inherits, "lm.rrpp")
  
  if(!any(lm.check) && effect.type == "LR")
    stop(c("\nThe likelihood ratio test can only be performed ",
    "with multiple models.\n"), call. = FALSE)
  
  if(doLR) effect.type <- "F"
  
  if(any(lm.check)) {
    lm.list <- dots[lm.check]
    out <- aov.multi.model(object, lm.list, 
                           effect.type = effect.type, 
                           print.progress = print.progress)
  } else out <- aov.single.model(object, ...,
                          effect.type = effect.type,
                          full.resid = object$PermInfo$full.resid,
                          error = error)
  if(any(lm.check) && doLR) {
    Tab <- out$table
    n <- max(Tab$ResDf) + 1
    Tab$logLR <- rep(NA, nrow(Tab))
    lr_rows <- 2:(nrow(Tab) - 1)
    
    for(i in lr_rows){
      Df <- Tab[i, 1:2]
      LR <- n / 2 * log(Tab$F[i] * Df[2] / Df[1] + 1)
      Tab$logLR[i] <- as.numeric(unlist(LR))
    }
    
    out$table <- Tab
  }
    
  out
}
  

#' Pairwise comparisons of lm.rrpp fits
#'
#' Function generates distributions of pairwise statistics for a lm.rrpp fit and
#' returns important statistics for hypothesis tests.
#'
#' Based on an lm.rrpp fit, this function will find fitted values over all 
#' permutations and based 
#' on a grouping factor, calculate either least squares (LS) means or 
#' slopes, and pairwise statistics among
#' them.  Pairwise statistics have multiple flavors, related to vector attributes: 
#' 
#' \describe{
#' \item{\bold{Distance between vectors, "dist"}}{ Vectors for LS means or 
#' slopes originate at the origin and point to some location, having both a 
#' magnitude
#' and direction.  A distance between two vectors is the inner-product of of 
#' the vector difference, i.e., the distance between their endpoints.  For
#' LS means, this distance is the difference between means.  For multivariate 
#' slope vectors, this is the difference in location between estimated change 
#' for the dependent variables, per one-unit change of the covariate considered.
#' For univariate slopes, this is the absolute difference between slopes.}
#' \item{\bold{Standardized distance between vectors, "stdist"}}{Same as the 
#' distance between vectors, but distances are divided by the model standard error 
#' (square-root of the trace of the residual covariance matrix, adjusted by sample size). 
#' Pairwise tests with this statistic should be consistent with ANOVA results.}
#' \item{\bold{Mahalanobis distance between vectors, "mdist"}}{Similar to the
#' standardized distance between vectors but the inverse of the 
#' residual covariance matrix is used in calculation of the distance, rather than
#' dividing the Euclidean distance between means and dividing by the standard error. 
#' If the residual covariance matrix is singular, Mahalanobis distances will not be 
#' calculated.  Pairwise tests with this statistic should be consistent with 
#' MANOVA results.}
#' \item{\bold{Vector correlation, "VC"}}{ If LS mean or slope vectors are 
#' scaled to unit size, the vector correlation is the inner-product of the 
#' scaled vectors.
#' The arccosine (acos) of this value is the angle between vectors, which can 
#' be expressed in radians or degrees.  Vector correlation indicates the 
#' similarity of 
#' vector orientation, independent of vector length.}
#' \item{\bold{Difference between vector lengths, "DL"}}{  If the length of a vector 
#' is an important attribute -- e.g., the amount of multivariate change per 
#' one-unit
#' change in a covariate -- then the absolute value of the difference in 
#' vector lengths is a practical statistic to compare vector lengths, rather
#' than the estimates the vectors make.  Let 
#' d1 and
#' d2 be the distances (length) of vectors.  Then |d1 - d2| is a statistic 
#' that compares their lengths.  For slope vectors, this is a comparison of rates.
#' For comparison, if vectors are rates, "dist" finds the difference between estimates per unit
#' change of, e.g., time, size, etc., which could be large, even for small rates of change, if 
#' vectors point in dissimilar directions.  "DL" is a comparison of rates, irrespective of direction.}
#' \item{\bold{Variance, "var"}}{  Vectors of residuals from a linear model 
#' indicate can express the distances of observed values from fitted values.  
#' Mean
#' squared distances of values (variance), by group, can be used to measure 
#' the amount of dispersion around estimated values for groups.  Absolute
#' differences between variances are used as test statistics to compare mean 
#' dispersion of values among groups.  Variance degrees of freedom equal n, 
#' the group size, rather than n-1, as the purpose is to compare mean dispersion 
#' in the sample.  (Additionally, tests with one subject in a group are 
#' possible, or at least not a hindrance to the analysis.)}
#' }
#' 
#' The \code{\link{summary.pairwise}} function is used to select a test 
#' statistic for the statistics described above, as
#' "dist", "VC", "DL", and "var", respectively.  If vector correlation is tested, 
#' the \code{angle.type} argument can be used to choose between radians and
#' degrees.
#' 
#' The null model is defined via \code{\link{lm.rrpp}}, but one can 
#' also use an alternative null model as an optional argument.
#' In this case, residual randomization in the permutation procedure 
#' (RRPP) will be performed using the alternative null model 
#' to generate fitted values.  If full randomization of values (FRPP) 
#' is preferred,
#' it must be established in the lm.rrpp fit and an alternative model 
#' should not be chosen. If one is unsure about the inherent
#' null model used if an alternative is not specified as an argument, 
#' the function \code{\link{reveal.model.designs}} can be used.
#' 
#' Observed statistics, effect sizes, P-values, and one-tailed confidence 
#' limits based on the confidence requested will
#' be summarized with the \code{\link{summary.pairwise}} function.  
#' Confidence limits are inherently one-tailed as
#' the statistics are similar to absolute values.  For example, a 
#' distance is analogous to an absolute difference.  Therefore,
#' the one-tailed confidence limits are more akin to two-tailed 
#' hypothesis tests.  (A comparable example is to use the absolute 
#' value of a t-statistic, in which case the distribution has a lower 
#' bound of 0.)  
#' 
#'  \subsection{Notes for RRPP 0.6.2 and subsequent versions}{ 
#'  In previous versions of pairwise, \code{\link{summary.pairwise}} had three 
#'  test types: "dist", "VC", and "var".  When one chose "dist", for LS mean 
#'  vectors, the statistic was the inner-product of the vector difference.  
#'  For slope vectors, "dist" returned the absolute value of the difference 
#'  between vector lengths, which is "DL" in 0.6.2 and subsequent versions.  This
#'  update uses the same calculation, irrespective of vector types.  Generally,
#'  "DL" is the same as a contrast in rates for slope vectors, but might not have
#'  much meaning for LS means.  Likewise, "dist" is the distance between vector
#'  endpoints, which might make more sense for LS means than slope vectors.  
#'  Nevertheless, the user has more control over these decisions with version 0.6.2
#'  and subsequent versions.
#' }
#' 
#'  \subsection{Notes for RRPP 2.0.4 and subsequent versions}{ 
#'  The test types, standardized distance between vectors, "stdist", and Mahalanobis distances
#'  between vectors were added.  The former
#'  simply divides the distance between vectors by model standard error (square-root 
#'  of the trace of the residual covariance matrix, adjusted by sample size).  
#'  This is a multivariate generalization
#'  of a t-statistic for the difference between means.  It is not the same as Hotelling 
#'  squared-T-statistic, which requires incorporating the inverse of the residual covariance 
#'  matrix in the calculation of the distance, a calculation that also requires a non-singular
#'  covariance matrix.  However, the Mahalanobis distances are similar (and proportional) 
#'  to the Hotelling squared-T-statistic. Pairwise tests using Mahalanobis distances represent
#'  a non-parametric analog to the parametric Hotelling squared-T test.  Both tests should be 
#'  better for GLS model fits compared
#'  to Euclidean distances between means, as the total sums of squares are more likely to vary
#'  across random permutations. In general, if ANOVA is 
#'  performed a pairwise test with "stdist" is a good association; if MANOVA is performed,
#'  a pairwise test with "mdist" is a good association.
#' }
#'  
#' @param fit A linear model fit using \code{\link{lm.rrpp}}.
#' @param fit.null An alternative linear model fit to use as a null 
#' model for RRPP, if the null model
#' of the fit is not desired.  Note, for FRPP this argument should 
#' remain NULL and FRPP
#' must be established in the lm.rrpp fit (RRPP = FALSE).  If the 
#' null model is uncertain, 
#' using \code{\link{reveal.model.designs}} will help elucidate the 
#' inherent null model used.
#' @param groups A factor or vector that is coercible into a factor, 
#' describing the levels of
#' the groups for which to find LS means or slopes.  Normally this 
#' factor would be part of the 
#' model fit, but it is not necessary for that to be the case in order 
#' to obtain results.
#' @param covariate A numeric vector for which to calculate slopes for 
#' comparison  If NULL, 
#' LS means will be calculated instead of slopes.  Normally this variable 
#' would be part of the 
#' model fit, but it is not necessary for that to be the case in order 
#' to obtain results.
#' @param verbose A logical value for whether to include extra results, specifically 
#' the Mahalanobis distances among means, which requires the calculation of residual 
#' covariance matrices for each permutation.  This should be generally
#' FALSE, unless Mahalanobis distances are desired, in which case it must be TRUE.  
#' Verbose computations can require much additional time.
#' @param print.progress If a null model fit is provided, a logical 
#' value to indicate whether analytical 
#' results progress should be printed on screen.  Unless large data 
#' sets are analyzed, this argument 
#' is probably not helpful.
#' @keywords analysis
#' @export
#' @author Michael Collyer
#' 
#' @return An object of class \code{pairwise} is a list containing the following
#' \item{LS.means}{LS means for groups, across permutations.}
#' \item{slopes}{Slopes for groups, across permutations.}
#' \item{means.dist}{Pairwise distances between means, across permutations.}
#' \item{std.means.dist}{Pairwise distances between means, across permutations, standardized.}
#' \item{mah.means.dist}{Pairwise Mahalanobis distances between means, across permutations.}
#' \item{means.vec.cor}{Pairwise vector correlations between 
#' means, across permutations.}
#' \item{means.lengths}{LS means vector lengths, by group, across permutations.}
#' \item{means.diff.length}{Pairwise absolute differences between 
#' mean vector lengths, across permutations.}
#' \item{slopes.dist}{Pairwise distances between slopes (end-points), 
#' across permutations.}
#' \item{slopes.vec.cor}{Pairwise vector correlations between slope 
#' vectors, across permutations.}
#' \item{slopes.lengths}{Slope vector lengths, by group, across permutations.}
#' \item{slopes.diff.length}{Pairwise absolute differences between 
#' slope vector lengths, across permutations.}
#' \item{n}{Sample size}
#' \item{p}{Data dimensions; i.e., variable number}
#' \item{PermInfo}{Information for random permutations, passed on 
#' from lm.rrpp fit and possibly
#' modified if an alternative null model was used.}
#' @references Collyer, M.L., D.J. Sekora, and D.C. Adams. 2015. A 
#' method for analysis of phenotypic change for phenotypes described
#' by high-dimensional data. Heredity. 115:357-365.
#' @references Adams, D.C and M.L. Collyer. 2018. Multivariate 
#' phylogenetic ANOVA: group-clade aggregation, biological 
#' challenges, and a refined permutation procedure. Evolution. In press.
#' @seealso \code{\link{lm.rrpp}}
#' @examples 
#' \dontrun{
#' # Examples use geometric morphometric data on pupfishes
#' # See the package, geomorph, for details about obtaining such data
#'
#' # Body Shape Analysis (Multivariate) --------------
#' 
#' data("Pupfish")
#' 
#' # Note:
#' 
#' dim(Pupfish$coords) # highly multivariate!
#' 
#' Pupfish$logSize <- log(Pupfish$CS) 
#'
#' # Note: one should use all dimensions of the data but with this 
#' # example, there are many.  Thus, only three principal components 
#' # will be used for demonstration purposes.
#' 
#' Pupfish$Y <- ordinate(Pupfish$coords)$x[, 1:3]
#' 
#' ## Pairwise comparisons among LS means
#' 
#' # Note: one should increase RRPP iterations but a 
#' # smaller number is used here for demonstration 
#' # efficiency.  Generally, iter = 999 will take less
#' # than 1s for these examples with a modern computer.
#' 
#' fit1 <- lm.rrpp(Y ~ logSize + Sex * Pop, SS.type = "I", 
#' data = Pupfish, print.progress = FALSE, iter = 199) 
#' summary(fit1, formula = FALSE)
#' anova(fit1) 
#'
#' pup.group <- interaction(Pupfish$Sex, Pupfish$Pop)
#' pup.group
#' PW1 <- pairwise(fit1, groups = pup.group)
#' PW1
#' 
#' # distances among means
#' summary(PW1, confidence = 0.95, test.type = "dist") 
#' summary(PW1, confidence = 0.95, test.type = "dist", stat.table = FALSE)
#' 
#' # standardized distances among means
#' summary(PW1, confidence = 0.95, test.type = "stdist") 
#' 
#' # Mahalanobis (generalized) distances among means
#' summary(PW1, confidence = 0.95, test.type = "mdist") 
#' 
#' # absolute difference between mean vector lengths
#' summary(PW1, confidence = 0.95, test.type = "DL") 
#' 
#' # correlation between mean vectors (angles in degrees)
#' summary(PW1, confidence = 0.95, test.type = "VC", 
#'    angle.type = "deg") 
#'
#' # Can also compare the dispersion around means
#' summary(PW1, confidence = 0.95, test.type = "var")
#' 
#' ## Pairwise comparisons of slopes
#' 
#' fit2 <- lm.rrpp(Y ~ logSize * Sex * Pop, SS.type = "I", 
#' data = Pupfish, print.progress = FALSE, iter = 199) 
#' summary(fit2, formula = FALSE)
#' anova(fit1, fit2)
#'
#' # Using a null fit that excludes all factor-covariate 
#' # interactions, not just the last one  
#' 
#' PW2 <- pairwise(fit2, fit.null = fit1, groups = pup.group, 
#' covariate = Pupfish$logSize, print.progress = FALSE) 
#' PW2
#' 
#' # distances between slope vectors (end-points)
#' summary(PW2, confidence = 0.95, test.type = "dist") 
#' summary(PW2, confidence = 0.95, test.type = "dist", stat.table = FALSE)
#' 
#' # absolute difference between slope vector lengths
#' summary(PW2, confidence = 0.95, test.type = "DL") 
#' 
#' # correlation between slope vectors (and angles)
#' summary(PW2, confidence = 0.95, test.type = "VC",
#'    angle.type = "deg") 
#'    
#' # Can also compare the dispersion around group slopes
#' summary(PW2, confidence = 0.95, test.type = "var")
#' }
#' 
pairwise <- function(fit, fit.null = NULL, groups, covariate = NULL, 
                     verbose = FALSE, print.progress = FALSE) {
  fitf <- fit
  term.labels <- fit$LM$term.labels
  n <- fit$LM$n
  p <- fit$LM$p
  
  res.df <- fit$ANOVA$df[(length(fit$ANOVA$df) - 1)]
  SE <- sqrt(fit$ANOVA$RSS.model[NROW(fit$ANOVA$RSS.model), ] / res.df)
  
  if(verbose){
    if(res.df < fit$LM$p) {
      cat("\nVerbose results were requested but the residual covariance matrix is singular.\n",
          "Therefore, residual covariance matrices will not be estimated for every random permutation.\n")
      verbose = FALSE
    }
  }
  
  PermInfo <- getPermInfo(fit, attribute = "all")
  Models <- getModels(fit, attribute = "all")
  ind <- PermInfo$perm.schedule
  perms <- length(ind)
  gls <- fitf$LM$gls
  if(!inherits(fitf, "lm.rrpp")) 
    stop("The model fit must be a lm.rrpp class object")
  
  if(!is.null(fit.null)) {
    PermInfo$perm.method <- "RRPP"
    if(!inherits(fit.null, "lm.rrpp")) 
      stop("The null model fit must be a lm.rrpp class object")
    
    if(print.progress){
      cat(paste("\nCoefficients estimation from RRPP on null model:", 
                perms, "permutations.\n"))
      pb <- txtProgressBar(min = 0, max = perms+1, initial = 0, style=3)
    }
  }
  
  fit <- NULL
  
  k <- length(term.labels)
  kk <- length(Models$full)
  if(k != kk) {
    cat("Because the linear model design matrix is not full rank, there might be an issue.\n")
    cat("If there is a subsequent error, this is probably why.\n\n")
  }
  
  Y <- fitf$LM$Y
  X <- Models$reduced[[max(1, kk)]]$X
  
  if(gls) {
    if(!is.null(fitf$LM$Cov) && is.null(fitf$LM$PCov))
       fitf$LM$Pcov <- Cov.proj(fitf$LM$Cov)
    Y <- if(!is.null(fitf$LM$Pcov)) fitf$LM$Pcov %*% Y else
      Y %*% sqrt(fitf$LM$weights)
    X <- if(!is.null(fitf$LM$Pcov)) fitf$LM$Pcov %*% X else
      X %*% sqrt(fitf$LM$weights)
  }
  
  if(!is.null(fit.null)) X <- model.matrix(fit.null)
  
  X.s <- drop0(Matrix(X, sparse = TRUE), 1e-10)
  if(length(X.s@x) < length(X)) X <- X.s
  rm(X.s)
  
  if(k > 0) {
    lmf <- LM.fit(X, Y)
    fitted <-lmf$fitted.values
    res <- lmf$residuals
  } else {
    fitted <- matrix(0, n, p)
    res <- Y
  }
  
  rrpp.args <- list(fitted = as.matrix(fitted), residuals = as.matrix(res),
                    ind.i = NULL, o = NULL)
  
  o <- if(!is.null(fitf$LM$offset)) fitf$LM$offset else NULL
  offset <- if(!is.null(o)) TRUE else FALSE
  rrpp.args$o <- o
  
  rrpp <- function(fitted, residuals, ind.i, offset = FALSE, o = NULL) {
    if(offset) fitted + residuals[ind.i, , drop = FALSE] - o else
      fitted + residuals[ind.i, , drop = FALSE]
  }
  
  rrpp.args$o <- if(!is.null(fitf$LM$offset)) fitf$LM$offset else NULL
  rrpp.args$offset <- if(!is.null(o)) TRUE else FALSE
  
  Qf <- QRforX(model.matrix(fitf), reduce = FALSE)

  H <- tcrossprod(solve(Qf$R), Qf$Q)
  getCoef <- function(y) H %*% y
  
  if(is.null(fitf$LM$random.coef) || !is.null(fit.null)) {
    
    coef.n <- lapply(1:perms, function(j){
      step <- j
      if(print.progress && !is.null(fit.null)) 
        setTxtProgressBar(pb,step)
      rrpp.args$ind.i <- ind[[j]]
      y <- do.call(rrpp, rrpp.args)
      getCoef(y)
    })
    
    fitf$LM$random.coef <- vector("list", length = kk)
    names(fitf$LM$random.coef) <- term.labels
    fitf$LM$random.coef[[max(1, kk)]] <- coef.n
  }
  
  step <- perms + 1
  if(print.progress && !is.null(fit.null)) {
    setTxtProgressBar(pb,step)
    close(pb)
  }
  
  groups <- factor(groups)
  n_groups <- tapply(groups, groups, length)
  
  
  if(length(groups) != n) 
    stop("The length of the groups factor does not match the number of observations 
         in the lm.rrpp fit")
  if(!is.null(covariate)) {
    if(!is.numeric(covariate)) 
      stop("The covariate argument must be numeric")
    if(inherits(covariate, "matrix")) 
      stop("The covariate argument can only be a single covariate, not a matrix")
    if(length(unique(covariate)) < 2) 
      stop("The covariate vector appears to have no variation.")
    if(length(covariate) != n)
      stop("The length of the covariate does not match the number of observations 
           in the lm.rrpp fit")
  }
  if(is.null(covariate)) {
    
    means <- getLSmeans(fit = fitf, g = groups)
    slopes <- NULL
    means.dist <- lapply(means, function(x) as.matrix(dist(x)))
    pw_matches <- combn(n_groups, 2, simplify = FALSE)
    pw.sample.adjust <- lapply(pw_matches, function(x) 1/x[1] + 1/x[2])
    ng <- length(n_groups)
    pw_correct <- as.dist(matrix(0, ng, ng))
    pw_correct[1:length(pw.sample.adjust)] <- sqrt(unlist(pw.sample.adjust))
    pw_correct <- as.matrix(pw_correct)
    diag(pw_correct) <- 1
    std.means.dist <- Map(function(m, s) m/(s * pw_correct), means.dist, as.list(SE))
    means.vec.cor <- lapply(means, vec.cor.matrix)
    means.length <- lapply(means, function(x) sqrt(rowSums(x^2)))
    means.diff.length <- lapply(means.length, function(x) 
      as.matrix(dist(as.matrix(x))))
    if(verbose){
      if(print.progress){
        cat(paste("\nEstimation of residual covariance matrices:", 
                  perms, "permutations.\n"))
        pb <- txtProgressBar(min = 0, max = perms+1, initial = 0, style=3)
      }
      
      ResCovs <- lapply(1:perms, function(j){
        rrpp.args$ind.i <- ind[[j]]
        y <- do.call(rrpp, rrpp.args)
        yh <- fastFit(Qf$Q, y, n, p)
        R <- y - yh
        step <- j
        if(print.progress) 
          setTxtProgressBar(pb,step)
        crossprod(R) / res.df
      })
      
      step <- perms + 1
      if(print.progress) {
        setTxtProgressBar(pb,step)
        close(pb)
      }
        
      if(print.progress){
        cat(paste("\nCalculation of pairwise Mahalanobis distances:", 
                  perms, "permutations.\n"))
        pb <- txtProgressBar(min = 0, max = perms+1, initial = 0, style=3)
      }
      
      mah.means.dist <- lapply(1:perms, function(j){
        m <- means[[j]]
        s <- ResCovs[[j]]
        step <- j
        md <- mahal_dist(m, s, method = "euclidean")
        if(print.progress) 
          setTxtProgressBar(pb,step)
        as.matrix(md)
      })
      step <- perms + 1
      if(print.progress) {
        setTxtProgressBar(pb,step)
        close(pb)
      }
    } else mah.means.dist <- NULL
    
    slopes.length <- NULL
    slopes.dist <- NULL
    std.slopes.dist <- NULL
    slopes.diff.length <- NULL
    slopes.vec.cor <- NULL
    
  } else {
    
    means <- NULL
    slopes <- getSlopes(fit = fitf, x = covariate, g = groups)
    means.dist <- NULL
    std.means.dist <- NULL
    mah.means.dist <- NULL
    means.vec.cor <- NULL
    means.length <- NULL
    means.diff.length <- NULL
    
    slopes.length <- lapply(slopes, function(x) sqrt(rowSums(x^2)))
    slopes.dist <- lapply(slopes, function(x) as.matrix(dist(as.matrix(x))))
    pw_matches <- combn(n_groups, 2, simplify = FALSE)
    pw.sample.adjust <- lapply(pw_matches, function(x) 1/x[1] + 1/x[2])
    ng <- length(n_groups)
    pw_correct <- as.dist(matrix(0, ng, ng))
    pw_correct[1:length(pw.sample.adjust)] <- sqrt(unlist(pw.sample.adjust))
    pw_correct <- as.matrix(pw_correct)
    diag(pw_correct) <- 1
    std.slopes.dist <- Map(function(m, s) m/(s * pw_correct), 
                           slopes.dist, as.list(SE))
    slopes.diff.length <- lapply(slopes.length, 
                                 function(x) as.matrix(dist(as.matrix(x))))
    slopes.vec.cor <- lapply(slopes, vec.cor.matrix)
  }
  
  if(gls) res <- fitf$LM$gls.residuals else res <- fitf$LM$residuals
  disp.args <- list(res = as.matrix(res), ind.i = NULL, x = model.matrix(~groups + 0))
  if(gls) disp.args$x <- fitf$LM$Pcov %*% disp.args$x
  g.disp <- function(res, ind.i, x) {
    r <- res[ind.i, , drop = FALSE]
    if(NCOL(r) > 1) d <- apply(r, 1, function(x) sum(x^2)) else
      d <- r^2
    coef(lm.fit(as.matrix(x), d))
  }
  
  vars <- sapply(1:perms, function(j){
    disp.args$ind.i <- ind[[j]]
    do.call(g.disp, disp.args)
  })
  
  rownames(vars) <- levels(groups)
  colnames(vars) <- c("obs", paste("iter", 1:(perms -1), sep = "."))
  
  out <- list(LS.means = means, slopes = slopes, 
              means.dist = means.dist, std.means.dist = std.means.dist,
              mah.means.dist = mah.means.dist, 
              means.vec.cor = means.vec.cor, 
              means.length = means.length, 
              means.diff.length = means.diff.length,
              slopes.dist = slopes.dist, 
              std.slopes.dist = std.slopes.dist,
              slopes.vec.cor = slopes.vec.cor,
              slopes.length = slopes.length, 
              slopes.diff.length = slopes.diff.length,
              vars = vars, SE = SE, n = n, p = p, PermInfo = fitf$PermInfo)
  
  class(out) <- "pairwise"
  out
}

## rrpp.data.frame

#' Handle missing values in rrpp.data.frame objects
#'
#' @param object object (from \code{\link{rrpp.data.frame}})
#' @param ... further arguments (currently not used)
#' @method na.omit rrpp.data.frame
#' @export
#' @author Michael Collyer
#' @keywords utilities
#' @examples
#' y <- matrix(rnorm(15), 5, 3)
#' x <- rnorm(5)
#' rdf <- rrpp.data.frame(x = x, y = y, d = dist(y))
#' rdf$x[1] <- NA # create missing data
#' rdf
#' 
#' ndf <- na.omit(rdf)
#' ndf

na.omit.rrpp.data.frame <- function(object, ...) {
  nms <- names(object)
  classes <- unlist(lapply(object, function(x) class(x)[1]))
  subDF <- object[!is.na(match(classes, c("numeric", "factor", "integer", 
                                          "character", "matrix", "logical")))]
  sub.classes <- classes[!is.na(match(classes, c("numeric", "factor", "integer", 
                                                 "character", "matrix", "logical")))]
  oDF <- object[is.na(match(classes, c("numeric", "factor", "integer", 
                                       "character", "matrix", "logical")))]
  o.classes <- classes[is.na(match(classes, c("numeric", "factor", "integer", 
                                              "character", "matrix", "logical")))]
  subDF <- as.data.frame(subDF)
  newDF <- na.omit(subDF)
  omits <- attr(newDF, "na.action")
  
  for(i in 1:length(o.classes)){
    
    if(o.classes[[i]] == "array") {
      dims <- dim(oDF[[i]])
      if(length(dims) != 3)
        stop("Data are neither a vector, matrix, nor appopriate array.\n",
             call. = FALSE)
      oDF[[i]] <- oDF[[i]][,,-omits]
    }
    
    if(o.classes[[i]] == "phylo") {
      
      cat("Part of this data frame is a class phylo object\n")
      cat("It is currently not possible to prune the tree according to missing data\n")
      cat("The following actions are recommended:\n")
      cat("1. Make a data frame with all objects or variables except the phylo object\n")
      cat("2. Omit missing data to create a new data frame\n")
      cat("3. Add a pruned tree to the new data frame; e.g., newDF$tree <- myPrunedTree\n")
      stop("na.omit terminated", call. = FALSE)

        stop("Data are netieher a vector, matrix, nor appopriate array.\n",
             call. = FALSE)
      oDF[[i]] <- oDF[[i]][,,-omits]
    }
    
    if(o.classes[[i]] == "dist") {
      
      d <- as.matrix(oDF[[i]])
      d <- d[-omits, -omits]
      oDF[[i]] <- as.dist(d)
    }
    
  }
  
  outDF <- c(as.list(newDF), oDF)
  class(outDF) <- "rrpp.data.frame"
  attr(outDF, "na.action") <- omits
  outDF[nms]
  
}


## lm.rrpp

#' Print/Summary Function for RRPP
#'
#' @param x print/summary object (from \code{\link{lm.rrpp}})
#' @param ... other arguments passed to print/summary
#' @method print lm.rrpp
#' @export
#' @author Michael Collyer
#' @keywords utilities
print.lm.rrpp <- function(x, ...){
  cat("\nLinear Model fit with lm.rrpp\n")
  LM <- x$LM
  PI <- getPermInfo(x, attribute = "all")
  AN <- x$ANOVA
  if(is.null(AN$Fs)) {
    AN2 <- getANOVAStats(x, stat = "all")
    AN[c("MS", "Rsq", "F", "cohenf")] <- AN2[c("MS", "Rsq", "F", "cohenf")]
  }
    
  if(!is.null(x$LM$dist.coefficients)) 
    dv <- "(dimensions of data after PCoA of distance matrix)" else
    dv <- " "
  cat(paste("\nNumber of observations:", LM$n))
  cat(paste("\nNumber of dependent variables:", LM$p, dv))
  cat(paste("\nData space dimensions:", LM$p.prime, dv))
  
  est <- if(isTRUE(x$LM$LMM)) "mixed" else if(x$LM$gls) "gls" else "ols"
  if(est == "mixed") {
    cat("\nEstimation: Mixed Model")
    fixed.est <- if(x$LM$gls) "Generalized Least Squares" else 
      "Ordinary Least Squares"
    random.est <- if(x$LM$estimation == "LS") fixed.est else
      if(x$LM$estimation == "ML") "Maximum Likelihood" else
        "Restricted Maximum Likelihood "
    cat("\n    Fixed Effects:", fixed.est)
    cat("\n   Random Effects:", random.est)
  }
  if(est == "gls")
    cat("\nEstimation: Generalized Least Squares (via OLS projection)")
  if(est == "ols")
    cat("\nEstimation: Ordinary Lease Squares")
  
  if(!is.null(AN$SS.type)) cat(paste("\nSums of Squares and Cross-products: Type", 
                                     AN$SS.type))
  if(!is.null(PI)) cat(paste("\nNumber of permutations:", PI$perms))
  cat("\nCall: ")
  cat(strsplit(deparse1(x$call), " ")[[1]], 
      fill = 40)
  invisible(x)
}

## lm.rrpp

#' Print/Summary Function for RRPP
#'
#' @param object print/summary object (from \code{\link{lm.rrpp}})
#' @param formula Logical argument for whether to include formula in summary table
#' @param ... other arguments passed to print/summary
#' @method summary lm.rrpp
#' @export
#' @author Michael Collyer
#' @keywords utilities
summary.lm.rrpp <- function(object, formula = TRUE, ...){
  if(inherits(object, "manova.lm.rrpp")) out <- summary.manova.lm.rrpp(object) else {
    x <- object
    LM <- x$LM
    PI <- getPermInfo(x, attribute = "all")
    AN <- x$ANOVA
    if(is.null(AN$Fs)) {
      AN2 <- getANOVAStats(object, stat = "all")
      AN[c("SS", "MS", "Rsq", "Fs", "cohenf")] <-
        AN2[c("SS", "MS", "Rsq", "Fs", "cohenf")]
    }
      
    Models <- getModels(x, attribute = "all")
    perms <- PI$perms
    dv <- LM$dist.coefficients
    n <- LM$n
    p <- LM$p
    p.prime <- LM$p.prime
    SS <- AN$SS
    SSM.obs <- SS[,1]
    RSS <- AN$RSS
    TSS <- AN$TSS[nrow(AN$RSS.model),]
    RSS.model <- AN$RSS.model[nrow(AN$RSS.model),]

    if(is.null(RSS)) RSS <- RSS.model
    if(is.null(RSS)) TSS <- RSS.model
    SS.type <- AN$SS.type
    trms <- LM$term.labels
    k <- length(trms)
    kk <- length(Models$full)
    if(k > kk){
      k <- kk
      trms <- names(Models$full)
    }
    
    if(k > 0) {
      df <- AN$df
      dfe <- df[k+1]
      dfM <- sum(df[1:k])
      SSM <- (TSS- RSS.model)
      SSM.obs <- SSM[1]
      Rsq <- SSM.obs/TSS[1]
      Fs <- (SSM/dfM)/(RSS.model/dfe)
      Fs.obs <- Fs[1]
      P <- pval(as.vector(Fs))
      Z <- effect.size(as.vector(Fs))
      
    } else {
      df <- 0
      dfe <- AN$df
      dfM <- 1
      SSM <- Fs <- Z <- Rsq <- SSM.obs <- Fs.obs <- P <- ""
    }
    
    tab <- data.frame(dfM = dfM, dfe = dfe,
                      SSM = SSM.obs, RSS = RSS[1], Rsq = Rsq,
                      F = Fs.obs, Z = Z, P = P)
    dimnames(tab)[[2]] <- c("Df",
                            "Residual Df",
                            "SS",
                            "Residual SS",
                            "Rsq",
                            "F",
                            "Z (from F)",
                            "Pr(>F)")
    
    if(isTRUE(x$LM$LMM)) formula <- FALSE
    if(formula) dimnames(tab)[[1]] <- deparse(formula(x$call$f1)[[3]]) else
      dimnames(tab)[[1]] <- deparse(substitute(object))
    
    if(LM$ols){
      pca.fitted <- prcomp(LM$fitted)
      pca.residuals <- prcomp(LM$residuals)
      pca.total <- prcomp(LM$Y)
    }
    
    
    if(LM$gls) {
      int <- attr(LM$Terms, "intercept")
      
      if(!is.null(LM$Cov) && is.null(LM$Pcov))
        LM$Pcov <- Cov.proj(LM$Cov)
      
      if(is.null(LM$Pcov)) {
        w <- sqrt(LM$weights)
        PY <- LM$Y * w
        PX <- LM$X * w
        Pint <- as.matrix(rep(int, n) * w)
        RM <- lm.fit(PX, PY)$residuals
        Sr <- crossprod(RM)
      }
      if(!is.null(LM$Pcov)) {
        Pcov <- LM$Pcov
        PY <- as.matrix(Pcov %*% LM$Y)
        PX <- as.matrix(Pcov %*% LM$X)
        Pint <- as.matrix(Pcov %*% rep(int, n))
        RM <- lm.fit(PX, PY)$residuals
        Sr <- crossprod(RM)
      }
      
      RT <- lm.fit(Pint, PY)$residuals
      Sy <- crossprod(RT)
      Sf <- Sy - Sr
      
      Cf <- Sf/(n - 1)
      Cr <- Sr/(n - 1)
      Cy <- Sy/(n - 1)
      
      pca.fitted <- pca.residuals <- pca.total <- list()
      
      svd.Y <- svd(Cy)
      keep <- which(zapsmall(svd.Y$d) > 0)
      pca.total$sdev <- sqrt(svd.Y$d[keep])
      pca.total$rotation <- as.matrix(svd.Y$v[, keep, drop = FALSE])
      pca.total$x <- RT %*% as.matrix(svd.Y$v[, keep, drop = FALSE])
      
      svd.f <- svd(Cf)
      keep <- which(zapsmall(svd.f$d) > 0)
      if(length(keep) == 0) {
        pca.fitted$sdev <- pca.fitted$rotation <- pca.fitted$x <- 0
      } else {
        pca.fitted$sdev <- sqrt(svd.f$d[keep])
        pca.fitted$rotation <- as.matrix(svd.f$v[, keep])
        pca.fitted$x <- LM$gls.fitted %*% as.matrix(svd.f$v[, keep])
      }
      
      svd.r <- svd(Cr)
      keep <- which(zapsmall(svd.r$d) > 0)
      pca.residuals$sdev <- sqrt(svd.r$d[keep])
      pca.residuals$rotation <- as.matrix(svd.r$v[, keep, drop = FALSE])
      pca.residuals$x <- RM %*% as.matrix(svd.r$v[, keep, drop = FALSE])
      
    }
    
    
    d.f <- pca.fitted$sdev^2
    d.r <- pca.residuals$sdev^2
    d.t <- pca.total$sdev^2
    
    rank.f <- length(which(zapsmall(d.f) > 0))
    rank.r <- length(which(zapsmall(d.r) > 0))
    rank.t <- length(which(zapsmall(d.t) > 0))
    
    Trace <- zapsmall(c(sum(d.f), sum(d.r), sum(d.t)))
    Proportion <- zapsmall(Trace/sum(d.t))
    Rank <- c(rank.f, rank.r, rank.t)
    
    redund <- data.frame(Trace, Proportion, Rank)
    rownames(redund) <- c("Fitted", "Residuals", "Total")
    eigs.f <- eigs.r <- eigs.t <- rep(NA, rank.t)
    if(rank.f > rank.t) rank.f <- rank.t
    if(rank.r > rank.t) rank.r <- rank.t
    eigs.f[1:rank.f] <- d.f[1:rank.f]
    eigs.r[1:rank.r] <- d.r[1:rank.r]
    eigs.t[1:rank.t] <- d.t[1:rank.t]
    eigs <- as.table(zapsmall(rbind(eigs.f, eigs.r, eigs.t)))
    rownames(eigs) <- c("Fitted", "Residuals", "Total")
    colnames(eigs) <- paste("PC", 1:rank.t, sep="")
    
    reduced <- Models$reduced
    full <- Models$full
    
    if(k > 0) {
      
      if(LM$gls) {
        TY <- if(!is.null(LM$Pcov)) LM$Pcov %*% LM$Y else
          sqrt(LM$weights)
      } else TY <- LM$Y
      
      Ur <- lapply(reduced, function(x) x$qr$Q)
      Uf <- lapply(full, function(x) x$qr$Q)
      
      RR <- Map(function(u) TY - fastFit(u, TY, n, p), Ur)
      RF <- Map(function(u) TY - fastFit(u, TY, n, p), Uf)
      
      SSCP <- lapply(1:length(RF), function(j) crossprod(RR[[j]] - RF[[j]]))
      names(SSCP) <- trms
      SSCP <- c(SSCP, list(Residuals = as.matrix(crossprod(RF[[k]]))))
      
    } else {
      RR <- reduced[[1]]$residuals
      RF <- full[[1]]$residuals
      
      if(LM$gls) {
        if(!is.null(LM$Cov)) {
          if(!is.null(LM$Cov) && is.null(LM$Pcov))
            LM$Pcov <- Cov.proj(LM$Cov)
          RR <- LM$Pcov %*% RR
          RF <- LM$Pcov %*% RF
        } else {
          RR <- RR * sqrt(LM$weights)
          RF <- RF * sqrt(LM$weights)
        }
      }
      
      SSCP <- list(Residuals = as.matrix(crossprod(RF)))
      
    }
    
    out <- list(table = tab, SSCP = SSCP, n = n, p = p, p.prime = p.prime, k = k, 
                perms = perms, dv = dv, SS = SS, SS.type = SS.type, 
                redundancy = redund,
                eigenvalues = eigs, gls = x$LM$gls, fit = x)
    class(out) <- "summary.lm.rrpp"
  }
  out
}

#' Print/Summary Function for RRPP
#'
#' @param x print/summary object (from \code{\link{summary.lm.rrpp}})
#' @param ... other arguments passed to print/summary
#' @method print summary.lm.rrpp
#' @export
#' @author Michael Collyer
#' @keywords utilities
#' 
print.summary.lm.rrpp <- function(x, ...) {
  print.lm.rrpp(x$fit, ...)
  cat("\n\nFull Model Analysis of Variance\n\n")
  print(x$table)
  cat("\n\nRedundancy Analysis (PCA on fitted values and residuals)\n\n")
  if(x$gls) {
    cat("\nGLS mean used rather than center of gravity.  
        Projection is not orthogonal.\n\n")
  }
  print(x$redundancy)
  cat("\nEigenvalues\n\n")
  print(x$eigenvalues)
  cat("\n")
  invisible(x)
}


## terms.lm.rrpp

#' Extract the terms from an lm.rrpp object
#' 
#' \code{terms.lm.rrpp} returns the terms constructed for an \code{lm.rrpp} object.
#'
#' @param x Object from \code{\link{lm.rrpp}}
#' @param ...	further arguments passed to or from other methods
#' @export
#' @author Michael Collyer
#' @keywords utilities

terms.lm.rrpp <- function(x, ...) return(x$LM$Terms)

## model.frame.lm.rrpp

#' Extract model frame from a lm.rrpp object
#' 
#' \code{model.frame.lm.rrpp} returns the model frame constructed for 
#' an \code{lm.rrpp} object.
#'
#' @param formula Object from \code{\link{lm.rrpp}}
#' @param ...	further arguments passed to or from other methods
#' @export
#' @author Michael Collyer
#' @keywords utilities

model.frame.lm.rrpp <- function(formula, ...) {
  res <- if(inherits(formula, "lm.rrpp")) formula$LM$data else
    model.frame.default(formula)
  return(res)
}

## model.matrix.lm.rrpp

#' Extract the model design matrix from an lm.rrpp object
#' 
#' \code{model.matrix.lm.rrpp} returns the design matrix constructed for 
#' an \code{lm.rrpp} object.
#'
#' @param object Object from \code{\link{lm.rrpp}}
#' @param ...	further arguments passed to or from other methods
#' @export
#' @author Michael Collyer
#' @keywords utilities

model.matrix.lm.rrpp <- function(object, ...) {
  res <- if(isTRUE(object$LM$LMM)){
    list(fixed = object$LM$X,
         random = object$LM$Z)
  } else object$LM$X
  
  return(res)
}

## coef.lm.rrpp

#' Print/Summary Function for RRPP
#'
#' @param x Object from \code{\link{coef.lm.rrpp}}
#' @param ... Other arguments passed onto coef.lm.rrpp
#' @method print coef.lm.rrpp
#' @export
#' @author Michael Collyer
#' @keywords utilities
print.coef.lm.rrpp <- function(x, ...){
  
  if(!x$test) {
    cat("\n\nObserved coefficients\n\n")
    print(x$coef.obs)
    if(!is.null(x$coef.se)) {
      cat("\n\nCoefficient standard errors\n\n")
      print(x$coef.se)
    }
  }
  
  if(x$test){
    
    cat("\nPlease be aware that the test of coefficients will be deprecated.")
    cat("\nUse betaTest, which is more flexible;")
    cat("\ne.g., betaTest(fit, null.method = 'terms')\n")
    
    cat("\nLinear Model fit with lm.rrpp\n")
    cat(paste("\nNumber of observations:", x$n))
    cat(paste("\nNumber of dependent variables:", x$p))
    cat(paste("\nData space dimensions:", x$p.prime))
    cat(paste("\nSums of Squares and Cross-products: Type", x$SS.type))
    cat(paste("\nNumber of permutations:", x$nperms))
    
    if(is.null(x$stat.table)) {
      cat("\n\nTests on coefficients are not possible (only an intercept).")
      cat("\n\nObserved coefficients\n\n")
      print(x$coef.obs)
    } else {
      rrpp.type <- x$RRPP
      cat("\n\nStatistics (distances) of coefficients with \n")
      cat(x$confidence*100, "percent confidence intervals, effect sizes,") 
      cat("\nand probabilities of exceeding observed values,")
      cat("\nbased on", x$nperms, "random permutations, using", rrpp.type, "\n\n") 
      print(x$stat.tab)
      cat("\n\n")
    }
  }
  invisible(x)
}

#' Print/Summary Function for RRPP
#'
#' @param object Object from \code{\link{coef.lm.rrpp}}
#' @param ... Other arguments passed onto coef.lm.rrpp
#' @method summary coef.lm.rrpp
#' @export
#' @author Michael Collyer
#' @keywords utilities
summary.coef.lm.rrpp <- function(object, ...){
  print.coef.lm.rrpp(object, ...)
}

## predict.lm.rrpp

#' Print/Summary Function for RRPP
#'
#' @param x Object from \code{\link{predict.lm.rrpp}}
#' @param PC Logical argument for whether to use predicted values 
#' rotated to their PCs
#' @param ... Other arguments passed onto predict.lm.rrpp
#' @method print predict.lm.rrpp
#' @export
#' @author Michael Collyer
#' @keywords utilities
print.predict.lm.rrpp <- function(x, PC = FALSE, ...){
  cat("\nLinear Model fit with lm.rrpp\n")
  cat(paste("\nNumber of predictions:", NROW(x$mean)))
  cat(paste("\nConfidence level:", x$confidence*100, "%"))
      cat(paste("\nNumber of bootstrap permutations:", 
                length(x$random.predicted)))
      if(PC)  cat(paste("\nPredicted values are rotated to their PCs"))
      cat("\n\nPredicted values (mean of bootstrapped values):\n\n")
      if(PC) print(x$pc.mean) else print(x$mean)
      cat("\n\n", x$confidence*100, "% Lower confidence limits:\n\n") 
      if(PC) print(x$pc.lcl) else print(x$lcl)
      cat("\n\n", x$confidence*100, "% Upper confidence limits:\n\n") 
      if(PC) print(x$pc.ucl) else  print(x$ucl) 
      cat("\n\n")
      invisible(x)
}

#' Print/Summary Function for RRPP
#'
#' @param object Object from \code{\link{predict.lm.rrpp}}
#' @param ... Other arguments passed onto predict.lm.rrpp
#' @method summary predict.lm.rrpp
#' @export
#' @author Michael Collyer
#' @keywords utilities
summary.predict.lm.rrpp <- function(object, ...){
  print.predict.lm.rrpp(object, ...)
}

## anova.lm.rrpp

#' Print/Summary Function for RRPP
#'
#' @param x print/summary object (from \code{\link{lm.rrpp}})
#' @param ... other arguments passed to print/summary
#' @method print anova.lm.rrpp
#' @export
#' @author Michael Collyer
#' @keywords utilities
print.anova.lm.rrpp <- function(x, ...) {
  tab <- x$table
  pm <- x$perm.method
  perms <- x$perm.number
  est <- x$est.method
  SS.type <- x$SS.type
  effect.type <- x$effect.type
  
  if(pm == "RRPP") pm <- "Randomization of null model residuals" else
    if(pm == "FMRP") pm <- "Randomization of the full model residuals" else 
      pm <- ("Randomization of raw values (residuals of mean)")
  
  if(est == "mixed") est <- "Mixed Model" else
    if(est == "GLS") 
      est <- "Generalized Least-Squares (via OLS projection)" else
        est <- "Ordinary Least Squares"
  if(est == "Mixed Model") {
    est.fixed <- if(x$fixed.est == "GLS")
      "Generalized Least-Squares (via OLS projection)" else
        "Ordinary Least Squares"
    est.random<- if(x$random.est == "ML")
      "Maximum Likelihood" else if(x$random.est == "REML")
        "Restricted Maximum Likelihood" else "Least Squares"
  }
  
  if(!is.null(SS.type)) {
    cat("\nAnalysis of Variance, using Residual Randomization\n")
    cat(paste("Permutation procedure:", pm, "\n"))
    cat(paste("Number of permutations:", perms, "\n"))
    cat(paste("Estimation method:", est, "\n"))
    if(est == "Mixed Model"){
      cat(paste("    Fixed effects:", est.fixed, "\n"))
      cat(paste("   Random effects:", est.random, "\n"))
    }
    cat(paste("Sums of Squares and Cross-products: Type", SS.type, "\n"))
    cat(paste("Effect sizes (Z) based on", effect.type, "distributions\n\n"))
    print(tab)
    cat("\nCall: ")
    cat(strsplit(deparse1(x$call), " ")[[1]], 
        fill = 40)
  } else {
    cat("\nAnalysis of Variance, using Residual Randomization\n")
    cat(paste("Permutation procedure:", pm, "\n"))
    cat(paste("Number of permutations:", perms, "\n"))
    cat(paste("Estimation method:", est, "\n"))
    if(NROW(tab) == 1) cat("No model effects; simple summary provided\n\n")
    if(NROW(tab) > 1) cat(paste("Effect sizes (Z) based on", 
                                effect.type, "distributions\n\n"))
    print(tab)
    if(NCOL(tab) == 7) {
      cat("\nCall: ")
      cat(strsplit(deparse1(x$call), " ")[[1]], 
          fill = 40)
    }
  }
}

#' Print/Summary Function for RRPP
#'
#' @param object Object from \code{\link{predict.lm.rrpp}}
#' @param ... Other arguments passed onto predict.lm.rrpp
#' @method summary anova.lm.rrpp
#' @export
#' @author Michael Collyer
#' @keywords utilities
summary.anova.lm.rrpp <- function(object, ...){
  print.anova.lm.rrpp(object, ...)
}



## plot functions

#' Plot Function for RRPP
#' 
#' @param x plot object (from \code{\link{lm.rrpp}})
#' @param type Indicates which type of plot, choosing among diagnostics,
#' regression, or principal component plots.  Diagnostic plots are similar to 
#' \code{\link{lm}} diagnostic plots, but for multivariate data.  Regression plots
#' plot multivariate dispersion in some fashion against predictor values. PC plots
#' project data onto the eigenvectors of the covariance matrix for fitted values.
#' @param resid.type If type = "diagnostics", an optional argument for whether Pearson ("p") or normalized ("n")
#' residuals should be used.  These residuals are the same for ordinary least-squares (OLS) estimation
#' but differ for generalized least-squares (GLS) estimation.  For the latter, normalizing residuals
#' requires multiplying them by the transformation matrix obtained for GLS estimation.
#' @param fitted.type As with resid.type, whether fitted values use observed ("o") or transformed ("t")
#' values.  
#' @param predictor An optional vector if "regression" plot type is chosen, 
#' and is a variable likely used in \code{\link{lm.rrpp}}.
#' This vector is a vector of covariate values equal to the number of observations.
#' @param reg.type If "regression" is chosen for plot type, this argument
#' indicates whether prediction line 
#' (PredLine) or regression score (RegScore) plotting is performed.  
#' For explanation of prediction line,
#' see Adams and Nistri (2010).  For explanation of regression score, see 
#' Drake and Klingenberg (2008).
#' @param ... other arguments passed to plot (helpful to employ
#' different colors or symbols for different groups).  See
#' \code{\link{plot.default}} and \code{\link{par}}
#' @method plot lm.rrpp
#' @export
#' @author Michael Collyer
#' @keywords utilities
#' @keywords visualization
#' @references Drake, A. G., and C. P. Klingenberg. 2008. 
#' The pace of morphological change: Historical 
#' transformation of skull shape in St Bernard dogs. Proc. R. Soc. B. 275:71-76.
#' @references Adams, D. C., and A. Nistri. 2010. Ontogenetic convergence 
#' and evolution of foot morphology 
#' in European cave salamanders (Family: Plethodontidae). BMC Evol. Biol. 10:1-10.
#' @examples 
#' \dontrun{
#' # Univariate example
#' data(PlethMorph)
#' fitGLS <- lm.rrpp(TailLength ~ SVL, data = PlethMorph, Cov = PlethMorph$PhyCov, 
#'  print.progress = FALSE, iter = 0)
#'  
#'  par(mfrow = c(2, 2))
#'  plot(fitGLS)
#'  plot(fitGLS, resid.type = "n") # use normalized (transformed) residuals
#'  plot(fitGLS, resid.type = "n", fitted.type = "t") # use also transformed fitted values
#'  
#'  # Multivariate example
#'  
#' Y <- as.matrix(cbind(PlethMorph$TailLength,
#' PlethMorph$HeadLength,
#' PlethMorph$Snout.eye,
#' PlethMorph$BodyWidth,
#' PlethMorph$Forelimb,
#' PlethMorph$Hindlimb))
#' PlethMorph$Y <- Y
#' fitGLSm <- lm.rrpp(Y ~ SVL, data = PlethMorph, 
#' Cov = PlethMorph$PhyCov,
#' print.progress = FALSE, iter = 0)
#' 
#' par(mfrow = c(2, 2))
#'  plot(fitGLSm)
#'  plot(fitGLSm, resid.type = "n") # use normalized (transformed) residuals
#'  plot(fitGLSm, resid.type = "n", fitted.type = "t") # use also transformed fitted values
#'  par(mfrow = c(1, 1))
#'  }
plot.lm.rrpp <- function(x, type = c("diagnostics", "regression",
                                      "PC"), 
                         resid.type = c("p", "n"),
                         fitted.type = c("o", "t"),
                         predictor = NULL,
                          reg.type = c("PredLine", "RegScore"), ...){
  plot_args <- list(...)
  type <- match.arg(type)
  resid.type <- match.arg(resid.type)
  fitted.type <- match.arg(fitted.type)
  
  if(!is.null(x$LM$Cov) && is.null(x$LM$Pcov))
    x$LM$Pcov <- Cov.proj(x$LM$Cov)
  
  if(x$LM$gls) {
    
    r <- as.matrix(x$LM$gls.residuals)
    f <- as.matrix(x$LM$gls.fitted)
    
    if(resid.type == "n"){
      if(!is.null(x$LM$Pcov)) r <- x$LM$Pcov %*% r
      if(!is.null(x$LM$weights)) r <- r * sqrt(x$LM$weights)
    }
    if(fitted.type == "t"){
      if(!is.null(x$LM$Pcov)) f <- x$LM$Pcov %*% f
      if(!is.null(x$LM$weights)) f <- f * sqrt(x$LM$weights)
    }
    
  } else {
    r <- as.matrix(x$LM$residuals)
    f <- as.matrix(x$LM$fitted)
  }
  
  if(is.na(match(type, c("diagnostics", "regression", "PC")))) 
    type <- "diagnostics"
  PL <- Reg.proj <- PC.points <- NULL
  
  if(type == "diagnostics") {
    
    if(x$LM$p == 1) {
      plot_args <- NULL
      
      y <- x$LM$Y
      if(resid.type == "n"){
        if(!is.null(x$LM$Pcov)) y <- x$LM$Pcov %*% y
        if(!is.null(x$LM$weights)) y <- y * sqrt(x$LM$weights)
      }
      
      X <- x$LM$X
      if(fitted.type == "t"){
        if(!is.null(x$LM$Pcov)) X <- x$LM$Pcov %*% X
        if(!is.null(x$LM$weights)) X <- X * sqrt(x$LM$weights)
      }

      X <- as.matrix(X)
      y <- as.matrix(y)
      lm.new <- lm(y ~ X + 0)
      lm.new$call <- x$call
      plot(lm.new, ...)
      
    } else {
      
      plot_args <- NULL
      pca.r <- prcomp(r)
      var.r <- round(pca.r$sdev^2/sum(pca.r$sdev^2)*100,2)
      plot(pca.r$x, pch=19, asp =1,
           xlab = paste("PC 1", var.r[1],"%"),
           ylab = paste("PC 2", var.r[2],"%"),
           main = "PCA Residuals")
      pca.f <- prcomp(f)
      var.f <- round(pca.f$sdev^2/sum(pca.f$sdev^2)*100,2)
      dr <- scale(sqrt(diag(tcrossprod(center(r)))))
      plot_QQ(r)
      plot(pca.f$x[,1], dr, pch=19, 
           xlab = paste("PC 1", var.f[1],"%"),
           ylab = "Standardized Euclidean Distance Residuals",
           main = "Residuals vs. PC 1 fitted")
      abline(h = 0, col = "gray", lty =3)
      if(length(unique(round(pca.f$x[,1], 7))) <= 2) {
        lfr <- list()
        lfr$x <- pca.f$x[,1]
        lfr$y <- dr
        fit <- lm(dr ~ pca.f$x[,1])
        lfr$fitted <- fit$fitted.values
        lfr$residuals <- fit$residuals
      } else {
        options(warn = -1)
        lfr <- loess(dr~pca.f$x[,1], span = 1)
        options(warn = 0)
      } 
      lfr <- cbind(lfr$x, lfr$fitted); lfr <- lfr[order(lfr[,1]),]
      points(lfr, type="l", col="red")
      plot_het(r,f)
      p <- ncol(r)
    }
    
  }
    
  if(type == "regression"){
    PL <- prcomp(f)$x[,1]
    reg.type <- match.arg(reg.type)
    if(is.na(match(reg.type, c("PredLine", "RegScore")))) 
      if(is.null(predictor))
        stop("This plot type is not available without a predictor.")
    n <- NROW(r); p <- NCOL(r)
    if(!is.vector(predictor)) stop("Predictor must be a vector")
    if(length(predictor) != n) 
      stop("Observations in predictor must equal observations if procD.lm fit")
    
    plot_args$x <- predictor
    
    plot_args$ylab <- "Regression Score"
    if(is.null(plot_args$xlab)) plot_args$xlab <- deparse(substitute(predictor))

    xc <- predictor
    if(x$LM$gls) {
      if(!is.null(x$LM$weights)) X <- cbind(xc, x$LM$X) * sqrt(x$LM$weights) else
        X <- cbind(xc, x$LM$Pcov %*% x$LM$X)
    } else X <- cbind(xc, x$LM$X)
    
    X <- as.matrix(X)
    b <- as.matrix(lm.fit(X, f)$coefficients)[1, ]
    Reg.proj <- center(x$LM$Y) %*% b %*% sqrt(solve(crossprod(b)))
    plot_args$y <- Reg.proj
    if(reg.type == "RegScore") {
      do.call(plot, plot_args)
    } else {
      plot_args$y <- PL
      plot_args$ylab <- "PC 1 for fitted values"
      do.call(plot, plot_args)
    }
  }
  if(type == "PC"){
    pca <- prcomp(f)
    eigs <- pca$rotation
    P <- center(x$LM$Y)%*%eigs
    v <- pca$sdev^2
    ev <- round(v[1:2]/sum(v)*100, 2)
    plot_args <- if(NCOL(P) > 1) list(x = P[,1], y = P[,2],  
                        xlab = paste("PC 1 for fitted values: ",ev[1],"%", sep = "") ,
                        ylab = paste("PC 2 for fitted values: ",ev[2],"%", sep = "") ,
                        ...) else list(x = 1:length(P), y = P, 
                                       xlab = "Index", 
                                       ylab = "PC 1 for fitted values: 100%",...)
    do.call(plot, plot_args)
    PC.points <- P
    rownames(P) <- rownames(x$LM$data)
  }
  out <- list(PredLine = PL, RegScore = Reg.proj, PC.points = PC.points, 
              plot_args = plot_args)
  invisible(out)
}

plot_het <- function(r,f){
  r <- center(r)
  f <- center(f)
  r <- sqrt(diag(tcrossprod(r)))
  f <- sqrt(diag(tcrossprod(f)))
  if(length(unique(round(f, 7))) <= 2) {
    lfr <- list()
    lfr$x <- f
    lfr$y <- scale(r)
    fit <- lm(scale(r) ~ f)
    lfr$fitted <- fit$fitted.values
    lfr$residuals <- fit$residuals
  } else {
    options(warn = -1)
    lfr <- loess(scale(r)~f, span = 1)
    options(warn = 0)
  }
  lfr <- cbind(lfr$x, lfr$y, lfr$fitted)
  lfr <- lfr[order(lfr[,1]), , drop = FALSE]
  plot(lfr, pch=19,  
       xlab = "Euclidean Distance Fitted Values",
       ylab = "Standardized Euclidean Distance Residuals", 
       main = "Residuals vs. Fitted")
  abline(h = 0, col = "gray", lty =3)
  points(lfr[,1], lfr[,3], type="l", col="red")
}

plot_QQ <- function(r){
  r <- center(r)
  r <- sqrt(diag(tcrossprod(r)))
  r <- sort(r)
  n <- length(r)
  tq <- (seq(1,n)-0.5)/n
  tq <- qnorm(tq)
  plot(tq, r, pch=19, xlab = "Theoretical Quantiles",
       ylab = "Euclidean Distance Residuals", 
       main = "Q-Q plot")
}

#' Plot Function for RRPP
#' 
#' @param x plot object (from \code{\link{predict.lm.rrpp}})
#' @param PC A logical argument for whether the data space should be 
#' rotated to its 
#' principal components
#' @param ellipse A logical argument to change error bars to ellipses 
#' in multivariate plots.  
#' It has no function for univariate plots or is abscissa is not NULL.
#' @param abscissa An optional vector (numeric of factor) equal in length 
#' to predictions to use for 
#' plotting as the abscissa (x-axis), in which case predictions are the 
#' ordinate (y-axis).  This might be 
#' helpful if predictions are made for a continuous independent variable.  
#' The abscissa would be the
#' same variable used to make predictions (and can be the data.frame used 
#' for 
#' newdata in \code{\link{predict.lm.rrpp}}).
#' @param label A logical argument for whether points should be labeled 
#' (in multivariate plots).
#' @param ... other arguments passed to plot, arrows, points, or text (helpful 
#' to employ
#' different colors or symbols for different groups).  See
#' \code{\link{plot.default}},  \code{\link{arrows}}, \code{\link{points}},
#' \code{\link{par}}, and \code{\link{text}}
#' @method plot predict.lm.rrpp
#' @export
#' @author Michael Collyer
#' @keywords utilities
#' @keywords visualization
#' @examples 
#' # See lm.rrpp help file for examples.
plot.predict.lm.rrpp <- function(x, PC = FALSE, ellipse = FALSE,
                                 abscissa = NULL,
                                 label = TRUE, ...){
  oldw <- getOption("warn")
  options(warn = -1)
  
  m <- if(is.matrix(x$mean)) rbind(x$mean, x$lcl, x$ucl) else 
    c(x$mean, x$lcl, x$ucl)
  mpc <- if(is.matrix(x$pc.mean)) rbind(x$pc.mean, x$pc.lcl, x$pc.ucl) else 
    c(x$pc.mean, x$pc.lcl, x$pc.ucl)
  conf <- x$confidence
  k <- NROW(x$mean)
  plot_args <- dots <- list(...)
  arrow_args <- text_args <- eP <- NULL
  
  plot_names <- names(plot_args)
  arrows.names <- c("angle", "length", "code", "col",
                    "lty", "lwd")
  text.names <- c("adj","offset", "pos", "vfont", "labels", "cex")
  
  plot_args <- plot_args[!(plot_names %in% arrows.names)]
  arrow_args <- dots[names(dots) %in% arrows.names]
  arrow_args <- dots[names(dots) %in% text.names]
  
  absx <- FALSE
  
  if(!is.null(abscissa)) {
    absx <- TRUE
    if(is.list(abscissa)) {
      xlabel <- names(abscissa)[[1]]
      abscissa <- unlist(abscissa)
    } else xlabel <- deparse(substitute(abscissa))
    if(length(abscissa) != k)
      stop("\n The length of the abscissa does not match the number of 
           predictions\n",
           call. = FALSE)
  } else xlabel <-  "Predicted values"
  
  plot_type <- if(NCOL(mpc) == 1) "uni" else "multi"
  if(absx) plot_type <- "uni"
  response.type <- if(NCOL(mpc) == 1) "uni" else "multi"
  
  if(plot_type == "uni") {

    if(response.type == "uni") {
      
      xx <- seq(1:k)
      xf <- as.factor(rownames(x$mean))
      if(absx && is.numeric(abscissa)) xx <- abscissa
      
      resp <- if(PC) x$pc.mean[,1] else x$mean[,1]
      mt <- if(PC) paste("Predicted PC1 Values, +/-", 
                         conf*100, "percent confidence levels",
                         sep = " ") else
                           paste("Predicted Values, +/-", 
                                 conf*100, "percent confidence levels",
                                 sep = " ")
      lcl <- if(PC) x$pc.lcl[,1] else x$lcl[,1]
      ucl <- if(PC) x$pc.ucl[,1] else x$ucl[,1]
      
      plot_args$x <- xx
      plot_args$y <- resp
      if(is.null(plot_args$xlab)) plot_args$xlab <- xlabel
      if(is.null(plot_args$ylab)) plot_args$ylab <- if(PC) "PC1: 100%" else  
        x$data.name
      if(is.null(plot_args$ylim)) plot_args$ylim <- c(min(lcl), max(ucl))
      if(is.null(plot_args$main)) plot_args$main <- mt
      if(is.null(plot_args$pch)) plot_args$pch <- 19
      if(is.null(plot_args$cex.main)) plot_args$cex.main <- 0.7
      plot_args$xaxt <- "n"
      
      do.call(plot, plot_args)

      if(absx && is.numeric(abscissa)) axis(1, xx) else
        axis(1, at = xx, labels = as.character(xf)) 
      
      arrow_args$x0 <- xx
      arrow_args$y0 <- resp
      arrow_args$x1 <- xx
      arrow_args$y1 <- lcl
      if(is.null(arrow_args$angle)) arrow_args$angle <- 90
      if(is.null(arrow_args$lemgth)) arrow_args$length <- 0.1
   
      do.call(arrows, arrow_args)
      arrow_args$y1 <- ucl
      do.call(arrows, arrow_args)
      do.call(points, plot_args)
      
    }
    
    if(response.type == "multi") {
      
      if(PC){
        pca <- x$pca
        mr <- mpc
        if(is.matrix(mr)) mr <- mr[,1]
        v <- pca$sdev^2
        ve <- round(v/sum(v)*100, 2)[1:2]
        ylm <- c(min(mr), max(mr))
        ylb <- paste("PC1: ", ve, "%", sep = "")
        type <- "uni"
      } else {
        mr <- m[,1]
        cn <- colnames(x$mean)[[1]]
        if(is.null(cn)) cn <- paste("V", 1, sep=".")
        ylb <- cn
        ylm <- c(min(mr), max(mr))
      }
      
      mt <- if(PC) 
        paste("PC1 predicted Values, +/-", conf*100, 
              "percent confidence levels",
                         sep = " ") else 
        paste("Predicted Values (variable 1), +/-", conf*100, 
              "percent confidence levels",
                        sep = " ")
      xx <- seq(1:k)
      xf <- as.factor(rownames(x$mean))
      if(absx && is.numeric(abscissa)) xx <- abscissa
      
      plot_args$x <- xx
      plot_args$y <- mr[1:k]
      if(is.null(plot_args$xlab)) plot_args$xlab <- xlabel
      if(is.null(plot_args$ylab)) plot_args$ylab <- ylb
      if(is.null(plot_args$ylim)) plot_args$ylim <- c(min(mr), max(mr))
      if(is.null(plot_args$main)) plot_args$main <- mt
      if(is.null(plot_args$pch)) plot_args$pch <- 19
      if(is.null(plot_args$cex.main)) plot_args$cex.main <- 0.7
      plot_args$xaxt <- "n"
      
      do.call(plot, plot_args)
      
      if(absx && is.numeric(abscissa)) axis(1, xx) else
        axis(1, at = xx, labels = as.character(xf)) 
      
      la <- (k + 1):(2 * k)
      ua <- (2 * k +1):(3 * k)
      
      arrow_args$x0 <- xx
      arrow_args$y0 <- mr[1:k]
      arrow_args$x1 <- xx
      arrow_args$y1 <- mr[la]
      if(is.null(arrow_args$angle)) arrow_args$angle <- 90
      if(is.null(arrow_args$lemgth)) arrow_args$length <- 0.1
      
      do.call(arrows, arrow_args)
      arrow_args$y1 <- mr[ua]
      do.call(arrows, arrow_args)
      do.call(points, plot_args)
      
    }
    
  }
  
  if(plot_type == "multi") {
    
    if(PC) {
      
      pca <- x$pca
      mr <- mpc[,1:2]
      v <- pca$sdev^2
      ve <- round(v/sum(v)*100, 2)[1:2]
      xlb <- paste("PC1: ", ve[1], "%", sep = "")
      ylb <- paste("PC2: ", ve[2], "%", sep = "")
      
    }
    
    if(!PC) {
      
      mr <- m[,1:2]
      cn <- colnames(x$mean)
      if(is.null(cn)) cn <- paste("V", 1:2, sep=".")
      xlb <- cn[1]
      ylb <- cn[2]
    }
    
    mt <- if(PC) "Among-prediction PC rotation" else 
      "Plot of first two variables"
    mt <- paste(mt, "; ", conf*100, "% confidence limits", sep = "")
  
    eP <- if(PC) ellipse.points(m = x$pc.mean[,1:2],
                                pr = x$random.predicted.pc, conf) else
                                       ellipse.points(m = x$mean[,1:2],
                                                pr = x$random.predicted, conf)
    span <- apply(eP$ellP, 2, "c")
    xlim <- c(min(span[,1]), max(span[,1]))
    ylim <- c(min(span[,2]), max(span[,2]))
    
    plot_args$x <- mr[1:k, 1]
    plot_args$y <- mr[1:k, 2]
    if(is.null(plot_args$xlab)) plot_args$xlab <- xlb
    if(is.null(plot_args$ylab)) plot_args$ylab <- ylb
    if(is.null(plot_args$xlim)) plot_args$xlim <- xlim
    if(is.null(plot_args$ylim)) plot_args$ylim <- ylim
    if(is.null(plot_args$main)) plot_args$main <- mt
    if(is.null(plot_args$cex.main)) plot_args$cex.main <- 0.7
    if(is.null(plot_args$asp)) plot_args$asp <- 1
    
    do.call(plot, plot_args)
    
    if(ellipse) {
      for(i in 1:(dim(eP$ellP)[[3]])){
        points(eP$ellP[,,i], type = "l", ...)
      }
      if(length(plot_args) == 0) points(eP$means, pch=19, cex = 0.7) else
        points(eP$means, ...)
      
    } else {
      
      la <- (k + 1):(2 * k)
      ua <- (2 * k +1):(3 * k)
      
      if(is.null(arrow_args$angle)) arrow_args$angle <- 90
      if(is.null(arrow_args$lemgth)) arrow_args$length <- 0.05

      arrow_args$x0 <- mr[1:k,1]
      arrow_args$y0 <- mr[1:k,2]
      arrow_args$x1 <- mr[1:k,1]
      arrow_args$y1 <- mr[la, 2]
      
      do.call(arrows, arrow_args)
      
      arrow_args$y1 <- mr[ua, 2]
      do.call(arrows, arrow_args)
      
      arrow_args$x1 <- mr[la, 1]
      arrow_args$y1 = mr[1:k,2]
      do.call(arrows, arrow_args)
      
      arrow_args$x1 <- mr[ua, 1]
      do.call(arrows, arrow_args)
      
      do.call(points, plot_args)
      
      arrow_args$y1 <- mr[la, 2] # return to original
    }
    
    
    if(label) {
      
      text_args <- list(x = NULL, y = NULL, cex = 1, col = 1,
                        adj = NULL, offset = 0.5, pos = 1, vfont = NULL,
                        labels = rownames(x$mean))
      
      dot.match <- intersect(c("x", "y", "cex", "col", "adj", 
                               "offset", "pos", "vfont", "labels"), 
                             names(dots))
      if(length(dot.match) > 0)
        text_args[dot.match] <- dots[dot.match]
      
      text_args$x <- plot_args$x
      text_args$y <- plot_args$y
      
      do.call(text, text_args)
      
    }
    
  }

  options(warn = oldw)
  out <- list(plot_args = plot_args, arrow_args = arrow_args, 
           text_args = text_args, ellipse.points = eP)
  class(out) <- "plot.predict.lm.rrpp"
  invisible(out)
}

## resid and fitted

# residuals.lm.rrpp
# S3 generic for lm.rrpp

#' Extract residuals
#' 
#' @param object plot object (from \code{\link{lm.rrpp}})
#' @param ... Arguments passed to other functions 
#' @export
#' @author Michael Collyer
#' @keywords utilities
#' @examples 
#' # See examples for lm.rrpp
residuals.lm.rrpp <- function(object, ...) {
  if(object$LM$gls) out <- object$LM$gls.residuals else 
    out <- object$LM$residuals
  out
}

# fitted.lm.rrpp
# S3 generic for lm.rrpp

#' Extract fitted values
#' 
#' @param object plot object (from \code{\link{lm.rrpp}})
#' @param ... Arguments passed to other functions 
#' @export
#' @author Michael Collyer
#' @keywords utilities
#' @examples 
#' # See examples for lm.rrpp
fitted.lm.rrpp <- function(object,  ...) {
  if(object$LM$gls) out <- object$LM$gls.fitted else 
    out <- object$LM$fitted
  out
}


#' Print/Summary Function for RRPP
#'
#' @param x Object from \code{\link{pairwise}}
#' @param ... Other arguments passed onto pairwise
#' @method print pairwise
#' @export
#' @author Michael Collyer
#' @keywords utilities
print.pairwise <- function(x, ...){
  if(is.null(x$LS.means)) type = "slopes"
  if(is.null(x$slopes)) type = "means"
  if(type == "means") {
    means <- x$LS.means
    perms <- length(means)
    groups <- rownames(means[[1]])
  }
  if(type == "slopes") {
    slopes <- x$slopes
    perms <- length(x$slopes)
    groups <- rownames(slopes[[1]])
  }
  perm.type <- x$PermInfo$perm.method
  cat("\nPairwise comparisons\n")
  cat("\nGroups:", groups, "\n\n")
  cat(paste(perm.type,":", sep=""), perms, "permutations\n")
}


#' Print/Summary Function for RRPP
#'
#' See \code{\link{pairwise}} for further description.  
#' 
#' The following summarize the test that can be performed: 
#' 
#' \describe{
#' \item{\bold{Distance between vectors, "dist"}}{ Vectors for LS means or 
#' slopes originate at the origin and point to some location, having both a 
#' magnitude
#' and direction.  A distance between two vectors is the inner-product of of 
#' the vector difference, i.e., the distance between their endpoints.  For
#' LS means, this distance is the difference between means.  For multivariate 
#' slope vectors, this is the difference in location between estimated change 
#' for the dependent variables, per one-unit change of the covariate considered.
#' For univariate slopes, this is the absolute difference between slopes.}
#' \item{\bold{Standardized distance between vectors, "stdist"}}{Same as the 
#' distance between vectors, but distances are divided by the model standard error 
#' (square-root of the trace of the residual covariance matrix). Pairwise tests
#' with this statistic should be consistent with ANOVA results.}
#' \item{\bold{Mahalanobis distance between vectors, "mdist"}}{Similar to the
#' standardized distance between vectors but the inverse of the 
#' residual covariance matrix is used in calculation of the distance, rather than
#' dividing the Euclidean distance between means and dividing by the standard error. 
#' If the residual covariance matrix is singular, Mahalanobis distances will not be 
#' calculated.  Pairwise tests with this statistic should be consistent with 
#' MANOVA results.}
#' \item{\bold{Vector correlation, "VC"}}{ If LS mean or slope vectors are 
#' scaled to unit size, the vector correlation is the inner-product of the 
#' scaled vectors.
#' The arccosine (acos) of this value is the angle between vectors, which 
#' can be expressed in radians or degrees.  Vector correlation indicates 
#' the similarity of 
#' vector orientation, independent of vector length.}
#' \item{\bold{Difference between vector lengths, "DL"}}{  If the length of a vector 
#' is an important attribute -- e.g., the amount of multivariate change per 
#' one-unit
#' change in a covariate -- then the absolute value of the difference in 
#' vector lengths is a practical statistic to compare vector lengths, rather
#' than the estimates the vectors make.  Let 
#' d1 and
#' d2 be the distances (length) of vectors.  Then |d1 - d2| is a statistic 
#' that compares their lengths.  For slope vectors, this is a comparison of rates.
#' For comparison, if vectors are rates, "dist" finds the difference between estimates per unit
#' change of, e.g., time, size, etc., which could be large, even for small rates of change, if 
#' vectors point in dissimilar directions.  "DL" is a comparison of rates, irrespective of direction.}
#' \item{\bold{Variance, "var"}}{  Vectors of residuals from a linear 
#' model indicate can express the distances of observed values from 
#' fitted values.  Mean
#' squared distances of values (variance), by group, can be used to 
#' measure the amount of dispersion around estimated values for groups.  
#' Absolute
#' differences between variances are used as test statistics to compare 
#' mean dispersion of values among groups.  Variance degrees of freedom 
#' equal n, 
#' the group size, rather than n-1, as the purpose is to compare mean 
#' dispersion 
#' in the sample.  (Additionally, tests with one subject in a group 
#' are possible, or at least not a hindrance to the analysis.)}
#' }
#' 
#' The argument, \code{test.type} is used to select one of the tests 
#' above.  See \code{\link{pairwise}} for examples.
#' 
#'  \subsection{Notes for RRPP 0.6.2 and subsequent versions}{ 
#'  In previous versions of pairwise, \code{\link{summary.pairwise}} had three 
#'  test types: "dist", "VC", and "var".  When one chose "dist", for LS mean 
#'  vectors, the statistic was the inner-product of the vector difference.  
#'  For slope vectors, "dist" returned the absolute value  of the difference 
#'  between vector lengths, which is "DL" in 0.6.2 and subsequent versions.  This
#'  update uses the same calculation, irrespective of vector types.  Generally,
#'  "DL" is the same as a contrast in rates for slope vectors, but might not have
#'  much meaning for LS means.  Likewise, "dist" is the distance between vector
#'  endpoints, which might make more sense for LS means than slope vectors.  
#'  Nevertheless, the user has more control over these decisions with version 0.6.2
#'  and subsequent versions.
#' }
#' 
#'  \subsection{Notes for RRPP 2.0.4 and subsequent versions}{ 
#'  The test types, standardized distance between vectors, "stdist", and Mahalanobis distances
#'  between vectors were added.  The former
#'  simply divides the distance between vectors by model standard error (square-root 
#'  of the trace of the residual covariance matrix).  This is a multivariate generalization
#'  of a t-statistic for the difference between means.  It is not the same as Hotelling 
#'  squared-T-statistic, which requires incorporating the inverse of the residual covariance 
#'  matrix in the calculation of the distance, a calculation that also requires a non-singular
#'  covariance matrix.  However, the Mahalanobis distances are similar (and proportional) 
#'  to the Hotelling squared-T-statistic. Pairwise tests using Mahalanobis distances represent
#'  a non-parametric analog to the parametric Hotelling squared-T test.  Both tests should be 
#'  better for GLS model fits compared
#'  to Euclidean distances between means, as the total sums of squares are more likely to vary
#'  across random permutations. In general, if ANOVA is 
#'  performed a pairwise test with "stdist" is a good association; if MANOVA is performed,
#'  a pairwise test with "mdist" is a good association.
#' }
#' 
#' @param object Object from \code{\link{pairwise}}
#' @param stat.table Logical argument for whether results should be 
#' returned in one table 
#' (if TRUE) or separate pairwise tables (if FALSE)
#' @param test.type the type of statistic to test.  See below
#' should be used in the test.
#' @param angle.type If test.type = "VC", whether angle results are 
#' expressed in radians or degrees.
#' @param confidence Confidence level to use for upper confidence 
#' limit; default = 0.95 (alpha = 0.05)
#' @param show.vectors Logical value to indicate whether vectors 
#' should be printed.
#' @param ... Other arguments passed onto pairwise
#' @method summary pairwise
#' @export
#' @author Michael Collyer
#' @keywords utilities
summary.pairwise <- function(object, stat.table = TRUE, 
                             test.type = c("dist", "stdist", "mdist", "VC", "DL", "var"),
                             angle.type = c("rad", "deg"),
                             confidence = 0.95, show.vectors = FALSE, ...){
  test.type <- match.arg(test.type)
  angle.type <- match.arg(angle.type)
  x <- object
  if(test.type != "var") {
    if(is.null(x$LS.means)) type = "slopes"
    if(is.null(x$slopes)) type = "means"
  } else {
    type <- "var"
    tab <- NULL
  }
 
  vars <- object$vars
  if(type == "var") {
    var.diff <- lapply(1:NCOL(vars), function(j){
      v <- as.matrix(vars[,j, drop = FALSE])
      as.matrix(dist(v))
    })
    L <- d.summary.from.list(var.diff, confidence = confidence)
    tab <- makePWDTable(L)
  }
  
  if(type == "means") {
    if(test.type == "dist") {
      L <- d.summary.from.list(x$means.dist, confidence = confidence)
      if(stat.table) tab <- makePWDTable(L)
    }
    if(test.type == "stdist") {
      L <- d.summary.from.list(x$std.means.dist, confidence = confidence)
      if(stat.table) tab <- makePWDTable(L)
    }
    if(test.type == "mdist") {
      if(is.null(x$mah.means.dist))
        stop("Mahalanobis distance tests require verbose = TRUE in pairwise.\n")
      L <- d.summary.from.list(x$mah.means.dist, confidence = confidence)
      if(stat.table) tab <- makePWDTable(L)
    }
    
    if(test.type == "VC") {
      L <- r.summary.from.list(x$means.vec.cor, confidence = confidence)
      if(angle.type == "deg") {
        options(warn = -1)
        L$angle <- L$angle * 180 / pi
        L$aCL <- L$aCL * 180 / pi
        options(warn = 0)
      }
      if(stat.table) tab <- makePWCorTable(L)
    }
    
    if(test.type == "DL") {
      L <- d.summary.from.list(x$means.diff.length, confidence = confidence)
      if(stat.table) tab <- makePWDTable(L)
    }
    
  }
  
  if(type == "slopes") {

    if(test.type == "dist") {
      L <- d.summary.from.list(x$slopes.dist)
      if(stat.table) tab <- makePWDTable(L)
    }
    
    if(test.type == "stdist") {
      L <- d.summary.from.list(x$std.slopes.dist)
      if(stat.table) tab <- makePWDTable(L)
    }
    
    if(test.type == "VC") {
      L <- r.summary.from.list(x$slopes.vec.cor) 
      if(angle.type == "deg") {
        options(warn = -1)
        L$angle <- L$angle * 180 / pi
        L$aCL <- L$aCL * 180 / pi
        options(warn = 0)
      }
      if(stat.table) tab <- makePWCorTable(L)
    }
    
    if(test.type == "DL") {
      L <- d.summary.from.list(x$slopes.diff.length, confidence = confidence)
      if(stat.table) tab <- makePWDTable(L)
    }
    
  }
  
  out <- list()
  out$pairwise.tables <- L
  if(stat.table){
    out$stat.table <- TRUE
    out$summary.table <- tab
  } else {
    out$stat.table <- FALSE
    out$summary.table <- NULL
  }

  out$type <- type
  out$test.type <- test.type
  out$angle.type <- angle.type 
  out$confidence <- confidence
  if(!is.null(vars)) out$vars <- vars else out$vars <- NULL
  out$show.vectors <- show.vectors
  out$x <- x
  
  class(out) <- "summary.pairwise"
  out
}

#' Print/Summary Function for RRPP
#'
#' @param x Object from \code{\link{summary.pairwise}}
#' @param ... Other arguments passed onto summary.pairwise
#' @method print summary.pairwise
#' @export
#' @author Michael Collyer
#' @keywords utilities
#' 
print.summary.pairwise <- function(x, ...) {
  type <- x$type
  test.type <- x$test.type
  stat.table <- x$stat.table
  print.pairwise(x$x)
  cat("\n")
  
  L <- x$pairwise.tables
  if(stat.table) tab <- x$summary.table

  if(type == "var") {
    
    cat("\nObserved variances by group\n\n")
    print(x$vars[,1])
    
    if(stat.table) {
      cat("\nPairwise distances between variances, plus statistics\n")
      print(tab)
    } else {
      cat("\nPairwise distances between variances\n")
      print(L$D)
      cat("\nPairwise", paste(L$confidence*100, "%", sep=""), 
          "upper confidence limits between variances\n")
      print(L$CL)
      cat("\nPairwise effect sizes (Z) between variances\n")
      print(L$Z)
      cat("\nPairwise P-values between variances\n")
      print(L$P)
    }
  }
  
  if(type == "means") {
    
    cat("LS means:\n")
    if(x$show.vectors) print(x$x$LS.means[[1]]) else 
      cat("Vectors hidden (use show.vectors = TRUE to view)\n")
    
    if(test.type == "dist") {
      if(stat.table) {
        cat("\nPairwise distances between means, plus statistics\n")
        print(tab)
      } else {
        cat("\nPairwise distances between means\n")
        print(L$D)
        cat("\nPairwise", paste(L$confidence*100, "%", sep=""), 
            "Upper confidence limits between means\n")
        print(L$CL)
        cat("\nPairwise effect sizes (Z) between means\n")
        print(L$Z)
        cat("\nPairwise P-values between means\n")
        print(L$P)
      }
    }
    
    if(test.type == "stdist") {
      if(stat.table) {
        cat("\nPairwise standardized distances between means, plus statistics\n")
        print(tab)
      } else {
        cat("\nPairwise standardized distances between means\n")
        print(L$D)
        cat("\nPairwise", paste(L$confidence*100, "%", sep=""), 
            "Upper confidence limits between means\n")
        print(L$CL)
        cat("\nPairwise effect sizes (Z) between means\n")
        print(L$Z)
        cat("\nPairwise P-values between means\n")
        print(L$P)
      }
    }
    
    if(test.type == "mdist") {
      if(stat.table) {
        cat("\nPairwise Mahalanobis distances between means, plus statistics\n")
        print(tab)
      } else {
        cat("\nPairwise Mahalanobis distances between means\n")
        print(L$D)
        cat("\nPairwise", paste(L$confidence*100, "%", sep=""), 
            "Upper confidence limits between means\n")
        print(L$CL)
        cat("\nPairwise effect sizes (Z) between means\n")
        print(L$Z)
        cat("\nPairwise P-values between means\n")
        print(L$P)
      }
    }
    
    if(test.type == "VC") {
      if(stat.table) {
        cat("\nPairwise statistics based on mean vector correlations\n")
        print(tab)
      } else {
        cat("\nPairwise vector correlations between mean vectors\n")
        print(L$r)
        cat("\nPairwise angles between mean vectors\n")
        print(L$angle)
        cat("\nPairwise", paste(L$confidence*100, "%", sep=""), 
            "Upper confidence limits for angles between mean vectors\n")
        print(L$aCL)
        cat("\nPairwise effect sizes (Z) for angles between mean vectors\n")
        print(L$Z)
        cat("\nPairwise P-values for angles between mean vectors\n")
        print(L$P)
      }
    }
    
    if(test.type == "DL") {
      if(stat.table) {
        cat("\nPairwise absolute difference (d) between vector lengths, 
            plus statistics\n")
        print(tab)
      } else {
        cat("\nPairwise absolute differences (d) between mean vector lengths\n")
        print(L$D)
        cat("\nPairwise", paste(L$confidence*100, "%", sep=""), 
            "Upper confidence limits between mean vector lengths\n")
        print(L$CL)
        cat("\nPairwise effect sizes (Z) between mean vector lengths\n")
        print(L$Z)
        cat("\nPairwise P-values between mean vector lengths\n")
        print(L$P)
      }
    }
    
  }
  
  if(type == "slopes") {
    cat("Slopes (vectors of variate change per one unit of covariate 
        change, by group):\n")
    if(x$show.vectors) print(x$x$slopes[[1]]) else 
      cat("Vectors hidden (use show.vectors = TRUE to view)\n")
    
    if(test.type == "dist") {
      if(stat.table) {
        cat("\nPairwise distances between slope vector 
            (end-points), plus statistics\n")
        print(tab)
      } else {
        cat("\nPairwise distances between slope vector (end-points\n")
        print(L$D)
        cat("\nPairwise", paste(L$confidence*100, "%", sep=""), 
            "Upper confidence limits between slopes\n")
        print(L$CL)
        cat("\nPairwise effect sizes (Z) between slopes\n")
        print(L$Z)
        cat("\nPairwise P-values between slopes\n")
        print(L$P)
      }
    }
      
      if(test.type == "stdist") {
        if(stat.table) {
          cat("\nStandardized pairwise distances between slope vector 
            (end-points), plus statistics\n")
          print(tab)
        } else {
          cat("\nStandardized pairwise distances between slope vector (end-points\n")
          print(L$D)
          cat("\nPairwise", paste(L$confidence*100, "%", sep=""), 
              "Upper confidence limits between slopes\n")
          print(L$CL)
          cat("\nPairwise effect sizes (Z) between slopes\n")
          print(L$Z)
          cat("\nPairwise P-values between slopes\n")
          print(L$P)
        }
      }
    
    if(test.type == "VC") {
      cat("\nPairwise statistics based on slopes vector correlations (r) 
          and angles, acos(r)")
      cat("\nThe null hypothesis is that r = 1 (parallel vectors).")
      cat("\nThis null hypothesis is better treated as the angle 
          between vectors = 0\n")
      if(stat.table) print(tab)
        else {
        cat("\nPairwise vector correlations between slope vectors\n")
        print(L$r)
        cat("\nPairwise angles between slope vectors\n")
        print(L$angle)
        cat("\nPairwise", paste(L$confidence*100, "%", sep=""), 
            "upper confidence limits for angles between mean vectors\n")
        print(L$aCL)
        cat("\nPairwise effect sizes (Z) for angles between slope vectors\n")
        print(L$Z)
        cat("\nPairwise P-values for angles between slope vectors\n")
        print(L$P)
      }
    }
    
    if(test.type == "DL") {
      cat("\nSlope vector lengths\n")
      print(x$x$slopes.length[[1]])
      if(stat.table) {
        cat("\nPairwise absolute difference (d) between vector 
            lengths, plus statistics\n")
        print(tab)
      } else {
        cat("\nPairwise absolute differences (d) between slope vector lengths\n")
        print(L$D)
        cat("\nPairwise", paste(L$confidence*100, "%", sep=""), 
            "Upper confidence limits between slope vector lengths\n")
        print(L$CL)
        cat("\nPairwise effect sizes (Z) between slope vector lengths\n")
        print(L$Z)
        cat("\nPairwise P-values between slope vector lengths\n")
        print(L$P)
      }
    }
    
  }
  invisible(x)
}


#' Print/Summary Function for RRPP
#'
#' @param x Object from \code{\link{model.comparison}}
#' @param ... Other arguments passed onto model.comparison
#' @method print model.comparison
#' @export
#' @author Michael Collyer
#' @keywords utilities
print.model.comparison <- function(x ,...){
  print(x$table)
}

#' Print/Summary Function for RRPP
#'
#' @param object Object from \code{\link{model.comparison}}
#' @param ... Other arguments passed onto model.comparison
#' @method summary model.comparison
#' @export
#' @author Michael Collyer
#' @keywords utilities
summary.model.comparison <- function(object ,...){
  x <- object
  type <- colnames(x$table)[[1]]
  if(type == "cov.trace") type.name <- "traces of model covariance matrices"
  if(type == "logLik") type.name <- "model log-likelihoods"
  if(type == "Z") type.name <- "model Z-scores"
  n <- NROW(x$table)
  cat("\n\n Summary statistics for", type.name, "\n\n")
  cat(n, "Models compared.\n\n")
  tab <- x$table
  rownames(tab) <- x$names
  print(tab)
}

#' Plot Function for RRPP
#' 
#' @param x plot object (from \code{\link{model.comparison}})
#' @param ... other arguments passed to plot (helpful to employ
#' different colors or symbols for different groups).  See
#' \code{\link{plot.default}} and \code{\link{par}}
#' @method plot model.comparison
#' @export
#' @author Michael Collyer
#' @keywords utilities
#' @keywords visualization
plot.model.comparison <- function(x, ...){
  object <- x
  type <- names(object$table)[[1]]
  tab <- object$table
  nms <- object$names
  spln <- if(type == "logLik") ncol(tab) == 5 else 
    if(type == "cov.trace") ncol(tab) == 3 else ncol(tab) == 2
  
  if(type == "cov.trace") type.name <- "Trace"
  if(type == "logLik") {
    type.name <- if(spln) "log-likelihood" else "-2 * log-likelihood"
  }
  if(type == "Z") type.name <- "Z"
  
  if(type == "logLik") {
    x <- if(spln) tab[,5] else tab[,3]
    xLab <- if(spln) names(tab)[5] else names(tab)[3] 
  } else if(type == "cov.trace"){
    x <- if(spln) tab[,3] else tab[,2]
    xLab <- if(spln) names(tab)[3] else names(tab)[2]
  } else {
    x <- if(spln) tab[,2] else NULL
    xLab <- names(tab)[if(spln) 2 else NULL]
  }
    
  if(type == "Z" && is.null(x)){
    cat("No predictor.  Z-scores will be plotted in the order provided.\n")
    x <- 1:nrow(tab)
  }
  y <- tab[,1]
  if(all(is.na(x))) x <- 1:length(y)
  if(type == "logLik" && !spln) y <- -2 * y
  xrange <- range(x)
  dx <- xrange[2] - xrange[1]
  xlim <- xrange + c(-0.1*dx, 0.1*dx)
  
  yrange <- range(y)
  dy <- yrange[2] - yrange[1]
  ylim <- yrange + c(-0.1*dy, 0.1*dy)

  
  if(var(x) == 0)
    stop("There is no plot to make, as the parameter penalty is invariant.",
         call. = FALSE) 
  plot(x, y, xlab = xLab, 
       ylab = type.name, xlim = xlim, ylim = ylim, ...)
  
  if(spln){
    
    if(!any(is.na(x))) {
      sp <- spline(x, y)
      points(sp$x, sp$y, col = 2, type = "l")
      xmax <- sp$x[which.max(sp$y)]
      xmin <-sp$x[which.min(sp$y)]
      text(x, y, nms, pos = 1, cex = 0.4)
      cat("max", type, "found at", xLab, "=", round(xmax, 4))
      cat("\nmin", type, "found at", xLab, "=", round(xmin, 4), "\n")
    }
      
  } else {
    if(is.null(xLab))
      stop("There is no plot to make, as there is no predictor.",
           call. = FALSE) 
    f <- lm(y ~ x)
    abline(f, lty = 3, lwd = 0.8, col = "red")
    text(x, y, nms, pos = 1, cex = 0.4)
               
           
  }

}

#' Print/Summary Function for RRPP
#'
#' @param object Object from \code{\link{lm.rrpp}}, updated with 
#' \code{\link{manova.update}}
#' @param test Type of multivariate test statistic to use.
#' @param ... Other arguments passed onto manova.lm.rrpp
#' @method summary manova.lm.rrpp
#' @export
#' @author Michael Collyer
#' @keywords utilities
summary.manova.lm.rrpp <- function(object, test = c("Roy", "Pillai", "Hotelling-Lawley", "Wilks"), ...){
  if (!inherits(object, "manova.lm.rrpp")) 
    stop(gettextf("object must be of class %s", dQuote("manova.lm.rrpp"), domain = NA))
  test <- match.arg(test)
  if(test == "Hotelling-Lawley") test <- "Hotelling.Lawley"
  p <- object$LM$p
  p.prime <- object$LM$p.prime
  n <- object$LM$n
  Models <- getModels(object, attribute = "all")
  PermInfo <- getPermInfo(object, attribute = "all")
  perm.method <- object$PermInfo$perm.method
  if(perm.method == "RRPP") RRPP = TRUE else RRPP = FALSE
  ind <- PermInfo$perm.schedule
  perms <- length(ind)
  trms <- object$LM$term.labels
  k <- length(trms)
  kk <- length(Models$full)
  if(k > kk){
    k <- kk
    trms <- names(Models$full)
  }
  df <- object$ANOVA$df
  df.model <- sum(df[1:k])
  df <- c(df[1:k], df.model, df[k+1])
  names(df) <- c(trms, "Full.Model", "Residuals")
  
  MANOVA <- object$MANOVA
  eigs <- MANOVA$eigs
  error <- MANOVA$error
  stats <- as.data.frame(matrix(NA, nrow = k + 2, ncol = 5, byrow = FALSE,
                                dimnames <- list(names(df), 
                                    c("Df", "Rand", test, "Z", "Pr"))))
  stats$Df <- df
  if(!is.null(error)) stats$Rand[1:(k+1)] <- c(error, 
                            "Residuals") else 
                              stats$Rand[1:(k+1)] <- rep("Residuals", k+1)
  
  if(test == "Pillai") {
    
    rand.stats <- sapply(1:perms, function(j){
      y <- eigs[[j]]
      apply(y, 1, pillai)
    })
    
    test.stats <- rand.stats[, 1]
    Z <- apply(rand.stats, 1, effect.size)
    P <- apply(rand.stats, 1, pval)
    stats$Z[1:(k+1)] <- Z
    stats$Pr[1:(k+1)] <- P
    stats$Pillai[1:(k+1)] <- test.stats
  }
  else if(test == "Hotelling.Lawley") {
    
    rand.stats <- sapply(1:perms, function(j){
      y <- eigs[[j]]
      apply(y, 1, hot.law)
    })
    
    test.stats <- rand.stats[, 1]
    Z <- apply(rand.stats, 1, effect.size)
    P <- apply(rand.stats, 1, pval)
    stats$Z[1:(k+1)] <- Z
    stats$Pr[1:(k+1)] <- P
    stats$Hotelling.Lawley[1:(k+1)] <- test.stats
  }
  
  else if(test == "Wilks"){
    
    rand.stats <- sapply(1:perms, function(j){
      y <- eigs[[j]]
      apply(y, 1, wilks)
    })
    
    test.stats <- rand.stats[, 1]
    Z <- apply(-1 * rand.stats, 1, effect.size)
    P <- apply(-1 * rand.stats, 1, pval)
    stats$Z[1:(k+1)] <- Z
    stats$Pr[1:(k+1)] <- P
    stats$Wilks[1:(k+1)] <- test.stats
  }
  else {
    rand.stats <- sapply(1:perms, function(j){
      y <- eigs[[j]]
      apply(y, 1, max)
    })
    
    test.stats <- rand.stats[,1]
    Z <- apply(rand.stats, 1, effect.size)
    P <- apply(rand.stats, 1, pval)
    stats$Z[1:(k+1)] <- Z
    stats$Pr[1:(k+1)] <- P
    stats$Roy[1:(k+1)] <- test.stats
  }
  
  if(test == "Wilks") 
    names(stats)[[length(stats)]] <- paste("Pr(<", test, ")", sep = "") else
    names(stats)[[length(stats)]] <- paste("Pr(>", test, ")", sep = "")
  
  out <- list(stats.table = stats, rand.stats = rand.stats, stat.type = test,
              n = n, p = p, p.prime = p.prime, e.rank = MANOVA$e.rank, 
              manova.pc.dims = MANOVA$manova.pc.dims, PCA = MANOVA$PCA,
              SS.type = object$ANOVA$SS.type, SS.tot = MANOVA$SS.tot, 
              perms = object$PermInfo$perms)
  class(out) <- "summary.manova.lm.rrpp"
  out
}


#' Print/Summary Function for RRPP
#'
#' @param x Object from \code{\link{summary.manova.lm.rrpp}}
#' @param ... Other arguments passed onto summary.manova.lm.rrpp
#' @method print summary.manova.lm.rrpp
#' @export
#' @author Michael Collyer
#' @keywords utilities
print.summary.manova.lm.rrpp <- function(x, ...){
  
  cat("\nLinear Model fit with lm.rrpp\n")
  cat(paste("\nNumber of observations:", x$n))
  cat("\nNumber of dependent variables:", x$p)
  cat("\nData space dimensions:", x$p.prime)
  cat("\nResidual covariance matrix rank:", x$e.rank)
  pc.max <- x$manova.pc.dims
  
  if(pc.max < x$e.rank) {
    cat("\n   Data reduced to", pc.max, "PCs, as required or prescribed.")
    PCA <- x$PCA
    d.p <- sum((PCA$x[, 1:pc.max])^2) / x$SS.tot
    cat("\n  ", round(d.p*100, 1), 
        "% of overall variation explained by these PCs.")
    cat("\n   See $MANOVA$PCA from manova.lm.rrpp object for more information.")
  }
  
  if(!is.null(x$SS.type)) 
    cat(paste("\nSums of Squares and Cross-products: Type", x$SS.type))
  cat(paste("\nNumber of permutations:", x$perms), "\n\n")
  
  tab <- as.matrix(x$stats.table)
  class(tab) <- c("anova", class(tab))
  print.table(tab, na.print = "")
  invisible(x)
}

#' Print/Summary Function for RRPP
#'
#' @param x Object from \code{\link{lr_test}}
#' @param ... Other arguments passed onto print.lr_test
#' @method print lr_test
#' @export
#' @author Michael Collyer
#' @keywords utilities
print.lr_test <- function(x, ...){
  x <- if(!is.null(x$tab)) x$tab else as.data.frame(x)
  colnames(x)[4] <- "Pr(>LR)"
  class(x) <- c("anova", class(x))
  
  cat("\n\n")
  
  print(x)
  cat("\n\n")
}

#' Print/Summary Function for RRPP
#'
#' @param object Object from \code{\link{lr_test}}
#' @param ... Other arguments passed onto print.lr_test
#' @method summary lr_test
#' @export
#' @author Michael Collyer
#' @keywords utilities
summary.lr_test <- function(object, ...){
  print.lr_test(object)
}

#' Print/Summary Function for RRPP
#'
#' @param x Object from \code{\link{trajectory.analysis}}
#' @param ... Other arguments passed onto 
#' @method print trajectory.analysis
#' @export
#' @author Michael Collyer
#' @keywords utilities
print.trajectory.analysis<- function(x, ...){
  if(is.null(x$SD)) type = "vectors" else type = "trajectories"
  perms <- length(x$MD)
  pca = x$pca
  groups <- rownames(x$LS.means[[1]])
  pca <- x$pca
  cat("\nTrajectory analysis\n\n")
  cat(perms, "permutations.\n\n")
  if(!is.null(pca)) cat("Points projected onto trajectory PCs\n")
  
}

#' Print/Summary Function for RRPP
#'
#' @param object Object from \code{\link{trajectory.analysis}}
#' @param stat.table Logical argument for whether results should be 
#' returned in one table 
#' (if TRUE) or separate pairwise tables (if FALSE)
#' @param attribute Whether magnitude differences (MD, absolute difference 
#' in trajectory path lengths), 
#' trajectory correlations (TC), or trajectory shape differences (SD) are 
#' summarized.
#' @param angle.type If attribute = "TC", whether angle results are 
#' expressed in radians or degrees.
#' @param confidence Confidence level to use for upper confidence limit; 
#' default = 0.95 (alpha = 0.05)
#' @param show.trajectories Logical value to indicate whether trajectories 
#' should be printed.
#' @param ... Other arguments passed onto trajectory.analysis
#' @method summary trajectory.analysis
#' @export
#' @author Michael Collyer
#' @keywords utilities
summary.trajectory.analysis <- function(object, stat.table = TRUE, 
                             attribute = c("MD", "TC", "SD"),
                             angle.type = c("rad", "deg"),
                             confidence = 0.95, show.trajectories = FALSE, ...) {
  
  attribute <- match.arg(attribute)
  angle.type <- match.arg(angle.type)
  x <- object
  if(is.null(x$SD)) type = "vectors" else type = "trajectories"
  
  MD <- object$MD
  TC <- object$TC
  SD <- object$SD
  
  if(attribute == "MD"){
    L <- d.summary.from.list(MD, confidence = confidence)
    tab <- makePWDTable(L)
  }
  
  if(attribute == "TC"){
    if(is.null(TC)) stop("Trajectory correlations not available\n", 
                         call. = FALSE)
    L <- r.summary.from.list(TC, confidence = confidence)
    
    if(angle.type == "deg") {
      options(warn = -1)
      L$angle <- L$angle * 180 / pi
      L$aCL <- L$aCL * 180 / pi
      options(warn = 0)
    }
    
    tab <- makePWCorTable(L)
  }
  
  if(attribute == "SD"){
    if(type == "trajectories") {
      L <- d.summary.from.list(SD, confidence = confidence)
      tab <- makePWDTable(L)
    } else {
      L <- NULL
      tab <- NULL
    }
  }
  
 
  out <- list()
  out$pairwise.tables <- L
  if(stat.table){
    out$stat.table <- TRUE
    out$summary.table <- tab
  } else {
    out$stat.table <- FALSE
    out$summary.table <- NULL
  }
  
  out$type <- type
  out$attribute <- attribute
  out$angle.type <- angle.type 
  out$confidence <- confidence
  out$show.trajectories <- show.trajectories
  out$x <- x
  
  class(out) <- "summary.trajectory.analysis"
  out
}

#' Print/Summary Function for RRPP
#'
#' @param x Object from \code{\link{summary.trajectory.analysis}}
#' @param ... Other arguments passed onto summary.trajectory.analysis
#' @method print summary.trajectory.analysis
#' @export
#' @author Michael Collyer
#' @keywords utilities
#' 
print.summary.trajectory.analysis <- function(x, ...) {
  attribute <- x$attribute
  type <- x$type
  if(attribute == "SD" && type == "vectors") 
    cat("\n\nCannot summarize trajectory shape differences for vectors.\n\n")
  stat.table <- x$stat.table
  if(attribute != "SD") print.trajectory.analysis(x$x) else 
    if(type == "trajectories") print.trajectory.analysis(x$x)

  cat("\n")
  
  L <- x$pairwise.tables
  if(stat.table) tab <- x$summary.table
  
  if(attribute == "MD") {
    
    cat("Trajectories:\n")
    if(x$show.trajectories) print(x$x$trajectories[[1]]) else 
      cat("Trajectories hidden (use show.trajectories = TRUE to view)\n")
    
    cat("\nObserved path distances by group\n\n")
    print(x$x$PD[[1]])
    
    if(stat.table) {
      cat("\nPairwise absolute differences in path distances, plus statistics\n")
      print(tab)
    } else {
      cat("\nPairwise absolute differences in path distancess\n")
      print(L$D)
      cat("\nPairwise", paste(L$confidence*100, "%", sep=""), 
          "upper confidence limits, absolute differences in path distancess\n")
      print(L$CL)
      cat("\nPairwise effect sizes (Z) for absolute differences in path distancess\n")
      print(L$Z)
      cat("\nPairwise P-values for absolute differences in path distances\n")
      print(L$P)
    }
  }
  
  if(attribute == "TC") {
    
    cat("Trajectories:\n")
    if(x$show.trajectories) print(x$x$trajectories[[1]]) else 
      cat("Trajectories hidden (use show.trajectories = TRUE to view)\n")
    
    if(stat.table) {
      cat("\nPairwise correlations between trajectories, plus statistics\n")
      print(tab)
    } else {
      cat("\nPairwise statistics based on trajectory vector 
          correlations (r) and angles, acos(r)")
      cat("\nThe null hypothesis is that r = 1 (parallel vectors).")
      cat("\nThis null hypothesis is better treated as the angle 
          between vectors = 0\n")
      
      cat("\nPairwise vector correlations between trajectories\n")
      print(L$r)
      cat("\nPairwise angles between trajectories\n")
      print(L$angle)
      cat("\nPairwise", paste(L$confidence*100, "%", sep=""), 
          "upper confidence limits for angles\n")
      print(L$aCL)
      cat("\nPairwise effect sizes (Z) for angles\n")
      print(L$Z)
      cat("\nPairwise P-values for angles\n")
      print(L$P)
    }
  }
  
  if(type == "trajectories"  && attribute == "SD") {
    
    cat("Trajectories:\n")
    if(x$show.trajectories) print(x$x$trajectories[[1]]) else 
      cat("Trajectories hidden (use show.trajectories = TRUE to view)\n")

    if(stat.table) {
      cat("\nPairwise trajectory shape differences, plus statistics\n")
      print(tab)
    } else {
      cat("\nPairwise trajectory shape differences\n")
      print(L$D)
      cat("\nPairwise", paste(L$confidence*100, "%", sep=""), 
          "upper confidence limits, shape differnces\n")
      print(L$CL)
      cat("\nPairwise effect sizes (Z) for trajectory shape differencess\n")
      print(L$Z)
      cat("\nPairwise P-values for trajectory shape differences\n")
      print(L$P)
    }
  }
  
  cat("\n\n")
    
  invisible(x)
}


#' Plot Function for RRPP
#' 
#'  Function generates a principal component plot for trajectories
#'
#'  The function calculates and plots principal components of fitted values from 
#'  \code{\link{lm.rrpp}} that are passed onto \code{\link{trajectory.analysis}}, 
#'  and projects
#'  data onto them.  This function is a set.up, and \code{\link{add.trajectories}} 
#'  is needed to 
#'  add trajectories to the plot.  By having two stages of control, the plotting 
#'  functions are more 
#'  flexible.  This function also returns plotting information that can be 
#'  valuable for making
#'  individualized plots, if \code{\link{add.trajectories}} is not preferred.
#' @param x plot object (from \code{\link{trajectory.analysis}})
#' @param ... other arguments passed to plot (helpful to employ
#' different colors or symbols for different groups).  See
#' \code{\link{plot.default}} and \code{\link{par}}
#' 
#' @return If an object is assigned, it will return:
#'  \item{pca}{Principal component analysis performed using \code{\link{prcomp}}.}
#'  \item{pc.points}{Principal component scores for all data.}
#'  \item{trajectory.analysis}{Trajectory analysis passed on.}
#'  \item{trajectories}{pca Observed trajectories projected onto principal 
#'  components.}
#'  
#' @seealso 
#' \code{\link{plot.default}} and \code{\link{par}}
#' @method plot trajectory.analysis
#' @export
#' @author Michael Collyer
#' @keywords utilities
#' @keywords visualization
#' @references Adams, D. C., and M. M. Cerney. 2007. 
#' Quantifying biomechanical motion using Procrustes 
#'   motion analysis. J. Biomech. 40:437-444.
#' @references Adams, D. C., and M. L. Collyer. 2007. 
#' The analysis of character divergence along environmental 
#'   gradients and other covariates. Evolution 61:510-515.
#' @references Adams, D. C., and M. L. Collyer. 2009. 
#' A general framework for the analysis of phenotypic 
#'   trajectories in evolutionary studies. Evolution 63:1143-1154.
#' @references Collyer, M. L., and D. C. Adams. 2007. 
#' Analysis of two-state multivariate phenotypic change 
#'   in ecological studies. Ecology 88:683-692.
#' @references Collyer, M. L., and D. C. Adams. 2013. 
#' Phenotypic trajectory analysis: comparison of shape change patterns 
#' in evolution and ecology. Hystrix 24: 75-83.
#' @references Collyer, M.L., D.J. Sekora, and D.C. Adams. 2015. 
#' A method for analysis of phenotypic change for phenotypes described 
#' by high-dimensional data. Heredity. 115:357-365.
#' 
#' @examples 
#' # See trajectory.analysis help file for examples
plot.trajectory.analysis <- function(x, ...) {
  
  if(!is.null(x$pca)) {
    pca <- x$pca
    rot <- pca$rotation
    Y <- x$fit$LM$Y
    props <- pca$sdev^2 / sum(pca$sdev^2)
    pc.points <- center(Y) %*% rot
    trajectories <- x$trajectories[[1]]
  }
  
  if(is.null(x$pca) && x$type == "factorial") {
    f <- if(x$fit$LM$gls) x$fit$LM$gls.fitted else x$fit$LM$fitted
    pca <- prcomp(f)
    rot <- pca$rotation
    Y <- x$fit$LM$Y
    Y.cent <- colMeans(Y)
    props <- pca$sdev^2 / sum(pca$sdev^2)
    pc.points <- center(Y) %*% rot
    trajectories <- x$trajectories[[1]]
    if(is.matrix(trajectories)) trajectories <- list(trajectories)
    traj.c <- matrix(Y.cent, NROW(trajectories[[1]]), 
                     NCOL(trajectories[[1]]), byrow = TRUE)
    trajectories <- lapply(trajectories, function(x) (x - traj.c) %*% rot)
  }
  
  if(x$type == "single.factor") {
    f <- if(x$fit$LM$gls) x$fit$LM$gls.fitted else x$fit$LM$fitted
    tp <- x$n.points
    p <- NCOL(f)/tp
    n <- NROW(f)
    ft <- array(f, c(n, p, tp))
    ft2 <- ft[,,1]
    for(i in 2:tp) ft2 <- rbind(ft2, ft[,,i])
    pca <- prcomp(ft2)
    rot <- pca$rotation
    Y <- x$fit$LM$Y
    Y2 <- array(Y, c(n, p, tp))
    Y <- Y2[,,1]
    for(i in 2:tp) Y <- rbind(Y, Y2[,,i])
    Y.cent <- colMeans(Y)
    props <- pca$sdev^2 / sum(pca$sdev^2)
    pc.points <- center(Y) %*% rot
    trajectories <- x$trajectories[[1]]
    if(is.matrix(trajectories)) trajectories <- list(trajectories)
    traj.c <- matrix(Y.cent, NROW(trajectories[[1]]), 
                     NCOL(trajectories[[1]]), byrow = TRUE)
    trajectories <- lapply(trajectories, function(x) (x - traj.c) %*% rot)
  }
  
  
  dots <- list(...)
  if(is.null(dots$xlab))
    xlabel <- paste("PC 1 for fitted values: ", 
                    round(props[1] *100, 2), "%", sep = "")
  if(is.null(dots$ylab))
    ylabel <- paste("PC 2 for fitted values: ", 
                    round(props[2] *100, 2), "%", sep = "")
  
  if(!is.null(dots$xlab) && !is.null(dots$ylab)) 
    plot(pc.points[,1], pc.points[,2], asp = 1, ...)
  if(!is.null(dots$xlab) && is.null(dots$ylab)) 
    plot(pc.points[,1], pc.points[,2], asp = 1, ylab = ylabel, ...)
  if(is.null(dots$xlab) && !is.null(dots$ylab)) 
    plot(pc.points[,1], pc.points[,2], asp = 1, xlab = xlabel, ...)
  if(is.null(dots$xlab) && is.null(dots$ylab))
    plot(pc.points[,1], pc.points[,2], asp = 1, xlab = xlabel, ylab = ylabel, ...)
  out <- list(pca = pca, pc.points = pc.points, 
              trajectoy.analysis = x, trajectories = trajectories)
  invisible(out)
}

#' Plot Function for RRPP
#' 
#'  Function adds trajectories to a principal component plot
#'
#'  The function adds trajectories to a plot made by 
#'  \code{\link{plot.trajectory.analysis}}.
#'  This function has a restricted set of plot parameters 
#'  based on the number of trajectories
#'  to be added to the plot.
#'  
#' @param TP plot object (from \code{\link{plot.trajectory.analysis}})
#' @param traj.pch Plotting "character" for trajectory points.  
#' Can be a single value or vector 
#' of length equal to the number of trajectories.  
#' See \code{\link{par}} and its description 
#' for pch.
#' @param traj.col The color of trajectory lines.  
#' Can be a single value or vector 
#' of length equal to the number of trajectories.  
#' See \code{\link{par}} and its description 
#' for col.
#' @param traj.lty Trajectory line type.  Can be a single value or vector 
#' of length equal to the number of trajectories.  
#' See \code{\link{par}} and its description 
#' for lty.
#' @param traj.lwd Trajectory line width.  Can be a single value or vector 
#' of length equal to the number of trajectories.  
#' See \code{\link{par}} and its description 
#' for lwd.
#' @param traj.cex Trajectory point character expansion.  Can be a single value or vector 
#' of length equal to the number of trajectories.  
#' See \code{\link{par}} and its description 
#' for cex.
#' @param traj.bg Trajectory point background.  Can be a single value or vector 
#' of length equal to the number of trajectories.  
#' See \code{\link{par}} and its description 
#' for bg.
#' @param start.bg Trajectory point background, just the start points.  
#' Can be a single value or vector 
#' of length equal to the number of trajectories.  
#' See \code{\link{par}} and its description 
#' for bg.  Green start points are the default.
#' @param end.bg Trajectory point background, just the end points.  
#' Can be a single value or vector 
#' of length equal to the number of trajectories.  
#' See \code{\link{par}} and its description 
#' for bg.  Red end points are the default.
#' 
#' @seealso 
#' \code{\link{plot.default}} and \code{\link{par}}
#' @export
#' @author Michael Collyer
#' @keywords utilities
#' @keywords visualization
#' @references Adams, D. C., and M. M. Cerney. 2007. 
#' Quantifying biomechanical motion using Procrustes 
#'   motion analysis. J. Biomech. 40:437-444.
#' @references Adams, D. C., and M. L. Collyer. 2007. 
#' The analysis of character divergence along environmental 
#'   gradients and other covariates. Evolution 61:510-515.
#' @references Adams, D. C., and M. L. Collyer. 2009. 
#' A general framework for the analysis of phenotypic 
#'   trajectories in evolutionary studies. Evolution 63:1143-1154.
#' @references Collyer, M. L., and D. C. Adams. 2007. 
#' Analysis of two-state multivariate phenotypic change 
#'   in ecological studies. Ecology 88:683-692.
#' @references Collyer, M. L., and D. C. Adams. 2013. 
#' Phenotypic trajectory analysis: comparison of shape change patterns 
#' in evolution and ecology. Hystrix 24: 75-83.
#' @references Collyer, M.L., D.J. Sekora, and D.C. Adams. 2015. 
#' A method for analysis of phenotypic change for phenotypes described 
#' by high-dimensional data. Heredity. 115:357-365.
add.trajectories <- function(TP, 
                             traj.pch = 21,
                             traj.col = 1,
                             traj.lty = 1,
                             traj.lwd = 1,
                             traj.cex = 1.5,
                             traj.bg = 1,
                             start.bg = 3,
                             end.bg = 2) {
  
  traj <- TP$trajectories
  nt <- length(traj)
  np <- NROW(traj[[1]])
  
  if(length(traj.pch) != 1 && length(traj.pch) != nt)
    stop("For add.trajectories, traj.pch must be equal in length 
         to the number of trajectories or just one value\n",
         call. = FALSE) else if(length(traj.pch) == 1) 
           traj.pch <- rep(traj.pch, nt)
  
  if(length(traj.col) != 1 && length(traj.col) != nt)
    stop("For add.trajectories, traj.col must be equal in length 
         to the number of trajectories or just one value\n",
         call. = FALSE) else if(length(traj.col) == 1) 
           traj.col <- rep(traj.col, nt)
  
  if(length(traj.lty) != 1 && length(traj.lty) != nt)
    stop("For add.trajectories, traj.lty must be equal in length 
         to the number of trajectories or just one value\n",
         call. = FALSE) else if(length(traj.lty) == 1) 
           traj.lty <- rep(traj.lty, nt)
  
  if(length(traj.lwd) != 1 && length(traj.lwd) != nt)
    stop("For add.trajectories, traj.lwd must be equal in length 
         to the number of trajectories or just one value\n",
         call. = FALSE) else if(length(traj.lwd) == 1) 
           traj.lwd <- rep(traj.lwd, nt)
  
  if(length(traj.cex) != 1 && length(traj.cex) != nt)
    stop("For add.trajectories, traj.cex must be equal in length 
         to the number of trajectories or just one value\n",
         call. = FALSE) else if(length(traj.cex) == 1) 
           traj.cex <- rep(traj.cex, nt)
  
  if(length(traj.bg) != 1 && length(traj.bg) != nt)
    stop("For add.trajectories, traj.bg must be equal in length 
         to the number of trajectories or just one value\n",
         call. = FALSE) else if(length(traj.bg) == 1) 
           traj.bg <- rep(traj.bg, nt)
  
  if(length(start.bg) != 1 && length(start.bg) != nt)
    stop("For add.trajectories, start.bg must be equal in length 
         to the number of trajectories or just one value\n",
         call. = FALSE) else if(length(start.bg) == 1) 
           start.bg<- rep(start.bg, nt)
  
  if(length(end.bg) != 1 && length(end.bg) != nt)
    stop("For add.trajectories, end.bg must be equal in length 
         to the number of trajectories or just one value\n",
         call. = FALSE) else if(length(end.bg) == 1) 
           end.bg<- rep(end.bg, nt)     
  
  for(i in 1:nt){
    x <- traj[[i]][,1]
    y <- traj[[i]][,2]
    lines(x, y, col = traj.col[i], lwd = traj.lwd[i], lty = traj.lty[i])
    points(x, y, col = 1, pch = traj.pch[i], 
           lwd = traj.lwd[i], cex = traj.cex[i], bg = traj.bg[i])
    points(x[1], y[1], col = 1, pch = traj.pch[i], 
           lwd = traj.lwd[i], cex = traj.cex[i], bg = start.bg[i])
    points(x[np], y[np], col = 1, pch = traj.pch[i], 
           lwd = traj.lwd[i], cex = traj.cex[i], bg = end.bg[i])
  }
  
}

# ordinate

#' Print/Summary Function for RRPP
#'
#' @param x Object from \code{\link{ordinate}}
#' @param ... Other arguments passed onto print.ordinate
#' @method print ordinate
#' @export
#' @author Michael Collyer
#' @keywords utilities
#' 
print.ordinate <- function(x, ...){
  ord.type <- if(x$alignment == "principal") 
    "Principal Component Analysis" else
    "Alignment to an alternative matrix"
  cat("\nOrdination type:", ord.type, "\n")
  if(x$alignment != "principal")
    cat("Alignment matrix:", x$alignment, "\n")
  cen <- if(x$GLS) "GLS" else "OLS"
  cat("Centering by", cen, "mean\n")
  
  if(x$alignment == "principal") {
    if(x$GLS && x$transform)
      cat("GLS residuals transformed for orthogonal projection\n") else
        if(x$GLS) cat("Oblique projection of GLS-centered residuals\n") else
          cat("Orthogonal projection of OLS residuals\n")
  } else {
    if(x$GLS && x$transform)
      cat("GLS residuals transformed\n") else
        if(x$GLS) cat("GLS-centered residuals, not transformed\n") else
          cat("OLS residuals\n")
    cat("Alignment to ", x$alignment, 
        "means residual projection is not orthogonal.\n")
  }
  
  cat("Number of observations:", NROW(x$x), "\n")
  cat("Number of vectors", NCOL(x$x), "\n\n")
} 

#' Print/Summary Function for RRPP
#'
#' @param object Object from \code{\link{ordinate}}
#' @param ... Other arguments passed onto print.ordinate
#' @method summary ordinate
#' @export
#' @author Michael Collyer
#' @keywords utilities
#' 
summary.ordinate <- function(object, ...){
  x <- object
  print.ordinate(x, ...)
  d <- x$d
  p <- d/sum(d)
  cp <- cumsum(d)/sum(d)
  r <- as.data.frame(rbind(d, p, cp))
  r <- r[, 1:min(length(d), NCOL(x$x), NCOL(r))]
  colnames(r) <- colnames(x$x)[1:NCOL(r)]
  rownames(r) <- c("Singular Value", 
                   "Proportion of Covariance", "Cumulative Proportion")
  
  if(x$alignment == "principal") rownames(r)[1:2] <- c("Eigenvalues", 
                                                       "Proportion of Variance")
  
  if(x$alignment != "principal") {
    rv <- x$RV
    r <- rbind(r, rv, cumsum(rv))
    rownames(r)[4:5] <- c("RV by Component", "Cumulative RV")
  }

  cat("Importance of Components:\n")
  print(r)
  out <- r
  invisible(out)
}


#' Print/Summary Function for RRPP
#'
#' @param x Object from \code{\link{summary.ordinate}}
#' @param ... Other arguments passed onto print.ordinate
#' @method print summary.ordinate
#' @export
#' @author Michael Collyer
#' @keywords utilities
#' 
print.summary.ordinate <- function(x, ...){
  cat("\nImportance of Components:\n")
  print(x$table)
} 

#' Plot Function for RRPP
#' 
#' @param x An object of class \code{\link{ordinate}}
#' @param axis1 A value indicating which component should be 
#' displayed as the X-axis (default = C1)
#' @param axis2 A value indicating which component should be 
#' displayed as the Y-axis (default = C2)
#' @param flip An argument that if not NULL can be used to flip 
#' components in the plot.  
#' The values need to match axis1 or axis2.  For example, if axis1 = 3 
#' and axis2 = 4, flip = 1 will not
#' change either axis; flip = 3 will flip only the horizontal axis; 
#' flip = c(3, 4) will flip both axes.
#' @param include.axes A logical argument for whether axes should be shown at x = 0 and y = 0.  
#' This is different than the axes argument in the generic \code{\link{plot.default}} function, which
#' controls the edges of the plot (providing a box effect or not).  Using include.axes = TRUE does not 
#' allow aesthetic control of the axes.  If desired, it is better to use include.axes = FALSE and augment
#' the plot object with \code{\link{abline}} (choosing h = 0 and v = 0 in separate applications).
#' @param ... other arguments passed to plot (helpful to employ
#' different colors or symbols for different groups).  See
#' @return An object of class "plot.ordinate" is a list with components
#'  that can be used in other plot functions, such as the type of plot, points, 
#'  a group factor, and other information depending on the plot parameters used.
#' @method plot ordinate
#' @export
#' @author Michael Collyer
#' @keywords utilities
#' @keywords visualization
plot.ordinate <- function(x, axis1 = 1, axis2 = 2, flip = NULL, 
                          include.axes = TRUE, ...) {
  wrn <- options()$warn
  options(warn = -1)
  if(NCOL(x$x) == 1) 
    stop("Only one component.  No plotting capability with this function.\n", 
                          call. = FALSE)
  v <- x$d/sum(x$d)
  if(!is.null(x$RV)) rv <- x$RV
  
  pcdata <- x$x[, c(axis1, axis2)]
  if(!is.null(flip)) {
    if(length(flip) > 2) flip <- flip[1:2]
    flip <- flip[(flip %in% 1:ncol(pcdata))]
    if(length(flip > 0)) pcdata[, flip] <- pcdata[, flip] * -1
  }
  
  plot_args <- list(...)
  plot_args$x = pcdata[,1]
  plot_args$y = pcdata[,2]
  
  if(x$alignment == "principal") {
    xlabel <- paste("PC ", axis1, ": ", round(v[axis1] * 100, 2), "%", sep = "")
    ylabel <- paste("PC ", axis2, ": ", round(v[axis2] * 100, 2), "%", sep = "")
  } else {
    xlabel <- paste("C ", axis1,  sep = "")
    ylabel <- paste("C ", axis2,  sep = "")
  }

  if(is.null(plot_args$xlab)) plot_args$xlab <- xlabel
  if(is.null(plot_args$ylab)) plot_args$ylab <- ylabel
  if(is.null(plot_args$xlim)) plot_args$xlim <- 1.05*range(plot_args$x)
  if(is.null(plot_args$ylim)) plot_args$ylim <- 1.05*range(plot_args$y)
  if(is.null(plot_args$asp)) plot_args$asp <- 1
  
  do.call(plot.default, plot_args)
  
  if(include.axes){
    abline(h = 0, lty=2, ...)
    abline(v = 0, lty=2, ...)
  }
  
  out <- list(points = pcdata,   
              call = match.call())
  
  out$plot_args <- plot_args
  class(out) <- "plot.ordinate"
  options(warn = wrn)
  invisible(out)
  
}

#' Print/Summary Function for RRPP
#'
#' @param x Object from \code{\link{kcomp}}
#' @param ... Other arguments passed onto print.kcomp
#' @method print kcomp
#' @export
#' @author Michael Collyer
#' @keywords utilities
#' 
print.kcomp <- function(x, ...){
  cat("\nK-component analysis\n")
  cat("Components = ", length(x$values))
} 

#' Print/Summary Function for RRPP
#'
#' @param object Object from \code{\link{kcomp}}
#' @param ... Other arguments passed onto print.kcomp
#' @method summary kcomp
#' @export
#' @author Michael Collyer
#' @keywords utilities
#' 
summary.kcomp <- function(object, ...){
  x <- object
  print.kcomp(x, ...)
  d <- x$values
  p <- d/sum(d)
  cp <- cumsum(d)/sum(d)
  r <- as.data.frame(rbind(d, p, cp))
  r <- r[, 1:min(length(d), NCOL(x$vectors), NCOL(r))]
  r <- as.matrix(r)
  colnames(r) <- colnames(x$vectors)[1:NCOL(r)]

  rownames(r) <- c("Eigenvalues", "Proportion of Eigenvalue Sum",
                        "Cumulative Proportion")
  cat("\n\nImportance of Components:\n")
  print(r)
  out <- r
  invisible(out)
}


#' Plot Function for RRPP
#' 
#' @param x An object of class \code{\link{kcomp}}
#' @param axis1 A value indicating which component should be 
#' displayed as the X-axis (default = K1)
#' @param axis2 A value indicating which component should be 
#' displayed as the Y-axis (default = K2)
#' @param flip An argument that if not NULL can be used to flip 
#' components in the plot.  
#' The values need to match axis1 or axis2.  For example, if axis1 = 3 
#' and axis2 = 4, flip = 1 will not
#' change either axis; flip = 3 will flip only the horizontal axis; 
#' flip = c(3, 4) will flip both axes.
#' @param include.axes A logical argument for whether axes should be shown at x = 0 and y = 0.  
#' This is different than the axes argument in the generic \code{\link{plot.default}} function, which
#' controls the edges of the plot (providing a box effect or not).  Using include.axes = TRUE does not 
#' allow aesthetic control of the axes.  If desired, it is better to use include.axes = FALSE and augment
#' the plot object with \code{\link{abline}} (choosing h = 0 and v = 0 in separate applications).
#' @param ... other arguments passed to plot (helpful to employ
#' different colors or symbols for different groups).  See
#' @return An object of class "plot.kcomp" is a list with components
#'  that can be used in other plot functions, such as the type of plot, points, 
#'  a group factor, and other information depending on the plot parameters used.
#' @method plot kcomp
#' @export
#' @author Michael Collyer
#' @keywords utilities
#' @keywords visualization
plot.kcomp <- function(x, axis1 = 1, axis2 = 2, flip = NULL, 
                          include.axes = TRUE, ...) {
  wrn <- options()$warn
  options(warn = -1)
  if(NCOL(x$scores) == 1) 
    stop("Only one component.  No plotting capability with this function.\n", 
         call. = FALSE)
  v <- x$values/sum(x$values)
  
  kcdata <- x$scores[, c(axis1, axis2)]
  if(!is.null(flip)) {
    if(length(flip) > 2) flip <- flip[1:2]
    flip <- flip[(flip %in% 1:ncol(kcdata))]
    if(length(flip > 0)) kcdata[, flip] <- kcdata[, flip] * -1
  }
  
  plot_args <- list(...)
  plot_args$x = kcdata[,1]
  plot_args$y = kcdata[,2]
  
  xlabel <- paste("KC ", axis1, ": ", 
                  round(v[axis1] * 100, 2), "%", sep = "")
  ylabel <- paste("KC ", axis2, ": ", 
                  round(v[axis2] * 100, 2), "%", sep = "")
  
 

  if(is.null(plot_args$xlab)) plot_args$xlab <- xlabel
  if(is.null(plot_args$ylab)) plot_args$ylab <- ylabel
  if(is.null(plot_args$xlim)) plot_args$xlim <- 1.05*range(plot_args$x)
  if(is.null(plot_args$ylim)) plot_args$ylim <- 1.05*range(plot_args$y)
  if(is.null(plot_args$asp)) plot_args$asp <- 1
  
  do.call(plot.default, plot_args)
  
  if(include.axes){
    abline(h = 0, lty=2, ...)
    abline(v = 0, lty=2, ...)
  }
  
  out <- list(points = kcdata,   
              call = match.call())
  
  out$plot_args <- plot_args
  class(out) <- "plot.kcomp"
  options(warn = wrn)
  invisible(out)
  
}

# looCV

#' Print/Summary Function for RRPP
#'
#' @param x Object from \code{\link{looCV}}
#' @param ... Other arguments passed onto print.looCV
#' @method print looCV
#' @export
#' @author Michael Collyer
#' @keywords utilities
#' 
print.looCV <- function(x, ...){
  
  cat("Cross-validated results for", x$model, "\n")
  
  if(x$method == "PC"){
    
    cat("Results include cross-validated scores for", length(x$d$cv), "components,\n")
    cat("based on", NROW(x$scores$cv), "observations.\n\n")
    cat("Cross-validated scores should not be used as data in subsequent analyses.\n")

      } else {
    
    cat("Results include cross-validated fitted values, residuals,\n")
    cat("and RSS and MSE statistics\n")
    
  }
  
} 

#' Print/Summary Function for RRPP
#'
#' @param object Object from \code{\link{looCV}}
#' @param ... Other arguments passed onto print.looCV
#' @method summary looCV
#' @export
#' @author Michael Collyer
#' @keywords utilities
#' 
summary.looCV <- function(object, ...){
  
  x <- object
  print.looCV(x, ...)
  
  if(x$method == "PC"){
    
    dobs <- x$d$obs
    dcv <- x$d$cv
    pobs <- dobs/sum(dobs)
    cpobs <- cumsum(dobs)/sum(dobs)
    pcv <- dcv/sum(dcv)
    cpcv <- cumsum(dcv)/sum(dcv)
    
    robs <- as.data.frame(rbind(dobs, pobs, cpobs))
    robs <- robs[, 1:min(length(dobs), NCOL(x$scores$obs), NCOL(robs))]
    
    rcv <- as.data.frame(rbind(dcv, pcv, cpcv))
    rcv <- rcv[, 1:min(length(dcv), NCOL(x$scores$cv), NCOL(rcv))]
    
    colnames(robs) <- colnames(rcv) <- colnames(x$x)[1:NCOL(robs)]
    rownames(robs) <- rownames(rcv) <- c("Eigenvalue", 
                                         "Proportion of Variance", "Cumulative Proportion")
    
    cat("\nObserved eigenvalues")
    print(robs)
    cat("\nCross-validated eigenvalues")
    print(rcv)
  } else {
    
    cat("\n")
    tab <- as.table(matrix(
      c(x$RSS, x$MSE, x$cv_RSS, x$cv_MSE), 2, 2))
    dimnames(tab) <- list(c("RSS", "MSE"),
                          c("Observed", "Cross-validated"))
    print(tab)
    
  }
  
  
}


#' Plot Function for RRPP
#' 
#' @param x An object of class \code{\link{looCV}}
#' @param axis1 A value indicating which component should be 
#' displayed as the X-axis (default = C1)
#' @param axis2 A value indicating which component should be 
#' displayed as the Y-axis (default = C2)
#' @param flip An argument that if not NULL can be used to flip 
#' components in the plot.  
#' The values need to match axis1 or axis2.  For example, if axis1 = 3 
#' and axis2 = 4, flip = 1 will not
#' change either axis; flip = 3 will flip only the horizontal axis; 
#' flip = c(3, 4) will flip both axes.  Axis will only be flipped in first
#' plot.
#' @param ... other arguments passed to plot (helpful to employ
#' different colors or symbols for different groups).  See
#' @method plot looCV
#' @export
#' @author Michael Collyer
#' @keywords utilities
#' @keywords visualization
plot.looCV<- function(x, axis1 = 1, axis2 = 2, 
                      flip = NULL, ...) {
  owarn <- options()$warn
  options(warn = -1)
  if(NCOL(x$scores$obs) == 1) 
    stop("Only one component.  No plotting capability with this function.\n", 
         call. = FALSE)
  opars <- par()
  plot_args <- list(...)
  
  if(axis1 > length(x$d$obs) || axis2 > length(x$d$obs))
    stop("Choice of at least one axis exceeds total axes possible.\n",
         call. = FALSE)
  
  if(is.null(x$d))
    stop("Plotting for class looCV requires ordination scores (change method argument).\n",
         call. = FALSE)
  
  par(mfrow = c(1, 3))
  
  pcdata <- x$scores$obs[, c(axis1, axis2)]
  
  if(!is.null(flip)) {
    if(length(flip) > 2) flip <- flip[1:2]
    flip <- flip[(flip %in% 1:ncol(pcdata))]
    if(length(flip > 0)) pcdata[, flip] <- pcdata[, flip] * -1
  }
  
  plot_args$main <- NULL
  plot_args$x <- pcdata[, 1]
  plot_args$y <- pcdata[, 2]
  plot_args$xlab <- paste("PC", axis1, "for fitted values:", 
                          round(x$d$obs[axis1]/sum(x$d$obs) * 100, 2),
                          "%")
  plot_args$ylab <- paste("PC", axis2, "for fitted values:", 
                          round(x$d$obs[axis2]/sum(x$d$obs) * 100, 2),
                          "%")
  do.call(plot, plot_args)
  abline(h = 0, lty = 3)
  abline(v = 0, lty = 3)
  title("Observed PC values")
  
  plot_args$x <- as.matrix(x$scores$cv)[, axis1]
  plot_args$y <- as.matrix(x$scores$cv)[, axis2]
  plot_args$xlab <- paste("PC", axis1, "for fitted values:", 
                          round(x$d$cv[axis1]/sum(x$d$cv) * 100, 2),
                          "%")
  plot_args$ylab <- paste("PC", axis2, "for fitted values:", 
                          round(x$d$cv[axis2]/sum(x$d$cv) * 100, 2),
                          "%")
  do.call(plot, plot_args)
  abline(h = 0, lty = 3)
  abline(v = 0, lty = 3)
  title("Cross-validated PC values")
  
  k <- seq(1, min(c(length(x$d$obs), length(x$d$cv))))
  plot_args$x <-  x$d$obs[k]
  plot_args$y <- x$d$cv[k]
  plot_args$xlab <- "Observed eigenvalues"
  plot_args$ylab <- "Cross-validated eigenvalues"
  plot_args$pch <- 19
  plot_args$cex = 1
  plot_args$col = 1
  emax <- max(c(x$d$obs, x$d$cv))
  plot_args$xlim <- c(0, emax)
  plot_args$ylim <- c(0, emax)
  plot_args$asp <- 1
  do.call(plot, plot_args)
  title("Values close to 1:1 imply robust observed scores", 
        cex.main = 0.6)
  abline(0, 1, lty = 3)
 
  options(warn = owarn)
  par(opars)
}

#' Print/Summary Function for RRPP
#'
#' @param x Object from \code{\link{measurement.error}}
#' @param ... Other arguments passed onto measurement.error
#' @method print measurement.error
#' @export
#' @author Michael Collyer
#' @keywords utilities
#' @examples
#' \dontrun{
#' 
#' # Measurement error analysis on simulated data of fish shapes
#' 
#' data(fishy)
#' 
#' # Analysis unconcerned with groups 
#' 
#' ME1 <- measurement.error(
#'   Y = "coords",
#'   subjects = "subj",
#'   replicates = "reps",
#'   data = fishy)
#' 
#' anova(ME1)
#' ICCstats(ME1, subjects = "Subjects", with_in = "Systematic ME")
#' plot(ME1)
#' 
#' # Analysis concerned with groups 
#' 
#' ME2 <- measurement.error(
#'   Y = "coords",
#'   subjects = "subj",
#'   replicates = "reps",
#'   groups = "groups",
#'   data = fishy)
#' 
#' anova(ME2)
#' ICCstats(ME2, subjects = "Subjects", 
#'   with_in = "Systematic ME", groups = "groups")
#' P <- plot(ME2)
#' focusMEonSubjects(P, subjects = 18:20, shadow = TRUE)
#' }
print.measurement.error <- function(x, ...){
  
  print(anova(x))
  
  ev <- Re(eigen(x$SSCP.ME.product)$values)
  ev <- ev[zapsmall(ev) > 0]
  
  cat("\n\nNumber of SNR components (eigenvectors):", length(ev), "\n\n")
  cat("Use plot.measurement.error to visualize results\n\n")
  cat("Use function, ICCstats, to get ICC statistics.\n\n")
  
}

#' Print/Summary Function for RRPP
#'
#' @param object Object from \code{\link{measurement.error}}
#' @param ... Other arguments passed onto measurement.error
#' @method summary measurement.error
#' @export
#' @author Michael Collyer
#' @keywords utilities
summary.measurement.error <- function(object, ...){
  
  print.measurement.error(object)
  
}

#' Plot Function for RRPP
#' 
#' This function produces multivariate signal-to-noise ratio plots for  
#' \code{\link{measurement.error}} objects.  See the function, 
#' \code{\link{plot.interSubVar}} for plotting the inter-subject variability
#' from a \code{\link{measurement.error}} object, after applying the function, 
#' \code{\link{interSubVar}}.
#'
#' @param x Object from \code{\link{measurement.error}}
#' @param separate.by.groups A logical value for whether to make separate plots
#' for each group, if different groups are available.  If FALSE, groups
#' are still represented by different symbols in the plot, unless overridden
#' by plot arguments.
#' @param add.connectors A logical value for whether to add connectors, like
#' vectors, between replicate observations of the same subjects.  
#' @param titles An optional vector or list for augmenting the titles of plots produced.
#' The length of the vector or list should match the number of plots produced by 
#' other arguments.
#' @param add.labels A logical value for whether to label subjects.  
#' Labels are either subject name (if available) or number of occurrence
#' in the data set.
#' @param use.std.vectors A logical value for whether to use vectors obtained from
#' a standardized matrix, which are orthogonal.  This is not strictly necessary.
#' @param add.legend A logical value for whether to add a legend to plots.  If
#' separate.by.groups is TRUE, adding a legend to plots will be slightly redundant. If
#' certain parameters are augmented by user (point characters, colors), add.legend
#' will be made to be FALSE to prevent misinterpretation of intended plotting scheme.
#' @param ... Other arguments passed onto plot
#' @method plot measurement.error
#' @export
#' @author Michael Collyer
#' @keywords utilities
plot.measurement.error <- function(x, 
                                   separate.by.groups = TRUE,
                                   add.connectors = TRUE,
                                   add.labels = FALSE,
                                   use.std.vectors = FALSE,
                                   titles = NULL,
                                   add.legend = TRUE, ...){
  
  subj <- x$LM$data$subjects
  reps <- x$LM$data$replicates
  groups <- if(!is.null(x$LM$data$groups))
    as.factor(x$LM$data$groups) else NULL
  
  
  plot_args <- list(...)
  if(!is.null(titles)) titles <- as.list(titles)
  
  if(is.null(plot_args$pch)) {
    plot_args$pch <- 20 
    if(length(x$LM$data) > 3) {
      plot_args$pch <- plot_args$pch + as.numeric(groups)
      ldf.pch <- unique(data.frame(a = groups, 
                                  b = plot_args$pch))
    } else {
      plot_args$pch <- plot_args$pch + 1
      ldf.pch <- data.frame(a = 1, b = plot_args$pch)
    }
  } else {
    if(length(x$LM$data) > 3) {
      ldf.pch <- unique(data.frame(a = groups, 
                                  b = plot_args$pch))
    } else {
      ldf.pch <-unique(data.frame(a = 1, b = plot_args$pch))
    }
  }
  
  if(is.null(plot_args$bg)) plot_args$bg <- as.numeric(reps)
  ldf.bg <- unique(data.frame(a = reps, b = plot_args$bg))
  
  if(is.null(plot_args$col)) plot_args$col <- as.numeric(reps)
  ldf.col <- unique(data.frame(a = reps, b = plot_args$col))
  
  
  if(!is.null(x$SSCP.ME.product)) {
    S <- if(use.std.vectors) svd(x$SSCP.ME.product.std) else
      svd(x$SSCP.ME.product)
  } else S <- list(d = 1, u = 1, v = 1)
  
  Y <- center(x$LM$Y)
  if(length(x$LM$data) > 3){
    gp <- x$LM$data$groups
    Y <- resid(lm(Y ~ gp))
  }
    
  d <- S$d
  dx <- d[1]/sum(d)
  dy <- if(length(d) > 1) d[2]/sum(d) else 0
  
  pts <- as.matrix(Y) %*% S[[3]] %*% 
    diag(sqrt(S[[1]]))[, 1:(min(length(d), 2))]
  
  if(NCOL(pts) == 1) pts <- cbind(pts, 0)
  
  plot_args$asp <- 1
  if(is.null(plot_args$xlab))
    plot_args$xlab <- paste("EV 1:", round(dx *100, 2), "%")
  if(is.null(plot_args$ylab))
    plot_args$ylab <- paste("EV 2:", round(dy *100, 2), "%")
  if(is.null(plot_args$main))
    plot_args$main <- "Systematic ME / Random ME"
  if(!is.null(titles))
    plot_args$main <- titles[[1]]
  if(is.null(plot_args$cex.main))
    plot_args$cex.main <- 1
  
  add.me.connectors <- function(x, y, subj, d, maxy) {
    
    dindx <- which(zapsmall(d) > 0)
    if(length(dindx) == 1) {
      y <- y + 0.1 * maxy
      subjjig <- seq(0, 0.9 * maxy, (0.9 * maxy) / (nlevels(subj) - 1))
      for(i in 1:length(subjjig)) {
        y[which(subj == levels(subj)[i])] <- 
          y[which(subj == levels(subj)[i])] + subjjig[i]
      }
    }
    subjlv <- levels(subj)
    for(i in 1:length(subjlv)){
      xx <- x[subj == subjlv[i]]
      yy <- y[subj == subjlv[i]]
      xmean <- mean(xx)
      ymean <- mean(yy)
      for(j in 1:length(xx)) {
        arrows(xmean, ymean, xx[j], yy[j], length = 0, 
               angle = 0, lwd = 0.5, ...)
        
      }
    }
  }
  
  add.me.labels <- function(x, y, subj, d, maxy) {
    dindx <- which(zapsmall(d) > 0)
    if(length(dindx) == 1) {
      y <- y + 0.1 * maxy
      subjjig <- seq(0, 0.9 * maxy, (0.9 * maxy) / (nlevels(subj) - 1))
      for(i in 1:length(subjjig)) {
        y[which(subj == levels(subj)[i])] <- 
          y[which(subj == levels(subj)[i])] + subjjig[i]
      }
    }
    subjlv <- levels(subj)
    for(i in 1:length(subjlv)){
      xx <- x[subj == subjlv[i]]
      yy <- y[subj == subjlv[i]]
      xmean <- mean(xx)
      ymean <- mean(yy)
      for(j in 1:length(xx)) {
        text(xmean, ymean, subjlv[i], pos = 3, cex = 0.4, offset = 0.1)
      }
    }
  }
  
  plot_args$x <- as.matrix(pts)[,1]
  plot_args$y <- if(NCOL(as.matrix(pts)) == 1) 
    rep(0, length(pts)) else
      as.matrix(pts)[,2]
  if(is.null(plot_args$ylim)) {
    ymax <- max(c(plot_args$y, max(plot_args$x)))
    ymin <- min(plot_args$y)
    plot_args$ylim <- c(ymin, ymax)
  }
  
  if(separate.by.groups && is.null(x$LM$data$groups))
    separate.by.groups <- FALSE
  
  if(separate.by.groups) {
    nplots <- nlevels(x$LM$data$groups)
    point_args <- plot_args
    plot_args$type <- "n"
    plot_args$main <- NULL
    gps <- levels(x$LM$data$groups)
    if(!is.null(titles)) {
      if(length(titles) != nplots) {
        if(length(titles) > nplots)
          titles <- titles[1:nplots] else
            titles <- rep(titles, nplots)[1:nplots]
      }
    }
    
    for(i in 1:nplots) {
      do.call(plot, plot_args)
      if(is.null(titles)) {
        title(paste("Systematic ME / Random ME for", 
                    "Group", gps[[i]], sep = " "),
              cex.main = plot_args$cex.main)
      } else {
        title(titles[[i]], cex.main = plot_args$cex.main)
      }
      
      point_args.b <- point_args
      point_args.b$pch[which(x$LM$data$groups != gps[[i]])] <- NA
      
      do.call(points, point_args.b)
      
      if(add.connectors) add.me.connectors(
        point_args.b$x[which(!is.na(point_args.b$pch))],
        point_args.b$y[which(!is.na(point_args.b$pch))],
        factor(subj[which(!is.na(point_args.b$pch))]), 
        d, point_args.b$ylim[2])
      if(add.labels) add.me.labels(
        point_args.b$x[which(!is.na(point_args.b$pch))],
        point_args.b$y[which(!is.na(point_args.b$pch))],
        factor(subj[which(!is.na(point_args.b$pch))]), 
        d, point_args.b$ylim[2])
      
      if(add.legend) {
        
        legend("topleft", 
               legend = ldf.col$a,
               pch = unique(na.omit(point_args.b$pch)),
               col = ldf.col$b,
               pt.bg = ldf.bg$b,
               bg = "white")
      }
    }
    
  } else {
    do.call(plot, plot_args)
    
    if(add.connectors)
      add.me.connectors(plot_args$x, plot_args$y, subj, d, plot_args$ylim[2])
    if(add.labels)
      add.me.labels(plot_args$x, plot_args$y, subj, d, plot_args$ylim[2])
    
    if(add.legend) {
      legend("topleft", 
             legend = ldf.col$a,
             pch = min(ldf.pch$b),
             col = ldf.col$b,
             pt.bg = ldf.bg$b,
             bg = "white")

    }
  }
  
  plot_args$data <- x$LM$data
  class(plot_args) <- "plot.measurement.error"
  invisible(plot_args)
  
}

#' Plot Function for RRPP
#' 
#' This function produces a heat map for inter-subject variability, based on 
#' results from a \code{\link{measurement.error}} object.  The function, 
#' \code{\link{interSubVar}}, must first be used on the \code{\link{measurement.error}} 
#' object to obtain variability statistics.  This function use the \code{\link{image}}
#' function to produce plots.  It does little to manipulate such plots, but any
#' argument for \code{\link{image}} can be manipulated here, as well as the graphical
#' parameters that can be adjusted within \code{\link{image}}.
#'
#' @param x Object from \code{\link{interSubVar}}
#' @param ... Arguments passed onto \code{\link{image}} and \code{\link{plot}}.
#' @method plot interSubVar
#' @export
#' @author Michael Collyer
#' @keywords utilities
plot.interSubVar <- function(x, ...){
  subj <- x$subject.order
  n <- length(subj)
  names(subj) <- 1:n
  cat("If not apparent in the plot, the order of subjects on axes from 1 to",
      n, "is:\n")
  print(subj)
  cat("\n\n")
  image.args <- list(...)
  d <- as.matrix(x$var.map)
  image.args$x <- image.args$y <- 1:n
  image.args$z <- d
  image.args$xlab <- image.args$ylab <- "Subjects"
  do.call(image, image.args)
  out <- image.args
  out$subj <- subj
  class(out) <- "plot.interSubVar"
  invisible(out)
}

#' Plot Function for RRPP
#' 
#' Reduces a plot.measurement.error to a single research subject.  This can be 
#' for cases when many overlapping subjects in a plot obscure interpretation 
#' for specific subjects.
#'
#' @param x Plot from \code{\link{plot.measurement.error}}
#' @param subjects The specific subject to plot
#' @param shadow A logical value for whether to show other subject values as
#' shadows of their locations.
#' @param ... Other arguments passed onto plot
#' @export
#' @author Michael Collyer
#' @keywords utilities
#' 
focusMEonSubjects <- function(x, subjects = NULL, 
                              shadow = TRUE, ...){
  if(is.null(subjects))
    stop("Please specify at least 
         one subject to plot.\n", call. = FALSE)
  if(length(unique(subjects)) > length(x$x)) 
    stop("Please specify subjects within the vector origianlly used.\n",
         call. = FALSE)
  
  keep <- which(x$data$subj %in% unique(subjects))
  if(length(keep) == 0)
  stop("Subjects names not found within subject factor.\n", call. = FALSE)

  no.trm <- which(names(x) == "data")
  plot_args <- x[-no.trm]
  plot_args$type <- "n"
  do.call(plot, plot_args)
  if(shadow){
    plot_args$type <- "p"
    plot_args$cex <- if(!is.null(plot_args$cex)) 
      min(c(plot_args$cex / 2, 0.5)) else 0.5
    if(!is.null(plot_args$col)) plot_args$col <- alpha(plot_args$col, 0.1)
    if(!is.null(plot_args$bg)) plot_args$bg <- alpha(plot_args$bg, 0.1)
    do.call(points, plot_args)
  }

  point_args <- list(x = x$x[keep], y = x$y[keep])
  if(!is.null(x$pch)) point_args$pch <- if(length(x$pch) > 1)
    x$pch[keep] else x$pch
  if(!is.null(x$col)) point_args$col <- if(length(x$col) > 1)
    x$col[keep] else x$col
  if(!is.null(x$bg)) point_args$bg <- if(length(x$bg) > 1)
    x$bg[keep] else x$bg
  if(!is.null(x$lty)) point_args$lty <- if(length(x$lty) > 1)
    x$lty[keep] else x$lty
  if(!is.null(x$lwd)) point_args$lwd <- if(length(x$lwd) > 1)
    x$lwd[keep] else x$lwd
  if(!is.null(x$cex)) point_args$cex <- x$cex
  do.call(points, point_args)

  for(i in 1:length(subjects)) {
    keep <- which(x$data$subj %in% subjects[i])
    xx <- x$x[keep]
    yy <- x$y[keep]
    xmean <- mean(xx)
    ymean <- mean(yy)
    for(j in 1:length(xx)) {
      arrows(xmean, ymean, xx[j], yy[j], length = 0, 
             angle = 0, lwd = 0.5, ...)
      
    }
  }
  
}

#' Utility Function for RRPP
#' 
#' A function mostly for internal processing but can be used to extract
#' ANOVA statistics for other uses, such as plotting histograms
#'
#' @param fit Object from \code{\link{lm.rrpp}}.
#' @param stat The ANOVA statistic to extract.  Returns every RRPP permutation of 
#' the statistic.  If "all", a list of each statistic is returned.  
#' @export
#' @author Michael Collyer
#' @keywords utilities
#' @examples
#' \dontrun{
#' data(Pupfish)
#' fit <- lm.rrpp(coords ~ log(CS) + Sex*Pop, SS.type = "I", 
#' data = Pupfish, print.progress = FALSE, iter = 999) 
#' anova(fit)
#' Fstats <- getANOVAStats(fit, stat = "F")
#' par(mfrow = c(2, 2))
#' hist(Fstats$Fs[1,], breaks = 50, main = "log(CS)", xlab = "F")
#' abline(v = Fstats$Fs[1, 1])
#' hist(Fstats$Fs[2,], breaks = 50, main = "Sex", xlab = "F")
#' abline(v = Fstats$Fs[2, 1])
#' hist(Fstats$Fs[3,], breaks = 50, main = "Pop", xlab = "F")
#' abline(v = Fstats$Fs[3, 1])
#' hist(Fstats$Fs[4,], breaks = 50, main = "Sex:Pop", xlab = "F")
#' abline(v = Fstats$Fs[4, 1])
#' }
getANOVAStats <- function(fit, stat = c("SS", "MS", "Rsq", "F", "cohenf", "all")){
  ANOVA <- fit$ANOVA
  verbose <- fit$verbose
  SS <- ANOVA$SS
  MS <- ANOVA$MS
  Fs <- ANOVA$Fs
  cohenf <- ANOVA$cohenf
  Rsq <- ANOVA$Rsq
  Df <- ANOVA$df
  RSS <- ANOVA$RSS
  TSS <- ANOVA$TSS
  SS.type <- ANOVA$SS.type
  k <- length(fit$LM$term.labels)
  
  if(is.null(SS)){
    k <- 0
    SS <- ANOVA$RSS.model
    MS <- SS/Df
    Rsq <- rep(1, length(SS))
    names(Rsq) <- names(SS)
    Fs <- cohenf <- rep(0, length(SS))
    names(Fs) <- names(cohenf) <- names(SS)
    verbose <- TRUE
  }
  
  if(!verbose) {
    MS <- SS / Df[1:k]
    MSE <- RSS / Df[k + 1]
    Fs <- MS / MSE
    Rsq <- SS/TSS
    cohenf <- Rsq / (1 - Rsq)
    if(SS.type != "III"){
      etas <- Rsq
      if(k == 1) unexp <- 1 - etas else unexp <- 1 - apply(etas, 2, cumsum)
      cohenf <- etas/unexp
    }
  }
  out <- list(SS = SS, MS = MS, Rsq = Rsq, Fs = Fs, cohenf = cohenf)
  stat <- match.arg(stat)
  keep <- which(c("SS", "MS", "Rsq", "F", "cohenf", "all") %in% stat)
  if(keep == 6) keep <- 1:5
  out[keep]
}

#' Utility Function for RRPP
#' 
#' A function mostly for internal processing but can be used to extract
#' RRPP permutation information for other reasons.
#'
#' @param fit Object from \code{\link{lm.rrpp}}
#' @param attribute The various attributes that are used to generate RRPP
#'  permutation schedules.  If there are n observations, each iteration has
#'  some randomization of 1:n, restricted by the arguments that match attributes
#'  provided by this function.
#' @export
#' @author Michael Collyer
#' @keywords utilities

getPermInfo <- function(fit, attribute = c("perms", "perm.method",
                                           "block", "perm.seed",
                                           "perm.schedule", "all")){
  
  PermInfo <- fit$PermInfo
  perms <- PermInfo$perms
  perms <- max(c(perms, 2))
  perm.method <- PermInfo$perm.method
  block <- PermInfo$block
  perm.seed <- PermInfo$perm.seed
  perm.schedule <- PermInfo$perm.schedule
  n <- fit$LM$n
  
  if(is.null(perm.schedule)) {
    perm.schedule <- perm.index(n, perms - 1, block = block, 
                                seed = perm.seed)
  }
  
  out <- list(perms = perms, perm.method = perm.method, block = block,
              perm.seed = perm.seed, perm.schedule = perm.schedule)
  
  attribute <- match.arg(attribute)
  keep <- which(c("perms", "perm.method",
                  "block", "perm.seed",
                  "perm.schedule", "all") %in% attribute)
  if(keep == 6) keep <- 1:5
  out[keep]
}

#' Utility Function for RRPP
#' 
#' A function mostly for internal processing but can be used to extract
#' The terms for each reduced and full model used in an \code{\link{lm.rrpp}} fit.
#'
#' @param fit Object from \code{\link{lm.rrpp}}
#' @export
#' @author Michael Collyer
#' @keywords utilities

getTerms <- function(fit){
  Model.Terms <- .getTerms(fit)
  Model.Terms <- lapply(Model.Terms, function(x){
    lapply(x, function(y){
      attr(y, "dataClasses") <- NULL
      y
    })
  })
  
  names(Model.Terms) <- c("reduced", "full")
  Model.Terms
}

#' Utility Function for RRPP
#' 
#' A function mostly for internal processing but can be used to obtain terms,
#' design matrices, or QR decompositions used for each reduced or full
#' model that is fitted in an \code{\link{lm.rrpp}} fit.
#'
#' @param fit Object from \code{\link{lm.rrpp}}
#' @param attribute The various attributes that are used to extract RRPP
#'  Model information, including "terms", "X" (design matrices), "qr" (QR
#'  decompositions), or "all".
#' @export
#' @author Michael Collyer
#' @keywords utilities

getModels <- function(fit, attribute = c("terms", "X", "qr", "all")) {
  
  attribute <- match.arg(attribute)
  create <- is.null(fit$Models)
  if(!create) Models <- fit$Models else Models <- NULL
  
  Pcov <- if(!is.null(fit$LM$Pcov)) fit$LM$Pcov else
    if(!is.null(fit$LM$Cov)) Cov.proj(fit$LM$Cov) else NULL
  w <- if(!is.null(fit$LM$weights)) sqrt(fit$LM$weights) else NULL

  Model.Terms <- getTerms(fit)
  STerm <- if(fit$subTest)
    which(fit$LM$term.labels == fit$subjects.var) else NULL
  
  if(create){
    Xs <- suppressWarnings(getXs(Terms = fit$LM$Terms, 
                                  Y = fit$LM$Y, 
                                  SS.type = fit$ANOVA$SS.type,
                                  model = fit$LM$data,
                                 subjects.term = STerm)) 
  } else {
    Xs <- lapply(Models, function(x){
      lapply(x, function(y) y$X )
    })
  }
  names(Xs) <- c("reduced", "full")
  k <- length(Xs[[1]])
  Xs <- lapply(Xs, function(x){
    lapply(x, removeRedundant)
  })
  
  if(attribute == "all" || attribute == "qr") {
    
    if(create) {
      Models <-lapply(1:2, function(j){
        res <- lapply(1:max(1, k), function(jj){
          X <- Xs[[j]][[jj]]
          X <- removeRedundant(X)
          TX <- if(!is.null(Pcov)) Pcov %*% X else if(!is.null(w)) X*w else X
          qr <- QRforX(TX, reduce = FALSE)
          out <- list(X = X, qr = qr)
        })
        
        res
      })
      
      for(i in 1:2){
        for(j in 1:max(1, k)){
          Models[[i]][[j]]$terms <- Model.Terms[[i]][[j]]
        }
      }
      names(Models) <- c("reduced", "full")
      names(Models$reduced) <- names(Xs[[1]])
      names(Models$full) <- names(Xs[[2]])
    } else Models <- fit$Models
    
  } else Models <- NULL
  
  if(!is.null(Models)){
    Models <- lapply(Models, function(x){
      lapply(x, function(y){
        attr(y$terms, "dataClasses") <- NULL
        y
      })
    })
  }
  
  if(attribute == "all")
    out <- Models
  if(attribute == "terms")
    out <- Model.Terms
  if(attribute == "X")
    out <- Xs
  if(attribute == "qr"){
    QR <- lapply(Models, function(x){
      lapply(x, function(y) y$qr)
    })
    out <- QR
  }
  out
}

#' Utility Function for RRPP
#' 
#' A function mostly for internal processing but can be used to extract either
#' the model covariance matrix (Cov) or the projection matrix for transformations made 
#' from the covariance matrix (Pcov), which is basically the square-root of 
#' the covariance matrix.  This matrix is the model covariance used for estimation,
#' not the residual covariance matrix (see \code{\link{getResCov}}). 
#' There are also options for S3 or S4 format versions, or
#' a forcing of symmetry for Pcov.
#'
#' @param fit Object from \code{\link{lm.rrpp}}
#' @param type Whether the Cov or Pcov matrix is returned
#' @param format Whether an S3 or S4 format is returned
#' @param forceSym Logical value for whether a symmetric matrix should 
#' be returned for Pcov, even if Pcov was triangular as a solution.
#' @export
#' @author Michael Collyer
#' @keywords utilities

getModelCov <- function(fit, type = c("Cov", "Pcov"), 
                        format = c("S3", "S4"),
                        forceSym = TRUE) {
  if(is.null(fit$LM$Cov))
    stop("There was no model covariance matrix used in this lm.rrpp fit.\n",
         call. = FALSE)
  if(is.null(fit$LM$Pcov)) fit$LM$Pcov <- Cov.proj(fit$LM$Cov)
  
  type <- match.arg(type)
  format <- match.arg(format)
  
  res <- if(type == "Pcov") fit$LM$Pcov else fit$LM$Cov
  res <- if(format == "S4") Matrix(res, sparse = TRUE) else
    as.matrix(res)
  isSym <- (res[2, 1] == res[1, 2])
  if(!isSym && forceSym && type == "Pcov"){
    res <- Cov.proj(fit$LM$Cov, symmetric = TRUE)
    res <- if(format == "S4") Matrix(res, sparse = TRUE) else
      as.matrix(res)
  }
    
  
  res
}

#' Utility Function for RRPP
#' 
#' A function mostly for internal processing but can be used to extract the residual
#' covariance matrix.  This matrix is the residual covariance matrix,
#' not the model covariance matrix used for estimation (see \code{\link{getModelCov}}). 
#' Options for averaging over degrees of freedom or number of
#' observations, plus standardization, are also available.
#'
#' @param fit Object from \code{\link{lm.rrpp}}
#' @param useDf Logical value for whether the degrees of freedom from the linear model
#' fit should be used (if TRUE), as opposed to the number of observations (if FALSE).
#' @param standardize Logical value for whether residuals should be standardized.  If TRUE,
#' a correlation matrix is produced.
#' @export
#' @author Michael Collyer
#' @keywords utilities

getResCov <- function(fit, useDf = TRUE, standardize = FALSE) {
  gls <- fit$LM$gls
  if(gls && is.null(fit$LM$Pcov) && !is.null(fit$LM$Cov)) 
    fit$LM$Pcov <- Cov.proj(fit$LM$Cov)
  if(gls && !is.null(fit$LM$Pcov)) R <- fit$LM$Pcov %*% fit$LM$gls.residuals
  if(gls && is.null(fit$LM$Pcov)) R <- fit$LM$gls.residuals * sqrt(fit$LM$weights)
  if(!gls) R <-  fit$LM$residuals
  S <- as.matrix(crossprod(R))
  if(useDf){
    df <- fit$ANOVA$df
    if(length(df) > 1) df <- df[(length(df) - 1)] 
  } else df <- fit$LM$n
  
  S <- S / df
  
  if(standardize && length(S) > 1) {
    w <- diag(1 / sqrt(diag(S))) 
    S <- w %*% S %*% w
  } else if(standardize && length(S) == 1) S <- 1
    
    
  S
  
}

#' Print/Summary Function for RRPP
#'
#' @param x print/summary object (from \code{\link{ICCstats}})
#' @param ... other arguments passed to print/summary
#' @method print ICCstats
#' @export
#' @author Michael Collyer
#' @keywords utilities
print.ICCstats <-function(x, ...){
  
  cat("\nICC stats based on dispersion of values\n\n")
  print(x$ICC_disp)
  
  if(!is.null(x$ICC_mult)) {
    cat("\nICC stats by Eignevector of multivariate ICC matrix\n\n")
    print(x$ICC_mult)
  }
    
}

#' Print/Summary Function for RRPP
#'
#' @param object print/summary object (from \code{\link{ICCstats}})
#' @param ... other arguments passed to print/summary
#' @method summary ICCstats
#' @export
#' @author Michael Collyer
#' @keywords utilities
summary.ICCstats <-function(object, ...){
  print.ICCstats(object, ...)
}

#' Print/Summary Function for RRPP
#'
#' @param x print/summary object (from \code{\link{pairwise.model.Z}})
#' @param stats.table A logical value for whether to condense results into
#' a single table of multiple statistics.
#' @param ... other arguments passed to print/summary
#' @method print pairwise.model.Z
#' @export
#' @author Dean Adams and Michael Collyer
#' @keywords utilities
print.pairwise.model.Z <-function(x, stats.table = TRUE, ...){
  
  tab1 <- data.frame(Terms = names(x$sample.z),
                     Z = x$sample.z)
  
  nms <- paste("Mod", 1:length(x$sample.z), sep = "")
  nam.com <- combn(length(nms), 2)
  name.list <- list()
  for(i in 1:NCOL(nam.com)) 
    name.list[[i]]  <- paste(nms[nam.com[1,i]], nms[nam.com[2,i]], sep =":")
  
  rownames(tab1) <- nms
  cat("\nModels:\n\n")
  print(tab1)
  
  if(stats.table){

    Dz <- as.dist(x$pairwise.z)
    Dp <- as.dist(x$pairwise.P)
    tab2 <- data.frame(Z = as.vector(Dz), P = as.vector(Dp))
    colnames(tab2)[2] <- if(x$tails == 2) "Pr(>|Z|)" else "Pr(>Z)"
    rownames(tab2) <- unlist(name.list)
    
    cat("\nPairwise Statistics:\n\n")
    print(tab2)
    cat("\n\n")
    
  } else {
   
    cat("\nPairwise Z:\n")
    tb <- as.table(x$pairwise.z)
    dimnames(tb) <- list(nms, nms)
    print(tb)
    cat("\nPairwise P:\n")
    tb <- as.table(x$pairwise.P)
    dimnames(tb) <- list(nms, nms)
    print(tb)
    cat("\n\n")
  }
}

#' Print/Summary Function for RRPP
#'
#' @param object print/summary object (from \code{\link{pairwise.model.Z}})
#' @param ... other arguments passed to print/summary
#' @method summary pairwise.model.Z
#' @export
#' @author Dean Adams and Michael Collyer
#' @keywords utilities
summary.pairwise.model.Z <-function(object, ...){
  print.pairwise.model.Z(object, ...)
}

## betaTest

#' Print/Summary Function for RRPP
#'
#' @param x Object from \code{\link{betaTest}}
#' @param ... Other arguments passed onto betaTest
#' @method print betaTest
#' @export
#' @author Michael Collyer
#' @keywords utilities
print.betaTest <- function(x, 
                           ...){
  
  cat("Test of coefficients:", rownames(x$obs.B.mat)[x$coef.no])
  cat("\n equal to Beta = ")
  cat(as.vector(x$Beta))
  
  include.md <- !is.null(x$obs.md)
  perms <- if(include.md) NCOL(x$random.stats[[1]]) else 
    length(x$random.stats[[1]])
  
  cat("\n Test performed with", perms, 
      "permutations of residuals from specified null models.\n\n")
  
}

#' Print/Summary Function for RRPP
#'
#' @param object Object from \code{\link{betaTest}}
#' @param ... Other arguments passed onto betaTest
#' @param confidence The desired confidence limit to print with a table of 
#' summary statistics.  Because distances are directionless, confidence limits 
#' are one-tailed.
#' @method summary betaTest
#' @export
#' @author Michael Collyer
#' @keywords utilities
summary.betaTest <- function(object, 
                             confidence = 0.95,
                             ...){
  
  x <- object
  include.md <- !is.null(x$obs.md)
  perms <- if(include.md) NCOL(x$random.stats[[1]]) else 
    length(x$random.stats[[1]])

  Result <- lapply(x$random.stats, function(y){
    d <- if(include.md) y[1, ] else y
    dUCL = quantile(d, confidence)
    if(include.md){
      md <- y[2, ]
      mdUCL = quantile(md, confidence)
    }
    
    df = data.frame(d = d[1], UCLd = dUCL, 
                    Zd = effect.size(d),
                    Pd = pval(d))
    if(include.md){
      df$md = md[1] 
      df$UCLmd = mdUCL 
      df$Zmd = effect.size(md)
      df$Pmd = pval(md)
    }
    
    colnames(df)[2] <- paste("UCLd (", names(dUCL), ")", sep = "")
    colnames(df)[4] <- "Pr(>d)"
    if(include.md){
      colnames(df)[6] <- paste("UCLmd (", names(mdUCL), ")", sep = "")
      colnames(df)[8] <- "Pr(>md)"
    }
    
    df
  })
  
  names(Result) <- rownames(x$obs.B.mat)[x$coef.no]
  
  tab <- do.call(rbind, Result)
  out <- list(table = tab,
              confidence = confidence,
              perms = perms,
              Beta = as.vector(x$Beta))
  class(out) <- "summary.betaTest"
  out
}


#' Print/Summary Function for RRPP
#'
#' @param x Object from \code{\link{betaTest}}
#' @param ... Other arguments passed onto betaTest
#' @method print summary.betaTest
#' @export
#' @author Michael Collyer
#' @keywords utilities
#' 
print.summary.betaTest <- function(x,
                           ...){
  
  cat("Test of coefficients:", rownames(x$table))
  cat("\n equal to Beta = ")
  cat(as.vector(x$Beta))
  cat("\n Test performed with", x$perms, 
      "permutations of residuals from specified null models.\n\n")
  print(x$table)
  cat("\n\n")
}



## lmm.rrpp-specific functions


#' fixef for lmm.rrpp model fits
#'
#' @description Returns fixed effects from a lmm.rrpp object.
#'
#' @param object Object from \code{\link{lmm.rrpp}}
#' @param ... Other arguments passed.
#' @method fixef lmm.rrpp
#' @export
#' @author Michael Collyer
#' @keywords utilities
fixef.lmm.rrpp <- function(object, ...){
  out <- object$LM$coefficients[object$LM$coef.fixed, , drop = FALSE]
  class(out) <- "fixef.lmm.rrpp"
  out
}

#' ranef for lmm.rrpp model fits
#'
#' @description Returns random effects from a lmm.rrpp object.
#' @param object Object from \code{\link{lmm.rrpp}}
#' @param type Whether to combine random effects into one matrix or print a list
#' by variable.
#' @param ... Other arguments passed.
#' @method ranef lmm.rrpp
#' @export
#' @author Michael Collyer
#' @keywords utilities
ranef.lmm.rrpp <- function(object, 
                           type = c("matrix",
                                    "list"), ...){
  type <- match.arg(type)
  Brand <- as.matrix(object$LM$coefficients[object$LM$coef.random, , drop = FALSE])
  subjLevels <- levels(object$subjects)
  varNames <- colnames(object$LM$Y)
  effectNames <- object$LM$cnms[[1]]
  
  if(object$LM$estimation != "LS")
    Brand <- as.matrix(object$LM$Lambda %*% Brand)
  
  sp <- subjPartition(nrow(Brand), length(subjLevels))
  
  if(type == "list"){
    
    res <- lapply(1:ncol(Brand), function(j){
      b <- Brand[, j]
      m <- if(length(b[sp$slopeTerm]) > 0) cbind(b[sp$subjTerm], 
                                                 b[sp$slopeTerm]) else as.matrix(b)
      dimnames(m) <- list(subjLevels,
                          effectNames)
      m
    })
    names(res) <- colnames(Brand)
    if(length(res) == 1) 
      names(res)[[1]] <- object$subjects.var
  } else {
    res <- Brand
    if(length(res[sp$slopeTerm]) > 0) {
      rownames(res)[sp$subjTerm] <- subjLevels
      rownames(res)[sp$slopeTerm] <- paste(
        object$LM$ranef.slopeTerm, subjLevels, sep = ":")
    } else rownames(res) <- subjLevels
  }
  
  class(res) <- "ranef.lmm.rrpp"
  res
}

### -----------------------------------------------------------------------
###
### All parallel processing options must be run outside of CRAN check
###
### All tests that require dependencies should be run outside of CRAN check
###
### -----------------------------------------------------------------------


### lm.rrpp ---------------------------------------------------------------

test_that("fit01.works", {
  library(RRPP)
  data("Pupfish")
  succeed(lm.rrpp(coords[,1] ~ 1, data = Pupfish, 
                                print.progress = FALSE, iter = 3, verbose = TRUE))
})

test_that("fit02.works", {
  library(RRPP)
  data("Pupfish")
  succeed(lm.rrpp(coords ~ 1, data = Pupfish, 
                  print.progress = FALSE, iter = 3, verbose = TRUE))
})

test_that("fit03.works", {
  library(RRPP)
  data("Pupfish")
  succeed(lm.rrpp(coords ~ CS, data = Pupfish, 
                  print.progress = FALSE, iter = 3, verbose = TRUE))
})

test_that("fit04.works", {
  library(RRPP)
  data("Pupfish")
  succeed(lm.rrpp(coords ~ CS + Pop, data = Pupfish, 
                  print.progress = FALSE, iter = 3, verbose = TRUE))
})

test_that("fit05.works", {
  library(RRPP)
  data("Pupfish")
  succeed(lm.rrpp(coords ~ CS + Pop, data = Pupfish, 
                  SS.type = "II", print.progress = FALSE, iter = 3, verbose = TRUE))
})

test_that("fit06.works", {
  library(RRPP)
  data("Pupfish")
  succeed(lm.rrpp(coords ~ CS + Pop, data = Pupfish, 
                  SS.type = "III", print.progress = FALSE, iter = 3, verbose = TRUE))
})

test_that("fit07.works", {
  library(RRPP)
  data("Pupfish")
  succeed(lm.rrpp(coords ~ CS + Pop, data = Pupfish, 
                  turbo = TRUE, print.progress = FALSE, iter = 3, verbose = TRUE))
})


test_that("fit10.works", {
  library(RRPP)
  data("Pupfish")
  succeed(lm.rrpp(coords ~ CS * Pop * Sex, data = Pupfish, 
                  print.progress = FALSE, iter = 3, verbose = TRUE))
})

test_that("fit11.works", {
  library(RRPP)
  data("Pupfish")
  succeed(lm.rrpp(coords ~ CS * Pop * Sex, data = Pupfish, 
                  int.first = TRUE,
                  print.progress = FALSE, iter = 3, verbose = TRUE))
})

test_that("fit12.works", {
  library(RRPP)
  data("PlethMorph")
  succeed(lm.rrpp(TailLength ~ SVL, data = PlethMorph, 
                  print.progress = FALSE, iter = 3, verbose = TRUE))
})


test_that("fit13.works", {
  library(RRPP)
  data("PlethMorph")
  succeed(lm.rrpp(TailLength ~ 1, data = PlethMorph, 
                  print.progress = FALSE, 
                  Cov = PlethMorph$PhyCov,
                  iter = 3, verbose = TRUE))
})

test_that("fit14.works", {
  library(RRPP)
  data("PlethMorph")
  succeed(lm.rrpp(TailLength ~ SVL, data = PlethMorph, 
                  print.progress = FALSE, 
                  Cov = PlethMorph$PhyCov,
                  iter = 3, verbose = TRUE))
})

test_that("fit15.works", {
  library(RRPP)
  data("PlethMorph")
  succeed(lm.rrpp(cbind(TailLength, BodyWidth) ~ SVL, data = PlethMorph, 
                  print.progress = FALSE, 
                  Cov = PlethMorph$PhyCov,
                  iter = 3, verbose = TRUE))
})

test_that("fit16.works", {
  library(RRPP)
  data("PlethMorph")
  succeed(lm.rrpp(TailLength ~ SVL + 0, data = PlethMorph, 
                  print.progress = FALSE, 
                  Cov = PlethMorph$PhyCov,
                  iter = 3, verbose = TRUE))
})

test_that("fit17.works", {
  library(RRPP)
  data("PlethMorph")
  succeed(lm.rrpp(cbind(TailLength, BodyWidth) ~ SVL + 0, data = PlethMorph, 
                  print.progress = FALSE, 
                  Cov = PlethMorph$PhyCov,
                  iter = 3, verbose = TRUE))
})

### coef.lm.rrpp ----------------------------------------------------------

test_that("coef.fit01.works", {
  library(RRPP)
  data("Pupfish")
  succeed(coef(lm.rrpp(coords[,1] ~ 1, data = Pupfish, 
                  print.progress = FALSE, iter = 3, verbose = TRUE)))
})

test_that("coef.fit02.works", {
  library(RRPP)
  data("Pupfish")
  succeed(coef(lm.rrpp(coords ~ 1, data = Pupfish, 
                  print.progress = FALSE, 
                  iter = 3, verbose = TRUE)))
})

test_that("coef.fit03.works", {
  library(RRPP)
  data("Pupfish")
  succeed(coef(lm.rrpp(coords ~ CS, data = Pupfish, 
                  print.progress = FALSE, 
                  iter = 3, verbose = TRUE)))
})

test_that("coef.fit04.works", {
  library(RRPP)
  data("Pupfish")
  succeed(coef(lm.rrpp(coords ~ CS + Pop, data = Pupfish, 
                  print.progress = FALSE, 
                  iter = 3, verbose = TRUE)))
})

test_that("coef.fit05.works", {
  library(RRPP)
  data("Pupfish")
  succeed(coef(lm.rrpp(coords ~ CS + Pop, data = Pupfish, 
                  SS.type = "II", 
                  print.progress = FALSE, 
                  iter = 3, verbose = TRUE)))
})

test_that("coef.fit06.works", {
  library(RRPP)
  data("Pupfish")
  succeed(coef(lm.rrpp(coords ~ CS + Pop, data = Pupfish, 
                  SS.type = "III", 
                  print.progress = FALSE, 
                  iter = 3, verbose = TRUE)))
})

test_that("coef.fit07.works", {
  library(RRPP)
  data("Pupfish")
  succeed(coef(lm.rrpp(coords ~ CS + Pop, data = Pupfish, 
                  turbo = TRUE, 
                  print.progress = FALSE, 
                  iter = 3, verbose = TRUE)))
})


test_that("coef.fit10.works", {
  library(RRPP)
  data("Pupfish")
  succeed(coef(lm.rrpp(coords ~ CS * Pop * Sex, data = Pupfish, 
                  print.progress = FALSE, 
                  iter = 3, verbose = TRUE)))
})

test_that("coef.fit11.works", {
  library(RRPP)
  data("Pupfish")
  succeed(coef(lm.rrpp(coords ~ CS * Pop * Sex, data = Pupfish, 
                  int.first = TRUE,
                  print.progress = FALSE, 
                  iter = 3, verbose = TRUE)))
})


test_that("coef.fit12.works", {
  library(RRPP)
  data("PlethMorph")
  succeed(coef(lm.rrpp(TailLength ~ SVL, data = PlethMorph, 
                  print.progress = FALSE, 
                  iter = 3, verbose = TRUE)))
})


test_that("coef.fit13.works", {
  library(RRPP)
  data("PlethMorph")
  succeed(coef(lm.rrpp(TailLength ~ 1, data = PlethMorph, 
                  print.progress = FALSE, 
                  Cov = PlethMorph$PhyCov,
                  iter = 3, verbose = TRUE)))
})

test_that("coef.fit14.works", {
  library(RRPP)
  data("PlethMorph")
  succeed(coef(lm.rrpp(TailLength ~ SVL, data = PlethMorph, 
                  print.progress = FALSE, 
                  Cov = PlethMorph$PhyCov,
                  iter = 3, verbose = TRUE)))
})

test_that("coef.fit15.works", {
  library(RRPP)
  data("PlethMorph")
  succeed(coef(lm.rrpp(cbind(TailLength, BodyWidth) ~ SVL, data = PlethMorph, 
                  print.progress = FALSE, 
                  Cov = PlethMorph$PhyCov,
                  iter = 3, verbose = TRUE)))
})

test_that("coef.t.fit01.works", {
  library(RRPP)
  data("Pupfish")
  succeed(coef(lm.rrpp(coords[,1] ~ 1, data = Pupfish, 
                       print.progress = FALSE, iter = 3, verbose = TRUE),
          test = TRUE))
})

test_that("coef.t.fit02.works", {
  library(RRPP)
  data("Pupfish")
  succeed(coef(lm.rrpp(coords ~ 1, data = Pupfish, 
                       print.progress = FALSE, iter = 3, verbose = TRUE),
          test = TRUE))
})

test_that("coef.t.fit03.works", {
  library(RRPP)
  data("Pupfish")
  succeed(coef(lm.rrpp(coords ~ CS, data = Pupfish, 
                       print.progress = FALSE, iter = 3, verbose = TRUE),
          test = TRUE))
})

test_that("coef.t.fit04.works", {
  library(RRPP)
  data("Pupfish")
  succeed(coef(lm.rrpp(coords ~ CS + Pop, data = Pupfish, 
                       print.progress = FALSE, iter = 3, verbose = TRUE),
          test = TRUE))
})

test_that("coef.t.fit05.works", {
  library(RRPP)
  data("Pupfish")
  succeed(coef(lm.rrpp(coords ~ CS + Pop, data = Pupfish, 
                       SS.type = "II", print.progress = FALSE, iter = 3, verbose = TRUE),
          test = TRUE))
})

test_that("coef.t.fit06.works", {
  library(RRPP)
  data("Pupfish")
  succeed(coef(lm.rrpp(coords ~ CS + Pop, data = Pupfish, 
                       SS.type = "III", print.progress = FALSE, iter = 3, verbose = TRUE),
          test = TRUE))
})

test_that("coef.t.fit07.works", {
  library(RRPP)
  data("Pupfish")
  succeed(coef(lm.rrpp(coords ~ CS + Pop, data = Pupfish, 
                       turbo = TRUE, print.progress = FALSE, iter = 3, verbose = TRUE),
          test = TRUE))
})



test_that("coef.t.fit10.works", {
  library(RRPP)
  data("Pupfish")
  succeed(coef(lm.rrpp(coords ~ CS * Pop * Sex, data = Pupfish, 
                       print.progress = FALSE, iter = 3, verbose = TRUE),
          test = TRUE))
})

test_that("coef.t.fit11.works", {
  library(RRPP)
  data("Pupfish")
  succeed(coef(lm.rrpp(coords ~ CS * Pop * Sex, data = Pupfish, 
                       int.first = TRUE,
                       print.progress = FALSE, iter = 3, verbose = TRUE),
          test = TRUE))
})

test_that("coef.t.fit12.works", {
  library(RRPP)
  data("PlethMorph")
  succeed(coef(lm.rrpp(TailLength ~ SVL, data = PlethMorph, 
                  print.progress = FALSE, iter = 3, verbose = TRUE),
               test = TRUE))
})


test_that("coef.t.fit13.works", {
  library(RRPP)
  data("PlethMorph")
  succeed(coef(lm.rrpp(TailLength ~ 1, data = PlethMorph, 
                       print.progress = FALSE, 
                       Cov = PlethMorph$PhyCov,
                       iter = 3, verbose = TRUE),
               test = TRUE))
})

test_that("coef.t.fit14.works", {
  library(RRPP)
  data("PlethMorph")
  succeed(coef(lm.rrpp(TailLength ~ SVL, data = PlethMorph, 
                       print.progress = FALSE, 
                       Cov = PlethMorph$PhyCov,
                       iter = 3, verbose = TRUE),
               test = TRUE))
})

test_that("coef.t.fit15.works", {
  library(RRPP)
  data("PlethMorph")
  succeed(coef(lm.rrpp(cbind(TailLength, BodyWidth) ~ SVL, data = PlethMorph, 
                       print.progress = FALSE, 
                       Cov = PlethMorph$PhyCov,
                       iter = 3, verbose = TRUE),
               test = TRUE))
})

# needed: Cov application

### betaTest---------------------------------------------------------------

test_that("beta01.works", {
  library(RRPP)
  data("Pupfish")
  mod <- lm.rrpp(coords[,1] ~ 1, data = Pupfish, 
                  print.progress = FALSE, iter = 3, verbose = TRUE)
  succeed(betaTest(mod))
  succeed(betaTest(mod, Beta = rep(1, mod$LM$p)))
  succeed(betaTest(mod, Beta = rep(1, mod$LM$p), include.md = TRUE))
})

test_that("beta02.works", {
  library(RRPP)
  data("Pupfish")
  mod <- lm.rrpp(coords ~ 1, data = Pupfish, 
                  print.progress = FALSE, iter = 3, verbose = TRUE)
  succeed(betaTest(mod))
  succeed(betaTest(mod, Beta = rep(1, mod$LM$p)))
})

test_that("beta03.works", {
  library(RRPP)
  data("Pupfish")
  mod <- lm.rrpp(coords ~ CS, data = Pupfish, 
                  print.progress = FALSE, iter = 3, verbose = TRUE)
  succeed(betaTest(mod))
  succeed(betaTest(mod, Beta = rep(1, mod$LM$p)))
  succeed(betaTest(mod, Beta = rep(1, mod$LM$p), include.md = TRUE))
})

test_that("beta04.works", {
  library(RRPP)
  data("Pupfish")
  mod <- lm.rrpp(coords ~ CS + Pop, data = Pupfish, 
                  print.progress = FALSE, iter = 3, verbose = TRUE)
  succeed(betaTest(mod))
  succeed(betaTest(mod, Beta = rep(1, mod$LM$p)))
  succeed(betaTest(mod, Beta = rep(1, mod$LM$p), include.md = TRUE))
})

test_that("beta05.works", {
  library(RRPP)
  data("Pupfish")
  mod <- lm.rrpp(coords ~ CS + Pop, data = Pupfish, 
                  SS.type = "II", print.progress = FALSE, iter = 3, verbose = TRUE)
  succeed(betaTest(mod))
  succeed(betaTest(mod, Beta = rep(1, mod$LM$p)))
  succeed(betaTest(mod, Beta = rep(1, mod$LM$p), include.md = TRUE))
})

test_that("beta06.works", {
  library(RRPP)
  data("Pupfish")
  mod <- lm.rrpp(coords ~ CS + Pop, data = Pupfish, 
                  SS.type = "III", print.progress = FALSE, iter = 3, verbose = TRUE)
  succeed(betaTest(mod))
  succeed(betaTest(mod, Beta = rep(1, mod$LM$p)))
  succeed(betaTest(mod, Beta = rep(1, mod$LM$p), include.md = TRUE))
})

test_that("beta07.works", {
  library(RRPP)
  data("Pupfish")
  mod <- lm.rrpp(coords ~ CS + Pop, data = Pupfish, 
                  turbo = TRUE, print.progress = FALSE, iter = 3, verbose = TRUE)
  succeed(betaTest(mod))
  succeed(betaTest(mod, Beta = rep(1, mod$LM$p)))
  succeed(betaTest(mod, Beta = rep(1, mod$LM$p), include.md = TRUE))
})


test_that("beta10.works", {
  library(RRPP)
  data("Pupfish")
  mod <- lm.rrpp(coords ~ CS * Pop * Sex, data = Pupfish, 
                  print.progress = FALSE, iter = 3, verbose = TRUE)
  succeed(betaTest(mod))
  succeed(betaTest(mod, Beta = rep(1, mod$LM$p)))
  succeed(betaTest(mod, Beta = rep(1, mod$LM$p), include.md = TRUE))
})

test_that("beta11.works", {
  library(RRPP)
  data("Pupfish")
  mod <- lm.rrpp(coords ~ CS * Pop * Sex, data = Pupfish, 
                  int.first = TRUE,
                  print.progress = FALSE, iter = 3, verbose = TRUE)
  succeed(betaTest(mod))
  succeed(betaTest(mod, Beta = rep(1, mod$LM$p)))
  succeed(betaTest(mod, Beta = rep(1, mod$LM$p), include.md = TRUE))
})

test_that("beta12.works", {
  library(RRPP)
  data("PlethMorph")
  mod <- lm.rrpp(TailLength ~ SVL, data = PlethMorph, 
                  print.progress = FALSE, iter = 3, verbose = TRUE)
  succeed(betaTest(mod))
  succeed(betaTest(mod, Beta = rep(1, mod$LM$p)))
  succeed(betaTest(mod, Beta = rep(1, mod$LM$p), include.md = TRUE))
})


test_that("beta13.works", {
  library(RRPP)
  data("PlethMorph")
  mod <- lm.rrpp(TailLength ~ 1, data = PlethMorph, 
                  print.progress = FALSE, 
                  Cov = PlethMorph$PhyCov,
                  iter = 3, verbose = TRUE)
  succeed(betaTest(mod))
  succeed(betaTest(mod, Beta = rep(1, mod$LM$p)))

})

test_that("beta14.works", {
  library(RRPP)
  data("PlethMorph")
  mod <- lm.rrpp(TailLength ~ SVL, data = PlethMorph, 
                  print.progress = FALSE, 
                  Cov = PlethMorph$PhyCov,
                  iter = 3, verbose = TRUE)
  succeed(betaTest(mod))
  succeed(betaTest(mod, Beta = rep(1, mod$LM$p)))
  succeed(betaTest(mod, Beta = rep(1, mod$LM$p), include.md = TRUE))
})

test_that("beta15.works", {
  library(RRPP)
  data("PlethMorph")
  mod <- lm.rrpp(cbind(TailLength, BodyWidth) ~ SVL, data = PlethMorph, 
                  print.progress = FALSE, 
                  Cov = PlethMorph$PhyCov,
                  iter = 3, verbose = TRUE)
  succeed(betaTest(mod))
  succeed(betaTest(mod, Beta = rep(1, mod$LM$p)))
  succeed(betaTest(mod, Beta = rep(1, mod$LM$p), include.md = TRUE))
})

test_that("beta16.works", {
  library(RRPP)
  data("PlethMorph")
  mod <- lm.rrpp(TailLength ~ SVL + 0, data = PlethMorph, 
                  print.progress = FALSE, 
                  Cov = PlethMorph$PhyCov,
                  iter = 3, verbose = TRUE)
  succeed(betaTest(mod))
  succeed(betaTest(mod, Beta = rep(1, mod$LM$p)))
  succeed(betaTest(mod, Beta = rep(1, mod$LM$p), include.md = TRUE))
})

test_that("beta17.works", {
  library(RRPP)
  data("PlethMorph")
  mod <- lm.rrpp(cbind(TailLength, BodyWidth) ~ SVL + 0, data = PlethMorph, 
                  print.progress = FALSE, 
                  Cov = PlethMorph$PhyCov,
                  iter = 3, verbose = TRUE)
  
  succeed(betaTest(mod))
  succeed(betaTest(mod, Beta = rep(1, mod$LM$p)))
  succeed(betaTest(mod, Beta = rep(1, mod$LM$p), include.md = TRUE))
})

### anova.lm.rrpp ----------------------------------------------------------


test_that("anova.fit01.works", {
  library(RRPP)
  data("Pupfish")
  succeed(anova(lm.rrpp(coords[,1] ~ 1, data = Pupfish, 
                       print.progress = FALSE, iter = 3, verbose = TRUE)))
})

test_that("anova.fit02.works", {
  library(RRPP)
  data("Pupfish")
  succeed(anova(lm.rrpp(coords ~ 1, data = Pupfish, 
                       print.progress = FALSE, iter = 3, verbose = TRUE)))
})

test_that("anova.fit03.works", {
  library(RRPP)
  data("Pupfish")
  succeed(anova(lm.rrpp(coords ~ CS, data = Pupfish, 
                       print.progress = FALSE, iter = 3, verbose = TRUE)))
})

test_that("anova.fit04.works", {
  library(RRPP)
  data("Pupfish")
  succeed(anova(lm.rrpp(coords ~ CS + Pop, data = Pupfish, 
                       print.progress = FALSE, iter = 3, verbose = TRUE)))
})

test_that("anova.fit05.works", {
  library(RRPP)
  data("Pupfish")
  succeed(anova(lm.rrpp(coords ~ CS + Pop, data = Pupfish, 
                       SS.type = "II", print.progress = FALSE, iter = 3, verbose = TRUE)))
})

test_that("anova.fit06.works", {
  library(RRPP)
  data("Pupfish")
  succeed(anova(lm.rrpp(coords ~ CS + Pop, data = Pupfish, 
                       SS.type = "III", print.progress = FALSE, iter = 3, verbose = TRUE)))
})

test_that("anova.fit07.works", {
  library(RRPP)
  data("Pupfish")
  succeed(anova(lm.rrpp(coords ~ CS + Pop, data = Pupfish, 
                       turbo = TRUE, print.progress = FALSE, iter = 3, verbose = TRUE)))
})


test_that("anova.fit10.works", {
  library(RRPP)
  data("Pupfish")
  succeed(anova(lm.rrpp(coords ~ CS * Pop * Sex, data = Pupfish, 
                       print.progress = FALSE, iter = 3, verbose = TRUE)))
})

test_that("anova.fit11.works", {
  library(RRPP)
  data("Pupfish")
  succeed(anova(lm.rrpp(coords ~ CS * Pop * Sex, data = Pupfish, 
                       int.first = TRUE,
                       print.progress = FALSE, iter = 3, verbose = TRUE)))
})

test_that("anova.fit11.b.works", {
  library(RRPP)
  data("Pupfish")
  succeed(anova(lm.rrpp(coords ~ CS * Pop * Sex, data = Pupfish, 
                        int.first = TRUE,
                        print.progress = FALSE, iter = 3, verbose = TRUE),
                effect.type = "cohenf"))
})

test_that("anova.fit11.c.works", {
  library(RRPP)
  data("Pupfish")
  succeed(anova(lm.rrpp(coords ~ CS * Pop * Sex, data = Pupfish, 
                        int.first = TRUE,
                        print.progress = FALSE, iter = 3, verbose = TRUE),
                effect.type = "Rsq"))
})

test_that("anova.fit11.d.works", {
  library(RRPP)
  data("Pupfish")
  succeed(anova(lm.rrpp(coords ~ CS * Pop * Sex, data = Pupfish, 
                        int.first = TRUE,
                        print.progress = FALSE, iter = 3, verbose = TRUE),
                effect.type = "MS"))
})

test_that("anova.fit11.e.works", {
  library(RRPP)
  data("Pupfish")
  succeed(anova(lm.rrpp(coords ~ Pop * Sex, data = Pupfish, 
                        int.first = TRUE,
                        print.progress = FALSE, iter = 3, verbose = TRUE),
                error = c("Pop:Sex", "Pop:Sex", "Residuals")))
})


test_that("anova.fit12.works", {
  library(RRPP)
  data("PlethMorph")
  succeed(anova(lm.rrpp(TailLength ~ SVL, data = PlethMorph, 
                  print.progress = FALSE, iter = 3, verbose = TRUE)))
})


### predict.lm.rrpp -------------------------------------------------------


test_that("predict1.works", {
  library(RRPP)
  data("PupfishHeads")
  suppressWarnings(fit <- lm.rrpp(log(headSize) ~ sex + locality/year, SS.type = "III", 
                 data = PupfishHeads, print.progress = FALSE, iter = 3, verbose = TRUE))
  succeed(predict(fit))
})


### manova.update ---------------------------------------------------------

test_that("manova1.works", {
  library(RRPP)
  data("Pupfish")
  fit <- lm.rrpp(coords ~ CS + Sex, SS.type = "I", 
          data = Pupfish, print.progress = FALSE, iter = 3, verbose = TRUE)
  succeed(manova.update(fit, print.progress = FALSE))
})

### pairwise --------------------------------------------------------------

test_that("pairwise1.works", {
  library(RRPP)
  data("Pupfish")
  fit <- lm.rrpp(coords ~ CS + Sex, SS.type = "I", 
          data = Pupfish, print.progress = FALSE, iter = 3, verbose = TRUE)
  succeed(pairwise(fit, groups = Pupfish$Sex, print.progress = FALSE))
})


### trajectory.analysis ---------------------------------------------------

test_that("trajectory.analysis1.works", {
  library(RRPP)
  data("Pupfish")
  fit <- lm.rrpp(coords ~ Pop * Sex, data = Pupfish, print.progress = FALSE,
                 iter = 3, verbose = TRUE)
  succeed(trajectory.analysis(fit, groups = Pupfish$Pop, 
                              traj.pts = Pupfish$Sex, print.progress = FALSE))
})

test_that("trajectory.analysis2.works", {
  library(RRPP)
  data("motionpaths")
  fit <- lm.rrpp(trajectories ~ groups, data = motionpaths, iter = 3, verbose = TRUE)
  succeed(trajectory.analysis(fit, groups = motionpaths$groups, traj.pts = 5))
})

### ordinate --------------------------------------------------------------


test_that("ordinate1.works", {
  library(RRPP)
  data("PlethMorph")
  Y <- as.data.frame(PlethMorph[c("TailLength", "HeadLength", 
                                  "Snout.eye", "BodyWidth", 
                                  "Forelimb", "Hindlimb")])
  PlethMorph$Y <- as.matrix(Y)
  R <- lm.rrpp(Y ~ SVL, data = PlethMorph, 
               iter = 0, print.progress = FALSE)$LM$residuals
  succeed(ordinate(R, scale. = FALSE))
})

test_that("ordinate2.works", {
  library(RRPP)
  data("PlethMorph")
  Y <- as.data.frame(PlethMorph[c("TailLength", "HeadLength", 
                                  "Snout.eye", "BodyWidth", 
                                  "Forelimb", "Hindlimb")])
  PlethMorph$Y <- as.matrix(Y)
  R <- lm.rrpp(Y ~ SVL, data = PlethMorph, 
               iter = 0, print.progress = FALSE)$LM$residuals
  succeed(ordinate(R, scale. = TRUE))
})

test_that("ordinate3.works", {
  library(RRPP)
  data("PlethMorph")
  Y <- as.data.frame(PlethMorph[c("TailLength", "HeadLength", 
                                  "Snout.eye", "BodyWidth", 
                                  "Forelimb", "Hindlimb")])
  PlethMorph$Y <- as.matrix(Y)
  R <- lm.rrpp(Y ~ SVL, data = PlethMorph, 
               iter = 0, print.progress = FALSE)$LM$residuals
  succeed(ordinate(R, scale. = TRUE, 
                   transform. = FALSE, 
                   Cov = PlethMorph$PhyCov))
})

test_that("ordinate4.works", {
  library(RRPP)
  data("PlethMorph")
  Y <- as.data.frame(PlethMorph[c("TailLength", "HeadLength", 
                                  "Snout.eye", "BodyWidth", 
                                  "Forelimb", "Hindlimb")])
  PlethMorph$Y <- as.matrix(Y)
  R <- lm.rrpp(Y ~ SVL, data = PlethMorph, 
               iter = 0, print.progress = FALSE)$LM$residuals
  succeed(ordinate(R, scale. = TRUE, 
                   transform. = TRUE, 
                   Cov = PlethMorph$PhyCov))
})

test_that("ordinate5.works", {
  library(RRPP)
  data("PlethMorph")
  Y <- as.data.frame(PlethMorph[c("TailLength", "HeadLength", 
                                  "Snout.eye", "BodyWidth", 
                                  "Forelimb", "Hindlimb")])
  PlethMorph$Y<- as.matrix(Y)
  R <- lm.rrpp(Y ~ SVL, data = PlethMorph, 
               iter = 0, print.progress = FALSE)$LM$residuals
  succeed(ordinate(R, A = PlethMorph$PhyCov, scale. = TRUE))
})

test_that("ordinate6.works", {
  library(RRPP)
  data("PlethMorph")
  Y <- as.data.frame(PlethMorph[c("TailLength", "HeadLength", 
                                  "Snout.eye", "BodyWidth", 
                                  "Forelimb", "Hindlimb")])
  PlethMorph$Y <- as.matrix(Y)
  R <- lm.rrpp(Y ~ SVL, data = PlethMorph, 
               iter = 0, print.progress = FALSE)$LM$residuals
  succeed(ordinate(R, A = PlethMorph$PhyCov, 
                   scale. = TRUE,
                   transform. = FALSE, 
                   Cov = PlethMorph$PhyCov))
})

### measurement.error ----------------------------------------------------


test_that("measurement.error1.works", {
  library(RRPP)
  data("fishy")
  succeed(ME1 <- measurement.error(Y = "coords", subjects = "subj",
                           replicates = "reps", data = fishy, 
                           iter = 3, Parallel = FALSE,
                           verbose = TRUE))
})

test_that("measurement.error2.works", {
  library(RRPP)
  data("fishy")
  succeed(ME2 <- measurement.error(Y = "coords", subjects = "subj",
                                   replicates = "reps", data = fishy,
                                   groups = "groups",
                                   iter = 3, Parallel = FALSE,
                                   verbose = TRUE))
})

test_that("measurement.error3.works", {
  library(RRPP)
  data("fishy")
  ME2 <- measurement.error(Y = "coords", subjects = "subj",
                                   replicates = "reps", data = fishy,
                                   groups = "groups",
                                   iter = 3, Parallel = FALSE,
                                   verbose = TRUE)
  succeed(anova(ME2))
})

test_that("ICCstats1.works", {
  library(RRPP)
  data("fishy")
  ME2 <- measurement.error(Y = "coords", subjects = "subj",
                                   replicates = "reps", data = fishy,
                                   groups = "groups",
                                   iter = 3, Parallel = FALSE,
                           verbose = TRUE)
  succeed(ICCstats(ME2, subjects = "Subjects"))
})

test_that("ICCstats2.works", {
  library(RRPP)
  data("fishy")
  ME2 <- measurement.error(Y = "coords", subjects = "subj",
                           replicates = "reps", data = fishy,
                           groups = "groups",
                           iter =3, Parallel = FALSE,
                           verbose = TRUE)
  succeed(ICCstats(ME2, subjects = "Subjects", 
                   with_in = "Systematic ME"))
})

test_that("ICCstats3.works", {
  library(RRPP)
  data("fishy")
  ME2 <- measurement.error(Y = "coords", subjects = "subj",
                           replicates = "reps", data = fishy,
                           groups = "groups",
                           iter = 3, Parallel = FALSE,
                           verbose = TRUE)
  succeed(ICCstats(ME2, subjects = "Subjects", 
                   with_in = c("Systematic ME", "Systematic ME:Groups")))
})

### getModels---------------------------------------------------------------

test_that("getModels.works", {
  library(RRPP)
  data("fishy")
  ME2 <- measurement.error(Y = "coords", subjects = "subj",
                           replicates = "reps", data = fishy,
                           groups = "groups",
                           iter = 3, Parallel = FALSE,
                           verbose = TRUE)
  succeed(getModels(ME2, "all"))
})

### getANOVAstats ------------------------------------------------------------

test_that("getANOVAStats.works", {
  library(RRPP)
  data("fishy")
  ME2 <- measurement.error(Y = "coords", subjects = "subj",
                           replicates = "reps", data = fishy,
                           groups = "groups",
                           iter = 3, Parallel = FALSE,
                           verbose = TRUE)
  succeed(getANOVAStats(ME2, "all"))
})

### getPermInfo ----------------------------------------------------------

test_that("getANOVAStats.works", {
  library(RRPP)
  data("fishy")
  ME2 <- measurement.error(Y = "coords", subjects = "subj",
                           replicates = "reps", data = fishy,
                           groups = "groups",
                           iter = 3, Parallel = FALSE,
                           verbose = TRUE)
  succeed(getPermInfo(ME2, "all"))
})


### getModelCov---------------------------------------------------------------

test_that("getModelCov1.works", {
  library(RRPP)
  data("PlethMorph")
  fitGLS <- lm.rrpp(TailLength ~ SVL, 
                    data = PlethMorph, 
                    Cov = PlethMorph$PhyCov, 
                    print.progress = FALSE, iter = 3)
  
  succeed(getModelCov(fitGLS, "Cov"))
})

test_that("getModelCov2.works", {
  library(RRPP)
  data("PlethMorph")
  fitGLS <- lm.rrpp(TailLength ~ SVL, 
                    data = PlethMorph, 
                    Cov = PlethMorph$PhyCov, 
                    print.progress = FALSE, iter = 3)
  
  succeed(getModelCov(fitGLS, "Pcov"))
})


### getResCov ---------------------------------------------------------------

test_that("getResCov1.works", {
  library(RRPP)
  data("PlethMorph")
  fitGLS <- lm.rrpp(TailLength ~ SVL, 
                    data = PlethMorph, 
                    Cov = PlethMorph$PhyCov, 
                    print.progress = FALSE, iter = 3)
  
  succeed(getResCov(fitGLS, useDf = TRUE, standardize = FALSE))
})

test_that("getResCov2.works", {
  library(RRPP)
  data("PlethMorph")
  fitGLS <- lm.rrpp(TailLength ~ SVL, 
                    data = PlethMorph, 
                    Cov = PlethMorph$PhyCov, 
                    print.progress = FALSE, iter = 3)
  
  succeed(getResCov(fitGLS, useDf = TRUE, standardize = TRUE))
})

test_that("getResCov3.works", {
  library(RRPP)
  data("PlethMorph")
  fitGLS <- lm.rrpp(TailLength ~ SVL, 
                    data = PlethMorph, 
                    Cov = PlethMorph$PhyCov, 
                    print.progress = FALSE, iter = 3)
  
  succeed(getResCov(fitGLS, useDf = FALSE, standardize = FALSE))
})

test_that("getResCov4.works", {
  library(RRPP)
  data("PlethMorph")
  fitGLS <- lm.rrpp(TailLength ~ SVL, 
                    data = PlethMorph, 
                    Cov = PlethMorph$PhyCov, 
                    print.progress = FALSE, iter = 3)
  
  succeed(getResCov(fitGLS, useDf = FALSE, standardize = TRUE))
})


### lmm.rrpp

test_that("lmm.rrpp1.works", {
  library(RRPP)
  data("ReptAbund")
  fit <- lmm.rrpp(fixed = reptile_abundance ~ Year, 
           subjects = "Site",      
           type = "intercepts", 
           estimation = "LS", 
           data = ReptAbund,
           iter = 3)
  
  succeed(coef(fit, type = "matrix"))
  succeed(coef(fit, type = "list"))
  succeed(fixef(fit))
  succeed(ranef(fit, type = "matrix"))
  succeed(ranef(fit, type = "list"))
  succeed(model.matrix(fit))
  succeed(anova(fit))
  succeed(fitted(fit))
  succeed(resid(fit))
  succeed(getModels(fit, "all"))
  succeed(terms(fit))
  succeed(getTerms(fit))
  succeed(ICCstats(fit, subjects = "Site"))
  succeed(betaTest(fit))
  succeed(betaTest(fit, Beta = 1))
})

test_that("lmm.rrpp2.works", {
  library(RRPP)
  data("ReptAbund")
  fit <- lmm.rrpp(fixed = reptile_abundance ~ Year, 
                  subjects = "Site",      
                  type = "intercepts", 
                  estimation = "ML", 
                  data = ReptAbund,
                  iter = 3)
  
  succeed(coef(fit, type = "matrix"))
  succeed(coef(fit, type = "list"))
  succeed(fixef(fit))
  succeed(ranef(fit, type = "matrix"))
  succeed(ranef(fit, type = "list"))
  succeed(model.matrix(fit))
  succeed(anova(fit))
  succeed(fitted(fit))
  succeed(resid(fit))
  succeed(getModels(fit, "all"))
  succeed(terms(fit))
  succeed(getTerms(fit))
  succeed(ICCstats(fit, subjects = "Site"))
  succeed(betaTest(fit))
  succeed(betaTest(fit, Beta = 1))
})

test_that("lmm.rrpp3.works", {
  library(RRPP)
  data("ReptAbund")
  fit <- lmm.rrpp(fixed = reptile_abundance ~ Year, 
                  subjects = "Site",      
                  type = "intercepts", 
                  estimation = "REML", 
                  data = ReptAbund,
                  iter = 3)
  
  succeed(coef(fit, type = "matrix"))
  succeed(coef(fit, type = "list"))
  succeed(fixef(fit))
  succeed(ranef(fit, type = "matrix"))
  succeed(ranef(fit, type = "list"))
  succeed(model.matrix(fit))
  succeed(anova(fit))
  succeed(fitted(fit))
  succeed(resid(fit))
  succeed(getModels(fit, "all"))
  succeed(terms(fit))
  succeed(getTerms(fit))
  succeed(ICCstats(fit, subjects = "Site"))
  succeed(betaTest(fit))
  succeed(betaTest(fit, Beta = 1))
})

# Add random slopes using lme4::sleepstudy

#' Likelihood ratio test for a linear model, based on RRPP
#' 
#' @description 
#' Function performs likelihood ratio tests on an lm.rrpp fit, using 
#' RRPP or FRPP.  Likelihood ratio statistics are calculated for every random 
#' permutation, and the effect size is estimated from the distribution of 
#' random statistics.  The likelihood ratio tests has some resemblance to
#' MANOVA, especially using Wilks' lambda.  Sums of squares and cross-products 
#' (SSCP) matrices are calculated over the random permutations of 
#' a \code{\link{lm.rrpp}} fit.  SSCP matrices are 
#' computed, as are the inverse of R times H (invR.H), where R is a SSCP 
#' for the residuals or random effects and H is
#' the difference between SSCP matrices of full and reduced models 
#' (see \code{\link{manova.update}}).   From invR.H, Wilks lambda is first estimated, and the 
#' likelihood ratio stat is then estimated as -n * log(Wilks).
#' 
#' This function does one of two things.  It either performs an update using
#' \code{\link{manova.update}}, using Wilks' lambda as the test statistic, converting 
#' Wilks' lambda to likelihood ratio statistics or it uses the results from
#' a previously performed update to calculate new statistics.  
#' 
#' 
#' @references Adams, D. C., and M. L. Collyer. 2024. Extended phylogenetic 
#' regression models for comparing within-species patterns across the 
#' tree of life. Methods in Ecology and Evolution. In review.
#' @param fit Linear model fit from \code{\link{lm.rrpp}} or a fit
#' that has already been updated with \code{\link{manova.update}}.
#' @param verbose Logical value for whether to include all random 
#' Wilks' lambda and likelihood ratio statistics from random permutations.
#' @param ... Arguments passed onto \code{\link{manova.update}}.

#' @keywords analysis
#' @export
#' @author Michael Collyer
#' @examples 
#'    
#' # Body Shape Analysis (Multivariate) ----------------
#' 
#' \dontrun{
#' data(Pupfish)
#' 
#' # Although not recommended as a practice, this example will use only
#' # three principal components of body shape for demonstration.  
#' # A larger number of random permutations should also be used.
#' 
#' Pupfish$shape <- ordinate(Pupfish$coords)$x[, 1:3]
#' 
#' 
#' fit <- lm.rrpp(shape ~ log(CS) + Sex, SS.type = "I", 
#' data = Pupfish, print.progress = FALSE, iter = 499) 
#' summary(fit, formula = FALSE)
#' anova(fit) # ANOVA table
#' 
#' # MANOVA
#' 
#' fit.m <- manova.update(fit, print.progress = FALSE, tol = 0.001)
#' summary(fit.m, test = "Roy")
#' summary(fit.m, test = "Wilks")
#' 
#' 
#' # Likelihood Ratio Test
#' 
#' LRT <- lr_test(fit.m)
#' summary(LRT)
#' 
#' }


lr_test <- function(fit, verbose = FALSE, ...){
  
  manova.args <- list(...)
  need.invR.H <- is.null(fit$MANOVA$invR.H)
  n <- fit$LM$n
  
  if(need.invR.H) {
    if(inherits(fit, "manova.lm.rrpp")) {
      man.loc <- which(class(fit) == "manova.lm.rrpp")
      class(fit) <- class(fit)[-man.loc]
    }
    
    manova.args$fit <- fit
    fit <- NULL
    manova.args$error <- NULL
    if(is.null(manova.args$tol)) manova.args$tol <- 1e-07
    if(is.null(manova.args$print.progess)) manova.args$print.progress <- TRUE
    manova.args$verbose <- TRUE
    fit <- do.call(manova.update, manova.args)
  }
  
  tol <- min(c(1e-07, manova.args$tol))
  eigs <- fit$MANOVA$eigs
  fit <- NULL
  
  Wilks <- sapply(eigs, function(x){
    apply(x, 1, wilks)
  })
  
  eigs <- NULL
  LR <- -n * log(Wilks + tol)
  
  tab <- data.frame(
    Wilks  = Wilks[,1],
    LR = LR[,1],
    Z = apply(LR, 1, effect.size),
    pval = apply(LR, 1, pval)
  )
  
  out <- list(tab = tab)
  
  if(verbose){
    out$wilks <- Wilks
    out$lr <- LR
  }
  
  class(out) <- "lr_test"
  out
    
}

#' Obtain P-value from a vector of statistics
#'
#' A function to find the probability of values greater or lesser than target,
#' from a vector of statistics presumably obtained in random permutations.
#'
#' @param s The sampling distribution vector to use.
#' @param target The value to target in the distribution.  (If null, the first value
#' in the vector is used.).  If the target exists outside the range of s,
#' a probability of 0 or 1 is certain.
#' @param greater Logical value for whether the probability should be "greater than 
#' or equal to".  Change to greater = FALSE for "less than or equal to".
#' @export
#' @author Michael Collyer
#' @keywords utilities
pval <- function(s, target = NULL, greater = TRUE){
  if(is.null(target)) target <- s[1]
  p <- length(s)
  pv <- if(greater) 
    length(which(s >= target)) else 
      length(which(s <= target))
  pv / p
}


box.cox.true <- function(y, eps = 0.001){
  
  n <- length(y) - 1
  y.obs <- y[1]
  y <- y[-1]
  
  # Note the code below (to get yy) is short-cut for Jacobian adjustment 
  yy <- y / exp(mean(log(y)))
  logy <- log(yy)
  
  lambda <- seq(-2, 2, 0.001)
  m <- length(lambda)
  
  loglik <- sapply(1:m, function(j){ # same as MASS::boxcox loglik 
    la <- lambda[j]
    yt <- if(abs(la) > eps) yt <- (yy^la - 1)/la else
      logy * (1 + (la * logy)/2 * (1 + (la * logy)/3 * (1 + (la * logy)/4)))
    
    -n/2 * log(sum(center(yt)^2))
  })
  
  lambda.opt <- lambda[which.max(loglik)][[1]]
  
  if(abs(lambda.opt) < eps) lambda.opt <- 0
  y <- c(y.obs, y)
  res <- if(lambda.opt == 0) log(y) else (y^lambda.opt - 1)/lambda.opt
  
  list(opt.lambda = lambda.opt, transformed = res, lambda = lambda, 
       loglik = loglik)
  
}

box.cox.spline <- function(y, eps = 0.001) {
  
  n <- length(y) - 1
  y.obs <- y[1]
  y <- y[-1]
  
  yy <- y / exp(mean(log(y)))
  logy <- log(yy)
  m <- 20
  lambda <- seq(-1, 1.5, length.out = m)
  
  loglik <- sapply(1:m, function(j){ # same as MASS::boxcox loglik 
    la <- lambda[j]
    yt <- if(abs(la) > eps) yt <- (yy^la - 1)/la else
      logy * (1 + (la * logy)/2 * (1 + (la * logy)/3 * (1 + (la * logy)/4)))
    
    -n/2 * log(sum(center(yt)^2))
  })
  
  lambda.opt <- lambda[which.max(loglik)][[1]]
  
  sp <- spline(lambda, loglik, n = 300)
  lambda.opt <- sp$x[which.max(sp$y)]
  if(abs(lambda.opt) < eps) lambda.opt <- 0
  y <- c(y.obs, y)
  res <- if(lambda.opt == 0) log(y) else (y^lambda.opt - 1)/lambda.opt
  
  list(opt.lambda = lambda.opt, transformed = res, lambda = sp$x, loglik = sp$y)
  
}

box.cox.interp <- function(y, eps = 0.001) {
  bc <- box.cox.spline(y, eps = eps)
  if(bc$opt.lambda == -1 || bc$opt.lambda == 1.5) 
    bc <- box.cox.true(y, eps = eps)
  return(bc)
}

box.cox.fast <- function(y, eps = 0.001) {
  
  n <- length(y) - 1
  y.obs <- y[1]
  y <- y[-1]

  yy <- y / exp(mean(log(y)))
  logy <- log(yy)
  
  logLik <- function(lambda) {
    la <- lambda
    yt <- if(abs(la) > eps) yt <- (yy^la - 1)/la else
      logy * (1 + (la * logy)/2 * (1 + (la * logy)/3 * (1 + (la * logy)/4)))
    
    -n/2 * log(sum(center(yt)^2))
  }
  
  result <- optimise(logLik, lower = -2, upper = 2, maximum = TRUE)
  lambda.opt <- result$maximum
  
  if(abs(lambda.opt) < eps) lambda.opt <- 0
  y <- c(y.obs, y)
  res <- if(lambda.opt == 0) log(y) else (y^lambda.opt - 1)/lambda.opt
  list(opt.lambda = lambda.opt, transformed = res, lambda = NULL, loglik = NULL)
}



#' Power Transformation
#'
#' A function that transforms a vector of statistics or data to be 
#' approximately normally distributed.
#' 
#' This function uses a Box-Cox power transformation for a vector of values,
#' (presumably statistics from RRPP) with the following conditions:
#' 
#' (1) If any values are 0 or negative, a Yeo-Johnson transformation is performed
#' to optimize lambda with only positive values.
#' 
#' (2) The lambda parameter is optimized over a range of -2 to +2,
#' as is typical for most applications.
#' 
#' (3) The first (or targeted) value (assumed to be the observed value in a 
#' distribution of random values) is removed from the vector to 
#' perform optimization of lambda, but is returned to the vector for
#' transformation.  This assures that large, aberrant values, such
#' as might be found via RRPP for large effects, do not have 
#' leverage for the optimization.
#' 
#' A Yeo-Johnson transformation can be forced, which for positive values
#' merely shifts the distribution by adding 1.  For most RRPP statistics
#' (SS. MS, Rsq, F, d), this is not needed for effect sizes, as values are positive.  
#' However, for log-likelihoods, this might be a good idea for comparisons, 
#' because values might be strongly positive or strongly negative.
#' 
#' Using standard deviates (useStDev) is a good idea for statistics with
#' aberrant ranges (like 0 to 1, or 1,000 to 2,000).  If used, a Yeo-Johnson
#' transformation is implicit, since standardizing introduces negative values.
#' 
#'
#' @param y A vector of values to use.
#' @param eps Tolerance for lambda = 0.
#' @param target The value to target in the distribution.  (If null, the first value
#' in the vector is used.)  This will be moved to the first value in results.
#' @param interp Logical value for whether to spline interpolate lambda 
#' (ideal for vectors of small length).  This will default if the number
#' of values is less than 100, as in \link[MASS]{boxcox}.
#' @param forceYJ A logical value whether to force a Yeo-Johnson transformation for non-negative
#' values.  Negative values trigger
#' the transformation, but positive values can be transformed similarly.
#' @param useStDev A logical value whether to initially use standard deviates of y.
#' This might be useful if the range of y is restricted, say between 0 and 1.
#' @return A list containing the following:
#' \item{opt.lambda}{The optimized lambda parameter.}
#' \item{transformed}{The power-transformed values.}
#' \item{lambda}{If spline interpolation is used, the values of lambda used.}
#' \item{loglik}{If spline interpolation is used, the log-liklihoods calculated for lambda.}
#' @export
#' @author Michael Collyer
#' @keywords utilities
#' @references Box, G. E. P. and Cox, D. R. (1964) An analysis of transformations 
#' (with discussion). Journal of the Royal Statistical Society B, 26, 211–252.
#' @references Yeo, I. and Johnson, R. (2000) A new family of power transformations 
#' to improve normality or symmetry. Biometrika, 87, 954-959.
powerTrans <- function(y, 
                       eps = 0.001, 
                       target = NULL,
                       interp = FALSE,
                       forceYJ = FALSE,
                       useStDev = FALSE) {
  
  whichNegs <- NULL
  which0 <- NULL
  useYJ <- FALSE
  
  if(!is.null(target))
    y <- c(target, y)
  
  if(forceYJ && !useStDev) {
    useYJ <- TRUE
    y <- abs(y) + 1
  }
    
  if(any(y <= 0)) {
    useYJ <- TRUE
    whichNegs <- which(y < 0)
    which0 <- which(abs(y) <= eps)
    y <- abs(y) + 1
  }
  
  if(useStDev){
    useYJ <- TRUE
    n <- length(y) - 1
    yc <- y - mean(y[-1])
    sy <- sqrt(sum(center(y[-1])^2)/ n)
    y <- yc / sy
    whichNegs <- which(y < 0)
    which0 <- which(abs(y) <= eps)
    y <- abs(y) + 1
  }
  
  n <- length(y)
  if(n < 100) interp <- TRUE
  result <- if(interp) 
    try(box.cox.interp(y, eps = eps), silent = TRUE) else
    box.cox.fast(y, eps = eps)
  if(inherits(result, "try-error"))
    result <- suppressWarnings(
      box.cox.fast(y, eps = eps))
  
  if(useYJ){
    tf <- result$transformed
    lambda <- result$opt.lambda
    whichPos <- setdiff(1:n, union(whichNegs, which0))
    tf[whichPos] <- (y[whichPos]^lambda - 1)/lambda
    tf[which0] <- log(y[which0])
    if(lambda == 2){
      tf[whichNegs] <- -log(tf[whichNegs])
    } else tf[whichNegs] <- 
      -(y[whichNegs]^(2 - lambda) -1)/(2 - lambda)
  }
  
  return(result)
}

#' Obtain Effect-size from a vector of values
#'
#' A function to find the effect size (Z-score) of a target,
#' from a vector of values presumably obtained in random permutations.
#'
#' @param x The vector of data to use.
#' @param center Logical value for whether to center x.
#' @param ... Arguments passed to \link{powerTrans}.
#' @export
#' @author Michael Collyer
#' @keywords utilities
effect.size <- function(x, center = TRUE, ...) {
  es.args <- c(list(y = x), list(...))
  if(length(unique(x)) == 1) {
    sdx <- 1
    x <- 0
  } else {
    bc <- do.call(powerTrans, es.args)
    x <- bc$transformed
    n <- length(x)
    if(center) x <- center(x)
    sdx <- sqrt((sum(x^2)/n))
  }
  
  (x[1]- mean(x)) / sdx
}

#' ANOVA for lm.rrpp model fits used in measurement.error
#'
#' @description Computes an analysis of variance (ANOVA) table using 
#' distributions of random statistics from \code{\link{lm.rrpp}}.  
#' This function is the same as \code{\link{anova.lm.rrpp}} but includes statistics
#' specific to \code{\link{measurement.error}}, and with restrictions on
#' how P-values and effect.sizes are calculated.
#'
#' @param object Object from \code{\link{lm.rrpp}}
#' @param ... Additional lm.rrpp model fits or other arguments passed to anova. 
#' @export anova.measurement.error
#' @export
#' @author Michael Collyer
#' @keywords utilities
#' @examples 
#' # See measurement.error help file examples for use.
#' 
anova.measurement.error <- function(object, ...) {
  out <- aov.me(object)
  out
}
  

#' Calculate the pairwise Mahalanobis distances between observations
#'
#' This function emulates the \code{\link[stats]{dist}} function
#' but allows a covariance matrix (Cov) to be included for standardizing
#' distances.  It is assumed that the Covariance matrix makes sense with
#' respect to the data, and that the number of variables match between data and covariance matrix.
#' 
#' No tests are performed on distances but could be performed with the 
#' \code{\link{pairwise}} function.  Distances are only calculated if
#' the covariance matrix is not singular.
#'
#' @param x A numeric matrix of data frame.
#' @param Cov A covariance matrix with the same number of variables as the data.
#' @param ... Other arguments passed to \code{\link[stats]{dist}}.
#' @keywords analysis
#' @export
#' @author Michael Collyer
#' @return An object of class "dist".
#' 
#' @examples 
#' 
#' # Using the Pupfish data (see lm.rrpp help for more detail)
#' 
#' data(Pupfish)
#' Pupfish$Y <- ordinate(Pupfish$coords)$x[, 1:3]
#' fit <- lm.rrpp(Y ~ Sex * Pop, SS.type = "I", 
#' data = Pupfish, print.progress = FALSE, iter = 0)
#' X <- unique(as.matrix(model.matrix(fit)))
#' means <- X %*% coef(fit)
#' rownames(means) <- unique(interaction(Pupfish$Sex, Pupfish$Pop))
#' means
#' S <- getResCov(fit)
#' dist(means)
#' mahal_dist(means, S)

mahal_dist <- function(x, Cov, ...){
  x <- as.matrix(x)
  if(NCOL(x) != NCOL(Cov))
    stop("\nThe number of data variables do not match the number of covariance matrix variables.\n",
         call. = FALSE)
  Pcov <- Cov.proj(Cov)
  TX <- tcrossprod(x, Pcov)
  dist(TX, ...)
}

#' predict for lm.rrpp model fits
#'
#' @description Computes predicted values from an \code{\link{lm.rrpp}} 
#' model fit, using bootstrapped residuals
#' to generate confidence intervals.  (Residuals are the residuals of 
#' the lm.rppp fit, not its null model.  The bootstrap
#' procedure resamples residual vectors with replacement.)
#' The bootstrap permutations use the same number of iterations and 
#' seed as used
#' in the \code{\link{lm.rrpp}} model fit. A \code{\link{predict.lm.rrpp}} 
#' object can be plotted using various options.
#' See \code{\link{plot.predict.lm.rrpp}}.
#' 
#' Note that if data offsets are used (if the offset argument is used 
#' when fitting a \code{\link{lm.rrpp}} model),
#' they are ignored for estimating coefficients over iterations.  
#' Offsets are subtracted from data in \code{\link[stats]{lm}} and 
#' added to predicted values in \code{\link[stats]{predict.lm}}, 
#' effectively adjusting the intercept and then un-adjusting
#' it for predictions.  This causes problems if the newdata have a 
#' different number of observations than the original
#' model fit.
#' 
#'
#' @param object Object from \code{\link{lm.rrpp}}.
#' @param newdata Data frame of either class \code{\link{data.frame}} 
#' or \code{\link{rrpp.data.frame}}.  If null,
#' the data frame from the lm.rrpp fit will be used, effectively calculating 
#' all fitted values and
#' their confidence intervals.  If a numeric variable is missing from newdata, 
#' an attempt to average the values
#' will be made in prediction; i.e., least squares means for factor levels 
#' can be found.  All factors used in the
#' \code{\link{lm.rrpp}} fit should be represented in the newdata data frame, 
#' with appropriate factor levels.
#' @param confidence The desired confidence interval level for prediction.
#' @param block An optional factor for blocks within which to restrict resampling
#' permutations.
#' @param ... Other arguments (currently none)
#' @export
#' @author Michael Collyer
#' @keywords utilities
#' @examples 
#' \dontrun{
#' # See examples for lm.rrpp to see how predict.lm.rrpp works in conjunction
#' # with other functions
#' 
#' data(Pupfish)
#' 
#' # CS is centroid (fish) size
#' fit <- lm.rrpp(coords ~ log(CS)  + Sex*Pop, 
#' SS.type = "I", data = Pupfish, iter = 999) 
#'
#' # Predictions (holding alternative effects constant)
#' 
#' shapeDF <- expand.grid(Sex = levels(Pupfish$Sex), Pop = levels(Pupfish$Pop))
#' rownames(shapeDF) <- paste(shapeDF$Sex, shapeDF$Pop, sep = ".")
#' shapeDF
#' 
#' shapePreds <- predict(fit, shapeDF)
#' summary(shapePreds)
#' summary(shapePreds, PC = TRUE)
#' 
#' shapePreds99 <- predict(fit, shapeDF, confidence = 0.99)
#' summary(shapePreds99, PC = TRUE)
#' 
#' # Plot prediction
#' 
#' plot(shapePreds, PC = TRUE)
#' plot(shapePreds, PC = TRUE, ellipse = TRUE)
#' plot(shapePreds99, PC = TRUE)
#' plot(shapePreds99, PC = TRUE, ellipse = TRUE)
#' }
#' 
predict.lm.rrpp <- function(object, newdata = NULL, block = NULL,
                            confidence = 0.95, ...) {
  if(!inherits(object, "lm.rrpp")) stop("Object is not class lm.rrpp\n")
  if(isTRUE(object$LM$LMM))
    stop("predict.lm.rrpp is currently not available for lmm.rrpp objects.\n", call. = FALSE)
  Terms <- object$LM$Terms
  trms <- object$LM$term.labels
  k <- length(trms)
  TT <- delete.response(Terms)
  
  if(!missing(newdata) || !is.null(newdata)) {
    
    if(!inherits(newdata, "data.frame") && !inherits(newdata, "rrpp.data.frame"))
      stop("newdata must be an object of class data.frame or rrpp.data.frame")
    if(confidence < 0 || confidence > 1) 
      stop("Confidence level must be between 0 and 1")
    
    if(inherits(newdata, "rrpp.data.frame")) {
      vars <- all.vars(TT)
      refd <- names(newdata) %in% vars
      if(!all(refd))
        stop("\nOne or more variables in newdata are not model terms.\n",
             call. = FALSE)
      if(length(refd) != length(newdata))
        stop("\nIt's not possible to coerce the rrpp.data.frame into a 
             useable data frame for prediction.\n",
             call. = FALSE)
      
      class(newdata) <- "list"
    }
    
  }
  
  if(!is.null(newdata) && is.list(newdata)) newdata <- as.data.frame(newdata)
  
  if(missing(newdata) || is.null(newdata)) {
    nX <- object$LM$X
  } else {
    
    o.names <- all.vars(TT)
    n.names <- names(newdata)
    tm <- match(o.names, n.names)
    
    if(all(is.na(tm)))
      stop("\nVariables in newdata do not match variables used in lm.rrpp fit",
           call. = FALSE)
    
    if(any(is.na(tm))) {
      
      warning(
        paste(
          "\nThis is not an error!  It is a friendly warning.\n",
          "\nNot all variables in model accounted for in newdata.",
          "\nMissing variables will be averaged from observed data for prediction.\n",
          "\nUse suppressWarnings to turn off these warnings. \n\n", sep = " "),
        noBreaks. = TRUE, call. = FALSE, immediate. = TRUE) 
      
    }
    
    nX <- getXfromNewData(object, newdata)
    
  }
  
  o <- object$LM$offset
  if(!is.null(o)) offst = TRUE else offst = FALSE
  
  n <- NROW(nX)
  p <- NCOL(object$LM$Y)
  
  PI <- getPermInfo(object, attribute = "all")
  seed <- attr(PI$perm.schedule, "seed")
  perms <- PI$perms
  indb <- boot.index(length(PI$perm.schedule[[1]]), 
                     perms -1, block, seed)
  betas <- beta.boot.iter(object, indb)
  
  predM <- function(b) as.matrix(nX[, rownames(b), drop = FALSE] %*% b)
  preds <- lapply(betas, predM)
  
  alpha = 1 - confidence
  lcl <- function(x) quantile(x, prob = alpha/2)
  ucl <- function(x) quantile(x, prob = 1 - alpha/2)
  meanV <- Reduce("+", preds)/length(preds)
  
  getCL <- function(preds, CL) {
    x <- unlist(preds)
    ind <- rep(1:length(meanV), perms)
    as.vector(by(x, ind, CL))
  }
  
  LCL <- getCL(preds, lcl)
  UCL <- getCL(preds, ucl)
  
  LCL <- matrix(LCL, NROW(meanV), NCOL(meanV))
  UCL <- matrix(UCL, NROW(meanV), NCOL(meanV))
  
  dimnames(LCL) <- dimnames(UCL) <- dimnames(meanV)
  
  if(!is.null(dim(LCL))) {
    if(dim(LCL)[2] == n) LCL <- t(LCL)
  }
  if(!is.null(dim(UCL))) {
    if(dim(UCL)[2] == n) UCL <- t(UCL)
  }
  
  if(p == 1){
    pc.preds <- preds
    pcLCL <- LCL
    pcUCL <- UCL
    pc.meanV <- Reduce("+", pc.preds)/length(pc.preds)
    pca <- NULL
  } else {
    pca <- prcomp(preds[[1]])
    d <- length(which(zapsmall(pca$sdev) > 0))
    R <- pca$rotation[, 1:d, drop = FALSE]
    rotate <- function(x) center(x) %*% R
    pc.preds <- lapply(preds, rotate)
    pc.meanV <- Reduce("+", pc.preds)/length(pc.preds)
    pcLCL <- as.matrix(sapply(1:n, function(j){
      x <- sapply(1:length(pc.preds), function(jj){
        pc.preds[[jj]][j, , drop = FALSE]
      })
      if(!is.matrix(x)) z <- lcl(x) else z <- apply(x, 1, lcl)
      z
    }))
    
    pcUCL <- as.matrix(sapply(1:n, function(j){
      x <- sapply(1:length(pc.preds), function(jj){
        pc.preds[[jj]][j, , drop = FALSE]
      })
      if(!is.matrix(x)) z <- ucl(x) else z <- apply(x, 1, ucl)
      z
    }))
    
    if(!is.null(dim(pcLCL))) {
      if(dim(pcLCL)[2] == n) pcLCL <- t(pcLCL)
    }
    if(!is.null(dim(pcUCL))) {
      if(dim(pcUCL)[2] == n) pcUCL <- t(pcUCL)
    }
  }
  
  data.name = as.character(object$call[[2]][[2]])
  rn <- rownames(nX)
  if(is.null(rownames(nX))) rn <- as.character(1:NROW(nX))
  colnames(meanV) <- colnames(LCL) <- colnames(UCL) <- colnames(object$LM$Y)
  rownames(meanV) <- rownames(pc.meanV) <- rownames(LCL) <- rownames(UCL) <- 
    rownames(pcLCL) <- rownames(pcUCL) <- rn
  out <- list(mean = meanV, lcl = LCL, ucl = UCL, 
              pc.mean = pc.meanV, pc.lcl = pcLCL, pc.ucl = pcUCL,
              confidence = confidence, 
              data.name = data.name, random.predicted = preds,
              random.predicted.pc = pc.preds, pca=pca)
  class(out) <- "predict.lm.rrpp"
  out
}

#' Convert RRPP plots to ggplot objects
#'
#' Function attempts to coerce plot information from an RRPP plot object to an 
#' amenable ggplot object.  
#'
#' This function will attempt to use the plot arguments from an RRPP plot object 
#' to make a ggplot that can be additionally updated, as desired.  Not all plot 
#' characteristics can be converted.  For example, text arguments are not currently
#' passed to \code{\link[ggplot2]{ggplot}}, as the \code{\link{text}} function and geom_text
#' arguments do not easily align.  However, one can use text arguments produced by
#' a RRPP plot object and geom_text to augment a ggplot object the way they like.
#' 
#' This function assumes no responsibility for arguments made by \code{\link[ggplot2]{ggplot}}.
#' It merely produces a ggplot object that should resemble an RRPP plot default.  Any 
#' augmentation of ggplot objects can be done either by direct intervention of the ggplot 
#' produced or reformatting the initial RRPP plot produced.  One should not expect direct
#' correspondence between R base plot parameters and ggplot parameters.  For example,
#' error bars will generally appear as different widths, without an easy way to control them,
#' changing from one format to the other.
#' 
#' @param object A plot object produced from \code{\link{plot.lm.rrpp}} and type 
#' equals either "PC" or "regression, \code{\link{plot.predict.lm.rrpp}}, or 
#' \code{\link{plot.ordinate}}.  Essentially, any RRPP plot except a series of diagnostic
#' plots should work.
#' @keywords utilities
#' @export
#' @author Michael Collyer
#' @examples
#' \dontrun{
#' ### Linear Model Example
#' 
#' data(Pupfish)
#' fit <- lm.rrpp(coords ~ log(CS) + Sex*Pop, SS.type = "I", 
#' data = Pupfish, print.progress = FALSE) 
#' 
#' # Predictions (holding alternative effects constant)
#' 
#' shapeDF <- expand.grid(Sex = levels(Pupfish$Sex), 
#' Pop = levels(Pupfish$Pop))
#' rownames(shapeDF) <- paste(shapeDF$Sex, shapeDF$Pop, sep = ".")
#' 
#' shapePreds <- predict(fit, shapeDF)
#' summary(shapePreds, PC = TRUE)
#' 
#' # Plot prediction
#' 
#' P <- plot(shapePreds, PC = TRUE, ellipse = TRUE)
#' convert2ggplot(P)
#' 
#' ### Ordination Example
#' 
#' data("PlethMorph")
#' 
#' Y <- as.data.frame(PlethMorph[c("TailLength", "HeadLength", 
#'                                "Snout.eye", "BodyWidth", 
#'                                "Forelimb", "Hindlimb")])
#'Y <- as.matrix(Y)
#'R <- lm.rrpp(Y ~ SVL, data = PlethMorph, 
#'              print.progress = FALSE)$LM$residuals
#'
#' # PCA (on correlation matrix)
#'
#'
#' PCA.ols <- ordinate(R, scale. = TRUE)
#' PCA.ols$rot
#' prcomp(R, scale. = TRUE)$rotation # should be the same
#'
#'
#' PCA.gls <- ordinate(R, scale. = TRUE, 
#'                    transform. = FALSE, 
#'                    Cov = PlethMorph$PhyCov)
#'                
#' P <- plot(PCA.gls)
#' convert2ggplot(P)                   
#' }

convert2ggplot <- function(object){
  x <- y <- NULL
  pa <- object$plot_args
  if(!is.null(object$arrow_args)) {
    arrowed <- TRUE
    ellipses <- FALSE
    ea <- object$arrow_args
  } else if(!is.null(object$ellipse.points)) {
    arrowed <- FALSE
    ellipses <- TRUE
    ea <- object$ellipse.points
  } else {
    arrowed <- FALSE
    ellipses <- FALSE
  }
  
  if(is.null(pa)) 
    stop("Plot object does not appear to be a type that can be converted to ggplot.\n",
         call. = FALSE)
  
  if(arrowed){
    
    xbar <- ybar <- FALSE
    if(!identical(ea$x0, ea$x1)) xbar <- TRUE
    if(!identical(ea$y0, ea$y1)) ybar <- TRUE
    
    xb <- if(xbar) abs(ea$x1 - ea$x0) else NULL
    yb <- if(ybar) abs(ea$y1 - ea$y0) else NULL
    
    df <- data.frame(x = ea$x0, y = ea$y0)
    if(xbar) df$xb <- xb
    if(ybar) df$yb <- yb
    
  }  else if(ellipses) {
    df <- as.data.frame(ea$means)
    colnames(df) <- c("x", "y")
  } else  df <- data.frame(x = pa$x, y = pa$y)
                                
  if(is.null(pa$cex)) pa$cex <- 1
  if(is.null(pa$pch)) pa$pch <- 19
  if(is.null(pa$bg)) pa$bg <- NA
  if(is.null(pa$col)) pa$col <- 1
  if(is.null(pa$lty)) pa$lty <- 1
  if(is.null(pa$lwd)) pa$lwd <- 1
  
  if(arrowed || ellipses) {
    if(is.null(ea$cex)) ea$cex <- 1
    if(is.null(ea$pch)) ea$pch <- 19
    if(is.null(ea$bg)) ea$bg <- NA
    if(is.null(ea$col)) ea$col <- 1
    if(is.null(ea$lty)) ea$lty <- 1
    if(is.null(ea$lwd)) ea$lwd <- 1
    if(!is.null(ea$length) && ea$length < 0.5) ea$length <- 0.9
    if(is.null(ea$length)) ea$length <- 0
    
    if(is.null(pa$xlim)) xLim <- c(min(df$x), max(df$x))
    if(is.null(pa$ylim)) yLim <- c(min(df$y), max(df$y))
  }

  if(ellipses) {
    xLim <- ea$ellipse.points$pc1lim
    yLim <- ea$ellipse.points$pc2lim
  }
  if(!is.null(pa$xlim)) xLim <- pa$xlim
  if(!is.null(pa$ylim)) yLim <- pa$ylim
  
  # 5% span boost for graphics
  xboost <- 0.05 *(xLim[2] - xLim[1])
  yboost <- 0.05 *(yLim[2] - yLim[1])
  
  xLim[1] <- xLim[1] - xboost * abs(xLim[1])
  xLim[2] <- xLim[2] + xboost * abs(xLim[2])
  yLim[1] <- yLim[1] - yboost * abs(yLim[1])
  yLim[2] <- yLim[2] + yboost * abs(yLim[2])

  if(!is.null(pa$asp)){
    g <- ggplot(df, aes(x, y)) + 
      labs(title = pa$main, x = pa$xlab, y = pa$ylab) +
      xlim(xLim) + ylim(yLim) + coord_fixed(ratio = pa$asp)
  } else {
    g <- ggplot(df, aes(x, y)) + 
      labs(title = pa$main, x = pa$xlab, y = pa$ylab) +
      xlim(xLim) + ylim(yLim) 
    
  }

  if(ellipses) {
    ep <- ea$ellP
    k <- dim(ep)[3]
    for(i in 1:k){
      pts <- as.data.frame(ep[,,i])
      colnames(pts) <- c("x", "y")
      g <- g + geom_polygon(data = pts, aes(x, y), linetype = ea$lty, size = ea$lwd,
                         color = ea$col, fill=NA)
    }
  }
  
  if(arrowed){
    check <- match(names(df), c("x", "y", "xb", "yb"))
    if(length(check) == 4) {
      g <- g + geom_errorbar(data = df, aes(xmin = x - xb, 
                                            xmax = x + xb), 
                             size = ea$lwd,
                             linetype = ea$lty,
                             color = ea$col,
                             width = ea$length
      )
      g <- g + geom_errorbar(data = df, aes(ymin = y - yb,
                                            ymax = y + yb), 
                             size = ea$lwd,
                             linetype = ea$lty,
                             color = ea$col,
                             width = ea$length
      )
    }

    
    if(length(check) == 3) {
      if(check[3] == 3)
        g <- g + geom_errorbar(data = df, aes(xmin = x - xb, 
                                              xmax = x + xb),
                               size = ea$lwd,
                               linetype = ea$lty,
                               color = ea$col,
                               width = ea$length
        )
      if(check[3] == 4)
        g <- g + geom_errorbar(data = df, aes(ymin = y - yb, 
                                              ymax = y + yb),
                               size = ea$lwd,
                               linetype = ea$lty,
                               color = ea$col,
                               width = ea$length
        )
    }
  }
  
  pts <- data.frame(x = pa$x, y = pa$y)
  
  g <- g + geom_point(data = pts, aes(x, y), 
                      shape = pa$pch, size = pa$cex * 5, col = pa$col, fill = pa$bg) 
  g
}

### Functions that both RRPP and geomorph use, but should remain internal
### any alterations to these functions must be saved for both RRPP and geomorph

# get.names
# a universal subject name search
# used in lm.rrpp and its descendants
get.names <- function(Y) {
  nms <- if(is.vector(Y)) names(Y) else if(inherits(Y, "dist")) attr(Y, "Labels") else
    if(inherits(Y, "matrix")) rownames(Y) else dimnames(Y)[[3]]
  nms
}

# center
# centers a matrix faster than scale()
# used in various functions where mean-centering is required
center <- function(x){
  if(is.vector(x)) x - mean(x) else {
    x <- as.matrix(x)
    dims <- dim(x)
    fast.center(x, dims[1], dims[2])
  }
}

fast.center <- function(x, n, p){
  m <- colMeans(x)
  x - rep.int(m, rep_len(n, p))
}

fast.scale <- function(x, n, p){
  if(p > 1) {
    x <- fast.center(x, n, p)
    scale <- apply(x, 2, sd)
    x / rep.int(scale, rep_len(n, p))
  } else {
    x <- x - mean(x)
    x/sd(x)
  }
}

# csize
# calculates centroid size
# digitsurface
csize <- function(x) sqrt(sum(center(as.matrix(x))^2))

# cs.scale
# divide matrices by centroid size
# used in other functions for gpagen
cs.scale <- function(x) x/csize(x)

# center.scale
# center and divide matrices by centroid size; faster than scale()
# used in other functions for gpagen
center.scale <- function(x) {
  x <- center(x)
  cs <- sqrt(sum(x^2))
  y <- x/cs
  list(coords=y, CS=cs)
}

# apply.pPsup
# applies a partial Procrustes superimposition to matrices in a list
# used in gpagen functions
apply.pPsup<-function(M, Ya, rot.pts = NULL) {	# M = mean (reference); Ya all Y targets
  dims <- dim(Ya[[1]])
  k <- dims[2]; p <- dims[1]; n <- length(Ya)
  if(is.null(rot.pts)) rot.pts <- 1:p
  M <- cs.scale(M)
  lapply(1:n, function(j){
    y <- Ya[[j]]
    MY <- crossprod(M[rot.pts,], y[rot.pts, ])
    sv <- La.svd(MY,k,k)
    u <- sv$u; u[,k] <- u[,k]*determinant(MY)$sign
    tcrossprod(y,u%*%sv$vt)
  })
}

# fast.ginv
# same as ginv, but without traps (faster)
# used in any function requiring a generalized inverse
fast.ginv <- function(X, tol = sqrt(.Machine$double.eps)){
  k <- ncol(X)
  if(inherits(X, "matrix")) {
    Xsvd <- La.svd(X, k, k)
    Positive <- Xsvd$d > max(tol * Xsvd$d[1L], 0)
    rtu <-((1/Xsvd$d[Positive]) * t(Xsvd$u[, Positive, drop = FALSE]))
    v <-t(Xsvd$vt)[, Positive, drop = FALSE]
  } else {
    Xsvd <- svd(X, k, k)
    Positive <- Xsvd$d > max(tol * Xsvd$d[1L], 0)
    rtu <-((1/Xsvd$d[Positive]) * t(Xsvd$u[, Positive, drop = FALSE]))
    v <-Xsvd$v[, Positive, drop = FALSE]
  }

  v %*% rtu
}

# fast.solve
# same as solve, but without traps (faster)
# used in any function requiring a generalized inverse
fast.solve <- function(x) { 
  res <- try(solve(x), silent = TRUE)
  if(inherits(res, "try-error")) res <- fast.ginv(x)
  return(res)
}


# pcoa
# acquires principal coordinates from distance matrices
# used in all linear model functions with data input
pcoa <- function(D){
  options(warn=-1)
  if(!inherits(D, "dist")) stop("function only works with distance matrices")
  cmd <- cmdscale(D, k=attr(D, "Size") -1, eig=TRUE)
  options(warn=0)
  d <- cmd$eig
  min.d <- min(d)
  if(min.d < 0) {
    options(warn=-1)
    cmd.c <- cmdscale(D, k=attr(D, "Size") -1, eig=TRUE, add= TRUE)
    options(warn=0)
    d <- cmd.c$eig
  } else cmd.c <- cmd
  p <- length(cmd.c$eig[zapsmall(d) > 0])
  Yp <- cmd.c$points[,1:p]
  Yp
}


# perm.index
# creates a permutation index for resampling
# used in all functions with a resampling procedure
perm.index <-function(n, iter, block = NULL, seed = NULL){
  
  if(!is.null(block)){
    block <- as.factor(block)
    if(length(block) != n)
      stop("block factor not the same length as the number of observations.\n",
           call. = FALSE)
    n <- length(block)
    indx <- 1:n
    g <- nlevels(block)
    gp <- levels(block)
    loc.list <- lapply(1:g, function(j) {
      indx[block == gp[[j]]]
    })
    rindx <- unlist(loc.list)
    
  }
  
  if(is.null(seed)) seed = iter else
    if(seed == "random") seed = sample(1:iter,1) else
      if(!is.numeric(seed)) seed = iter
      set.seed(seed)
      get.samp <- function(n){
        if(!is.null(block)){
          out <- array(NA, n)
          r <- unlist(lapply(loc.list, sample, replace = FALSE))
          for(i in indx) out[rindx[i]] <- r[i]
        } else  out <- sample.int(n, n)
        out
      }
      ind <- c(list(1:n), (Map(function(x) get.samp(n), 1:iter)))
      rm(.Random.seed, envir=globalenv())
      attr(ind, "seed") <- seed
      names(ind) <- c("obs", paste("iter", 1:(length(ind) - 1), sep = "."))
      ind
      
}


# boot.index
# creates a bootstrap index for resampling
# used in lm.rrpp for intercept models
boot.index <-function(n, iter, block = NULL, seed = NULL){
  
  if(!is.null(block)){
    block <- as.factor(block)
    if(length(block) != n)
      stop("block factor not the same length as the number of observations.\n",
           call. = FALSE)
    n <- length(block)
    indx <- 1:n
    g <- nlevels(block)
    gp <- levels(block)
    loc.list <- lapply(1:g, function(j) {
      indx[block == gp[[j]]]
    })
    rindx <- unlist(loc.list)
  }
  
  if(is.null(seed)) seed = iter else
    if(seed == "random") seed = sample(1:iter,1) else
      if(!is.numeric(seed)) seed = iter
      set.seed(seed)
      get.samp <- function(n){
        if(!is.null(block)){
          out <- array(NA, n)
          r <- unlist(lapply(loc.list, sample, replace = TRUE))
          for(i in indx) out[rindx[i]] <- r[i]
        } else  out <- sample.int(n, n, replace = TRUE)
        out
      }
      ind <- c(list(1:n), (Map(function(x) get.samp(n), 
                              1:iter)))
      rm(.Random.seed, envir=globalenv())
      attr(ind, "seed") <- seed
      ind
}

# fastFit
# calculates fitted values for a linear model, after decomoposition of X to get U
# used in SS.iter
fastFit <- function(U,y,n,p){
  if(p > n) tcrossprod(U) %*% y else
    U %*% crossprod(U,y)
} 


# fastLM
# calculates fitted values and residuals, after fastFit
# placeholder in case needed later
fastLM<- function(U, y){
  p <- dim(y)[2]; n <- dim(y)[1]
  yh <- fastFit(U,y,n,p)
  list(fitted = yh, residuals = y-yh)
}

# Pval.matrix
# P-values form random outcomes that comprise matrices
# any analytical function with results in matrices
Pval.matrix = function(M){
  P = matrix(0,dim(M)[1],dim(M)[2])
  for(i in 1:dim(M)[1]){
    for(j in 1:dim(M)[2]){
      y = M[i,j,]
      p = pval(y)
      P[i,j]=p
    }
  }
  if(dim(M)[1] > 1 && dim(M)[2] >1) diag(P)=1
  rownames(P) = dimnames(M)[[1]]
  colnames(P) = dimnames(M)[[2]]
  P
}

# Effect.size.matrix
# Effect sizes form random outcomes that comprise matrices
# any analytical function with results in matrices
Effect.size.matrix <- function(M, center=TRUE){
  Z = matrix(0,dim(M)[1],dim(M)[2])
  for(i in 1:dim(M)[1]){
    for(j in 1:dim(M)[2]){
      y = M[i,j,]
      z = effect.size(y, center=center)
      Z[i,j]=z
    }
  }
  if(dim(M)[1] > 1 && dim(M)[2] >1) diag(Z)=0
  rownames(Z) = dimnames(M)[[1]]
  colnames(Z) = dimnames(M)[[2]]
  Z
}


# Cov.proj
# generates projection matrix from covariance matrix
# used in lm.rrpp

Cov.proj <- function(Cov, id = NULL, symmetric = FALSE){
  Cov <- if(is.null(id)) Cov else Cov[id, id]
  if(inherits(Cov, "matrix")) {
    Cov.s <- Matrix(Cov, sparse = TRUE)
    if(object.size(Cov.s) < object.size(Cov)) Cov <- Cov.s
    Cov.s <- NULL
  }
  ow <- options()$warn
  options(warn = -1)
  if(!symmetric) {
    Chol <- try(chol(Cov), silent = TRUE)
    if(inherits(Chol, "try-error")) 
      symmetric <- TRUE
  }
  
 if(symmetric) {
   sym <- isSymmetric(Cov)
   eigC <- eigen(Cov, symmetric = sym)
   eigC.vect = t(eigC$vectors)
   L <- eigC.vect * sqrt(abs(eigC$values))
   P <- fast.solve(crossprod(L, eigC.vect))
   dimnames(P) <- dimnames(Cov)
 } else P <- solve(t(Chol))
  options(warn = ow)
  P
}



# ape replacement functions below --------------------------------------------

# sim.char has a similar function to this but it is called in every simulation
# and defers to C for help.  This is done once only here. 
# (Produces a projection matrix)
phy.sim.mat <- function(phy) {
  if(is.null(phy$edge.length))
    stop("The tree has no branch lengths.\n", call. = FALSE)
  N <- length(phy$tip.label)
  n <- nrow(phy$edge)
  m <- matrix(0, N, n)
  edg <- phy$edge.length
  idx <- phy$edge[, 2]
  anc <- phy$edge[, 1]
  tips <- which(idx <= N)
  non.tips <- which(idx > N)
  for(i in 1:length(tips)) m[idx[tips[i]], tips[i]] <- sqrt(edg[tips[i]])
  for(i in 1:length(non.tips)) {
    x <- idx[non.tips[i]]
    anc.i <- which(anc == x)
    edg.i <- idx[anc.i]
    if(any(edg.i > N)) {
      edg.i <- as.list(edg.i)
      while(any(edg.i > N)) {
        edg.i <- lapply(1:length(edg.i), function(j){
          edg.i.j <- edg.i[[j]]
          if(edg.i.j > N) {
            idx[which(anc == edg.i.j)]
          } else edg.i.j <- edg.i.j
        })
        edg.i <- unlist(edg.i)
      }
    }
    m[edg.i, which(idx == x)] <- sqrt(edg[which(idx == x)])
  }
  rownames(m) <- phy$tip.label
  m
}

# fast.phy.vcv
# same as vcv.phylo but without options, in order to not use ape

fast.phy.vcv <- function (phy) {
  x <- phy.sim.mat(phy)
  tcrossprod(x)
}

# reorder.phy
# same as reorder function, but without options

reorder.phy <- function(phy){
  edge <- phy$edge
  edge.length <- phy$edge.length
  edge <- cbind(edge, edge.length)
  n <- nrow(edge)
  ind <-rank(edge[,1], ties.method = "last")
  edge <- edge[order(ind, decreasing = TRUE), ]
  edge.length <- edge[,3]
  phy$edge <- edge[,-3]
  phy$edge.length <- edge.length
  attr(phy, "order") <- "postorder"
  return(phy)
}


# anc.BM 
# via PICs
# same as ace, but multivariate
pic.prep <- function(phy, nx, px){
  phy <- reorder.phy(phy)
  ntip <- length(phy$tip.label)
  nnode <- phy$Nnode
  edge <- phy$edge
  edge1 <- edge[, 1]
  edge2 <- edge[,2]
  edge_len <- phy$edge.length
  phe <- matrix(0, ntip + nnode, px)
  contr <- matrix(0, nnode, px)
  var_contr <- rep(0, nnode)
  i.seq <- seq(1, ntip * 2 -2, 2)
  list(ntip = ntip, nnode = nnode, edge1 = edge1,
       edge2 = edge2, edge_len = edge_len, phe = phe,
       contr = contr, var_contr = var_contr, 
       tip.label = phy$tip.label,
       i.seq = i.seq)
}

ace.pics <- function(ntip, nnode, edge1, edge2, edge_len, phe, contr,
                     var_contr, tip.label, i.seq, x) {
  phe[1:ntip, ] <- if (is.null(rownames(x))) x else 
    x[tip.label, , drop = FALSE]
  N <- ntip + nnode
  for(ii in 1:nnode) {
    anc <- edge1[i.seq[ii]]
    ij <- which(edge1 == anc)
    i <- ij[1]
    j <- ij[2]
    d1 <- edge2[i]
    d2 <- edge2[j]
    sumbl <- edge_len[i] + edge_len[j]
    ic <- anc - ntip 
    ya <- (phe[d1,] - phe[d2,])/sqrt(sumbl)
    contr[ic, ] <- ya
    var_contr[ic] <- sumbl
    phe[anc,] <- (phe[d1, , drop = FALSE] * edge_len[j] + 
                    phe[d2, , drop = FALSE] * edge_len[i])/sumbl
    k <- which(edge2 == anc)
    edge_len[k] <- edge_len[k] + edge_len[i] * edge_len[j] / sumbl
  }
  phe
}

# anc.BM
# multivariate as opposed to fastAnc
# uses Ho and Ane (2014) algorithm

anc.BM <- function(phy, Y){
  phy <- reorder.phy(phy)
  Y <- as.matrix(Y)
  N <- length(phy$tip.label)
  Y <- as.matrix(Y[phy$tip.label, , drop = FALSE])
  edge <- cbind(phy$edge, phy$edge.length)
  ind <-rank(edge[,1], ties.method = "last")
  edge <- edge[order(ind, decreasing = TRUE), , drop = FALSE]
  ev <- edge[,3]
  anc <- edge[, 1]
  des <- edge[, 2]
  ne <- nrow(edge)
  if (is.null(phy$node.label)) {
    phy$node.label <- (length(phy$tip.label) + 1):(length(phy$tip.label) + 
                                                     phy$Nnode)
  }
  nl <- phy$node.label
  rm(phy, edge)
  
  Z <- matrix(0, ne + 1, NCOL(Y))
  p <- rep(0, ne + 1)
  
  for(i in 1:ne){
    a <- anc[i] 
    d <- des[i] 
    len <- ev[i] 
    
    if(d <= N){
      p[d] <- 1 / len
      Z[d, ] <- Y[d, , drop = FALSE]
    } else {
      pA <- p[d]
      Z[d, ] <- Z[d, , drop = FALSE] / pA
      p[d] <- pA / (1 + len * pA)
    }
    
    p[a] <- p[a] + p[d]
    Z[a, ] <- Z[a, , drop = FALSE] + Z[d, , drop = FALSE] * p[d]
  }
  
  Z[a, ] <- Z[a, , drop = FALSE] / p[a]
  
  for(i in ne:1){
    a <- anc[i] 
    d <- des[i] 
    len <- ev[i]
    
    if(d > N) {
      Z[d, ] <-  Z[d, , drop = FALSE] * p[d] * len + 
        Z[a, ] - Z[a, , drop = FALSE] * p[d] * len
    }
  }
  Z <- Z[-(1:N), , drop = FALSE]
  rownames(Z) <- nl
  if(!is.null(colnames(Y)))
    colnames(Z) <- colnames(Y)
  
  Z
  
}

# getNode Depth
# replaces node.depth.edgelength

getNodeDepth <- function(phy){
  phy <- reorder.phy(phy)
  E <- phy$edge
  anc <- E[,1]
  des <- E[,2]
  ntip <- length(phy$tip.label)
  nnode <- phy$Nnode
  N <- ntip + nnode
  L <- phy$edge.length
  base.tax <- ntip + 1
  full.depth.seq <- (ntip + 1):N
  full.node.depth <- 0
  tips <- which(des <= ntip)
  non.tips <- which(des > ntip)
  
  get.edge.ind <- function(tax){
    root.t <- ntip +1
    des.i <- which(des == tax)
    edge <- numeric()
    tax.i <- tax
    while(tax.i != root.t) {
      anc.i <- anc[which(des == tax.i)]
      tax.i <- anc[des.i]
      edge <- c(edge, des.i)
      des.i <- which(des == anc.i)
    }
    edge
  }
  
  tips.taxa <- lapply(as.list(tips), function(j) des[j])
  tips.edges <- lapply(tips.taxa, get.edge.ind)
  tips.depths <- sapply(1:ntip, function(j) sum(L[tips.edges[[j]]]))
  
  nodes <- as.list((ntip + 1):(N))
  nodes.edges <- lapply(nodes, get.edge.ind)
  nodes.depths <- sapply(1:nnode, function(j) sum(L[nodes.edges[[j]]]))
  
  c(tips.depths, nodes.depths)
}

#' Reveal model designs used in lm.rrpp fit
#'
#' Function returns every full and reduced model for model terms used in 
#' lm.rrpp fits.  This function is useful for revealing 
#' the null and full model that would be used in the pairwise function, 
#' if a specific null model is not declared as an argument
#' (fit.null in the \code{\link{pairwise}} function).
#' It also helps to demonstrate how sums of squares and cross-products 
#' (SSCP) are calculated in lm.rrpp permutations (iterations),
#' from the difference between fitted values for null and full designs.
#' 
#' @param fit A linear model fit from \code{\link{lm.rrpp}}.
#' @keywords analysis
#' @export
#' @author Michael Collyer
#' @examples
#'
#'data(Pupfish)
#'fit1 <- lm.rrpp(coords~ Pop*Sex, data = Pupfish, 
#'SS.type = "I", print.progress = FALSE, iter = 0)
#'fit2 <- lm.rrpp(coords~ Pop*Sex, data = Pupfish, 
#'SS.type = "II", print.progress = FALSE, iter = 0)
#'fit3 <- lm.rrpp(coords~ Pop*Sex, data = Pupfish, 
#'SS.type = "III", print.progress = FALSE, iter = 0)
#'
#'reveal.model.designs(fit1)
#'reveal.model.designs(fit2)
#'reveal.model.designs(fit3)
#' 
reveal.model.designs <- function(fit) {
  if(inherits(fit, "measurement.error")) fit <- fit$all.stats
  model.sets <- getModels(fit, attribute = "all")
  terms.f <- lapply(model.sets$full, function(x) x$terms)
  terms.r <- lapply(model.sets$reduced, function(x) x$terms)
  forms.r <- lapply(terms.r, function(x) formula(x)[[3]])
  forms.f <- lapply(terms.f, function(x) formula(x)[[3]])
  reduced <- lapply(forms.r, function(x) Reduce(paste, deparse(x)))
  full<- lapply(forms.f, function(x) Reduce(paste, deparse(x)))
  term.labels <- names(forms.f)
  k <- length(term.labels)
  blank <- target <- rep("", k)
  target[k] <- "<- Null/Full inherent in pairwise"
  df <- as.data.frame(cbind(blank, reduced = reduced, blank, full = full, target))
  names(df) <- c("", "Reduced", "", "Full", "")
  return(df)
}

#' Scaling of a Covariance Matrix
#'
#' Function performs linear and exponential scaling of a covariance, 
#' either including or excluding diagonals or off-diagonal elements.  
#' 
#' The function scales covariances as scale * cov ^exponent, where cov is any 
#' covariance or variance in the covariance matrix.  Arguments allow inclusion
#' or exclusion or either the diagonal or off-diagonal elements to be scaled.  It is 
#' assumed that a covariance matrix is scaled, but any square matrix will work.
#' @param Cov Square matrix to be scaled.
#' @param scale. The linear scaling parameter.  Values are multiplied by this numeric value.
#' @param exponent The exponentiation scaling parameter.  Values are raised to this power.
#' @param scale.diagonal Logical to indicate if diagonal should be included.
#' @param scale.only.diagonal Logical to indicate if only the diagonal should be scaled.
#' 
#' @keywords analysis
#' @export
#' @author Michael Collyer
#' @return A square matrix.
#' @examples 
#' \dontrun{
#' data(Pupfish)
#' Pupfish$logSize <- log(Pupfish$CS)
#' fit1 <- lm.rrpp(coords ~ logSize, data = Pupfish, iter = 0, 
#' print.progress = FALSE)
#' fit2 <- lm.rrpp(coords ~ Pop, data = Pupfish, iter = 0, 
#' print.progress = FALSE)
#' fit3 <- lm.rrpp(coords ~ Sex, data = Pupfish, iter = 0, 
#' print.progress = FALSE)
#' fit4 <- lm.rrpp(coords ~ logSize + Sex, data = Pupfish, iter = 0, 
#' print.progress = FALSE)
#' fit5 <- lm.rrpp(coords ~ logSize + Pop, data = Pupfish, iter = 0, 
#' print.progress = FALSE)
#' fit6 <- lm.rrpp(coords ~ logSize + Sex * Pop, data = Pupfish, iter = 0, 
#' print.progress = FALSE)
#' 
#' modComp1 <- model.comparison(fit1, fit2, fit3, fit4, fit5, 
#' fit6, type = "cov.trace")
#' modComp2 <- model.comparison(fit1, fit2, fit3, fit4, fit5, 
#' fit6, type = "logLik", tol = 0.01)
#' 
#' summary(modComp1)
#' summary(modComp2)
#' 
#' par(mfcol = c(1,2))
#' plot(modComp1)
#' plot(modComp2)
#' 
#' # Comparing fits with covariance matrices
#' # an example for scaling a phylogenetic covariance matrix with
#' # the scaling parameter, lambda
#' 
#' data("PlethMorph")
#' Cov <- PlethMorph$PhyCov
#' lambda <- seq(0, 1, 0.1)
#' 
#' Cov1 <- scaleCov(Cov, scale. = lambda[1])
#' Cov2 <- scaleCov(Cov, scale. = lambda[2])
#' Cov3 <- scaleCov(Cov, scale. = lambda[3])
#' Cov4 <- scaleCov(Cov, scale. = lambda[4])
#' Cov5 <- scaleCov(Cov, scale. = lambda[5])
#' Cov6 <- scaleCov(Cov, scale. = lambda[6])
#' Cov7 <- scaleCov(Cov, scale. = lambda[7])
#' Cov8 <- scaleCov(Cov, scale. = lambda[8])
#' Cov9 <- scaleCov(Cov, scale. = lambda[9])
#' Cov10 <- scaleCov(Cov, scale. = lambda[10])
#' Cov11 <- scaleCov(Cov, scale. = lambda[11])
#' 
#' 
#' fit1 <- lm.rrpp(SVL ~ 1, data = PlethMorph, Cov = Cov1)
#' fit2 <- lm.rrpp(SVL ~ 1, data = PlethMorph, Cov = Cov2)
#' fit3 <- lm.rrpp(SVL ~ 1, data = PlethMorph, Cov = Cov3)
#' fit4 <- lm.rrpp(SVL ~ 1, data = PlethMorph, Cov = Cov4)
#' fit5 <- lm.rrpp(SVL ~ 1, data = PlethMorph, Cov = Cov5)
#' fit6 <- lm.rrpp(SVL ~ 1, data = PlethMorph, Cov = Cov6)
#' fit7 <- lm.rrpp(SVL ~ 1, data = PlethMorph, Cov = Cov7)
#' fit8 <- lm.rrpp(SVL ~ 1, data = PlethMorph, Cov = Cov8)
#' fit9 <- lm.rrpp(SVL ~ 1, data = PlethMorph, Cov = Cov9)
#' fit10 <- lm.rrpp(SVL ~ 1, data = PlethMorph, Cov = Cov10)
#' fit11 <- lm.rrpp(SVL ~ 1, data = PlethMorph, Cov = Cov11)
#' 
#' par(mfrow = c(1,1))
#' 
#' MC1 <- model.comparison(fit1, fit2, fit3, fit4, fit5, fit6,
#' fit7, fit8, fit9, fit10, fit11,
#' type = "logLik")
#' MC1
#' plot(MC1)
#' 
#' MC2 <- model.comparison(fit1, fit2, fit3, fit4, fit5, fit6,
#' fit7, fit8, fit9, fit10, fit11,
#' type = "logLik", predictor = lambda)
#' MC2
#' plot(MC2)
#' 
#' 
#' MC3 <- model.comparison(fit1, fit2, fit3, fit4, fit5, fit6,
#' fit7, fit8, fit9, fit10, fit11,
#' type = "Z", predictor = lambda)
#' MC3
#' plot(MC3)
#' }
scaleCov <- function(Cov, scale. = 1, exponent = 1, 
                     scale.diagonal = FALSE, 
                     scale.only.diagonal = FALSE) {
  
  dims <- dim(Cov)
  if(length(dims) != 2)
    stop("Cov must be a matrix", call. = FALSE)
  if(dims[1] != dims[2])
    stop("Cov must be a square matrix", call. = FALSE)
  if(length(scale.) != 1)
    stop("The scale. argument must be a single numeric value.", 
         call. = FALSE)
  if(length(exponent) != 1)
    stop("The exponent argument must be a single numeric value.", 
         call. = FALSE)
  if(!is.numeric(scale.) || !is.numeric(exponent))
    stop("Scaling parameters must be numeric.", 
         call. = FALSE)
  
  D <- diag(diag(Cov))
  C <- Cov - D
  if(scale.only.diagonal) {
    D <- scale. * D^(exponent)
  } else {
    C <- scale. * C^(exponent)
    if(scale.diagonal) D <- scale. * D^(exponent)
  }
  
  C + D
}

#' @name RRPP-package
#' @aliases RRPP
#' @title Linear Model Evaluation with Randomized Residual Permutation Procedures
#' @author Michael Collyer and Dean Adams
#' @return Key functions for this package:
#' \item{\code{\link{lm.rrpp}}}{Fits linear models, using RRPP.
#' plus model comparisons.}
#' \item{\code{\link{coef.lm.rrpp}}}{Extract coefficients or perform test 
#' on coefficients, using RRPP.}
#' \item{\code{\link{predict.lm.rrpp}}}{Predict values from lm.rrpp fits 
#' and generate bootstrapped confidence intervals.}
#' \item{\code{\link{pairwise}}}{Perform pairwise tests, based on lm.rrpp 
#' model fits.}
#' 
#' @description Functions in this package allow one to evaluate linear models 
#' with residual randomization.
#' The name, "RRPP", is an acronym for, "Randomization of Residuals in a 
#' Permutation Procedure."  Through
#' the various functions in this package, one can use randomization of 
#' residuals to generate empirical probability
#' distributions for linear model effects, for high-dimensional data or 
#' distance matrices.
#' 
#' An especially useful option of this package is to fit models with 
#' either ordinary or generalized
#' least squares estimation (OLS or GLS, respectively), using theoretic 
#' covariance matrices.  Mixed linear
#' effects can also be evaluated.
#' 
#' @import parallel
#' @import ggplot2 
#' @import Matrix
#' @import lme4
#' @importFrom ape multi2di.phylo
#' @importFrom ape root.phylo
#' @importFrom ape collapse.singles
#' @importFrom stats na.omit anova as.dist as.formula cmdscale coef delete.response dist formula lm lm.fit lm.wfit loess logLik model.frame.default model.matrix optimise prcomp qnorm quantile resid sd spline var model.frame terms reformulate fitted resid
#' @importFrom graphics abline arrows axis legend lines par plot.default points text title
#' @importFrom utils combn object.size setTxtProgressBar txtProgressBar
#' @importFrom stats pnorm terms.formula
#' @export print.lm.rrpp
#' @export summary.lm.rrpp
#' @export print.summary.lm.rrpp
#' @export print.anova.lm.rrpp
#' @export summary.anova.lm.rrpp
#' @export print.coef.lm.rrpp
#' @export summary.coef.lm.rrpp
#' @export print.predict.lm.rrpp
#' @export summary.predict.lm.rrpp
#' @export plot.lm.rrpp
#' @export plot.predict.lm.rrpp
#' @export print.pairwise
#' @export summary.pairwise
#' @export print.summary.pairwise
#' @export print.trajectory.analysis
#' @export summary.trajectory.analysis
#' @export print.summary.trajectory.analysis
#' @export print.model.comparison
#' @export summary.model.comparison
#' @export print.ICCstats
#' @export summary.ICCstats
#' @export print.pairwise.model.Z
#' @export summary.pairwise.model.Z
#' @export plot.ordinate
#' @export plot.kcomp
#' @export print.ICCstats
#' @export summary.ICCstats
#' @export print.betaTest
#' @export summary.betaTest
#' @export print.summary.betaTest
#' @export ranef.lmm.rrpp
#' @export fixef.lmm.rrpp


NULL

#' @section RRPP TOC:
#' @docType package
#' RRPP-package
 
NULL

#####----------------------------------------------------------------------------------------------------

# HELP FUNCTIONs

#' Create a data frame for lm.rrpp analysis
#'
#' Create a data frame for lm.rrpp analysis, when covariance or distance 
#' matrices are used
#'
#' This function is not much different than \code{\link{data.frame}} but is 
#' more flexible to allow
#' distance matrices and covariance matrices to be included.  Essentially, 
#' this function creates a list,
#' much like an object of class \code{data.frame} is also a list.  However, 
#' \code{rrpp.data.frame} is
#' less concerned with coercing the list into a matrix and more concerned 
#' with matching the number of observations (n).
#' It is wise to use this function with any \code{lm.rrpp} analysis so that 
#' \code{\link{lm.rrpp}} does not have to search
#' the global environment for needed data.
#'
#' It is assumed that multiple data sets for the same subjects are in the 
#' same order.
#'
#' See \code{\link{lm.rrpp}} for examples.
#'
#' @param ... Components (objects) to combine in the data frame.
#' @export
#' @author Michael Collyer
#' @examples
#' # Why use a rrpp.data.frame?
#' y <- matrix(rnorm(30), 10, 3)
#' x <- rnorm(10)
#' df <- data.frame(x = x, y = y)
#' df
#' rdf <- rrpp.data.frame(x = x, y = y)
#' rdf # looks more like a list
#' 
#' is.list(df)
#' is.list(rdf)
#' 
#' d <- dist(y) # distance matrix as data
#' 
#' # One can try this but it will result in an error
#' # df <- data.frame(df, d = d) 
#' rdf <- rrpp.data.frame(rdf, d = d) # works
#' 
#' fit <- lm.rrpp(d ~ x, data = rdf, iter = 99)
#' summary(fit)

rrpp.data.frame<- function(...) {
  dots <- list(...)
  if(length(dots) == 1 && is.data.frame(dots[[1]])) {
    dots <- dots[[1]]
    class(dots) <- "rrpp.data.frame"
  } else if(length(dots) == 1 && inherits(dots[[1]], "geomorph.data.frame")) {
    dots <- dots[[1]]
    
    warning(
      paste(
        "\nThis is not an error!  It is a friendly warning.\n",
        "\nSome geomorph.data.frame objects might not be compatible with RRPP functions.",
        "\nIf any part of the geomorph.data.frame conatins a 3D array,",
        "consider converting it to a matrix before attempting to make an rrpp.data.frame\n",
        "Use suppressWarnings() to turn off these warnings.\n\n", sep = " "),
      noBreaks. = TRUE, call. = FALSE, immediate. = TRUE) 

    class(dots) <- "rrpp.data.frame"
  } else if(length(dots) == 1 && inherits(dots[[1]], "rrpp.data.frame")) {
    dots <- dots[[1]]
  } else {
    if(length(dots) > 1 && inherits(dots[[1]], "rrpp.data.frame")) {
      dots1 <- dots[[1]]
      dots2 <- dots[-1]
      dots <- c(dots1, dots2)
    }
    N <- length(dots)
    dots.ns <- array(NA,N)
    for(i in 1:N){
      if(is.array(dots[[i]])) {
        if(length(dim(dots[[i]])) == 3) dots.ns[i] <- dim(dots[[i]])[[3]]
        if(length(dim(dots[[i]])) == 2) dots.ns[i] <- dim(dots[[i]])[[2]]
        if(length(dim(dots[[i]])) == 1) dots.ns[i] <- dim(dots[[i]])[[1]]
      }
      if(inherits(dots[[i]], "matrix")) {
        dots.ns[i] <- dim(dots[[i]])[[1]]
        dt.nms <- rownames(dots[[i]])
        if(dots.ns[i] == length(dots[[i]])) {
          dots[[i]] <- as.vector(dots[[i]])
          names(dots[[i]]) <- dt.nms
        }
      }
      
      if(inherits(dots[[i]], "dist")) dots.ns[i] <- attr(dots[[i]], "Size")
      if(is.data.frame(dots[[i]])) dots.ns[i] <- dim(dots[[i]])[[1]]
      if(is.vector(dots[[i]])) dots.ns[i] <- length(dots[[i]])
      if(is.factor(dots[[i]])) dots.ns[i] <- length(dots[[i]])
      if(is.logical(dots[[i]])) dots.ns[i] <- length(dots[[i]])
    }
    if(any(is.na(dots.ns))) 
      stop("Some input is either dimensionless or inappropriate for data frames")
    if(length(unique(dots.ns)) > 1) 
      stop("Inputs have different numbers of observations")
    class(dots) <- c("rrpp.data.frame")
  }

  dots
}

#####--------------------------------------------------------------\

# SUPPORT FUNCTIONS

# Parallel.setup
# Function to help parallel library adjust to settings
# for parallel processing

Parallel.setup <- function(Parallel){
  
  ParLog <- is.logical(Parallel)
  usecluster <- inherits(Parallel, "cluster")
  if(usecluster){
    cluster <- Parallel
    ParLog <- Parallel <- TRUE
  } else cluster <- FALSE
  ParCores <- NULL
  
  if(is.numeric(Parallel)) {
    ParCores <- Parallel
    ParLog <- TRUE
    Parallel <- TRUE
  }
  
  if(ParLog && is.null(ParCores)) {
    ParCores <- detectCores() - 1
    if(usecluster) ParCores <- length(cluster)
    if(!Parallel) ParCores <- 1
  }
  
  if(is.numeric(ParCores)) {
    if(ParCores > detectCores() - 1) ParCores <- detectCores() - 1
  }
  
  Unix <- .Platform$OS.type == "unix"
  forking <- Unix && !usecluster
  
  if(is.null(ParCores)) ParCores <- 1
  
  if(ParCores == 1) {
    ParLog <- FALSE
    forking <- FALSE
    usecluster <- FALSE
    cluster <- NULL
  }
    
  if(ParCores > 1) {
    if(!Unix && !usecluster)
      cluster <- makeCluster(ParCores)
    if(Unix && usecluster)
      Unix <- FALSE
  }
  
  list(Parallel = ParLog,
       Unix = Unix, 
       forking = forking, 
       ParCores = ParCores, 
       usecluster = usecluster,
       cluster = cluster)
  
}

# lm.rrpp subfunctions
# lm-like fit modified for all submodels
# general workhorse for all 'lm.rrpp' functions

get.names.from.list <- function(L) {
  temp <- lapply(L, get.names)
  check <- which(!sapply(temp, is.null))
  temp <- if(length(check) > 0) temp[check] else NULL
  nms <- if(!is.null(temp)) temp[[1]] else NULL
  nms
}

add.names <- function(Y, nms) {
  if(is.vector(Y)) names(Y) <- nms
  if(inherits(Y, "matrix")) rownames(Y) <- nms
  if(inherits(Y, "dist")) attr(Y, "Labels") <- nms
  Y
}

makeDF <- function(form, data, n, nms) {
  
  if(!is.list(data)) 
    stop("\nThe data frame provide is not class rrpp.data.frame, 
         data.frame, or list, and therefore, unusable.\n", call. = FALSE)
  
  dat <- data
  class(dat) <- "list"
  
  form <- try(as.formula(form), silent = TRUE)
  if(inherits(form, "try-error"))
    stop("Formula is not coercible into a formula object.  
         Please fix the formula.\n",
         call. = FALSE)
              
  var.names <- if(length(form) == 3) all.vars(form)[-1] else if(length(form) == 2) 
      all.vars(form) else
        stop("\nFormula is not appropriately formatted.\n", call. = FALSE)
  
  dat <- dat[names(dat) %in% var.names]
  
  if(length(dat) > 0) {
    
    var.check <- sapply(seq_len(length(dat)), function(j) {
      x <- dat[[j]]
      nn <- if(is.matrix(x)) nrow(x) else length(x)
      nn == n
    })
    
    if(any(!var.check)) 
      stop("One or more independent variables does not match the number 
           of observations in the data.\n",
           call. = FALSE)
  }
   
  dat <- if(length(dat) == 0)  NULL else as.data.frame(dat)
  
  if(!is.null(dat)) rownames(dat) <- nms
  
  dat
}
  
lm.args.from.formula <- function(cl){
  
  lm.args <- list(formula = NULL, data = NULL, 
                  subset = NULL, weights = NULL,
                  na.action = na.omit, 
                  method = "qr", model = TRUE, 
                  qr = TRUE,
                  singular.ok = TRUE, 
                  contrasts = NULL, offset = NULL, 
                  tol = 1e-7)
  
  lm.nms <- names(lm.args)
  
  m1 <- match(names(cl), lm.nms)
  m2 <- match(lm.nms, names(cl))
  lm.args[na.omit(m1)] <- cl[na.omit(m2)]
  lm.args$x <- lm.args$y <- TRUE
  
  form <- lm.args$formula
  ko <- cl$keep.order
  if(is.null(form))
    stop("The formula is either missing or not formatted correctly.\n", 
         call. = FALSE)
  
  Dy <- NULL
  Y <- try(eval(lm.args$formula[[2]], lm.args$data, parent.frame()),
           silent = TRUE)
  nmsY <- get.names(Y)
  
  if(inherits(Y, "try-error"))
    stop("Data are missing from either the data frame or global environment.\n", 
         call. = FALSE)
  
  if(is.vector(Y)) {
    Y <- as.matrix(Y)
    Dy <- NULL
  }
  
  if(inherits(Y, "matrix") || is.data.frame(Y)) {
    if(isSymmetric(as.matrix(Y))) {
      Dy <- Y <- as.dist(Y)
      if(any(Dy < 0)) stop("Distances in distance matrix cannot be less than 0\n",
                           call. = FALSE)
    } else Dy <- NULL
  }
  
  if(inherits(Y, "dist")) {
    if(any(Y < 0)) stop("Distances in distance matrix cannot be less than 0")
    Dy <- Y
    Y <- pcoa(Y)
  }
  
  if(is.array(Y) && length(dim(Y)) > 2) 
    stop("Data are arranged in an array rather than a matrix.  
         Please convert data to a matrix. \n", 
         call. = FALSE)
  
  if(form[[3]] == ".") {
    xs <- paste(names(lm.args$data), collapse = "+")
    form <- as.formula(noquote(c("~", xs)))
  }
  
  trms <- attr(terms(form, keep.order = ko), "term.labels")
  if(length(trms) > 1)
    form <- reformulate(trms,
                      response = "Y") else form <- update(form, Y ~ .)
  lm.args$formula <- form
  n <- NROW(Y)
  
  if(!is.null(lm.args$data)) {
    nmsDF <- if(inherits(lm.args$data, "data.frame"))  
      attr(lm.args$data, "row.names") else 
        get.names.from.list(lm.args$data)

    lm.args$data <- makeDF(form, lm.args$data, n, nms = NULL)
    
  }
  
  if(is.null(lm.args$data)) {
    lm.args$data <- data.frame(Int = rep(1, n))
    lm.args$data$Y <- as.matrix(Y)
    lm.args$data <- lm.args$data[-1]
  }
  
  lm.args$data$Y <- Y
  
  Terms <- try(terms.formula(form, data = lm.args$data,
                             keep.order = ko),
               silent = TRUE)
  
  model <- try(model.frame(Terms, data = lm.args$data),
               silent = TRUE)
  
  
  if(inherits(model, "try-error") || inherits(Terms, "try-error"))
  stop("Variables or data might be missing from either the data frame or 
           global environment, or a linear model fit just does not work...\n", 
       call. = FALSE)
  
  Y <- add.names(Y, nmsY)
  out <- list(Terms = Terms, model = model, Y = Y)
  if(!is.null(Dy)) {
    d <- as.matrix(Dy)
    if(nrow(d) != NROW(out$Y)) d <- d[rownames(Y), rownames(Y)]
    out$D <- as.dist(d)
  }
  
  out
}

.getTerms <- function(fit = NULL, Terms = NULL, SS.type = NULL) {
  if(is.null(Terms)) Terms <-  fit$LM$Terms
  if(is.null(SS.type)) SS.type <- fit$ANOVA$SS.type
  if(length(attr(Terms, "factors")) == 0)
    SS.type <- "I"
  trms <- attr(Terms, "term.labels")
  k <- length(trms)
  mod.k <- if(k > 0) c(0, seq(1, k, 1)) else 0
  
  if(!is.null(fit) && SS.type == "Within-subject II") {
    SS.type <- "IIws"
    WS <- TRUE
  } else WS <- FALSE
  
  if(k > 0) {
    if(SS.type == "III"){
      k3 <- mod.k[-1]
      modf <- lapply(as.list(k3), function(.) Terms)
      modr <- lapply(as.list(k3), function(j) Terms[-j])
    } else if(SS.type == "II" || SS.type == "IIws"){
      k2 <- mod.k[-1]
      fac <- crossprod(attr(Terms, "factor"))
      modr <- lapply(as.list(k2), function(j){
        ind <- as.logical(ifelse(fac[j, ] < fac[j,j], 1, 0))
        Terms[ind]
      })
      modf <- lapply(as.list(k2), function(j){
        ind <- ifelse(fac[j, ] < fac[j,j], 1, 0)
        ind[j] <- 1
        ind <- as.logical(ind)
        Terms[ind]
      })
      
      
    } else {
      kf <- mod.k[-1]
      kr <- mod.k[-(max(mod.k) + 1)]
      modf <- lapply(as.list(kf), function(j) Terms[1:j])
      modr <- lapply(as.list(kr), function(j) Terms[0:j])
    }
    
    names(modf) <- names(modr) <- trms
    
    if(WS) {
      if(fit$subTest){
        trms.change <- which(trms == fit$subjects.var)
        modf[[trms.change]] <- Terms
        modr[[trms.change]] <- Terms[!trms %in% fit$subjects.var]
      }
    }
    
  } else {
    modr <- modf <- list("Intercept" = Terms)
  }
  list(terms.r = modr, terms.f = modf)
}

LM.fit <- function(x, y, offset = NULL, tol = 1e-07) {
  if(inherits(x, "matrix")) 
    x.s <- Matrix(x, sparse = TRUE) else {
      x.s <- x
      x <- as.matrix(x.s)
    }
  osx <- length(x)
  osxs <- if(inherits(x.s, "Matrix")) length(x.s@x) else osx
  X <- if(osx < osxs) x else x.s
  x <- x.s <- NULL
  Q <- QRforX(X, tol = tol)
  if(!is.null(offset)) y <- y - offset
  dims <- dim(y)
  n <- dims[1]
  p <- dims[2]
  fitted.values <- fastFit (Q$Q, y, n, p)
  residuals <- y - fitted.values
  if(!is.null(offset)) fitted.values <- fitted.values + offset
  list(qr = Q, fitted.values = as.matrix(fitted.values),
       residuals = as.matrix(residuals))
}

removeRedundant <- function(X){
  QRforX(X)$X

}

getPivot <- function(Xf, Xr){
  nf <- colnames(Xf)
  nr <- colnames(Xr)
  k <- seq(1, NCOL(Xf))
  p <- match(nr, nf)[1:NCOL(Xr)]
  k <- which(!k %in% p)
  c(p, k)
}

getXs <- function(Terms, Y, SS.type, tol = 1e-7,
                  model, subjects.term = NULL) {
  
  X <- model.matrix(Terms, data = model)
  n <- NROW(X)
  X.k <- X.k.obs <- attr(X, "assign")
  X.n.k.obs <- length(X.k.obs)
  Xred <- removeRedundant(X)
  X.rank <- NCOL(Xred)
  pivot <- getPivot(X, Xred)
  uk <- unique(c(0, X.k))
  term.labels <- attr(Terms, "term.labels")
  k <- length(term.labels)
  fix <- (X.rank < X.n.k.obs) 
  delete <- NULL
  if(fix) {
    Terms <- Terms[uk]
    warning(
      paste(
        "\nThis is not an error!  It is a friendly warning.\n",
        "\nBecause variables in the linear model are redundant,",
        "\nthe linear model design has been truncated (via QR decomposition).",
        "\nOriginal X columns:", X.n.k.obs,
        "\nFinal X columns (rank):", X.rank,
        "\nCheck coefficients or degrees of freedom in ANOVA to see changes.\n",
        "\nUse suppressWarnings() to turn off these warnings. \n\n", sep = " "),
      noBreaks. = TRUE, call. = FALSE, immediate. = TRUE) 
  } 
  
  if(fix) delete <- pivot[-(1:X.rank)]
  
  if(k > 0){
    if(SS.type == "III"){
      fix <- !(length(delete) == 0)
      Xfs <- lapply(2:length(uk), function(j)  if(fix) 
        X[, -delete, drop = FALSE] else X)
      Xrs <- lapply(2:length(uk), function(j){
        rmove <- c(which(X.k == (j - 1)), delete)
        Xout <- as.matrix(X[, -rmove, drop = FALSE])
        if(length(Xout) == 0) Xout <- matrix(0, n, 1)
        Xout
      })
      
    } else if(SS.type != "I"){
      fac <- as.matrix(crossprod(attr(Terms, "factor")))
      
      Xrs <- lapply(1:NROW(fac), function(j){
        index <- fac[, j, drop = FALSE]
        m <- max(index)
        ind <-  as.logical(ifelse(c(0, index) < m, 1, 0))
        rmove <- which(!X.k %in% uk[ind])
        rmove <- unique(c(rmove, delete))
        Xout <- as.matrix(X[, -rmove, drop = FALSE])
        if(length(Xout) == 0) Xout <- matrix(0, n, 1)
        Xout
      })
      
      Xfs <- lapply(1:NROW(fac), function(j){
        index <- fac[, j, drop = FALSE]
        m <- max(index)
        ind <-  as.logical(ifelse(c(0, index) < m, 1, 0))
        ind[j + 1] <- TRUE
        rmove <- which(!X.k %in% uk[ind])
        rmove <- unique(c(rmove, delete))
        fix <- !(length(rmove) == 0)
        if(fix) as.matrix(X[, -rmove, drop = FALSE]) else X
      })
      
    } else {
      Xrs <- lapply(2:length(uk), function(j){
        rmove <- which(!X.k %in% uk[1:(j - 1)])
        rmove <- unique(c(rmove, delete))
        Xout <- as.matrix(X[, -rmove, drop = FALSE])
        if(length(Xout) == 0) Xout <- matrix(0, n, 1)
        Xout
      })
      
      Xfs <- lapply(2:length(uk), function(j){
        rmove <- which(!X.k %in% uk[1:j])
        rmove <- unique(c(rmove, delete))
        fix <- !(length(rmove) == 0)
        if(fix) as.matrix(X[, -rmove, drop = FALSE]) else X
      })
    }
    names(Xrs) <- names(Xfs) <- term.labels
  } else {
    Xrs <- Xfs <- list(Intercept = X)
  }
  
  if(!is.null(subjects.term)) {
    Xfs[[subjects.term]] <- if(fix) X[, -delete, drop = FALSE] else X
    rmove <- c(which(X.k == subjects.term), delete)
    Xrs[[subjects.term]] <- as.matrix(X[, - rmove, drop = FALSE])
  }
  
  list(Xrs = Xrs, Xfs = Xfs)
}


lm.rrpp.fit <- function(x, y, Pcov = NULL, w = NULL, offset = NULL, tol = 1e-07){
  
  getGLSlm<- function(x, y, Pcov, offset = offset, method = "qr", tol = tol){
    PX <- as.matrix(Pcov %*% x)
    PY <- as.matrix(Pcov %*% y)
    fit <- LM.fit(x = PX, y = PY, offset = offset, tol = tol)
    fit
  }
  
  z <- if(!is.null(Pcov)) getGLSlm(x, y, Pcov, offset = offset, tol = tol) else 
    if(!is.null(w)) lm.wfit(x = as.matrix(x), y = y, w = w, offset = offset, tol = tol) else
      LM.fit(x = x, y = y, offset = offset, tol = tol)
  
  if(!is.null(Pcov)) {
    z$residuals <- fast.solve(Pcov) %*% z$residuals
    z$fitted.values <- y - z$residuals
  }
  z
}

# NO LONGER USED but retained for potential future use
lm.rrpp.exchange <- function(x, y, Pcov = NULL, w = NULL, offset = NULL, tol = 1e-07){
  
  getGLSlm<- function(x, y, Pcov, offset = offset, method = "qr", tol = tol){
    PX <- Pcov %*% x
    PY <- Pcov %*% y
    fit <- LM.fit(x = PX, y = PY, offset = offset, tol = tol)
    fit
  }
  
  getWlm<- function(x, y, w, offset = offset, method = "qr", tol = tol){
    wts <- sqrt(w)
    fit <- lm.fit(x = as.matrix(x * wts), 
                  y = as.matrix(y * wts), offset = offset, tol = tol)
    fit
  }
  
  z <- if(!is.null(Pcov)) getGLSlm(x = x, y = y, Pcov, offset = offset, tol = tol) else 
    if(!is.null(w)) getWlm(x = x, y = y, w = w, offset = offset, tol = tol) else
      LM.fit(x = x, y = y, offset = offset, tol = tol)
  z
}

# NO LONGER USED but retained for potential future use
package.exchanges <- function(Y, mods, Xs, Terms, model, 
                             offset = NULL, w = NULL,
                             Pcov = NULL, tol = 1e-7, SS.type) {
  Xrs <- Xs$Xrs
  Xfs <- Xs$Xfs
  exchange.args <- list(x = Xrs[[1]], y = Y, Pcov = Pcov, w = w,
                        offset = offset, tol = tol)
  
  reduced = lapply(Xrs, function(x) {
    exchange.args$x <- x
    do.call(lm.rrpp.exchange, exchange.args)
  })
  
  full = lapply(Xfs, function(x) {
    exchange.args$x <- x
    do.call(lm.rrpp.exchange, exchange.args)
  })
  
  model.sets <- list(terms.r = mods$terms.r, terms.f = mods$terms.f,
                     Xrs = Xrs, Xfs = Xfs)
  list(reduced = reduced, full = full, offset = offset, weights = w,
       Terms = Terms, model = model, Pcov = Pcov, SS.type = SS.type, 
       model.sets = model.sets)
  
}

# NO LONGER USED but retained for potential future use
package.fits <- function(Y, mods, Xs, Terms, model, 
                         offset = NULL, w = NULL,
                         Pcov = NULL, tol = 1e-7, SS.type) {
  Xrs <- Xs$Xrs
  Xfs <- Xs$Xfs
  fit.args <- list(x = Xrs[[1]], y = Y, Pcov = Pcov, w = w,
                   offset = offset, tol = tol)
  
  reduced = lapply(Xrs, function(x) {
    fit.args$x <- x
    do.call(lm.rrpp.fit, fit.args)
  })
  
  full = lapply(Xfs, function(x) {
    fit.args$x <- x
    do.call(lm.rrpp.fit, fit.args)
  })
  
  model.sets <- list(terms.r = mods$terms.r, terms.f = mods$terms.f,
                     Xrs = Xrs, Xfs = Xfs)
  list(reduced = reduced, full = full, offset = offset, weights = w,
       Terms = Terms, model = model, Pcov = Pcov, SS.type = SS.type, 
       model.sets = model.sets)
  
}

# droplevels.rrpp.data.frame
# same as droplevels for data.frame objects
# used in lm.rrpp

droplevels.rrpp.data.frame <- function (x, except = NULL, ...) {
  ix <- vapply(x, is.factor, NA)
  if (!is.null(except))
    ix[except] <- FALSE
  x[ix] <- lapply(x[ix], factor)
  x
}

# getHb
# function to find "hat" matrix for coefficients
# used in lm.rrpp/SS.iter/beta.iter
getHb <- function(Q) {
  Qnames <- Q$dimnames[[2]]
  k <- Q$rank
  R <- Q$R
  U <- Q$Q
  Rs <- try(fast.solve(R), silent = TRUE)
  if(inherits(Rs, "try-error")){
    Rs <- 1
  } 

  res <- as.matrix(tcrossprod(Rs, U))
  
  if(length(Rs) > 1 && !is.null(rownames(res)) &&
     !identical(rownames(Rs), Qnames))
    res <- res[Qnames, , drop = FALSE]
  
  if(is.null(rownames(res))) rownames(res) <- Qnames
 
  res
  
}

# checkDD
# helper function to determine if double decomposition should
# be used in checkers
checkDD <- function(Uf, Ur){
  
  DD <- matdiff(Ur, Uf)
  lf <- if(isS4(Uf)) length(Uf@x) else length(Uf)
  lr <- if(isS4(Ur)) length(Ur@x) else length(Ur)
  ld <- if(isS4(DD)) length(DD@x) else length(DD)
  ld <- length(DD)
  
  extend <- ((lf + lr) < ld)
  
  res <- if(extend) 
    list(Uf = Uf, Ur = Ur) else
      list(Uf = DD, Ur = NA)
  res
  
}
  
# matdff
# helper function to find U difference between Ur and Uf
# used in checkers/CheckDD 
matdiff <- function(U1, U2){
  dims1 <- dim(U1)
  dims2 <- dim(U2)
  if(dims1[1] != dims2[1])
    stop("Different numbers of rows in each matrix\n",
         call. = FALSE)
  ps <- c(dims1[2], dims2[2])
  if(ps[2] > ps[1]) {
    Uf <- U2
    Ur <- U1
  } else {
    Uf <- U1
    Ur <- U2
    ps <- ps[c(2,1)]
  }
  
  checkS <- colSums(round(as.matrix(crossprod(Ur, Uf)), 7))
  keep <- which(checkS == 0)
  
  res <- if(length(keep) > 0) 
    as.matrix(Uf[, keep, drop = FALSE]) else NA
  res

}


# checkers
# algorithms to facilitate RRPP iteration stats calculations
# used in lm.rrpp/SS.iter/beta.iter
checkers <- function(Y, Qs, Xs, turbo = FALSE, 
                     Terms, Pcov = NULL, w = NULL) {
  k <- length(attr(Terms, "term.labels"))
  Qr <- Qs$reduced
  Qf <- Qs$full
  n <- NROW(Y)
  Ur <- lapply(Qr, function(q) q$Q)
  Uf <- lapply(Qf, function(q) q$Q)
  kk <- length(Uf)
  
  if(k > 0 && k != kk) k <- kk

  if(!turbo) {
    Hbf <- lapply(Qf, getHb)
    Hbfs <- lapply(Hbf, function(x) Matrix(round(x, 12), sparse = TRUE))
    Hbr <- lapply(Qr, getHb)
    Hbrs <- lapply(Hbr, function(x) Matrix(round(x, 12), sparse = TRUE))  
    
    for(i in 1:max(1,k)) {
      if(object.size(Hbfs[[i]]) < object.size(Hbf[[i]]))
        Hbf[[i]] <- Hbfs[[i]]
      if(object.size(Hbrs[[i]]) < object.size(Hbr[[i]]))
        Hbr[[i]] <- Hbrs[[i]]
    }
    
  } else Hbr <- Hbf <- NULL
  
  Ufull <- Uf[[max(1, k)]]
  
  int <- attr(Terms, "intercept")
  intercept <- rep(int, n)
  Qint <- if(!is.null(Pcov))
    QRforX(Pcov %*% intercept) else if(!is.null(w))
      QRforX(intercept * sqrt(w)) else
        QRforX(intercept)

  Hbnull <- tcrossprod(fast.solve(Qint$R), Qint$Q) 
  
  # criteria SS U matrices
  
  #####
  Uss <- list()
  for(i in 1:max(1,k)) {
    
    ### Note this is intended but needs to be fixed
    # us <- checkDD(Uf[[i]], Ur[[i]])
    # if(all(is.na(us[[1]])))
      # us <- list(Uf = Uf[[i]], Ur = Ur[[i]])
    ### so this is defaulted
    us <- list(Uf = Uf[[i]], Ur = Ur[[i]])
    Uss[[i]] <- us
  }
  names(Uss) <- attr(Terms, "term.labels")
  ####
  
  out <- list(Y = Y, Uss = Uss, Ur = Ur, Uf = Uf, Unull = Qint$Q, Ufull = Ufull,
              Hbr = Hbr, Hbf = Hbf, Hbnull = Hbnull, QR = Qs, k = k,
              realized.trms = names(Xs$Xfs))
  
  out
  
}

# SS.iter
# three functions: main, and two for whether PP is used
# gets appropriate SS vectors for random permutations in lm.rrpp
# generates ANOVA stats

SS.iter <- function(checkrs, ind,  ind_s = NULL,
                    subTest = FALSE, STerm = NULL,
                    print.progress = TRUE, 
                    Parallel.args) {
  
  forking <- Parallel.args$forking
  usecluster <- Parallel.args$usecluster
  cluster <- Parallel.args$cluster
  no_cores <- Parallel.args$ParCores
  
    SS.iter.main(checkrs = checkrs, ind = ind,
                 ind_s = ind_s, subTest = subTest, 
                 STerm = STerm,
                 print.progress = print.progress,
                 no_cores = no_cores, usecluster = usecluster,
                 cluster = cluster, forking = forking)
    
  
}

SS.iter.main <- function(checkrs, ind, ind_s, subTest, STerm,
                         print.progress = TRUE, 
                         no_cores, cluster = NULL, forking = FALSE,
                         usecluster = FALSE) {
  
  Ur <- checkrs$Ur
  Uf <- checkrs$Uf
  Uss <- checkrs$Uss
  Unull <- checkrs$Unull
  Ufull <- checkrs$Ufull
  FR <- checkrs$FR
  Y <- checkrs$Y
  dims <- dim(Y)
  n <- dims[1]
  p <- dims[2]
  yh0 <- as.matrix(fastFit(Unull, Y, n, p))
  r0 <- as.matrix(Y - yh0)
  
  k <- checkrs$k
  trms <- checkrs$realized.trms
  rm(checkrs)
  
  perms <- length(ind)
  
  rrpp.args <- list(FR = FR, ind.i = NULL,
                    ind_s.i = NULL, subTest = subTest,
                    STerm = STerm)
  
  rrpp <- function(FR, ind.i, ind_s.i, subTest, STerm) {
    result <- lapply(FR, function(x) x$fitted + x$residuals[ind.i, , drop = FALSE])
    if(subTest)
      result[[STerm]] <- FR[[STerm]]$fitted + 
        FR[[STerm]]$residuals[ind_s.i, , drop = FALSE]
    result
  }
  
  
  ss <- function(u, y) {
    useUr <- !all(is.na(u[[2]]))
    ssu <- if(useUr) sum(crossprod(u[[2]], y)^2) else 0
    ssf <- sum(crossprod(u[[1]], y)^2)
    ssm <- sum(y^2) - sum(crossprod(Ufull, y)^2)
    c(ssu, ssf, ssm)
  } 
  
  pbbar <- FALSE
  if(print.progress && no_cores > 1){
    cat("\nProgress bar not available for Sums of Squares calculations...\n")
  } else   if(print.progress && no_cores == 1){
    cat(paste("\nSums of Squares calculations:", perms, "permutations.\n"))
    pb <- txtProgressBar(min = 0, max = perms, initial = 0, style=3)
    pbbar <- TRUE
  }
  
  if(forking && no_cores > 1) {
    result <- mclapply(1:perms, mc.cores = no_cores, function(j){
      x <-ind[[j]]
      rrpp.args$ind.i <- x
      if(subTest) rrpp.args$ind_s.i <- ind_s[[j]]
      Yi <- do.call(rrpp, rrpp.args)
      y <- as.matrix(yh0 + r0[x, , drop = FALSE])
      yy <- sum(y^2)
      if(k > 0) {
        res <- vapply(1:max(1, k), function(j){
          ss(Uss[[j]], Yi[[j]])
        }, numeric(3))
        
        SSr <- res[1, ]
        SSf <- res[2, ]
        RSS <- res[3, ]
        
        TSS <- yy - sum(crossprod(Unull, y)^2)
        TSS <- rep(TSS, k)
        SS = SSf - SSr
      } else SSr <- SSf <- SS <- RSS <- TSS <- NA
      RSS.model <- yy - sum(crossprod(Ufull, y)^2)
      if(k == 0) TSS <- RSS.model
      list(SS = SS, RSS = RSS, TSS = TSS, RSS.model = RSS.model)
    })
    
  } else if(usecluster && no_cores > 1) {
  
    result <- parLapply(cluster, 1:perms, function(j){
      x <-ind[[j]]
      rrpp.args$ind.i <- x
      if(subTest) rrpp.args$ind_s.i <- ind_s[[j]]
      Yi <- do.call(rrpp, rrpp.args)
      y <- as.matrix(yh0 + r0[x, , drop = FALSE])
      yy <- sum(y^2)
      if(k > 0) {
        res <- vapply(1:max(1, k), function(j){
          ss(Uss[[j]], Yi[[j]])
        }, numeric(3))
        
        SSr <- res[1, ]
        SSf <- res[2, ]
        RSS <- res[3, ]
        
        TSS <- yy - sum(crossprod(Unull, y)^2)
        TSS <- rep(TSS, k)
        SS = SSf - SSr
      } else SSr <- SSf <- SS <- RSS <- TSS <- NA
      RSS.model <- yy - sum(crossprod(Ufull, y)^2)
      if(k == 0) TSS <- RSS.model
      list(SS = SS, RSS = RSS, TSS = TSS, RSS.model = RSS.model)
    })
    stopCluster(cluster)
    
  } else {
    result <- lapply(1:perms, function(j){
      step <- j
      if(print.progress) setTxtProgressBar(pb,step)
      x <-ind[[j]]
      rrpp.args$ind.i <- x
      if(subTest) rrpp.args$ind_s.i <- ind_s[[j]]
      Yi <- do.call(rrpp, rrpp.args)
      y <- as.matrix(yh0 + r0[x, , drop = FALSE])
      yy <- sum(y^2)
      
      if(k > 0) {
        res <- vapply(1:max(1, k), function(j){
          ss(Uss[[j]], Yi[[j]])
        }, numeric(3))
        
        SSr <- res[1, ]
        SSf <- res[2, ]
        RSS <- res[3, ]
        
        TSS <- yy - sum(crossprod(Unull, y)^2)
        RSS.model <- yy - sum(crossprod(Ufull, y)^2)
        SS = SSf - SSr
      } else {
        SSr <- SSf <- SS <- RSS <- TSS <- NA
        RSS.model <- TSS <- yy - sum(crossprod(Ufull, y)^2)
      }
      list(SS = SS, RSS = RSS, TSS = TSS, RSS.model = RSS.model)
    })
  }
  
  SS <- matrix(sapply(result, "[[", "SS"), max(1, k), perms)
  RSS <- matrix(sapply(result, "[[", "RSS"), max(1, k), perms)
  TSS <- matrix(sapply(result, "[[", "TSS"), max(1, k), perms, 
                byrow = TRUE)
  RSS.model <- matrix(sapply(result, "[[", "RSS.model"), 
                      max(1, k), perms, byrow = TRUE)
  
  res.names <- list(if(k > 0) trms else "Intercept", 
                    c("obs", paste("iter", 1:(perms-1), sep=".")))
  dimnames(SS) <- dimnames(RSS) <- dimnames(TSS)  <- dimnames(RSS.model) <-
    res.names
  
  if(all(is.na(SS))) RSS <- SS <- NULL
  
  if(pbbar) close(pb)
  
  list(SS = SS, RSS = RSS, TSS = TSS, RSS.model = RSS.model)
}


# anova_parts
# construct an ANOVA table from random SS output
# used in lm.rrpp

getRank <- function(Q) {
  if(inherits(Q, "QR")){
    r <- Q$rank
  } else {
    r <- NCOL(removeRedundant(Q))
  }
  return(r)
}

anova_parts <- function(checkrs, SS, full.resid = FALSE){
  SS.type <- checkrs$SS.type
  perms <- NCOL(SS)
  trms <- checkrs$terms
  k <- checkrs$k
  dims <- dim(checkrs$Y)
  n <- dims[1]
  p <- dims[2]
  QRf <- checkrs$QR$full
  QRr <- checkrs$QR$reduced
  
  if(k > 0) {
    df <- unlist(Map(function(qf, qr) 
      getRank(qf) - getRank(qr), 
      QRf, QRr))
    dfe <- n - getRank(QRf[[k]])
    RSS <- SS$RSS
    TSS <- SS$TSS
    RSS.model <- SS$RSS.model
    SS <- SS$SS
    Rsq <- SS/TSS
    MS <- SS/df
    RMS <- RSS/dfe
    Fs <- MS/RMS
    dft <- n - 1
    df <- c(df, dfe, dft)
    if(SS.type == "III") {
      etas <- SS/TSS
      cohenf <- etas/(1-etas)
    } else {
      etas <- Rsq
      if(k == 1) unexp <- 1 - etas else unexp <- 1 - apply(etas, 2, cumsum)
      cohenf <- etas/unexp
    }
    
    rownames(Fs) <- rownames(cohenf) <- rownames(SS)
    
    if(full.resid) {
      SS <- abs(SS - cbind(0, matrix(SS[,1], nrow(SS), ncol(SS) - 1)))
      MS <- abs(MS - cbind(0, matrix(MS[,1], nrow(MS), ncol(MS) - 1)))
      Rsq <- abs(Rsq - cbind(0, matrix(Rsq[,1], nrow(Rsq), ncol(Rsq) - 1)))
      Fs <- MS/RMS
      rownames(Fs) <- rownames(SS)
      cohenf <- NULL
    }
    
    
  } else {
    RSS <- NULL
    TSS <- SS$TSS
    RSS.model <- SS$RSS.model
    SS <- NULL
    Rsq <- NULL
    MS <- NULL
    RMS <- NULL
    Fs <- NULL
    cohenf <- NULL
    df <- n - 1
    SS.type <- NULL
    
  }
  
  out <- list(SS.type = SS.type, SS = SS, MS = MS, RSS = RSS,
              TSS = TSS, RSS.model = RSS.model, Rsq = Rsq,
              Fs = Fs, cohenf = cohenf, full.resid = full.resid,
              n = n, p = p, df=df
  )
  out
}

# SS.mean
# support function for calculating SS quickly in SS.iter
# used in lm.rrpp
# NO LONGER USED but retained for potential future use

SS.mean <- function(x, n) if(is.vector(x)) sum(x)^2/n else sum(colSums(x)^2)/n


# getXfromNewData
# make a new X matrix from new data, for a lm.rrpp fit
# used in predict.lm.rrpp
getXfromNewData <- function(fit, newdata){
  Terms <- fit$LM$Terms
  o_vars <- all.vars(Terms)
  Terms <- delete.response(Terms)
  mf_vars <- all.vars(Terms)
  mf <- fit$LM$data
  mf <- mf[o_vars %in% mf_vars]
  nd_vars <- names(newdata)
  factors <- attr(Terms, "factors")
  X <- fit$LM$X
  if(is.null(attr(X, "assign"))){
    X <- try(model.matrix(Terms, mf),
             silent = TRUE)
    if(inherits(X, "try-error"))
      X <- try(model.matrix(Terms, newdata))
    if(inherits(X, "try-error"))
      stop("\n Could not construct a model matrix from new data.",
           call. = FALSE)
  }
  X <- try(model.matrix(Terms, mf),
           silent = TRUE)
  if(inherits(X, "try-error"))
    X <- try(model.matrix(Terms, newdata))
  if(inherits(X, "try-error"))
    stop("\n Could not construct a model matrix from new data.",
         call. = FALSE)
  asn <- attr(X, "assign")
  m <- colMeans(X)
  
  mfn <- as.data.frame(
    matrix(NA, NROW(newdata), NCOL(mf)))
  dimnames(mfn) <- list(rownames(newdata), 
                        mf_vars)
  mfn[, mf_vars %in% nd_vars] <- newdata[, nd_vars]
  
  NA_id <- which(is.na(mfn[1, , drop = FALSE]))
  if(length(NA_id) > 0) {
    for(i in 1:length(NA_id)) mfn[, NA_id[i]] <- mf[1, NA_id[i]]
  }
  
  N <- NROW(mf)
  mfnn <- rbind(mf, model.frame(Terms, mfn))
  mfnn[is.na(mfnn)] <- 0
  attr(mfnn, "terms") <- Terms
  
  fix <- which(!mf_vars %in% nd_vars)
  term_fix <- which(colSums(as.matrix(factors[fix, , drop = FALSE])) > 0)
  asn_fix <- which(asn %in% term_fix)
  
  b_ind <- rep(1, length(asn))
  b_ind[asn_fix] <- 0
  m_ind <- 1 - b_ind
  
  nXb <- model.matrix(Terms, mfnn)[-(1:N), , drop = FALSE]
  mfnn <- mfnn[-(1:N), , drop = FALSE]
  b_mat <- matrix(b_ind, NROW(mfnn), length(asn), byrow = TRUE)
  m_mat <- matrix(m_ind, NROW(mfnn), length(asn), byrow = TRUE)
  nXm <- matrix(m, NROW(mfnn), length(asn), byrow = TRUE)
  nX <- nXb * b_mat + nXm * m_mat
  rownames(nX) <- rownames(newdata)
  if(is.null(colnames(nX))) colnames(nX) <- colnames(X)
  nX
}


# beta.boot.iter
# gets appropriate beta vectors for coefficients via bootstrap
# used in predict.lm.rrpp

beta.boot.iter <- function(fit, ind) {

  gls <- fit$LM$gls
  QR <- getModels(fit, "qr")
  Qf <- QR$full[[length(QR$full)]]
  rm(QR)
  id <- Qf$dimnames[[2]]
  
  fitted <- if(gls) fit$LM$gls.fitted else fit$LM$fitted
  res <- if(gls) fit$LM$gls.residuals else fit$LM$residuals
  
  w <- fit$LM$weights
  if(!is.null(w)) weighted = TRUE else weighted = FALSE
  
  if(weighted) {
    fitted <- fitted * sqrt(w)
    res <- res * sqrt(w)
  }
  
  Y <- fitted + res
  dims <- dim(Y)
  n <- dims[1]
  p <- dims[2]
  perms <- length(ind)
  
  Pcov <- fit$LM$Pcov
  if(is.null(Pcov) && !is.null(fit$LM$Cov))
    Pcov <- Cov.proj(fit$LM$Cov)
  
  rrpp.args <- list(fitted = as.matrix(fitted), 
                    residuals = as.matrix(res),
                    ind.i = NULL)
  
  rrpp <- function(fitted, residuals, ind.i) as.matrix(fitted + 
                                                         residuals[ind.i, , drop = FALSE])
  
  Hf <- tcrossprod(fast.solve(Qf$R), Qf$Q)
  Hfs <- Matrix(round(Hf, 15), sparse = TRUE)
  if(object.size(Hfs) < object.size(Hf)) Hf <- Hfs
  b <- if(gls) fit$LM$gls.coefficients else
    fit$LM$coefficients
  Hf <- Hf[rownames(b), , drop = FALSE]
  
  betas <- lapply(1:perms, function(j){
    x <-ind[[j]]
    rrpp.args$ind.i <- x
    Yi <- do.call(rrpp, rrpp.args)
    if(!is.null(Pcov)) Yi <- Pcov %*% Yi
    res <- as.matrix(Hf %*% Yi)
    rownames(res) <- id
    res
  })
  betas
  
}

# beta.iter
# three functions: main, and two for whether PP is used
# gets appropriate beta vectors for random permutations in lm.rrpp
# generates distances as statistics for summary
beta.iter <- function(checkrs, ind, ind_s = NULL,
                      subTest = FALSE, STerm = NULL,
                      print.progress = TRUE, 
                      Parallel.args) {
  
  forking <- Parallel.args$forking
  cluster <- Parallel.args$cluster
  usecluster <- Parallel.args$usecluster
  no_cores <- Parallel.args$ParCores
    
    beta.iter.main(checkrs = checkrs, ind = ind, ind_s = ind_s, 
                   subTest = subTest, STerm = STerm,
                   print.progress = print.progress,
                   no_cores = no_cores, usecluster = usecluster,
                   cluster = cluster, forking = forking)
}


beta.iter.main <- function(checkrs, ind, ind_s, subTest,
                           STerm, print.progress = TRUE, 
                           no_cores, cluster = NULL, forking = FALSE,
                           usecluster = FALSE) {
  
  k <- checkrs$k
  trms <- checkrs$realized.trms
    
  Hr <- checkrs$Hbr 
  Hf <- checkrs$Hbf
  pert.rows <- lapply(1:max(1, k), function(j){
    br.nms <- try(rownames(Hr[[j]]), silent = TRUE)
    if(inherits(br.nms, "try-error"))
      br.nms <- "(Intercept)"
    bf.nms <- try(rownames(Hf[[j]]), silent = TRUE)
    if(inherits(bf.nms, "try-error"))
      bf.nms <- "(Intercept)"
    if(length(bf.nms) > length(br.nms)) {
      b <- bf.nms
      a <- br.nms
    } else {
      a <- bf.nms
      b <- br.nms
    }
    b.keep <- setdiff(bf.nms, br.nms)
    res <- which(b %in% b.keep)
    list(res = res, b.keep = b.keep)
  })
  
  b.names <- lapply(pert.rows, function(x) x$b.keep)
  pert.rows <- lapply(pert.rows, function(x) x$res)
  names(b.names) <- names(pert.rows) <- trms
  
  FR <- checkrs$FR
  o <- checkrs$offset
  offst <- !is.null(o)
  perms <- length(ind)
  Y <- checkrs$Y
  dims <- dim(Y)
  n <- dims[1]
  p <- dims[2]
  
  rrpp.args <- list(FR = FR, ind.i = NULL, 
                    ind_s.i = NULL, subTest = subTest,
                    STerm = STerm, offst = offst, o = o)
  
  rrpp <- function(FR, ind.i, ind_s.i, subTest, STerm, offst, o) {
    result <- if(offst)
      lapply(FR, function(x) x$fitted + x$residuals[ind.i, , drop = FALSE] - o) else
        lapply(FR, function(x) x$fitted + x$residuals[ind.i, , drop = FALSE]) 
    if(subTest) {
      result[[STerm]] <- FR[[STerm]]$fitted + 
        FR[[STerm]]$residuals[ind_s.i, , drop = FALSE]
      if(offst) result[[STerm]] <- result[[STerm]] - o
    }
    result
  }
  
  pbbar <- FALSE
  if(print.progress && no_cores > 1){
    cat("\nProgress bar not available for coefficients estimation...\n")
  } else   if(print.progress && no_cores == 1){
    cat(paste("\nCoefficients estimation:", perms, "permutations.\n"))
    pb <- txtProgressBar(min = 0, max = perms, initial = 0, style=3)
    pbbar <- TRUE
  }
  
  getBetas <- function(Hf, Yi, pert.rows){
    Bf <- Map(function(hf, y, p) {
      B <- as.matrix(hf %*% y)
      d <- rowSums(B^2)[p]
      res = list(B = B, d = sqrt(d))
      res
    }, Hf, Yi, pert.rows) 
  } 
  
  if(forking && no_cores > 1) {
    betas <- mclapply(1:perms, mc.cores = no_cores, function(j){
      x <-ind[[j]]
      rrpp.args$ind.i <- x
      if(subTest) rrpp.args$ind_s.i <- ind_s[[j]]
      Yi <- do.call(rrpp, rrpp.args)
      getBetas(Hf, Yi = Yi, pert.rows)
      
    })
    
  } else if(usecluster && no_cores > 1) {
    
    betas <- parLapply(cluster, 1:perms, function(j){
      x <-ind[[j]]
      rrpp.args$ind.i <- x
      if(subTest) rrpp.args$ind_s.i <- ind_s[[j]]
      Yi <- do.call(rrpp, rrpp.args)
      getBetas(Hf, Yi = Yi, pert.rows)
      
    })
    
  } else {
    betas <- lapply(1:perms, function(j){
      step <- j
      if(print.progress) setTxtProgressBar(pb,step)
      x <-ind[[j]]
      rrpp.args$ind.i <- x
      if(subTest) rrpp.args$ind_s.i <- ind_s[[j]]
      Yi <- do.call(rrpp, rrpp.args)
      getBetas(Hf, Yi = Yi, pert.rows)
      
    })
  }
  
  if(pbbar)  close(pb)

  cnms <- colnames(Y)
  
  betas.out <- lapply(1:max(1, k), function(j){
    res <- lapply(1:perms, function(jj){
      y <- betas[[jj]][[j]]$B
      colnames(y) <- cnms
      y
    })
    names(res) <- names(ind)
    res
  })
  
  names(betas.out) <-trms
  
  d.out <- lapply(1:max(1, k), function(j){
    res <- sapply(1:perms, function(jj){
      y <- betas[[jj]][[j]]$d
      y
    })
    
    res
  })
  
  if(is.list(d.out)) d.out <- do.call(rbind, d.out)
  try(dimnames(d.out) <- list(unlist(b.names), 
                              names(ind)), 
      silent = TRUE)

  if(is.matrix(d.out) && any(rownames(d.out) == "(Intercept)")){
    rmove <- which(rownames(d.out) == "(Intercept)")
    dnames <- rownames(d.out)[-rmove]
    d.out <- d.out[-rmove, , drop = FALSE]
    if(is.vector(d.out)){
      d.out <- matrix(d.out, 1, length(d.out))
      colnames(d.out) <- names(ind)
      rownames(d.out) <- dnames
    }
  }

  list(random.coef = betas.out, 
       random.coef.distances = d.out)
}

# ellipse.points
# A helper function for plotting ellipses from non-parametric CI data
# Used in predict.lm.rrpp

ellipse.points <- function(m, pr, conf) {
  m <- as.matrix(m)
  p <- NCOL(m)
  z <- qnorm((1 - conf)/2, lower.tail = FALSE)
  angles <- seq(0, 2*pi, length.out=200)
  ell    <- z* cbind(cos(angles), sin(angles))
  del <- lapply(pr, function(x) x[,1:p] - m)
  vcv <- lapply(1:NROW(m), function(j){
    x <- sapply(1:length(del), function(jj){
      del[[jj]][j, , drop = FALSE]
    })
    tcrossprod(x)/ncol(x)
  })
  R <- lapply(vcv, chol)
  ellP <- lapply(R, function(x) ell %*% x)
  np <- NROW(ellP[[1]])
  ellP <- lapply(1:length(ellP), function(j){
    x <- m[j,, drop = FALSE]
    mm <- matrix(rep(x, each = np), np, length(x))
    mm + ellP[[j]]
  })
  ellP <- simplify2array(ellP)
  pc1lim <- c(min(ellP[,1,]), max(ellP[,1,]))
  pc2lim <- c(min(ellP[,2,]), max(ellP[,2,]))
  list(ellP = ellP, pc1lim = pc1lim, pc2lim = pc2lim,
       means = m)
}

# aov.single.model
# performs ANOVA on a single model
# used in anova.lm.rrpp
aov.single.model <- function(object, ...,
                             effect.type = c("F", "cohenf", "SS", "MS", "Rsq"),
                             error = NULL) {
  x <- object$ANOVA
  if(is.null(x$Fs)) {
    AN <- getANOVAStats(object, stat = "all")
    x[c("SS", "MS", "Rsq", "Fs", "cohenf")] <-
       AN[ c("SS", "MS", "Rsq", "Fs", "cohenf")]
    AN <- NULL
  }
  
  est <- if(object$LM$ols)  "OLS" else "GLS"
  
  if(isTRUE(object$LM$LMM)){
    effect.type <- "F"
    fixed.est <- est
    est <- "mixed"
    random.est <- object$LM$estimation
  } else fixed.est <- random.est <- NULL
  
  if(is.null(x$Fs)) x$Fs <- x$F
  df <- x$df
  k <- length(df)-2
  kk <- length(object$LM$term.labels)
  if(k > 0 && k != kk) {
    warning(
      paste(
        "\nThis is not an error!  It is a friendly warning.\n",
        "\nANOVA is missing some terms, likely because",
        "\nsome independent variables were redundant.",
        "\nIf the residual SS is 0, results should not be trusted\n", 
        "\nUse suppressWarnings() to turn off these warnings. \n\n", sep = " "),
      noBreaks. = TRUE, call. = FALSE, immediate. = TRUE) 
  } 
    
  SS <- x$SS
  MS <- x$MS 
  RSS <-x$RSS
  TSS <- x$TSS
  perms <- object$PermInfo$perms
  pm <- object$PermInfo$perm.method
  if(pm == "RRPP" && object$PermInfo$full.resid)
    pm <- "FMRP"
  trms <- object$LM$term.labels
  
  if(!is.null(error)) {
    if(!inherits(error, "character")) 
      stop("The error description is illogical.  It should be a string of character 
           values matching ANOVA terms.",
                                           call. = FALSE)
    kk <- length(error)
    if(kk != k) 
      stop("The error description should match in length the number of ANOVA terms 
           (not including Residuals)",
                     call. = FALSE)
    MSEmatch <- match(error, c(trms, "Residuals"))
    if(any(is.na(MSEmatch))) 
      stop("At least one of the error terms is not an ANOVA term",
                                  call. = FALSE)
  } else MSEmatch <- NULL
  if(k >= 1) {
    Fs <- x$Fs
    
    if(!is.null(MSEmatch)){
      Fmatch <- which(MSEmatch <= k)
      Fs[Fmatch, ] <- MS[Fmatch,]/MS[MSEmatch[Fmatch],]
      F.effect.adj <- apply(Fs, 1, effect.size)
    }
    
    effect.type <- match.arg(effect.type)
    
    if(object$LM$gls) {
      
      if(effect.type == "SS") {
        
        warning(
          paste(
            "\nThis is not an error!  It is a friendly warning.\n",
            "\nCalculating effect size on SS is illogical with GLS.",
            "\nEffect type has been changed to F distributions.\n",
            "\nUse suppressWarnings() to turn off these warnings. \n\n", sep = " "),
          noBreaks. = TRUE, call. = FALSE, immediate. = TRUE) 
        
        effect.type = "F"
      }
      
      if(effect.type == "MS") {
        warning(
          paste(
            "\nThis is not an error!  It is a friendly warning.\n",
            "\nCalculating effect size on MS is illogical with GLS.",
            "\nEffect type has been changed to F distributions.\n",
            "\nUse suppressWarnings() to turn off these warnings. \n\n", sep = " "),
          noBreaks. = TRUE, call. = FALSE, immediate. = TRUE) 
        
        effect.type = "F"
      }
    } 
    
    ow <- options()$warn
    options(warn = -1)
    
    if(effect.type == "F") Z <- Fs
    if(effect.type == "SS") Z <- x$SS
    if(effect.type == "MS") Z <- x$MS
    if(effect.type == "Rsq") Z <- x$Rsq
    if(effect.type == "cohenf") Z <- x$cohenf
    if(effect.type == "Rsq") effect.type = "R-squared"
    if(effect.type == "cohenf") effect.type = "Cohen's f-squared"
    Fs <- Fs[,1]
    SS <- SS[,1]
    MS <- MS[,1]
    Rsq <- x$Rsq[,1]
    cohenf <- x$cohenf[,1]
    if(!is.null(Z)) {
      if(!inherits(Z, "matrix")) Z <- matrix(Z, 1, length(Z))
      P.val <- apply(Z, 1, pval) 
      Z <- apply(Z, 1, effect.size)
      } else P.val <- NULL
    
    Residuals <- c(df[k+1], RSS[[1]], RSS[[1]]/df[k+1], 
                   RSS[[1]]/TSS[[1]], rep(NA, 3))
    Total <- c(df[k+2], TSS[[1]], rep(NA, 5))
    tab <- data.frame(Df = df[1:k], SS=SS, MS = MS, Rsq = Rsq, 
                      F = Fs, Z = Z, P.val = P.val)
    tab <- rbind(tab, Residuals = Residuals, Total = Total)
    colnames(tab)[NCOL(tab)] <- paste("Pr(>", effect.type, ")", sep="")
    class(tab) = c("anova", class(tab))
    SS.type <- x$SS.type
    
    if(any(tab$Df == 0)){
      rmove <- which(tab$Df == 0)
      tab <- tab[-rmove, , drop = FALSE]
    }
    
    options(warn = ow)
    
    out <- list(table = tab, perm.method = pm, perm.number = perms,
                est.method = est, fixed.est = fixed.est, 
                random.est = random.est, SS.type = SS.type, 
                effect.type = effect.type,
                call = object$call)
    
      } else {
        RSS.model <- x$RSS.model
        tab <- data.frame(df = df, SS = RSS.model[[1]], MS = RSS.model[[1]]/df)
        rownames(tab) <- c("Residuals")
        class(tab) = c("anova", class(tab))
        if(object$LM$gls) est <-"GLS" else est <- "OLS"
        out <- list(table = tab, perm.method = pm, perm.number = perms,
                    est.method = est, SS.type = NULL, effect.type = NULL,
                    call = object$call)
        
      }
  class(out) <- "anova.lm.rrpp"
  out
  }

# aov.multi.model
# performs ANOVA on multiple models
# used in anova.lm.rrpp
aov.multi.model <- function(object, lm.list,
                            effect.type = c("F", "cohenf", "SS", "MS", "Rsq"),
                            print.progress = TRUE) {
  
  effect.type <- match.arg(effect.type)
  
  if(inherits(object, "lm.rrpp")) refModel <- object else 
    stop("The reference model is not a class lm.rrpp object")
  PermInfo <- getPermInfo(refModel, attribute = "all")
  ind <- PermInfo$perm.schedule
  perms <- length(ind)
  
  if(refModel$LM$gls) {
    X <- refModel$LM$X
    Y <- refModel$LM$Y
    Pcov <- refModel$LM$Pcov
    Cov <- refModel$LM$Cov  
    if(!is.null(Cov) && is.null(Pcov)) {
      Pcov <- Cov.proj(Cov)
    }
    X <- if(!is.null(Pcov)) Pcov %*% X else
      X * sqrt(refModel$LM$weights)
    Y <- if(!is.null(Pcov)) Pcov %*% Y else
      Y * sqrt(refModel$LM$weights)
    
  } else {
    X <- refModel$LM$X
    Y <- refModel$LM$Y
  }
  
  B <- if(refModel$LM$gls) refModel$LM$gls.coefficients else 
    refModel$LM$coefficients
  
  refQR <- getModels(refModel, "qr")
  refQR <- refQR$full[[length(refQR$full)]]
  
  K <- length(lm.list)
  QRlist <- lapply(1:K, function(j){
    m <- lm.list[[j]]
    mQR <- getModels(m, "qr")
    mQR <- mQR$full[[length(mQR$full)]]
    mQR
  })
  
  U <- as.matrix(refQR$Q)
  n <- refModel$LM$n
  p <- refModel$LM$p
  Yh <- as.matrix(fastFit(U, Y, n, p))
  R <- as.matrix(Y) - Yh
  
  
  Ulist <- lapply(1:K, function(j){
    QRlist[[j]]$Q
  })
  
  if(print.progress){
    if(K > 1)
    cat(paste("\nSums of Squares calculations for", K, "models:", 
              perms, "permutations.\n")) else
      cat(paste("\nSums of Squares calculations for", K, "model:", 
                perms, "permutations.\n"))
    pb <- txtProgressBar(min = 0, max = perms+5, initial = 0, style=3)
  }

  int <- attr(refModel$LM$Terms, "intercept")
  if(refModel$LM$gls) {
    Pcov <- refModel$LM$Pcov
    Cov <- refModel$LM$Cov  
    if(!is.null(Cov) && is.null(Pcov)) {
      Pcov <- Cov.proj(Cov)
    }
    int <- if(!is.null(Pcov))  Pcov %*% rep(int, n) else
      sqrt(refModel$LM$weights)
  } else int <- rep(int, n)
  
  Qint <- QRforX(int)
  U0 <- Qint$Q
  yh0 <- as.matrix(fastFit(U0, Y, n, p))
  r0 <- as.matrix(Y) - yh0
  
  rY <- function(ind.i) Yh + R[ind.i, , drop = FALSE]
  rY0 <- function(ind.i) yh0 + r0[ind.i, , drop = FALSE]
  
  RSS <- function(ind.i, U0, U, Ul, K, n, p, Y, yh0, r0) {
    y <- as.matrix(rY(ind.i))
    y0 <- as.matrix(rY0(ind.i))
    rss0  <- sum(y^2) - sum(crossprod(U, y)^2)
    rss00 <- sum(y0^2) - sum(crossprod(U0, y)^2)
    
    rss <- lapply(1:K, function(j){
      u <- Ul[[j]]
      sum(y^2) - sum(crossprod(u, y)^2)
    })
    tss <- sum(y0^2) - sum(crossprod(U0, y)^2)
    
    RSSp <- c(rss0, unlist(rss), tss)
    RSSp
  }
  
  rss.list <- list(ind.i = NULL, U0 = U0, U = U, 
                   Ul = Ulist, K = K, n = n , p = p, Y = Y,
                   yh0 = yh0, r0 = r0)
  
  RSSp <- sapply(1:perms, function(j){
    step <- j
    if(print.progress) setTxtProgressBar(pb,step)
    rss.list$ind.i <- ind[[j]]
    do.call(RSS, rss.list)
  })
  
  RSSy <- as.matrix(RSSp[nrow(RSSp), , drop = FALSE])
  RSSp <- as.matrix(RSSp[-(nrow(RSSp)), , drop = FALSE])
  RSSy <- as.matrix(matrix(RSSy, nrow(RSSp), perms, byrow = TRUE))
  
  fit.names <- c(refModel$call[[2]], lapply(1:K, function(j) 
    lm.list[[j]]$call[[2]]))
  rownames(RSSp) <- rownames(RSSy) <- fit.names
  
  SS <- rep(RSSp[1, , drop = FALSE], each = K + 1) - RSSp
  
  Rsq <-  SS / RSSy
  
  dfe <- n - c(getRank(refQR), unlist(lapply(1:K, 
                     function(j) getRank(QRlist[[j]]))))
  df <- dfe[1] - dfe
  df[1] <- 1
  
  MS <- SS/rep(df, perms)
  MSE <- RSSp/matrix(rep(dfe, perms), length(dfe), perms)
  Fs <- MS/MSE
  
  SS[which(zapsmall(SS) == 0)] <- 1e-32
  MS[which(zapsmall(MS) == 0)] <- 1e-32
  Rsq[which(zapsmall(Rsq) == 0)] <- 1e-32
  Fs[which(zapsmall(Fs) == 0)] <- 1e-32
  
  if(effect.type == "SS") {
    Pvals <- apply(SS, 1, pval)
    Z <- apply(SS, 1, effect.size)
  } else   if(effect.type == "MS") {
    Pvals <- apply(MS, 1, pval)
    Z <- apply(MS, 1, effect.size)
  } else   if(effect.type == "Rsq") {
    Pvals <- apply(Rsq, 1, pval)
    Z <- apply(Rsq, 1, effect.size)
  } else{
    Pvals <- apply(Fs, 1, pval)
    Z <- apply(Fs, 1, effect.size)
  }
  SS[1,] <- NA
  MS[1,] <- NA
  Fs[1,] <- NA
  Pvals[1] <- NA
  Z[1] <- NA
  
  RSS.obs <- RSSp[,1]
  SS.obs <- SS[,1]
  MS.obs <- MS[,1]
  Rsq.obs <- Rsq[,1]
  F.obs <- Fs[,1]
  
  tab <- data.frame(ResDf = dfe, Df = df, RSS = RSS.obs, SS = SS.obs, MS = MS.obs,
                    Rsq = Rsq.obs, F = F.obs, Z = Z, P = Pvals)
  tab$DF[1] <- NA
  tab$Rsq <- zapsmall(tab$Rsq, digits = 8)

  
  if(effect.type == "SS") p.type <- "Pr(>SS)" else
    if(effect.type == "MS") p.type <- "Pr(>MS)" else
      if(effect.type == "Rsq") p.type <- "Pr(>Rsq)" else
        if(effect.type == "cohenf") p.type <- "Pr(>cohenf)" else 
          p.type <- "Pr(>F)" 
  names(tab)[which(names(tab) == "P")] <- p.type
  rownames(tab)[1] <- paste(rownames(tab)[1], "(Null)")
  tab <- tab[, 1:9]
  tab <- rbind(tab, NA)
  
  nt <- nrow(tab)
  rownames(tab)[nt] <- "Total"
  tab$RSS[nt] <- RSSy[1]
  tab[nt, 1] <- n - 1
  
  class(tab) <- c("anova", class(tab))
  
  step <- perms + 5
  if(print.progress) {
    setTxtProgressBar(pb,step)
    close(pb)
  }
  pm <- "RRPP"
  if(refModel$LM$gls) est <- "GLS" else est <- "OLS"
  
  out <- list(table = tab, perm.method = pm, perm.number = perms,
              est.method = est, SS.type = NULL, effect.type = effect.type,
              SS = SS[-1,], MS = MS[-1,], Rsq = Rsq[-1,], F = Fs[-1,],
              call = object$call)
  
class(out) <- "anova.lm.rrpp"
out

}

aov.me <- function(object){
  
  perms <- getPermInfo(object, "perms")

  out <- list(table = object$AOV, perm.method = "RRPP", perm.number = perms,
              est.method = "OLS", SS.type = "Within-subject II", effect.type = "SNR",
              call = object$call)
  class(out) <- "anova.lm.rrpp"
  out
}

# getSlopes
# gets the slopes for groups from a lm.rrpp fit
# used in pairwise
getSlopes <- function(fit, x, g){
  
  beta <- fit$LM$random.coef
  k <- length(beta)
  beta <- beta[[k]]
  n <- fit$LM$n
  p <- fit$LM$p
  X <- fit$LM$X
  X <- X[, colnames(X)  %in% rownames(beta[[1]]), drop = FALSE]
  getFitted <- function(b) X %*% b
  fitted <- lapply(beta, getFitted)
  Xn <- model.matrix(~ g * x + 0)
  Q <- QRforX(Xn)
  H <- tcrossprod(fast.solve(Q$R), Q$Q)
  getCoef <- function(f) H %*% f
  Coef <- lapply(fitted, getCoef)
  group.slopes <- function(B){ # B of form ~ group * x + 0
    gp <- QRforX(model.matrix(~g))$rank
    gnames <- rownames(B)[1:gp]
    B <- as.matrix(B[-(1:gp), , drop = FALSE])
    B[2:gp, ] <- B[2:gp, , drop = FALSE] + 
      matrix(rep(B[1,], each = gp -1), gp -1, p)
    rownames(B) <- gnames
    B
  }
  slopes <- lapply(Coef, group.slopes)
  rename <- function(x) {
    dimnames(x)[[1]] <- levels(g)
    x
  }
  slopes <- lapply(slopes, rename)
  slopes
}

# getLSmeans
# gets the LS means for groups from a lm.rrpp fit, 
# after constraining covariates to mean values
# used in pairwise
getLSmeans <- function(fit, g){
  beta <- fit$LM$random.coef
  k <- length(beta)
  beta <- beta[[k]]
  n <- fit$LM$n
  dat <- fit$LM$data
  covCheck <- sapply(dat, class)
  for(i in 1:length(covCheck)) if(covCheck[i] == "numeric") 
    dat[[i]] <- mean(dat[[i]])
  L <- model.matrix(fit$LM$Terms, data = dat)
  L <- L[, colnames(L)  %in% rownames(beta[[1]]), drop = FALSE]
  getFitted <- function(b) {
    b <- b[rownames(b) %in% colnames(L), , drop = FALSE]
    L %*% b
  } 
  fitted <- lapply(beta, getFitted)
  Xn <- model.matrix(~ g + 0)
  Q <- QRforX(Xn)
  H <- tcrossprod(fast.solve(Q$R), Q$Q)
  getCoef <- function(f) as.matrix(H %*% f)
  means <- lapply(fitted, getCoef)
  rename <- function(x) {
    dimnames(x)[[1]] <- levels(g)
    x
  }
  means <- lapply(means, rename)
  names(means) <- c("obs", paste("iter", 1:(length(means) - 1), sep = "."))
  means
}

#' Support function for RRPP
#'
#' Calculate vector correlations for a matrix (by rows).  
#' Used for pairwise comparisons.
#'
#' @param M Matrix for vector correlations.
#' @keywords utilities
#' @export
#' @author Michael Collyer
vec.cor.matrix <- function(M) {
  options(warn = -1)
  M = as.matrix(M)
  w = 1/sqrt(rowSums(M^2))
  vc = tcrossprod(M*w)
  diag(vc) <- 1
  options(warn = 0)
  vc
}

# Pval.list
# P-values across a list
# used in pairwise
Pval.list <- function(M){
  
  y <- sapply(M, function(x) as.vector(as.dist(x)))
  if(is.vector(y)) y <- matrix(y, 1, length(y))
  P <- apply(y, 1, pval)
  D <- as.dist(M[[1]])
  D[1:length(D)] <- P
  P <- as.matrix(D)
  diag(P) <- 1
  P
}

# effect.list
# effect size across a list
# used in pairwise
effect.list <- function(M){
  
  y <- sapply(M, function(x) as.vector(as.dist(x)))
  if(is.vector(y)) y <- matrix(y, 1, length(y))
  Z <- apply(y, 1, effect.size)
  D <- as.dist(M[[1]])
  D[1:length(D)] <- Z
  as.matrix(D)
}

# percentile.list
# find percentiles across a list
# used in pairwise
percentile.list <- function(M, confidence = 0.95){
  P <- M[[1]]
  n <- length(M)
  for(i in 1:length(P)) {
    y <- sapply(1:n, function(j) M[[j]][i])
    P[i] <- quantile(y, confidence, na.rm = TRUE)
  }
  P
}

# d.summary.from.list
# find distance statistics from a list
# used in pairwise
d.summary.from.list <- function(M, confidence = 0.95){
  P <- Pval.list(M)
  Z <- effect.list(M)
  CL <- percentile.list(M, confidence)
  list(D=M[[1]], P=P, Z=Z, CL=CL, confidence = confidence)
}

# r.summary.from.list
# find vec correlation statistics from a list
# used in pairwise
r.summary.from.list <- function(M, confidence = 0.95){
  options(warn = -1)
  acos.mat <- function(x){
    a <- acos(x)
    diag(a) <- 1
    a
  }
  A <- lapply(M, acos.mat)
  options(warn = 0)
  P <- Pval.list(A)
  Z <- effect.list(A)
  aCL <- percentile.list(A, confidence)
  angle = A[[1]]
  diag(angle) <- 0
  list(r=M[[1]], angle = angle,
       P=P, 
       Z=Z, aCL=aCL, confidence = confidence)
}

# makePWDTable
# arrange distance statistics into a table
# used in pairwise
makePWDTable <- function(L) { # List from summary.from.list
  nms <- rownames(L$D)
  DD <- as.dist(L$D)
  DP <- as.dist(L$P)
  DZ <- as.dist(L$Z)
  DC <- as.dist(L$CL)
  nam.com <- combn(length(nms), 2)
  name.list <- list()
  for(i in 1:NCOL(nam.com)) 
    name.list[[i]]  <- paste(nms[nam.com[1,i]], nms[nam.com[2,i]], sep =":")
  name.list <- unlist(name.list)
  tab <- data.frame(d = as.vector(DD),
                    UCI = as.vector(DC),
                    Z = as.vector(DZ), 
                    P = as.vector(DP))
  rownames(tab) <- name.list
  colnames(tab)[2] <- paste("UCL (", L$confidence*100,"%)", sep = "")
  colnames(tab)[4] <- "Pr > d"
  tab
}

# makePWCorTable
# arrange vec cor statistics into a table
# used in pairwise
makePWCorTable <- function(L){
  nms <- rownames(L$r)
  DR <- as.dist(L$r)
  DA <- as.dist(L$angle)
  DP <- as.dist(L$P)
  DaZ <- as.dist(L$Z)
  DaC <- as.dist(L$aCL)
  nam.com <- combn(length(nms), 2)
  name.list <- list()
  for(i in 1:NCOL(nam.com)) 
    name.list[[i]]  <- paste(nms[nam.com[1,i]], nms[nam.com[2,i]], sep =":")
  name.list <- unlist(name.list)
  tab <- data.frame(r = as.vector(DR),
                    angle = as.vector(DA),
                    UCL = as.vector(DaC),
                    Z = as.vector(DaZ),
                    P = as.vector(DP))
  rownames(tab) <- name.list
  colnames(tab)[3] <- paste("UCL (", L$confidence*100,"%)", sep = "")
  colnames(tab)[5] <- "Pr > angle"
  tab

}

# leaveOneOut
# set-up for jackknife classification
# used in classify
leaveOneOut <- function(X, Y, n.ind) {
  x <- X[-n.ind, , drop = FALSE]
  QR <- QRforX(x)
  H <- tcrossprod(fast.solve(QR$R), QR$Q)
  y <- Y[-n.ind, , drop = FALSE]
  H %*% y
}

# RiReg
# Ridge Regularization of a covariance matrix, if needed
# used in classify
RiReg <- function(Cov, residuals){
  leads <- seq(0,1,0.005)[-1]
  leads <- leads[-length(leads)]
  I <- diag(1, NROW(Cov))
  N <- NROW(residuals)
  p <- NCOL(residuals)
  
  Covs <- lapply(1:length(leads), function(j){
    lambda <- leads[[j]]
    (lambda * Cov + (1 - lambda) * I)
  })
  logL <- sapply(1:length(leads), function(j){
    C <- Covs[[j]]
    N*p*log(2*pi) + N * log(det(C)) + sum(diag(
      residuals %*% fast.solve(C) %*% t(residuals) 
    ))
  })
  
  Covs[[which.min(logL)]]
  
}

cov.trace <- function(fit) {
  n <- fit$LM$n
  p <- fit$LM$p.prime
  if(fit$LM$gls){
    Pcov <- try(getModelCov(fit, "Pcov"), 
               silent = TRUE)
    if(inherits(Pcov, "try-error"))
      Pcov <- NULL
    
    Sig <- if(!is.null(Pcov))  crossprod(Pcov %*% fit$LM$gls.residuals)/n else
      crossprod(fit$LM$gls.residuals * sqrt(fit$LM$weights))/n
    
  }  else {
    
    Sig <- crossprod(fit$LM$residuals) /n
    
  }
  
  sum(Sig^2)
  
}

z.test <- function(aov.mm){
  effect.type = aov.mm$effect.type
  if(effect.type == "F") stat <- aov.mm$F
  if(effect.type == "cohenf") stat <- aov.mm$F
  if(effect.type == "SS") stat <- aov.mm$SS
  if(effect.type == "MS") stat <- aov.mm$MS
  if(effect.type == "Rsq") stat <- aov.mm$Rsq
  
  perms <- ncol(stat)
  m <- nrow(stat)
  stat.c <- sapply(1:m, function(j){
    s <- stat[j, , drop = FALSE]
    center(s)
  })
  
  index <- combn(m, 2)
  Dz <- Pz <- dist(matrix(0, m))

  zdj <- function(x, y, j) {
    obs <- x[j] - y[j]
    sigd <- sqrt(var(x) + var(y))
    obs/sigd
  }
  
  for(i in 1:ncol(index)) {
    x <- stat.c[,index[1,i]]
    y <- stat.c[,index[2,i]]
    res <- array(NA, perms)
    for(j in 1: perms) res[j] <- zdj(x, y, j)
    Dz[i] <- abs(res[1])
    Pz[i] <- pval(abs(res))
  }

Z = as.matrix(Dz)
P = as.matrix(Pz)

options(warn = -1)
mds <- cmdscale(Z, m-1, eig = TRUE)
options(warn = 0)
    
list(Z = as.matrix(Dz), P = as.matrix(Pz), 
     mds = mds,
     form.names = rownames(aov.mm$table)[1:m],
     model.names = paste("m", 0:(m-1), sep = ""))
  
}

wilks <- function(e) {
  e <- zapsmall(e)
  e <- e[e > 0]
  prod(1/(1 + e))
}

pillai <- function(e) {
  e <- zapsmall(e)
  e <- e[e > 0]
  sum(e/(1 + e))
}

hot.law <- function(e) {
  e <- zapsmall(e)
  e <- e[e > 0]
  sum(e)
}

# trajsize
# find path distance of trajectory
# used in: trajectory.analysis
trajsize <- function(y) {
  k <- NROW(y[[1]])
  tpairs <- cbind(1:(k-1),2:k)
  sapply(1:length(y), function(j) {
    d <- as.matrix(dist(y[[j]]))
    sum(d[tpairs])
  })
}

# trajorient
# find trajectory correlations from first PCs
# used in: trajectory.analysis
trajorient <- function(y, tn) {
  m <- t(sapply(1:tn, function(j){
    x <- y[[j]]
    La.svd(center.scale(x)$coords, 0, 1)$vt
  }))
  vec.cor.matrix(m)
}

# trajshape
# find shape differences among trajectories
# used in: trajectory.analysis
trajshape <- function(y){
  y <- Map(function(x) center.scale(x)$coords, y)
  M <- Reduce("+",y)/length(y)
  z <- apply.pPsup(M, y)
  z <- t(sapply(z, function(x) as.vector(t(x))))
  as.matrix(dist(z))
}

# lda.prep
# sets up lm.rrpp object to use in lda
# used in: lda.rrpp
lda.prep <- function(fit, tol = 1e-7, PC.no = NULL, newdata = NULL){
  dat <- fit$LM$data
  gls <- fit$LM$gls
  w <- fit$LM$weights
  Pcov <- fit$LM$Pcov
  
  dat.class <- sapply(dat, class)
  fac.list <- which(dat.class == "factor")
  if(length(fac.list) == 0) group <- factor(rep(1, n)) else {
    datf <- dat[fac.list]
    group <- factor(apply(datf, 1, paste, collapse = "."))
  }
  
  
  newY <- function(fit, group, w = NULL, Pcov=NULL) {
    Xg <- model.matrix(~group)
    Yg <- fit$LM$Y
    res <- if(gls) fit$LM$gls.residuals else fit$LM$residuals
    
    if(!is.null(w)) {
      nfit <- lm.fit(as.matrix(Xg * sqrt(w)), as.matrix(Yg * sqrt(w)))
      fitted <- Yg * sqrt(w) - res * sqrt(w)
      nY <- (fitted + nfit$residuals)/sqrt(w)
    } else if(!is.null(Pcov)) {
      PY <- Pcov %*% Yg
      nfit <- lm.fit(as.matrix(Pcov %*% Xg), PY)
      fitted <- PY - as.matrix(Pcov %*% res)
      nY <- fast.solve(Pcov) %*% (fitted + nfit$residuals)
    } else {
      nfit <- lm.fit(as.matrix(Xg), as.matrix(Yg))
      nY <- fit$LM$fitted + nfit$res
    }
    nY
  }
  
  Yn <- newY(fit, group, w, Pcov)
  dims <- dim(Yn)
  n <- dims[1]
  p <- dims[2]
  
  Yc <- if(gls) Yn - matrix(fit$LM$gls.mean, n, p, byrow = TRUE) else
    center(Yn)
  
  V <- crossprod(Yc)/n
  if(!is.null(Pcov)) V <- crossprod(Pcov %*% Yc)/n
  if(!is.null(w)) V <-  crossprod(Yc * sqrt(w))/n
  
  if(gls) s <- svd(V)
  
  PCA <- prcomp(Yn, tol = tol)
  if(gls) {
    PCA$rotation <- s$v
    PCA$sdev <- sqrt(s$d)
    PCA$x <- Yc %*% s$v
  }
  
  d <- zapsmall(PCA$sdev^2)
  d <- 1:length(d[d > 0])
  
  if(!is.null(PC.no)) {
    if(PC.no < length(d)) d <- 1:PC.no
  }
  
  PCA$rotation <- PCA$rotation[, d, drop = FALSE]
  PCA$sdev <- PCA$sdev[d]
  PCA$x <- as.matrix(PCA$x[, d, drop = FALSE])
  colnames(PCA$x) <- colnames(PCA$rotation) <- paste("PC", d, sep = "")
  
  Yn <- PCA$x
  
  if(!is.null(newdata)) {
    Yt <- newdata
    if(NCOL(newdata) != p) 
      stop("\nNumber of variables in newdata does not match the number 
           for the data in lm.rrpp fit.\n",
           call. = FALSE)
    Ytc <- if(gls) Yt - matrix(fit$LM$gls.mean, NROW(Yt), p, byrow = TRUE) else
      center(Yt)
    Yt <- Ytc %*% PCA$rotation
    
  } else Yt <- NULL
  
  list(Yn = Yn, Yt = Yt, group = group, gls = gls)
  
}

# looCVOne
# finds one predicted value from n-1 training
# values.  Used in looCV

looCVOne <-function(fit, n.ind, Pcov = NULL) {
  gls <- fit$LM$gls
  if(gls && is.null(Pcov)){
    Pcov <- try(getModelCov(fit, type = "Pcov"),
                silent = TRUE)
    if(gls && inherits(Pcov, "try-error")){
      w <- sqrt(fit$LM$weights)
      Pcov <- diag(1/w)
    }
  }
  
  X <- fit$LM$X
  Y <- fit$LM$Y
  x <- X[n.ind, , drop = FALSE]
  y <- Y[n.ind, , drop = FALSE]
  X <- X[-n.ind, , drop = FALSE]
  Y <- Y[-n.ind, , drop = FALSE]
  
  if(gls) Pcov <- Pcov[-n.ind, -n.ind, drop = FALSE]
  
  QR <- if(gls) QRforX(Pcov %*% X) else QRforX(X)
  H <- tcrossprod(fast.solve(QR$R), QR$Q)
  B <- if(gls)  H %*% (Pcov %*% Y) else
    H %*% Y
  x %*% B
  
}

looCVAll <- function(fit){
  gls <- fit$LM$gls
  Pcov <- try(getModelCov(fit, type = "Pcov"),
              silent = TRUE)
  if(gls && inherits(Pcov, "try-error")){
    w <- sqrt(fit$LM$weights)
    Pcov <- diag(1/w)
  }
  n <- fit$LM$n
  res <- lapply(1:n, function(j){
    r <- if(gls)
      looCVOne(fit, n.ind = j, 
               Pcov = Pcov) else
                 looCVOne(fit, n.ind = j)
    as.vector(r)
  })
  res <- if(length(res[[1]]) == 1) unlist(res) else
    do.call(rbind, res)
  return(res)
}

# looPCOne
# finds one PC vector via cross-validation
# used in looCV

looPCOne <-function(fit, n.ind) {
  
  gls <- fit$LM$gls
  Pcov <- try(getModelCov(fit, type = "Pcov"),
              silent = TRUE)
  if(gls && inherits(Pcov, "try-error")){
    w <- sqrt(fit$LM$weights)
    Pcov <- diag(1/w)
  }
  
  
  X <- fit$LM$X
  Y <- fit$LM$Y
  x <- X[n.ind, , drop = FALSE]
  y <- Y[n.ind, , drop = FALSE]
  X <- X[-n.ind, , drop = FALSE]
  Y <- Y[-n.ind, , drop = FALSE]
  
  if(gls) Pcov <- Pcov[-n.ind, -n.ind, drop = FALSE]
  
  QR <- if(gls) QRforX(Pcov %*% X) else QRforX(X)
  H <- tcrossprod(fast.solve(QR$R), QR$Q)
  B <- if(gls)  H %*% (Pcov %*% Y) else
    H %*% Y
  S <- svd(X %*% B)
  y %*% S$v
}

# looPCOne
# finds each PC vector via cross-validation for a model fit
# used in looCV
looPCAll<-function(fit, ...) {
  n <- fit$LM$n
  gls <- fit$LM$gls
  
  ord.args <- list(...)
  ord.args$Y <- fit$LM$Y
  QR <- getModels(fit, "qr")
  QR <- QR$full[[length(QR$full)]]
  ord.args$A <- tcrossprod(QR$Q)
  if(is.null(ord.args$tol)) 
    ord.args$tol <- 1e-6
  
  ord <- do.call(ordinate, ord.args)
  k <- 1:length(ord$d)
  
  res <- sapply(1:n, function(j) 
    looPCOne(fit, j))
  res <- if(is.matrix(res)) t(res) else
    matrix(res, length(res), 1)
  
  dimnames(res) <- if(gls) dimnames(fit$LM$gls.fitted) else
    dimnames(fit$LM$fitted)
  d <- svd(fit$LM$fitted)$d
  res <- res[,k]
  
  res <- ordinate(res, ord$x, rank. = max(k))
  list(raw = ord, cv = res)
}

## lmm.rrpp helper functions



# Not used but could in the future
getSubBlocks <- function(flmer){
  Zt <- getME(flmer, "Ztlist")[[1]]
  block <- as.factor(Zt@i + 1)
  block
}

# Not used but could in the future
indexZtoX <- function(X, Z){
  Z <- Z0 <- as.matrix(Z)
  kx <- ncol(X)
  kz <- ncol(Z)
  nx <- nrow(X)
  nz <- nrow(Z)
  if(kx != kz || nx != nz)
    stop("Dimensions of matrices do not match.\n",
         call. = FALSE)
  res <- array(NA, kx)
  z.index <- 1:kz
  
  for(i in seq_len(kx)){
    x <- X[, i, drop = FALSE]
    r <- sapply(1:kz, function(j){
      identical(x, Z[, j, drop = FALSE])
    })
    a <- which(r)
    res[i] <- z.index[a]
    Z <- as.matrix(Z[, -a, drop = FALSE])
    z.index <- z.index[-a]
    kz <- kz - 1
  }
  
  res
}


subjPartition <- function(kt, ns){ # kt = ncol(Z)
  # ns = n subjects
  step <- kt/ns
  ones <- seq(1, kt - step + 1, step)
  list(subjTerm = ones,
       slopeTerm = seq_len(kt)[-ones])
}

# Not used but could in the future
ZL_list <- function(init.fit, Y){
  Y <- as.matrix(Y)
  p <- NCOL(Y)
  Z <- getME(init.fit, "Z")
  result <- lapply(1:p, function(j){
    y <- Y[, j, drop = FALSE]
    ft <- suppressMessages(
      suppressWarnings(refit(init.fit, y)))
    Z %*% getME(ft, "Lambda")
  })
  
  result
  
}

getLMM_Hb <- function(X, Z = NULL) {
  big <- !is.null(Z)
  k1 <- ncol(X)
  k2 <- if(big) ncol(Z) else 0
  k <- k1 + k2
  XZ <- if(big) cbind(X, Z) else X
  out <- crossprod(XZ)
  if(big){
    D <- diag(k)
    diag(D)[1:k1] <- 0
    out <- out + D
  }
  tcrossprod(fast.solve(out), XZ)
}

---
title: "ANOVA versus MANOVA in RRPP"
author: "Michael L. Collyer"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{ANOVA versus MANOVA in RRPP}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
  
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
```

## Preliminaries

RRPP is an acronym for randomization of residuals in a permutation procedure.  *RRPP* refers to the package utilizing RRPP.

## What is ANOVA?

Analysis of variance (ANOVA) means different things to different people, but generally speaking, one recognizes an ANOVA table as a table of statistics including sums of squares, $SS$, mean squares , $MS$, $F$-statistics, and $P$-values.  In this table, such statistics might be found for different model terms and can be calculated for different types of $SS$.

## What is ANOVA in *RRPP*?

ANOVA in *RRPP* is a concept that generalizes the statistics used in univariate ANOVA to multivariate data.  The fundamental statistic is $SS$, calculated as the trace of the sums of squares and cross-products matrix, $\mathbf{S}$.  Thus, $SS$ is sum of each variable's $SS$, meaning for univariate data, $SS$ is found for a single variable, only. Dividing this trace form of $SS$ by appropriate degrees of freedom, produces $MS$ values, which can be used to calculate $F$-statistics; i.e., 

$$ F = \frac{MS_{effect}}{MS_{Random}} = \frac{SS_{effect}/df_{effect}}{SS_{random}/df_{random}}=
\frac{trace(\mathbf{S}_{effect})/df_{effect}}{trace(\mathbf{S}_{random})/df_{random}},$$

where $random$ can refer to a random effect or residuals, and $df$ is the appropriate degrees of freedom for effect and random $SS$.

In this manner, there is no fundamental difference in how univariate and multivariate statistics are calculated (the univariate statistics are a simplified version of the same multivariate statistics).  Therefore, ANOVA in *RRPP* does not distinguish between univariate and multivariate data, but refers to this consistency in calculation of statistics.

## What is MANOVA?

Multivariate analysis of variance (MANOVA) is perhaps typically thought of as an analog to ANOVA when data are multivariate.  In what is classically referred to as, "MANOVA," multivariate statistics are calculated, including Roy's maximum root, Pillai trace, Hotelling-Lawley trace, and Wilks $\Lambda$.  These statistics do not have probability distributions (density functions), so they are converted to statistics that are assumed to approximately follow $F$-distributions.  This works in cases that the number of observations are sufficiently larger than the number of response variables.  Therefore, the M in MANOVA can be thought of as an indicator that multivariate data are analyzed and/or that multivariate statistics are calculated as a means to estimate $F$-statistics, but the ANOVA in MANOVA is not different than parametric ANOVA for univariate data, once $F$-statistics are calculated.

## MANOVA in *RRPP*

In *RRPP*, the M in MANOVA references the calculation of multivariate statistics and generating their empirical sampling distributions.  Calculating multivariate statistics is not different than parametric MANOVA, but in *RRPP*, the statistics are calculated in every random permutation, eliminating the need to find an approximate probability distribution (the empirical sampling distribution is sufficient as an approximation).  Whereas parametric MANOVA could be thought of as the forcing of multivariate statistics into a univariate ANOVA-style framework, MANOVA in *RRPP* is better defined as using randomization of residuals in a permutation procedure on multivariate statistics in much the same way as the traces of $\mathbf{S}$ matrices in ANOVA.  Description of how RRPP works can be found in [another vignette](Using.RRPP.html)

All multivariate statistics are derived from eigenanalysis of

$$\mathbf{S}_{random}^{-1}\mathbf{S}_{effect},$$

where $\mathbf{S}_{random}$ is typically the $\mathbf{S}$ matrix for residuals.  Singular matrices are not a concern if data are projected into an Euclidean space of appropriate dimensions before matrix inversion.  However, Wilks $\Lambda$ might be less appropriate than other multivariate statistics, as it relies on products of eigenvalues rather than summation.

## Example of ANOVA and MANOVA in RRPP

When fitting a linear model in *RRPP* over many permutations, some ANOVA statistics are calculated automatically in every permutation.  Others
can be obtained from those statistics.

```{r}
library(RRPP)
data(Pupfish)
fit <- lm.rrpp(coords ~ Sex*Pop, SS.type = "I", 
               data = Pupfish, print.progress = FALSE) 
attributes(fit)
attributes(fit$ANOVA)
attributes(getANOVAStats(fit, stat = "all"))
```

The distributions of ANOVA statistics can then be used to construct an ANOVA table, via the anova.lm.rrpp S3 generic; i.e.,

```{r}
anova(fit)
```

To switch to MANOVA statistics, the linear model fit must be updated to include MANOVA statistics, in addition to the ANOVA statistics already generated.  The MANOVA statistics take more time to calculate, because of matrix inversion and eigenanalysis in every permutation, so it is not performed unless requested.  

```{r}
fitm <- manova.update(fit, print.progress = FALSE, tol = 0)
attributes(fitm)
attributes(fitm$MANOVA)
```

The "eigs" object returns all the eigenvalues of $\mathbf{S}_{random}^{-1}\mathbf{S}_{effect}$, because the default argument.  One can return the matrices, themselves, with "invR.H", where $\mathbf{H} = \mathbf{S}_{effect}$, and $\mathbf{R} = \mathbf{S}_{residual}$, in line with common notation.  ($\mathbf{H}$ and $\mathbf{R}$ are SSCP matrices for model effects and residuals, respectively.)  The class lm.rrpp fit has class manova.lm.rrpp added, and the S3 summary function produces a MANOVA-like table.

```{r}
summary(fitm, test = "Roy")
summary(fitm, test = "Pillai")

```

It should be apparent that although the ANOVA and MANOVA results have some similarities, the effect sizes ($Z$-scores) and $P$-values can vary.  Comparatively, the results of the MANOVA statistics can be influenced by variable covariances more so than ANOVA results (which are influenced solely by variances; i.e., the dispersion of values in the data space).  The results are also influenced by dimensionality.  In this case, there are more shape variables (112) than residual degrees of freedom (50), so data are projected into a lower-dimension Euclidean space to allow matrix inversion; i.e., $\mathbf{S}_{residual}^{-1}\mathbf{S}_{effect}$.  A tolerance of 0 means all possible PCs were retained for analysis.  (Tolerance is the permissible relative change in eigenvalues.  For example, tol = 0.01 means that if the k + 1st eigenvalue is not at least 1% smaller than the kth eigenvalue, retain only k eigenvectors.  The k + 1, k + 2, k + 3, ... eigenvalues decay at such a small rate that they are considered inconsequential.)

When the number of observations far exceeds the data space dimensions, this is okay, and the results will be qualitatively more similar to ANOVA results.  Consider this example (although reducing dimensions is not recommended as a typical solution):

```{r}
fitm <- manova.update(fit, print.progress = FALSE, tol = 0.001)
anova(fit)
summary(fitm)
```

Alternatively, one could specify an exact number of PCs to use in analysis.

```{r}
fitm <- manova.update(fit, print.progress = FALSE, PC.no  = 10)
anova(fit)
summary(fitm)
```

## Dimensionality Warning for MANOVA on high-dimensional data

If the maximum number of possible PCs is chosen (full data space dimensionality) **AND** the number of variables exceeds the number of observations, *RRPP* will attempt to assess the number of real dimensions in each random permutation and adjust SSCP matrices, accordingly.  (It is possible to randomly generate SSCP matrices of lower rank in random permutations, via RRPP.)  However, it cannot be guaranteed that eigenvalues will be positive in such cases.  If a warning message is delivered, "NaNs produced," this might be the reason.  Using a slightly lower number of PCs will probably resolve the issue.

## Conclusions and Suggestions

ANOVA in *RRPP* generalizes univariate ANOVA to multivariate data, its statistics are directly associated with the amount of dispersion in multivariate data spaces, irrespective of variable covariances, and data dimensionality has no effect or limitation on their calculation.

MANOVA in *RRPP* produces the same multivariate statistics found in parametric MANOVA (when the number of observations exceeds residual degrees of freedom), but unlike parametric MANOVA, $P$-values are estimated from empirical sampling distributions of statistics (rather than estimating values that approximately follow $F$-distributions, with constraints).  The choice of multivariate statistic is inconsequential for $P$-values, but makes a difference for effect sizes.  

Because high-dimensional data force a projection of data into Euclidean subspaces, to make matrix inversion and eigenanalysis possible, MANOVA results could differ from ANOVA results (both qualitatively and in terms of effect sizes).  For multivariate data with many more observations than variables (ideal conditions), results using either ANOVA or MANOVA will be comparable.  For high-dimensional data, ANOVA results might be more reliable.

---
title: "Using RRPP" 
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Using RRPP}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
  
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
```

# APPENDIX S2

This is a more detailed version of the section, "**Analytical Demonstrations**," within the Methods in Ecology and Evolution article, "**RRPP: An R package for fitting linear models to high-dimensional data using residual randomization**," by Michael L. Collyer and Dean C. Adams.  The purpose of this vignette is to offer an opportunity for people reading that article to see code and results together.  Some changes to code have been made, especially to enhance features that are difficult to introduce in the article, but the overall presentation should be basically the same.  As the package has been updated, this document has also been slightly edited to make sure any functional changes have been illustrated.

(Note parallel processing is available on Unix systems for `lm.rrpp`; see `lm.rrpp` help page for description on its use.  This is helpful for large data sets.)

Three data sets are provided in `RRPP` (with examples also in help pages), and are used here as examples for various analyses: 
    
    Pupfish (Collyer et al., 2015)
    PupfishHeads (Gilbert, 2016)
    PlethMorph (Adams & Collyer, 2018)
    
The first two datasets contain landmark-based geometric morphometric data collected from museum samples of Pecos pupfish (_Cyprinodon pecosensis_), representing body shape and cranial morphology, respectively.  Within both data sets, the `$coords`  objects are matrices of Procrustes residuals obtained from generalized Procrustes analysis (GPA) of configurations of anatomical landmarks. For the purposes of this example, it is sufficient to recognize that Procrustes residuals embody a highly multivariate dataset representing shape (see the R package, geomorph).  The third data set contains averaged linear measurements of 37 species of Plethodontid salamanders, plus a covariance matrix based on a Brownian model of evolution, given the phylogenetic relationship among the species (Adams & Collyer, 2018, for details).

***Example: Pupfish Cranial Morphology and Mixed-Model ANOVA***

In the first example we use a univariate dependent variable (head size, measured as the centroid size of the cranial landmark configuration; Bookstein, 1991) with a mixed-model design.  The following code highlights the analytical steps, with results categorized in Fig. 1:
	
```{r}
library(RRPP)
data("PupfishHeads")
PupfishHeads$logHeadSize <- log(PupfishHeads$headSize)
fit <- lm.rrpp(logHeadSize ~ sex + locality/year, 
               SS.type = "I", data = PupfishHeads, 
               print.progress = FALSE,
               turbo = FALSE, verbose = TRUE)
summary(fit)
anova(fit, effect.type = "F") 
```

Of important note, we choose to log-transform head size and include it as a separate variable in the RRPP data frame, `PupfishHeads`.  We accomplished this via the code above – rather than using `log(headSize)` - because downstream functions like `predict.lm.rrpp` work better without functions in the formula.   ANOVA was performed using random distributions of _F_-statistics to calculate _z_-scores and _P_-values (but one could use alternative distributions - see the `RRPP` help page). The S3 Generic functions (`summary`, `anova`) return summaries that remind the user how random data were generated, the type of _SS_, and how _z_-scores were calculated.  This particular ANOVA summary is a default that fails to consider the year fish were sampled as a random effect.   A mixed-model ANOVA update can be performed by changing the expected mean-square (_MS_) error estimates in each _F_ calculation: 
    
```{r}
anova(fit, effect.type = "F", 
  error = c("Residuals", "locality:year", "Residuals"))
```

This adjustment illustrates that the head size variation does not significantly differ between localities, with respect to the variation among sampling events.  The anova function can also be used for multi-model inference.

```{r}
fit.sex <- lm.rrpp(logHeadSize ~ sex, 
                   data = PupfishHeads, 
                   print.progress = FALSE)
fit.sex.loc<- lm.rrpp(logHeadSize ~ sex + locality, 
                      data = PupfishHeads, 
                      print.progress = FALSE)
fit.sex.loc.year<- lm.rrpp(logHeadSize ~ sex + locality/year, 
                           data = PupfishHeads, 
                           print.progress = FALSE)
anova(fit.sex, fit.sex.loc, fit.sex.loc.year, print.progress= FALSE)
```

One might wish to also look at individual model coefficients, and ascertain which have the largest effect:

```{r}
coef(fit, test = TRUE)
```

This function produces a table much like `summary.lm` output, but with bootstrap-generated confidence intervals of coefficients.  

It might be of interest to visualize model predictions for certain effects, holding constant other effects.  For example, if we want to look at confidence intervals to compare male and female head sizes, holding constant the effects of locality and sampling period, we could do the following:

```{r, fig.height=3, fig.width=5}
sizeDF <- data.frame(sex = c("Female", "Male"))
rownames(sizeDF) <- c("Female", "Male")
sizePreds <- predict(fit, sizeDF)
plot(sizePreds)

```

The plots are perfectly amenable (e.g., point type and color, line thickness, alternative labels, and additional text can be added or adjusted with typical `par` arguments).  
```{r, fig.height=3, fig.width=5}
plot(sizePreds, pch = 21, cex = 3, bg = c(2,4), lwd = 2)

```

Finally, the _SS_ type can be also toggled easily by refitting the model:

```{r}
fit2 <- lm.rrpp(logHeadSize ~ sex + locality/year, 
                SS.type = "II", data = PupfishHeads, print.progress = FALSE)
fit3 <- lm.rrpp(logHeadSize ~ sex + locality/year, 
                SS.type = "III", data = PupfishHeads, print.progress = FALSE)

anova(fit)
anova(fit2)
anova(fit3)

```

***Example: Pupfish Body Shape and High-Dimensional Data***

In the second example, we highlight the `RRPP` ability to efficiently handle large data computations.  For this demonstration, a 54(_n_) × 112 (_p_) matrix of Procrustes residuals are the data.  In every one of the 1,000 random permutations, RRPP shuffles residual vectors the same way for four different null models, estimates coefficients for four different full models, estimates the _SS_ as the difference between residual _SS_ (_RSS_) for four null-full model comparisons, and calculates the total _SS_, before calculating _MS_, _R_^2^, _F_, Cohen's _f_^2^, and Euclidean distances of coefficient vectors across all 1,000 permutations.  This process, plus packaging of results, took approximately 0.5 seconds on a notebook computer, without any parallel processing.
	In the second example, the initial steps are quite the same as the first example:

```{r}
data(Pupfish)
Pupfish$logSize <- log(Pupfish$CS) 
fit <- lm.rrpp(coords ~ logSize + Sex*Pop, SS.type = "I", 
               data = Pupfish, print.progress = FALSE,
               turbo = FALSE, verbose = TRUE) 
summary(fit, formula = FALSE)
anova(fit) 
coef(fit, test = TRUE) 

```
ANOVA results reveal that after accounting for body size allometry, not only are there significant inter-population differences in body shape and sexual dimorphism in body shape, but sexual dimorphism also significantly varies between the two populations.  A fuller evaluation of these results is available in Collyer et al. (2015).  It is worth taking a moment to realize why this analysis is a valuable alternative to a parametric M-ANOVA.  The following code attempts to perform a parametric M-ANOVA:

```{r, error = TRUE}
fit$LM$data$coords <- Pupfish$coords
fit.par <- lm(fit$call$f1, data = fit$LM$data)
all.equal(fit$LM$coefficients, fit.par$coefficients)
summary(manova(fit.par))
```
Although both functions return the same coefficients, the error summarizing the attempted parametric M-ANOVA clearly indicates the limitation of having residual degrees of freedom (rank) lower than the number of variables.   Returning to the `lm.rrpp` fit, which does not suffer this problem, we can look at the precision of group mean estimation, accounting for allometric shape variation, by doing the following:

```{r, fig.height=5, fig.width=6}
shapeDF <- expand.grid(Sex = levels(Pupfish$Sex), Pop = levels(Pupfish$Pop))
rownames(shapeDF) <- paste(shapeDF$Sex, shapeDF$Pop, sep = ".")
shapePreds <- predict(fit, shapeDF, confidence = 0.95)
plot(shapePreds, PC = TRUE, ellipse = TRUE) # generic 
plot(shapePreds, PC = TRUE, ellipse = TRUE, 
     pch = 19, col = 1:NROW(shapeDF)) # with added par arguments

groups <- interaction(Pupfish$Sex, Pupfish$Pop)
plot(fit, type = "PC") # generic
plot(fit, type = "PC", pch = 19, col = groups) # with added par arguments

```

These plots differ as there is a rotational difference between the covariance matrices estimated with 4 predicted and 54 fitted values.  Additionally, the former illustrates prediction precision and the latter sample dispersion.  Both functions allow passing `par` arguments to the plot as well as saving plot data for more advanced plotting.  The following is a regression-type plot:

```{r, fig.height=5, fig.width=6}
plot(fit, type = "regression", reg.type = "PredLine", 
    predictor = Pupfish$logSize, pch=19,
    col = as.numeric(groups))
```

The function, pairwise, can be used to test pairwise differences between least-squares means with:
```{r}
PWT <- pairwise(fit, groups = interaction(Pupfish$Sex, Pupfish$Pop))
summary(PWT, confidence = 0.95)

```
Much like the tukeyHSD function in the R stats package, `pairwise` will generate tables with confidence intervals and _P_-values for the pairwise statistic, Euclidean distance between least-squares means.  This function could also be used for pairwise comparison of slopes in analysis of covariance (ANCOVA) designs.

```{r}
fit2 <- lm.rrpp(coords ~ logSize * Sex * Pop, SS.type = "I", 
                data = Pupfish, print.progress = FALSE, iter = 999) 
summary(fit2, formula = FALSE)
anova(fit, fit2, print.progress = FALSE)

PW2 <- pairwise(fit2, fit.null = fit, groups = groups, 
                covariate = Pupfish$logSize, print.progress = FALSE) 
PW2
summary(PW2, confidence = 0.95, 
        test.type = "dist") # distances between slope vector lengths
summary(PW2, confidence = 0.95, 
        test.type = "dist", stat.table = FALSE)
summary(PW2, confidence = 0.95, 
        test.type = "VC",
        angle.type = "deg") # correlation between slope vectors (and angles)

```

Because the Procrustes residuals are projected into a Euclidean tangent space (see geomorph function, gpagen; Adams et al., 2017), this analysis could be performed with an object of class `dist` (values from lower half of a distance matrix) representing the inter-specimen shape (Euclidean) distances, using the following code:

```{r}
D <- dist(Pupfish$coords) # inter-observation Euclidean distances
Pupfish$D <- D

fitD <- lm.rrpp(D ~ logSize + Sex*Pop, SS.type = "I", 
                data = Pupfish, print.progress = FALSE) 

summary(fitD)
summary(fit)

anova(fitD)
anova(fit)
```
The ANOVA results with either method are exactly the same.


***Example: Plethodontid Morphology, Phylogenetics, and GLS Estimation***

In the third example, we highlight GLS estimation.  The following code creates two `lm.rrpp` fits using OLS and GLS, respectively, and evaluates them as in previous examples:

```{r}
data(PlethMorph)
fitOLS <- lm.rrpp(TailLength ~ SVL, 
                  data = PlethMorph,
                  print.progress = FALSE,
                  turbo = FALSE, verbose = TRUE)
fitGLS <- lm.rrpp(TailLength ~ SVL, 
                  data = PlethMorph, 
                  Cov = PlethMorph$PhyCov,
                  print.progress = FALSE,
                  turbo = FALSE, verbose = TRUE)

anova(fitOLS)
anova(fitGLS)

coef(fitOLS, test = TRUE)
coef(fitGLS, test = TRUE)

```

Although analyses on either model fit indicate a significant relationship between tail length and snout-to-vent length (SVL), the GLS coefficients test and ANOVA show how phylogenetic auto-correlation among species augments the OLS-estimated relationship.  The following is a multivariate example:

```{r, fig.height=4, fig.width=8}
Y <- as.matrix(cbind(PlethMorph$TailLength,
PlethMorph$HeadLength,
PlethMorph$TailLength,
PlethMorph$Snout.eye,
PlethMorph$BodyWidth,
PlethMorph$Forelimb,
PlethMorph$Hindlimb))
PlethMorph$Y <- Y

fitOLSm <- lm.rrpp(Y ~ SVL, data = PlethMorph,
                   print.progress = FALSE,
                  turbo = FALSE, verbose = TRUE)
fitGLSm <- lm.rrpp(Y ~ SVL, data = PlethMorph, 
                   Cov = PlethMorph$PhyCov,
                   print.progress = FALSE,
                  turbo = FALSE, verbose = TRUE)

anova(fitOLSm)
anova(fitGLSm)

sizeDF <- data.frame(SVL = sort(PlethMorph$SVL))

plot(predict(fitOLSm, sizeDF), PC= TRUE) # Correlated error
plot(predict(fitGLSm, sizeDF), PC= TRUE) # Independent error

```


***Analytical Summary***

On the surface, these three examples and their analyses should seem intuitive to any user of R who has used the `lm` function plus its associated S3 generics (`coef, predict, resid, fitted, summary`, and `anova`), all of which can be used on `lm.rrpp` model fits.  The functions, `pairwise` (not an S3 generic) and `anova`, also allow pairwise comparisons of least-squares means or slopes and multi-model inferences, respectively. Advanced users will recognize, however, much more extensive usable results for adaptive programming.  The output from a `lm.rrpp` fit is arranged hierarchically, i.e.:

```{r}
attributes(fit)

```

Within the `$LM` partition, all attributes of the lm function are found, in addition to coefficients for every random permutation.  Within the `$ANOVA` partition, the SS type, plus `SS`, `MS`, `R`^2^, `F`, and Cohen's `f`^2^ for all permutations, as well as effect sizes estimated for each of these are provided.  Within the `$PermInfo` partition, the number of permutations, type (RRPP or randomization of "full" data values, FRPP), and sampling frame in every permutation (schedule) are provided.  Thus, lm.rrpp is the workhorse that makes all downstream analysis efficient.

*

***References***

Adams, D. C., & Collyer, M. L. (2018). Multivariate phylogenetic comparative methods: Evaluations, comparisons, and recommendations. Systematic Biology, 67, 14–31.

Adams, D. C., Collyer, M. L., Kaliontzopoulou, A., & Sherratt, E. (2017). Geomorph: Software for geometric morphometric analyses. R package version 3.0.6. 

Adams, D. C., Rohlf, F. J., & Slice, D. E. 2013. A field comes of age: Geometric morphometrics in the 21st century. Hystrix, 24, 7–14.

Bookstein, F. L. (1991). Morphometric tools for landmark data: geometry and biology. Cambridge: Cambridge University Press.

Collyer, M. L., Sekora, D. J., & Adams, D. C. (2015). A method for analysis of phenotypic change for phenotypes described by high-dimensional data. Heredity, 115, 357–365.

Gilbert, M. C. (2016). Impacts of habitat fragmentation on the cranial morphology of a threatened desert fish (_Cyprinodon pecosensis_). Masters thesis.  Western Kentucky University, Bowling Green, KY, USA.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.support.code.r
\name{rrpp.data.frame}
\alias{rrpp.data.frame}
\title{Create a data frame for lm.rrpp analysis}
\usage{
rrpp.data.frame(...)
}
\arguments{
\item{...}{Components (objects) to combine in the data frame.}
}
\description{
Create a data frame for lm.rrpp analysis, when covariance or distance 
matrices are used
}
\details{
This function is not much different than \code{\link{data.frame}} but is 
more flexible to allow
distance matrices and covariance matrices to be included.  Essentially, 
this function creates a list,
much like an object of class \code{data.frame} is also a list.  However, 
\code{rrpp.data.frame} is
less concerned with coercing the list into a matrix and more concerned 
with matching the number of observations (n).
It is wise to use this function with any \code{lm.rrpp} analysis so that 
\code{\link{lm.rrpp}} does not have to search
the global environment for needed data.

It is assumed that multiple data sets for the same subjects are in the 
same order.

See \code{\link{lm.rrpp}} for examples.
}
\examples{
# Why use a rrpp.data.frame?
y <- matrix(rnorm(30), 10, 3)
x <- rnorm(10)
df <- data.frame(x = x, y = y)
df
rdf <- rrpp.data.frame(x = x, y = y)
rdf # looks more like a list

is.list(df)
is.list(rdf)

d <- dist(y) # distance matrix as data

# One can try this but it will result in an error
# df <- data.frame(df, d = d) 
rdf <- rrpp.data.frame(rdf, d = d) # works

fit <- lm.rrpp(d ~ x, data = rdf, iter = 99)
summary(fit)
}
\author{
Michael Collyer
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{summary.model.comparison}
\alias{summary.model.comparison}
\title{Print/Summary Function for RRPP}
\usage{
\method{summary}{model.comparison}(object, ...)
}
\arguments{
\item{object}{Object from \code{\link{model.comparison}}}

\item{...}{Other arguments passed onto model.comparison}
}
\description{
Print/Summary Function for RRPP
}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict.lm.rrpp.r
\name{predict.lm.rrpp}
\alias{predict.lm.rrpp}
\title{predict for lm.rrpp model fits}
\usage{
\method{predict}{lm.rrpp}(object, newdata = NULL, block = NULL, confidence = 0.95, ...)
}
\arguments{
\item{object}{Object from \code{\link{lm.rrpp}}.}

\item{newdata}{Data frame of either class \code{\link{data.frame}} 
or \code{\link{rrpp.data.frame}}.  If null,
the data frame from the lm.rrpp fit will be used, effectively calculating 
all fitted values and
their confidence intervals.  If a numeric variable is missing from newdata, 
an attempt to average the values
will be made in prediction; i.e., least squares means for factor levels 
can be found.  All factors used in the
\code{\link{lm.rrpp}} fit should be represented in the newdata data frame, 
with appropriate factor levels.}

\item{block}{An optional factor for blocks within which to restrict resampling
permutations.}

\item{confidence}{The desired confidence interval level for prediction.}

\item{...}{Other arguments (currently none)}
}
\description{
Computes predicted values from an \code{\link{lm.rrpp}} 
model fit, using bootstrapped residuals
to generate confidence intervals.  (Residuals are the residuals of 
the lm.rppp fit, not its null model.  The bootstrap
procedure resamples residual vectors with replacement.)
The bootstrap permutations use the same number of iterations and 
seed as used
in the \code{\link{lm.rrpp}} model fit. A \code{\link{predict.lm.rrpp}} 
object can be plotted using various options.
See \code{\link{plot.predict.lm.rrpp}}.

Note that if data offsets are used (if the offset argument is used 
when fitting a \code{\link{lm.rrpp}} model),
they are ignored for estimating coefficients over iterations.  
Offsets are subtracted from data in \code{\link[stats]{lm}} and 
added to predicted values in \code{\link[stats]{predict.lm}}, 
effectively adjusting the intercept and then un-adjusting
it for predictions.  This causes problems if the newdata have a 
different number of observations than the original
model fit.
}
\examples{
\dontrun{
# See examples for lm.rrpp to see how predict.lm.rrpp works in conjunction
# with other functions

data(Pupfish)

# CS is centroid (fish) size
fit <- lm.rrpp(coords ~ log(CS)  + Sex*Pop, 
SS.type = "I", data = Pupfish, iter = 999) 

# Predictions (holding alternative effects constant)

shapeDF <- expand.grid(Sex = levels(Pupfish$Sex), Pop = levels(Pupfish$Pop))
rownames(shapeDF) <- paste(shapeDF$Sex, shapeDF$Pop, sep = ".")
shapeDF

shapePreds <- predict(fit, shapeDF)
summary(shapePreds)
summary(shapePreds, PC = TRUE)

shapePreds99 <- predict(fit, shapeDF, confidence = 0.99)
summary(shapePreds99, PC = TRUE)

# Plot prediction

plot(shapePreds, PC = TRUE)
plot(shapePreds, PC = TRUE, ellipse = TRUE)
plot(shapePreds99, PC = TRUE)
plot(shapePreds99, PC = TRUE, ellipse = TRUE)
}

}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.data.r
\docType{data}
\name{PupfishMR}
\alias{PupfishMR}
\title{Experimental metabolic rate data from pupfish}
\description{
Simulated metabolic rate data from experimental pupfish
}
\details{
These data are simulated data meant to mimic a subset of the 
data from Collyer & Stockwell (2004).  Variables include experimental treatment 
(Control versus artificially infected), mass in grams, standard length in mm,
and metabolic rate (oxygen consumption, mg O2 per L per hour).  These data do not have
a temporal component like the original data, and do not include aberrant or missing
observations.  These data are suitable data for demonstrating linear regression
and analysis of covariance 
(ANCOVA), used in the book, Permutational Biometry (Collyer & Adams, 2025).
}
\references{
Collyer, M. L., & Stockwell, C. A. (2004). Experimental 
evidence for costs of parasitism for a threatened species, 
White Sands pupfish (Cyprinodon tularosa). Journal of 
Animal Ecology, 73(5), 821-830.

Collyer, M. L., & Adams D. C. (2025). Permutational Biometry: Volume
1, Univariate Data. Iowa State University Digital Press.  NOT YET PUBLISHED.
}
\author{
Michael Collyer
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{print.anova.lm.rrpp}
\alias{print.anova.lm.rrpp}
\title{Print/Summary Function for RRPP}
\usage{
\method{print}{anova.lm.rrpp}(x, ...)
}
\arguments{
\item{x}{print/summary object (from \code{\link{lm.rrpp}})}

\item{...}{other arguments passed to print/summary}
}
\description{
Print/Summary Function for RRPP
}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{summary.ICCstats}
\alias{summary.ICCstats}
\title{Print/Summary Function for RRPP}
\usage{
\method{summary}{ICCstats}(object, ...)
}
\arguments{
\item{object}{print/summary object (from \code{\link{ICCstats}})}

\item{...}{other arguments passed to print/summary}
}
\description{
Print/Summary Function for RRPP
}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prep.lda.r
\name{prep.lda}
\alias{prep.lda}
\title{Linear discriminant function for lm.rrpp model fits}
\usage{
prep.lda(
  fit,
  tol = 1e-07,
  PC.no = NULL,
  newdata = NULL,
  inherent.groups = FALSE,
  ...
)
}
\arguments{
\item{fit}{A linear model fit using \code{\link{lm.rrpp}}.}

\item{tol}{A tolerance used to decide if the matrix of data is singular.
This value is passed onto both
\code{\link[MASS]{lda}} and \code{\link[stats]{prcomp}}, internally.}

\item{PC.no}{An optional argument to define the specific number of 
principal components (PC) used in analysis.
The minimum of this value or the number of PCs resulting from the tol 
argument will be used.}

\item{newdata}{An optional matrix (or object coercible to a matrix) for 
classification.  If NULL,
all observed data are used.}

\item{inherent.groups}{A logical argument in case one wishes to have the 
inherent groups in the model fit revealed.  If 
TRUE, no other analysis will be done than to reveal the groups.  This 
argument should always be FALSE to perform 
a classification analysis.}

\item{...}{Arguments passed to \code{\link[MASS]{lda}}.
See \code{\link[MASS]{lda}} for details}
}
\value{
A list of arguments that can be passed to \code{\link[MASS]{lda}}.  
As a minimum, these arguments include
$x, $grouping, and $tol.  If newdata is not NULL, $newdata, using the same 
transformation and PCs as for the data,
will also be included.
}
\description{
Function creates arguments for \code{\link[MASS]{lda}} 
or \code{\link[MASS]{qda}} from a \code{\link{lm.rrpp}} fit.
}
\details{
This function uses a \code{\link{lm.rrpp}} fit to produce the 
data and the groups to use in \code{\link[MASS]{lda}} or
\code{\link[MASS]{qda}}.There are two general purposes of this 
function that are challenging when using \code{\link[MASS]{lda}}, directly.
First, this function finds the inherent groups in the \code{\link{lm.rrpp}} 
fit, based on factor levels.  Second,
this function find pseudodata - rather than the observed data - 
that involve either or both a principal component projection
with appropriate (or user-prescribed) dimensions and a transformation.  
The principal component projection incorporates GLS 
mean-centering, where appropriate.  Transformation involves holding 
non-grouping model terms constant.  This is accomplished by using
the fitted values from the \code{\link{lm.rrpp}} fit and the residuals 
of a \code{\link{lm.rrpp}} fit with grouping factors, alone.  When,
the \code{\link{lm.rrpp}} fit contains only grouping factors, this 
function produces raw data projected on principal components.

Regardless of variables input, data are projected onto PCs.  The 
purpose of this function is to predict 
group association, and working in PC space facilitates this objective.

This is a new function and not all limits and scenarios have been 
tested before its release.  Please report 
any issues or limitations or strange results to the maintainer.  

 \subsection{Notes for RRPP 0.5.0 and subsequent versions}{ 
 Prior to version 0.5.0, the function, \code{\link{classify}}, was 
 available.  This function has been deprecated.
 It mimicked \code{\link[MASS]{lda}} with added features that are 
 largely retained with \code{prep.lda}.  However,
 \code{prep.lda} facilitates the much more diverse options available 
 with \code{\link[MASS]{lda}}.
 
}
}
\examples{

# Using the Pupfish data (see lm.rrpp help for more detail)

data(Pupfish)
Pupfish$logSize <- log(Pupfish$CS) 
fit <- lm.rrpp(coords ~ logSize + Sex * Pop, SS.type = "I", 
data = Pupfish, print.progress = FALSE, iter = 0)

prep.lda(fit, inherent.groups = TRUE) # see groups available
lda.args <- prep.lda(fit, CV = TRUE, PC.no = 6)
lda.args$x
lda.args$grouping

# not run:
# library(MASS)
# LDA <- do.call(lda, lda.args)
# LDA$posterior
# table(lda.args$grouping, LDA$class)

}
\seealso{
\code{\link[MASS]{lda}}, \code{\link[MASS]{predict.lda}}, 
\code{\link[MASS]{qda}},
\code{\link[MASS]{predict.qda}}
}
\author{
Michael Collyer
}
\keyword{analysis}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.data.r
\docType{data}
\name{mammalHR}
\alias{mammalHR}
\title{Mammal Home Range Data}
\description{
Home range and body mass data for 49 species of mammals.
}
\details{
The data set is from a study of mammal species home range size 
(squared kilometers) and its covariation with body mass or whether mammals
are carnivores or ungulates (Garland et al., 1993).  the data set also includes a
covariance matrix based on phylogenetic relatedness among species, to account for the
non-independence of observations.
These data are suitable data for demonstrating generalized least squares (GLS)
estimation, used in the book, 
Permutational Biometry (Collyer & Adams, 2025).
}
\references{
Garland, T., Dickerman, A. W., Janis, C. M., and Jones, J. A. (1993). 
Phylogenetic analysis of covariance by computer simulation. Systematic Biology,
42:265–292.

Collyer, M. L., & Adams D. C. (2025). Permutational Biometry: Volume
1, Univariate Data. Iowa State University Digital Press.  NOT YET PUBLISHED.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add.tree.r
\name{add.tree}
\alias{add.tree}
\title{Plot tool to add phylogenetic trees to ordination plots}
\usage{
add.tree(
  OP,
  tree,
  edge.col = 1,
  edge.lty = 1,
  edge.lwd = 1,
  anc.pts = FALSE,
  return.ancs = FALSE,
  ...
)
}
\arguments{
\item{OP}{An object with class \code{\link{plot.ordinate}}.}

\item{tree}{An object of class phylo.}

\item{edge.col}{A single value or vector equal to the number of edges for edge colors.}

\item{edge.lty}{A single value or vector equal to the number of edges for edge line type}

\item{edge.lwd}{A single value or vector equal to the number of edges for edge line weight.}

\item{anc.pts}{A logical value for whether to add points for ancestral values.}

\item{return.ancs}{A logical value for whether ancestral values should be printed.}

\item{...}{Arguments passed onto \code{\link{points}}, used only for ancestral points.}
}
\description{
Function adds a tree based on a description of edges from a class phylo object 
to an existing plot made from an ordinate object.
}
\details{
With some \code{\link{ordinate}} plots, it might be desirable to add a tree 
connecting points in a prescribed way, which would be tedious using 
\code{\link{points}} or \code{\link{lines}}.  This function will project a 
tree from an object of class phylo into a plot with class, 
\code{\link{plot.ordinate}}.  Using an edges matrix from a phylo object, 
this function will systematically connect plot points with lines that pass 
through estimated ancestral character points in the same plot space.  
Ancestral states are estimated assuming a Brownian motion model 
of evolutionary divergence.
}
\examples{

# Examples use residuals from a regression of salamander morphological 
# traits against body size (snout to vent length, SVL).
# Observations are species means and a phylogenetic covariance matrix
# describes the relatedness among observations.

data("PlethMorph")
Y <- as.data.frame(PlethMorph[c("TailLength", "HeadLength", 
"Snout.eye", "BodyWidth", 
"Forelimb", "Hindlimb")])
Y <- as.matrix(Y)
R <- lm.rrpp(Y ~ SVL, data = PlethMorph, 
iter = 0, print.progress = FALSE)$LM$residuals

PCA <- ordinate(R, scale. = TRUE)
pc.plot <- plot(PCA, pch = 19, col = "blue")

add.tree(pc.plot, tree = PlethMorph$tree, anc.pts = TRUE, 
pch = 19, cex = 0.5, col = "red")

}
\seealso{
\code{\link{lines}} and \code{\link{points}}
}
\author{
Michael Collyer
}
\keyword{graphics}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.data.r
\docType{data}
\name{Pupfish}
\alias{Pupfish}
\title{Landmarks on pupfish}
\description{
Landmark data from Cyprinodon pecosensis body shapes, with 
indication of Sex and
Population from which fish were sampled (Marsh or Sinkhole).
}
\details{
These data were previously aligned with GPA.  Centroid size (CS) 
is also provided.  
See the \pkg{geomorph} package for details.
}
\references{
Collyer, M.L., D.J. Sekora, and D.C. Adams. 2015. A method for 
analysis of phenotypic
change for phenotypes described by high-dimensional data. Heredity. 113: 
doi:10.1038/hdy.2014.75.
}
\author{
Michael Collyer
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{summary.anova.lm.rrpp}
\alias{summary.anova.lm.rrpp}
\title{Print/Summary Function for RRPP}
\usage{
\method{summary}{anova.lm.rrpp}(object, ...)
}
\arguments{
\item{object}{Object from \code{\link{predict.lm.rrpp}}}

\item{...}{Other arguments passed onto predict.lm.rrpp}
}
\description{
Print/Summary Function for RRPP
}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/anova.measurement.error.r
\name{anova.measurement.error}
\alias{anova.measurement.error}
\title{ANOVA for lm.rrpp model fits used in measurement.error}
\usage{
\method{anova}{measurement.error}(object, ...)
}
\arguments{
\item{object}{Object from \code{\link{lm.rrpp}}}

\item{...}{Additional lm.rrpp model fits or other arguments passed to anova.}
}
\description{
Computes an analysis of variance (ANOVA) table using 
distributions of random statistics from \code{\link{lm.rrpp}}.  
This function is the same as \code{\link{anova.lm.rrpp}} but includes statistics
specific to \code{\link{measurement.error}}, and with restrictions on
how P-values and effect.sizes are calculated.
}
\examples{
# See measurement.error help file examples for use.

}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{summary.coef.lm.rrpp}
\alias{summary.coef.lm.rrpp}
\title{Print/Summary Function for RRPP}
\usage{
\method{summary}{coef.lm.rrpp}(object, ...)
}
\arguments{
\item{object}{Object from \code{\link{coef.lm.rrpp}}}

\item{...}{Other arguments passed onto coef.lm.rrpp}
}
\description{
Print/Summary Function for RRPP
}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{print.looCV}
\alias{print.looCV}
\title{Print/Summary Function for RRPP}
\usage{
\method{print}{looCV}(x, ...)
}
\arguments{
\item{x}{Object from \code{\link{looCV}}}

\item{...}{Other arguments passed onto print.looCV}
}
\description{
Print/Summary Function for RRPP
}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{na.omit.rrpp.data.frame}
\alias{na.omit.rrpp.data.frame}
\title{Handle missing values in rrpp.data.frame objects}
\usage{
\method{na.omit}{rrpp.data.frame}(object, ...)
}
\arguments{
\item{object}{object (from \code{\link{rrpp.data.frame}})}

\item{...}{further arguments (currently not used)}
}
\description{
Handle missing values in rrpp.data.frame objects
}
\examples{
y <- matrix(rnorm(15), 5, 3)
x <- rnorm(5)
rdf <- rrpp.data.frame(x = x, y = y, d = dist(y))
rdf$x[1] <- NA # create missing data
rdf

ndf <- na.omit(rdf)
ndf
}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{plot.model.comparison}
\alias{plot.model.comparison}
\title{Plot Function for RRPP}
\usage{
\method{plot}{model.comparison}(x, ...)
}
\arguments{
\item{x}{plot object (from \code{\link{model.comparison}})}

\item{...}{other arguments passed to plot (helpful to employ
different colors or symbols for different groups).  See
\code{\link{plot.default}} and \code{\link{par}}}
}
\description{
Plot Function for RRPP
}
\author{
Michael Collyer
}
\keyword{utilities}
\keyword{visualization}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{ranef.lmm.rrpp}
\alias{ranef.lmm.rrpp}
\title{ranef for lmm.rrpp model fits}
\usage{
\method{ranef}{lmm.rrpp}(object, type = c("matrix", "list"), ...)
}
\arguments{
\item{object}{Object from \code{\link{lmm.rrpp}}}

\item{type}{Whether to combine random effects into one matrix or print a list
by variable.}

\item{...}{Other arguments passed.}
}
\description{
Returns random effects from a lmm.rrpp object.
}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.support.code.r
\name{vec.cor.matrix}
\alias{vec.cor.matrix}
\title{Support function for RRPP}
\usage{
vec.cor.matrix(M)
}
\arguments{
\item{M}{Matrix for vector correlations.}
}
\description{
Calculate vector correlations for a matrix (by rows).  
Used for pairwise comparisons.
}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{fitted.lm.rrpp}
\alias{fitted.lm.rrpp}
\title{Extract fitted values}
\usage{
\method{fitted}{lm.rrpp}(object, ...)
}
\arguments{
\item{object}{plot object (from \code{\link{lm.rrpp}})}

\item{...}{Arguments passed to other functions}
}
\description{
Extract fitted values
}
\examples{
# See examples for lm.rrpp
}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mahal_dist.R
\name{mahal_dist}
\alias{mahal_dist}
\title{Calculate the pairwise Mahalanobis distances between observations}
\usage{
mahal_dist(x, Cov, ...)
}
\arguments{
\item{x}{A numeric matrix of data frame.}

\item{Cov}{A covariance matrix with the same number of variables as the data.}

\item{...}{Other arguments passed to \code{\link[stats]{dist}}.}
}
\value{
An object of class "dist".
}
\description{
This function emulates the \code{\link[stats]{dist}} function
but allows a covariance matrix (Cov) to be included for standardizing
distances.  It is assumed that the Covariance matrix makes sense with
respect to the data, and that the number of variables match between data and covariance matrix.
}
\details{
No tests are performed on distances but could be performed with the 
\code{\link{pairwise}} function.  Distances are only calculated if
the covariance matrix is not singular.
}
\examples{

# Using the Pupfish data (see lm.rrpp help for more detail)

data(Pupfish)
Pupfish$Y <- ordinate(Pupfish$coords)$x[, 1:3]
fit <- lm.rrpp(Y ~ Sex * Pop, SS.type = "I", 
data = Pupfish, print.progress = FALSE, iter = 0)
X <- unique(as.matrix(model.matrix(fit)))
means <- X \%*\% coef(fit)
rownames(means) <- unique(interaction(Pupfish$Sex, Pupfish$Pop))
means
S <- getResCov(fit)
dist(means)
mahal_dist(means, S)
}
\author{
Michael Collyer
}
\keyword{analysis}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{getResCov}
\alias{getResCov}
\title{Utility Function for RRPP}
\usage{
getResCov(fit, useDf = TRUE, standardize = FALSE)
}
\arguments{
\item{fit}{Object from \code{\link{lm.rrpp}}}

\item{useDf}{Logical value for whether the degrees of freedom from the linear model
fit should be used (if TRUE), as opposed to the number of observations (if FALSE).}

\item{standardize}{Logical value for whether residuals should be standardized.  If TRUE,
a correlation matrix is produced.}
}
\description{
A function mostly for internal processing but can be used to extract the residual
covariance matrix.  This matrix is the residual covariance matrix,
not the model covariance matrix used for estimation (see \code{\link{getModelCov}}). 
Options for averaging over degrees of freedom or number of
observations, plus standardization, are also available.
}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{print.betaTest}
\alias{print.betaTest}
\title{Print/Summary Function for RRPP}
\usage{
\method{print}{betaTest}(x, ...)
}
\arguments{
\item{x}{Object from \code{\link{betaTest}}}

\item{...}{Other arguments passed onto betaTest}
}
\description{
Print/Summary Function for RRPP
}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/betaTest.r
\name{betaTest}
\alias{betaTest}
\title{Test of beta parameters for lm.rrpp model fits}
\usage{
betaTest(
  fit,
  X.null = NULL,
  include.md = FALSE,
  coef.no = NULL,
  Beta = NULL,
  print.progress = FALSE
)
}
\arguments{
\item{fit}{Object from \code{\link{lm.rrpp}}}

\item{X.null}{Optional object that is either a linear model design matrix or a model
fit from \code{\link{lm.rrpp}}, from which a linear model design matrix can be extracted. Note
that if any transformation of a design matrix is required (GLS estimation), 
it is assumed that the matrix was transformed prior to analysis.  If X.null is a \code{\link{lm.rrpp}}
object, transformation is inherent.}

\item{include.md}{A logical vector for whether to include Mahalanobis distances in the results.
For highly multivariate data, this will slow down computations, significantly.}

\item{coef.no}{The row or rows of a matrix of coefficients for which to perform the test.
This can be learned by performing coef(fit), prior to the test.  If left NULL, 
the analysis will cycle through every possible vector of coefficients (rows of a coefficients matrix).}

\item{Beta}{A single value (for univariate data) or a numeric vector with length equal to
the number of variables used in the fit object.  If left NULL, 0 is used for each parameter.  
This should not be a matrix.  If one wishes to use different Beta vectors for different coefficients,
then multiple tests should be performed.  (Because tests are performed sequentially, multiple tests
using the same Beta vector produces results that are the same as for multiple rows of coefficients,
using the same Beta vector.)}

\item{print.progress}{A logical value for whether to print test progress to the screen.  This might
be useful if a large number of coefficient vectors are tested, so that one can track completion.}
}
\value{
Function returns a list with the following components: 
  \item{obs.d}{Length of observed b - Beta vector} 
  \item{obs.md}{The observed b - Beta vector length, after accounting for 
  residual covariance matrix; the Mahalanobis distance}
  \item{Beta}{Hypothesized beta values in the Beta vector.}
  \item{obs.B.mat}{The observed matrix of coefficients (before subtracting Beta).}
  \item{coef.no}{The rows of the observed matrix of coefficients, for which to subtract Beta.}
  \item{random.stats}{Random distances produced with RRPP.}
}
\description{
For any \code{\link{lm.rrpp}} object, a vector of coefficients
can be used for a specific test of a vector of betas (specific population parameters).
This test follows the form of (b - beta) in the numerator of a t-statistic, where 
beta can be a value other than 0 (or 0).  However, for this test, a vector (Beta) of length, p,
is used for the p variables in the \code{\link{lm.rrpp}} fit.  If Beta is a vector of 0s, this
test is essentially the same as the test performed for \code{\link{coef.lm.rrpp}}.  However,
it is possible to test null hypotheses for beta values other than 0, sensu Cicuéndez et al. (2023).

This function can use either the square-root of the inner-product of vectors of coefficients (distance, d)
or generalized inner-product based on the inverse of the residual covariance matrix 
(Mahalanobis distance, md) as statistics.  In most cases, either will likely yield similar (or same) 
P-values.  However, Mahalanobis distance might be preferred for generalized least squares fits, which 
do not have consistent residual covariance matrices for null (intercept only) models over
RRPP permutations (the distances are thus
standardized by the residual covariances).  If high-dimensional data are analyzed, a generalized inverse 
of the residual covariance matrix will be used because of singular covariance matrices.  Results are less 
trustworthy with Mahalanobis distances, in these cases.

The coefficient number should be provided for specific tests.  One can determine this with, e.g., 
coef(fit).  If it is not provided (NULL), tests will be performed on all possible vectors of coefficients
(rows of coefficients matrix).  These tests will be performed sequentially.  If a null model is not specified,
then for each vector of coefficients, the corresponding parameter is dropped from the linear model
design matrix to make a null model.  
This process is analogous in some ways to a leave-one-out 
cross-validation (LOOCV) analysis, testing each coefficient against models containing parameters for all other
coefficients.  For example, for a linear model fit, y ~ x1 + x2 + 1, where x1 and x2 are single-parameter 
covariates,
the analysis would first drop the intercept, then x1, then x2, performing three sequential analyses.  This 
option could require large amounts of computation time for large models, high-dimensional data, many RRPP
permutations, or any combination of these.  
The test results previously reported via coef.lm.rrpp can be found using X.null.
One would have to be cognizant of the null model used for each coefficient, based on
which term it represents.  The function, \code{\link{reveal.model.designs}} could help determine
terms to include in a null model.  Regardless, such tests have to be performed iteratively now,
but do not require verbose results for initial lm.rrpp fits.

 \subsection{Difference between coef.lm.rrpp test and betaTest}{ 
 The test for coef.lm.rrpp uses the square-root of inner-products of vectors (d) as a 
 test statistic and only tests the null hypothesis that the length of the vector is 0.  
 The significance of the test is based on random values produced by RRPP, based on the
 matrices of coefficients that are produced in all permutations.  The null models for generating
 RRPP distributions are consistent with those used for ANOVA, as specified in the 
 \code{\link{lm.rrpp}} fit by choice of SS type.  Therefore, the random coefficients are
 consistent with those produced by RRPP for generating random estimates used in ANOVA.
 
 The betaTest analysis allows different null hypotheses to be used (vector length is not necessarily 0) 
 and unless otherwise specified, uses a null model that lacks one vector of parameters and a full
 model that contains all vectors of parameters, for the parameter for which coefficients are estimated.
 This is closest to a type III SS method of estimation, but each parameter is dropped from the model,
 rather than terms potentially comprising several parameters.  Additionally, betaTest calculates
 Mahalanobis distance, in addition to Euclidean distance, for vectors of coefficients.  
 This statistic is probably 
 better for more types of models (like generalized least squares fits).
}

 \subsection{High-dimensional data}{ 
 If data are high-dimensional (more variables than observations), or even highly multivariate,
 using Mahalanobis distance can require significant computation time and will require
 using a generalized inverse.  One might wish to consider first whether using principal component 
 scores or other ordinate scores could achieve the same goal.  (See \code{\link{ordinate}}.)
 For example, one could use the first few principal components as a surrogate for a high-dimensional
 trait, and test whether the surrogate trait is different than Beta.  This would require that
 the PC scores make sense compared to the original variables, but it would be more
 computationally tractable.
} 

 \subsection{Generalized Least Squares}{ 
 To the extent that is possible, tests for GLS estimated coefficients should use 
 Mahalanobis distance.  The reason is that the covariance matrix for the data 
 (not to be confused with the residual covariance matrix of a linear model)
 might not be consistent across RRPP permutations.  To assure that random distances are 
 comparable in terms of scale, a generalized (Mahalanobis) distance is safer.  However,
 this can impose a computational burden for high-dimensional data (see above).
}
}
\examples{
\dontrun{
data(PlethMorph)
fit <- lm.rrpp(TailLength ~ SVL, 
data = PlethMorph,
verbose = TRUE)

## Allometry test (Beta = 0)

T1 <- betaTest(fit, coef.no = 2, Beta = 0)
summary(T1)

# Including Mahalanobis distance

T1 <- betaTest(fit, coef.no = 2, 
Beta = 0, include.md = TRUE)
summary(T1)


# compare to
coef(fit, test = TRUE)

# Note that if Beta is not provided

T1 <- betaTest(fit, coef.no = 2)
summary(T1)

# Note that if coef.no is not provided

T1 <- betaTest(fit)
summary(T1)

# Note that if X.null is provided

T1 <- betaTest(fit, X.null = model.matrix(fit)[, 1],
coef.no = 2)
summary(T1)


## Isometry test (Beta = 1)
# Failure to reject H0 suggests isometric-like association.

T2 <- betaTest(fit, coef.no = 2, Beta = 1)
summary(T2)


## More complex tests

# Multiple covariates

fit2 <- lm.rrpp(HeadLength ~ SVL + TailLength, 
data = PlethMorph,
SS.type = "II",
verbose = TRUE)

fit.null1 <- lm.rrpp(HeadLength ~ SVL, 
data = PlethMorph,
verbose = TRUE)

fit.null2 <- lm.rrpp(HeadLength ~ TailLength, 
data = PlethMorph,
verbose = TRUE)

## allometries
T3 <- betaTest(fit2, fit.null2, coef.no = 2, Beta = 0)
T4 <- betaTest(fit2, fit.null1, coef.no = 3, Beta = 0)
summary(T3)
summary(T4)

# compare to
coef(fit2, test = TRUE)

## isometries
T5 <- betaTest(fit2, fit.null2, coef.no = 2, Beta = 1)
T6 <- betaTest(fit2, fit.null1, coef.no = 3, Beta = 1)

summary(T5)
summary(T6) 

# Intercept test
T7 <- betaTest(fit2, fit.null1, coef.no = 1)
summary(T7)

# multivariate data

PlethMorph$Y <- cbind(PlethMorph$HeadLength, PlethMorph$TailLength)
fit3 <- lm.rrpp(Y ~ SVL, 
data = PlethMorph,
verbose = TRUE)

T8 <- betaTest(fit3, coef.no = 2, Beta = c(0, 0))
T9 <- betaTest(fit3, coef.no = 2, Beta = c(1, 1))

summary(T8)
summary(T9)

## GLS example

fit4 <- lm.rrpp(TailLength ~ SVL, 
data = PlethMorph,
Cov = PlethMorph$PhyCov,
verbose = TRUE)

T10 <- betaTest(fit4, include.md = TRUE)

summary(T10)

# compare to
coef(fit4, test = TRUE)

anova(fit4)

}
}
\references{
Tejero-Cicuéndez, H., I. Menéndez, A. Talavera, G. Riaño, B. Burriel-Carranza, 
M. Simó-Riudalbas, S. Carranza, and D.C. Adams. 2023. 
Evolution along allometric lines of least resistance: 
Morphological differentiation in Pristurus geckos. Evolution. 77:2547–2560.
}
\seealso{
\code{\link{coef.lm.rrpp}}
}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lm.rrpp.r
\name{lm.rrpp}
\alias{lm.rrpp}
\title{Linear Model Evaluation with a Randomized Residual Permutation Procedure}
\usage{
lm.rrpp(
  f1,
  iter = 999,
  turbo = FALSE,
  seed = NULL,
  int.first = FALSE,
  RRPP = TRUE,
  full.resid = FALSE,
  block = NULL,
  SS.type = c("I", "II", "III"),
  data = NULL,
  Cov = NULL,
  print.progress = FALSE,
  Parallel = FALSE,
  verbose = FALSE,
  ...
)
}
\arguments{
\item{f1}{A formula for the linear model (e.g., y~x1+x2).  Can also 
be a linear model fit
from \code{\link{lm}}.}

\item{iter}{Number of iterations for significance testing}

\item{turbo}{A logical value that if TRUE, suppresses coefficient estimation 
in every random permutation.  This will affect subsequent analyses that 
require random coefficients (see \code{\link{coef.lm.rrpp}})
but might be useful for large data sets for which only ANOVA is needed.}

\item{seed}{An optional argument for setting the seed for random 
permutations of the resampling procedure.
If left NULL (the default), the exact same P-values will be found 
for repeated runs of the analysis (with the same number of iterations).
If seed = "random", a random seed will be used, and P-values will vary.  
One can also specify an integer for specific seed values,
which might be of interest for advanced users.}

\item{int.first}{A logical value to indicate if interactions of first 
main effects should precede subsequent main effects}

\item{RRPP}{A logical value indicating whether residual randomization 
should be used for significance testing}

\item{full.resid}{A logical value for whether to use the full model residuals, only 
(sensu ter Braak, 1992). This only works if RRPP = TRUE and SS.type = III.  
Rather than permuting reduced model residuals,
this option permutes only the full model residuals in every random permutation of RRPP.}

\item{block}{An optional factor for blocks within which to restrict resampling
permutations.}

\item{SS.type}{A choice between type I (sequential), type II 
(hierarchical), or type III (marginal)
sums of squares and cross-products computations.}

\item{data}{A data frame for the function environment, see 
\code{\link{rrpp.data.frame}}}

\item{Cov}{An optional argument for including a covariance matrix 
to address the non-independence
of error in the estimation of coefficients (via GLS).  If included, 
any weights are ignored.}

\item{print.progress}{A logical value to indicate whether a progress 
bar should be printed to the screen.
This is helpful for long-running analyses.}

\item{Parallel}{Either a logical value to indicate whether parallel processing 
should be used, a numeric value to indicate the number of cores to use, or a predefined
socket cluster.  This argument defines parallel processing via the \code{parallel} library. 
If TRUE, this argument invokes forking or socket cluster assignment of all processor cores, 
except one.  If FALSE, only one core is used. A numeric value directs the number of cores to 
use, but one core will always be spared.  If a predefined socket cluster (Windows) is provided,
the cluster information will be passed to \code{parallel}.}

\item{verbose}{A logical value to indicate if all possible output from an analysis 
should be retained.  Generally this should be FALSE, unless one wishes to extract, e.g.,
all possible terms, model matrices, QR decomposition, or random permutation schemes.}

\item{...}{Arguments typically used in \code{\link{lm}}, such as 
weights or offset, passed on to
\code{LM.fit} (an internal RRPP function) for estimation of coefficients.  
If both weights and 
a covariance matrix are included,
weights are ignored (since inverses of weights are the diagonal elements 
of weight matrix, used in lieu
of a covariance matrix.)}
}
\value{
An object of class \code{lm.rrpp} is a list containing the 
following
\item{call}{The matched call.}
\item{LM}{Linear Model objects, including data (Y), coefficients, 
design matrix (X), sample size
(n), number of dependent variables (p), dimension of data space (p.prime),
QR decomposition of the design matrix, fitted values, residuals,
weights, offset, model terms, data (model) frame, random coefficients 
(through permutations),
random vector distances for coefficients (through permutations), 
whether OLS or GLS was performed, 
and the mean for OLS and/or GLS methods. Note that the data returned 
resemble a model frame rather than 
a data frame; i.e., it contains the values used in analysis, which 
might have been transformed according to 
the formula.  The response variables are always labeled Y.1, Y.2, ..., 
in this frame.}
\item{ANOVA}{Analysis of variance objects, including the SS type, 
random SS outcomes, random MS outcomes,
random R-squared outcomes, random F outcomes, random Cohen's f-squared 
outcomes, P-values based on random F
outcomes, effect sizes for random outcomes, sample size (n), number of 
variables (p), and degrees of freedom for
model terms (df).  These objects are used to construct ANOVA tables.}
\item{PermInfo}{Permutation procedure information, including the number 
of permutations (perms), The method
of residual randomization (perm.method), and each permutation's sampling 
frame (perm.schedule), which
is a list of reordered sequences of 1:n, for how residuals were 
randomized.}
\item{Models}{Reduced and full model fits for every possible model 
combination, based on terms
of the entire model, plus the method of SS estimation.}
}
\description{
Function performs a linear model fit over many random permutations of 
data, using a randomized residual permutation procedure.
}
\details{
The function fits a linear model using ordinary least squares (OLS) or 
generalized least squares (GLS) estimation of coefficients over any 
number of random permutations of the data.  A permutation procedure that 
randomizes vectors of residuals is employed.  This
procedure can randomize two types of residuals: residuals from null 
models or residuals from
an intercept model.  The latter is the same as randomizing full values, 
and is referred to as
as a full randomization permutation procedure (FRPP); the former uses 
the residuals from null
models, which are defined by the type of sums of squares and cross-products 
(SSCP) sought in an
analysis of variance (ANOVA), and is referred to as a randomized residual 
permutation procedure (RRPP).
Types I, II, and III SSCPs are supported.

Users define the SSCP type, the permutation procedure type, whether a 
covariance matrix is included
(GLS estimation), and a few arguments related to computations.   
Results comprise observed linear model
results (coefficients, fitted values, residuals, etc.), random sums of 
squares (SS) across permutation iterations,
and other parameters for performing ANOVA and other hypothesis tests, 
using empirically-derived probability distributions.

\code{lm.rrpp} emphasizes estimation of standard deviates of observed 
statistics as effect sizes
from distributions of random outcomes.  When performing ANOVA, using 
the \code{\link{anova}} function,
the effect type (statistic choice) can be varied.  See 
\code{\link{anova.lm.rrpp}} for more details.  Please
recognize that the type of SS must be chosen prior to running 
\code{lm.rrpp} and not when applying \code{\link{anova}}
to the \code{lm.rrpp} fit, as design matrices for the linear model 
must be created first.  Therefore, SS.type
is an argument for \code{lm.rrpp} and effect.type is an argument for 
\code{\link{anova.lm.rrpp}}.  If MANOVA
statistics are preferred, eigenvalues can be added with 
\code{\link{manova.update}} and statistics summarized with
\code{\link{summary.manova.lm.rrpp}}.  See \code{\link{manova.update}} 
for examples.

The \code{\link{coef.lm.rrpp}} function can be used to test the specific 
coefficients of an lm.rrpp fit.  The test
statistics are the distances (d), which are also standardized (Z-scores).  
The Z-scores might be easier to compare,
as the expected values for random distances can vary among coefficient 
vectors (Adams and Collyer 2016).

 \subsection{ANOVA vs. MANOVA}{ 
 Two SSCP matrices are calculated for each linear model effect, for 
 every random permutation: R (Residuals or Random effects) and
 H, the difference between SSCPs for "full" and "reduced" models. 
 (Full models contain and reduced models lack
 the effect tested; SSCPs are hypothesized to be the same under a 
 null hypothesis, if there is no effect.  The 
 difference, H, would have a trace of 0 if the null hypothesis were 
 true.)  In RRPP, ANOVA and MANOVA correspond to
 two different ways to calculate statistics from R and H matrices.
 
 ANOVA statistics are those that find the trace of R and H SSCP 
 matrices before calculating subsequent statistics,
 including sums of squares (SS), mean squares (MS), and F-values.  
 These statistics can be calculated with univariate data
 and provide univariate-like statistics for multivariate data.  
 These statistics are dispersion measures only (covariances
 among variables do not contribute) and are the same as "distance-based" 
 stats proposed by Goodall (1991) and Anderson (2001).
 MANOVA stats require multivariate data and are implicitly affected 
 by variable covariances.  For MANOVA, the inverse of R times
 H (invR.H) is first calculated for each effect, then eigenanalysis 
 is performed on these matrix products.  Multivariate
 statistics are calculated from the positive, real eigenvalues.  In 
 general, inferential
 conclusions will be similar with either approach, but effect sizes 
 might differ.
 
 ANOVA tables are generated by \code{\link{anova.lm.rrpp}} on lm.rrpp 
 fits and MANOVA tables are generated
 by \code{\link{summary.manova.lm.rrpp}}, after running manova.update 
 on lm.rrpp fits.
 
 Currently, mixed model effects are only possible with $ANOVA statistics, 
 not $MANOVA.
 
 More detail is found in the vignette, ANOVA versus MANOVA.
}

 \subsection{Notes for RRPP 0.5.0 and subsequent versions}{ 
 The output from lm.rrpp has changed, compared to previous versions.  
 First, the $LM
 component of output no longer includes both OLS and GLS statistics, 
 when GLS fits are 
 performed.  Only GLS statistics (coefficients, residuals, fitted values) 
 are provided 
 and noted with a "gls." tag.  GLS statistics can include those calculated
 when weights are input (similar to the \code{\link[stats]{lm}} argument).  
 Unlike previous 
 versions, GLS and weighted LS statistics are not labeled differently, 
 as weighted
 LS is one form of generalized LS estimation.  Second, a new object, 
 $Models, is included
 in output, which contains the linear model fits (\code{\link[stats]{lm}} 
 attributes ) for
 all reduced and full models that are possible to estimate fits.
 
}

 \subsection{Notes for RRPP 0.3.1 and subsequent versions}{ 
 F-values via RRPP are calculated with residual SS (RSS) found uniquely 
 for any model terms, as per
 Anderson and ter Braak (2003).  This method uses the random pseudo-data 
 generated by each term's
 null (reduced) model, meaning RSS can vary across terms.  Previous 
 versions used an intercept-only 
 model for generating random pseudo-data.  This generally has appropriate 
 type I error rates but can have
 elevated type I error rates if the observed RSS is small relative 
 to total SS.  Allowing term by term
 unique RSS alleviates this concern.
}
}
\examples{
\dontrun{

# Examples use geometric morphometric data
# See the package, geomorph, for details about obtaining such data

data("PupfishHeads")
names(PupfishHeads)

# Head Size Analysis (Univariate)-------------------------------------------------------

fit <- lm.rrpp(log(headSize) ~ sex + locality/year, SS.type = "I", 
data = PupfishHeads, print.progress = FALSE, iter = 999)
summary(fit)
anova(fit, effect.type = "F") # Maybe not most appropriate
anova(fit, effect.type = "Rsq") # Change effect type, but still not 
# most appropriate

# Mixed-model approach (most appropriate, as year sampled is a random 
# effect:

anova(fit, effect.type = "F", error = c("Residuals", "locality:year", 
"Residuals"))

# Change to Type III SS

fit <- lm.rrpp(log(headSize) ~ sex + locality/year, SS.type = "III", 
data = PupfishHeads, print.progress = FALSE, iter = 999,
verbose = TRUE)
summary(fit)
anova(fit, effect.type = "F", error = c("Residuals", "locality:year", 
"Residuals"))

# Coefficients Test

coef(fit, test = TRUE)

# Predictions (holding alternative effects constant)

sizeDF <- data.frame(sex = c("Female", "Male"))
rownames(sizeDF) <- c("Female", "Male")
sizePreds <- predict(fit, sizeDF)
summary(sizePreds)
plot(sizePreds)

# Diagnostics plots of residuals

plot(fit)

# Body Shape Analysis (Multivariate) -----------

data(Pupfish)
names(Pupfish)

# Note:

dim(Pupfish$coords) # highly multivariate!
fit <- lm.rrpp(coords ~ log(CS) + Sex*Pop, SS.type = "I", 
data = Pupfish, print.progress = FALSE, iter = 999,
verbose = TRUE) 
summary(fit, formula = FALSE)
anova(fit) 
coef(fit, test = TRUE)

# Predictions (holding alternative effects constant)

shapeDF <- expand.grid(Sex = levels(Pupfish$Sex), 
Pop = levels(Pupfish$Pop))
rownames(shapeDF) <- paste(shapeDF$Sex, shapeDF$Pop, sep = ".")
shapeDF

shapePreds <- predict(fit, shapeDF)
summary(shapePreds)
summary(shapePreds, PC = TRUE)

# Plot prediction

plot(shapePreds, PC = TRUE)
plot(shapePreds, PC = TRUE, ellipse = TRUE)

# Diagnostics plots of residuals

plot(fit)

# PC-plot of fitted values

groups <- interaction(Pupfish$Sex, Pupfish$Pop)
plot(fit, type = "PC", pch = 19, col = as.numeric(groups))

# Regression-like plot

plot(fit, type = "regression", reg.type = "PredLine", 
    predictor = log(Pupfish$CS), pch=19,
    col = as.numeric(groups))

# Body Shape Analysis (Distances) ----------

D <- dist(Pupfish$coords) # inter-observation distances
length(D)
Pupfish$D <- D

fitD <- lm.rrpp(D ~ log(CS) + Sex*Pop, SS.type = "I", 
data = Pupfish, print.progress = FALSE, iter = 999) 

# These should be the same:
summary(fitD, formula = FALSE)
summary(fit, formula = FALSE) 

# GLS Example (Univariate) -----------

data(PlethMorph)
fitOLS <- lm.rrpp(TailLength ~ SVL, data = PlethMorph, 
print.progress = FALSE, iter = 999)
fitGLS <- lm.rrpp(TailLength ~ SVL, data = PlethMorph, Cov = PlethMorph$PhyCov, 
print.progress = FALSE, iter = 999)

anova(fitOLS)
anova(fitGLS)

sizeDF <- data.frame(SVL = sort(PlethMorph$SVL))

# Prediction plots

# By specimen
plot(predict(fitOLS, sizeDF)) # Correlated error
plot(predict(fitGLS, sizeDF)) # Independent error

# With respect to independent variable (using abscissa)
plot(predict(fitOLS, sizeDF), abscissa = sizeDF) # Correlated error
plot(predict(fitGLS, sizeDF), abscissa = sizeDF) # Independent error


# GLS Example (Multivariate) -----------

Y <- as.matrix(cbind(PlethMorph$TailLength,
PlethMorph$HeadLength,
PlethMorph$Snout.eye,
PlethMorph$BodyWidth,
PlethMorph$Forelimb,
PlethMorph$Hindlimb))
PlethMorph$Y <- Y
fitOLSm <- lm.rrpp(Y ~ SVL, data = PlethMorph, 
print.progress = FALSE, iter = 999)
fitGLSm <- lm.rrpp(Y ~ SVL, data = PlethMorph, 
Cov = PlethMorph$PhyCov,
print.progress = FALSE, iter = 999)

anova(fitOLSm)
anova(fitGLSm)

# Prediction plots

# By specimen
plot(predict(fitOLSm, sizeDF)) # Correlated error
plot(predict(fitGLSm, sizeDF)) # Independent error

# With respect to independent variable (using abscissa)
plot(predict(fitOLSm, sizeDF), abscissa = sizeDF) # Correlated error
plot(predict(fitGLSm, sizeDF), abscissa = sizeDF) # Independent error
}
}
\references{
Anderson MJ. 2001. A new method for non-parametric 
multivariate analysis of variance.
   Austral Ecology 26: 32-46.

Anderson MJ. and C.J.F. ter Braak. 2003. Permutation 
tests for multi-factorial analysis of variance. Journal of Statistical 
Computation and Simulation 73: 85-113.

Collyer, M.L., D.J. Sekora, and D.C. Adams. 2015. A method 
for analysis of phenotypic change for phenotypes described
by high-dimensional data. Heredity. 115:357-365.

Adams, D.C. and M.L. Collyer. 2016.  On the comparison of 
the strength of morphological integration across morphometric
datasets. Evolution. 70:2623-2631.

Adams, D.C and M.L. Collyer. 2018. Multivariate phylogenetic 
anova: group-clade aggregation, biological 
challenges, and a refined permutation procedure. Evolution. 72:1204-1215.

ter Braak, C.J.F. 1992. Permutation versus bootstrap significance tests in 
multiple regression and ANOVA. pp .79–86 In Bootstrapping and Related Techniques. eds K-H. Jockel, 
G. Rothe & W. Sendler.Springer-Verlag, Berlin.
\code{\link[stats]{lm}} for more on linear model fits.
}
\seealso{
\code{procD.lm} and \code{procD.pgls} within \code{geomorph};
}
\author{
Michael Collyer
}
\keyword{analysis}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pairwise.model.Z.r
\name{pairwise.model.Z}
\alias{pairwise.model.Z}
\title{Pairwise comparisons of model effects}
\usage{
pairwise.model.Z(
  ...,
  nsamp = NULL,
  two.tailed = TRUE,
  predictor = NULL,
  tol = NULL,
  pc.no = NULL,
  gls.null = FALSE
)
}
\arguments{
\item{...}{Either an object of class \code{\link{model.comparison}}, 
or several objects of class \code{\link{lm.rrpp}}.  If the former, arguments of
type = 'Z' and verbose = TRUE are required.  If the latter, a model comparison analysis
will first be performed with these arguments.}

\item{nsamp}{An optional vector containing the sample sizes for each model fit}

\item{two.tailed}{A logical value to indicate whether a two-tailed test (typical and default) should be performed.}

\item{predictor}{An optional vector to be passed to \code{\link{model.comparison}}, if used.}

\item{tol}{An optional value to be passed to \code{\link{model.comparison}}, if used.}

\item{pc.no}{An optional value to be passed to \code{\link{model.comparison}}, if used.}

\item{gls.null}{An optional logical value to be passed to \code{\link{model.comparison}}, if used.}
}
\value{
A list containing the following
\item{sample.z}{A vector of model effect sizes.}
\item{pairwise.z}{A matrix of pairwise test statistics comparing model effect sizes.}
\item{pairwise.P}{A matrix of pairwise significance levels.}
\item{tails}{Number of tails used for P-value calculation.}
}
\description{
Function generates pairwise statistics for comparing model fits and
returns important statistics for hypothesis tests.
}
\details{
The function statistically compares the effect sizes of two or more models fit
and evaluated using RRPP.  Input for the function is a list of fitted models
of the class 'model.comparison', whose options included type = 'Z' and 
verbose = TRUE when the models were compared with that function.

A two-sample test is performed on each pair of models, comparing the strength 
of model fits to one another (Collyer and Adams 2025). This might be used to 
compare the strength of fit of the data to differing statistical models (as in 
model selection) or for comparing the fit across differing 
datasets for the same model to determine whether the strength of fit in one dataset
is greater than that found in another (see Collyer and Adams 2025). In the latter
case, one is advised to include a vector containing the sample sizes of each
dataset, so that two-sample tests may account for differences in sample size.
}
\examples{
\dontrun{
 data(Pupfish)
 Pupfish$logSize <- log(Pupfish$CS)
 fit1 <- lm.rrpp(coords ~ logSize, data = Pupfish,
 print.progress = FALSE)
 fit2 <- lm.rrpp(coords ~ Pop, data = Pupfish,
 print.progress = FALSE)
 fit3 <- lm.rrpp(coords ~ Sex, data = Pupfish, 
 print.progress = FALSE)
 fit6 <- lm.rrpp(coords ~ logSize + Sex * Pop, data = Pupfish, 
 print.progress = FALSE)
 Mod.C <- model.comparison(fit1, fit2, fit3, fit6,
 pc.no = 4, type = "Z", verbose = TRUE)
 res <- pairwise.model.Z(Mod.C)
 summary(res, stats.table = TRUE)
 summary(res, stats.table = FALSE)
 }
}
\references{
Collyer, M.L., and D.C. Adams. 2025. Permutational Biometry. 
  Volume 1: Univariate Data. Iowa State University Digital Press. (Forthcoming).
}
\author{
Dean Adams and Michael Collyer
}
\keyword{analysis}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{print.summary.lm.rrpp}
\alias{print.summary.lm.rrpp}
\title{Print/Summary Function for RRPP}
\usage{
\method{print}{summary.lm.rrpp}(x, ...)
}
\arguments{
\item{x}{print/summary object (from \code{\link{summary.lm.rrpp}})}

\item{...}{other arguments passed to print/summary}
}
\description{
Print/Summary Function for RRPP
}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.data.r
\docType{data}
\name{ReptAbund}
\alias{ReptAbund}
\title{Reptile Abundance Data in Western Australia}
\description{
Reptile abundance data from several sites in Western Australia,
originally collected to examine recovery after a large fire.
}
\details{
The data set is from a study examining reptile abundance from
several sites in Western Australia (Davis and Doherty, 2015). The original 
study examined the recovery of reptile communities in ten remnant
localities following a large fire in 2009. Half of localities were in the burn zone 
and the remaining half were unaffected by the fire. Data were sampled over five years.
These data are suitable data for demonstrating mixed linear models, used in the book, 
Permutational Biometry (Collyer & Adams, 2025).
}
\references{
Davis, R. A. and Doherty, T. S. (2015). Rapid recovery of an urban remnant
reptile community following summer wildfire. PLOS ONE, 10(5):e0127925.

Collyer, M. L., & Adams D. C. (2025). Permutational Biometry: Volume
1, Univariate Data. Iowa State University Digital Press.  NOT YET PUBLISHED.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{plot.measurement.error}
\alias{plot.measurement.error}
\title{Plot Function for RRPP}
\usage{
\method{plot}{measurement.error}(
  x,
  separate.by.groups = TRUE,
  add.connectors = TRUE,
  add.labels = FALSE,
  use.std.vectors = FALSE,
  titles = NULL,
  add.legend = TRUE,
  ...
)
}
\arguments{
\item{x}{Object from \code{\link{measurement.error}}}

\item{separate.by.groups}{A logical value for whether to make separate plots
for each group, if different groups are available.  If FALSE, groups
are still represented by different symbols in the plot, unless overridden
by plot arguments.}

\item{add.connectors}{A logical value for whether to add connectors, like
vectors, between replicate observations of the same subjects.}

\item{add.labels}{A logical value for whether to label subjects.
Labels are either subject name (if available) or number of occurrence
in the data set.}

\item{use.std.vectors}{A logical value for whether to use vectors obtained from
a standardized matrix, which are orthogonal.  This is not strictly necessary.}

\item{titles}{An optional vector or list for augmenting the titles of plots produced.
The length of the vector or list should match the number of plots produced by 
other arguments.}

\item{add.legend}{A logical value for whether to add a legend to plots.  If
separate.by.groups is TRUE, adding a legend to plots will be slightly redundant. If
certain parameters are augmented by user (point characters, colors), add.legend
will be made to be FALSE to prevent misinterpretation of intended plotting scheme.}

\item{...}{Other arguments passed onto plot}
}
\description{
This function produces multivariate signal-to-noise ratio plots for  
\code{\link{measurement.error}} objects.  See the function, 
\code{\link{plot.interSubVar}} for plotting the inter-subject variability
from a \code{\link{measurement.error}} object, after applying the function, 
\code{\link{interSubVar}}.
}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/interSubVar.r
\name{interSubVar}
\alias{interSubVar}
\title{Reveal the inter-subject variability from a measurement error analysis}
\usage{
interSubVar(ME, type = c("range", "sd", "var", "cv"))
}
\arguments{
\item{ME}{A measurement error object}

\item{type}{A value to indicate the type of variability (statistic)
to measure, which can be
one of range (the maximum value minus the minimum value, not the two values),
standard deviation (sd), variance (var), or coefficient of variation (cv).  No 
attempt is made to assure the distribution of values is appropriate for the 
statistics.  For example, if only two replicates are available, using sd, var, or
cv might not be wise.  Or if the replicated values are exact, cv 
will be NA (and other stats will be 0).  Choice of statistic should consider
the distribution of values.}
}
\value{
An object of class \code{interSubVar} is a list containing the 
following
\item{var.map}{A distance matrix object with values that map the variability
statistic used for inter-subject Euclidean distances.}
\item{distance.mats}{The inter-subject distance matrices for every replicate.}
\item{subject.order}{A vector of subject levels in the order that was used to
guarantee consistent sorting across distance matrices.}
\item{var.map}{The variability type (statistic) that was used.}
}
\description{
Function produces both a list of inter-subject Euclidean distance matrices, 
based on replicate measurements of the same subjects, and one matrix that 
summarizes the variability among the inter-subject distances, across subjects.  
This function can be considered a tool for the evaluation of subject 
estimate precision.  The function, \code{\link{plot.interSubVar}} can produce a 
heat map of the inter-subject variability.
}
\examples{

\dontrun{
# Measurement error analysis on simulated data of fish shapes

data(fishy)

# Analysis unconcerned with groups 

ME1 <- measurement.error(
  Y = "coords",
  subjects = "subj",
  replicates = "reps",
  data = fishy)

anova(ME1)
ICCstats(ME1, subjects = "Subjects", with_in = "Systematic ME")
plot(ME1)

# Analysis concerned with groups 

ME2 <- measurement.error(
  Y = "coords",
  subjects = "subj",
  replicates = "reps",
  groups = "groups",
  data = fishy)

anova(ME2)
ICCstats(ME2, subjects = "Subjects", 
  with_in = "Systematic ME", groups = "groups")
P <- plot(ME2)
focusMEonSubjects(P, subjects = 18:20, shadow = TRUE)
 }
 
}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.data.r
\docType{data}
\name{snakeHS}
\alias{snakeHS}
\title{Snake Head Size and Regional Variation}
\description{
A data set containing variables for snake head size, 
snout-to-vent-length, sex, and geographic information.
}
\details{
These data were collected from prairie rattlesnakes (Crotalus viridis viridis),
from several geographic locations in the upper Midwest of the United States (Smith and 
Collyer, 2008).  Variables include a spatial measure of head size (HS), 
snout-to-vent-length (SVL, cm), sex, and the site and region where 107
snakes were captured.  These 
data are suitable for demonstrating model selection  in the book,
Permutational Biometry (Collyer & Adams, 2025).
}
\references{
Smith, M. T. and Collyer, M. L. (2008). The biology of rattlesnakes, chapter
Regional variation and sexual dimorphism in head form of the prairie
rattlesnake (Crotalus viridid viridis): Comparisons using new analytical techniques
and collection methods, pages 79–90. Loma Linda University Press.

Collyer, M. L., & Adams D. C. (2025). Permutational Biometry: Volume
1, Univariate Data. Iowa State University Digital Press.  NOT YET PUBLISHED.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.data.r
\docType{data}
\name{motionpaths}
\alias{motionpaths}
\title{Simulated motion paths}
\description{
Simulated motion paths
}
\references{
Adams, D. C., and M. L. Collyer. 2009. A general framework for 
the analysis of phenotypic
  trajectories in evolutionary studies. Evolution 63:1143-1154.
}
\author{
Dean Adams
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{print.coef.lm.rrpp}
\alias{print.coef.lm.rrpp}
\title{Print/Summary Function for RRPP}
\usage{
\method{print}{coef.lm.rrpp}(x, ...)
}
\arguments{
\item{x}{Object from \code{\link{coef.lm.rrpp}}}

\item{...}{Other arguments passed onto coef.lm.rrpp}
}
\description{
Print/Summary Function for RRPP
}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{getANOVAStats}
\alias{getANOVAStats}
\title{Utility Function for RRPP}
\usage{
getANOVAStats(fit, stat = c("SS", "MS", "Rsq", "F", "cohenf", "all"))
}
\arguments{
\item{fit}{Object from \code{\link{lm.rrpp}}.}

\item{stat}{The ANOVA statistic to extract.  Returns every RRPP permutation of 
the statistic.  If "all", a list of each statistic is returned.}
}
\description{
A function mostly for internal processing but can be used to extract
ANOVA statistics for other uses, such as plotting histograms
}
\examples{
\dontrun{
data(Pupfish)
fit <- lm.rrpp(coords ~ log(CS) + Sex*Pop, SS.type = "I", 
data = Pupfish, print.progress = FALSE, iter = 999) 
anova(fit)
Fstats <- getANOVAStats(fit, stat = "F")
par(mfrow = c(2, 2))
hist(Fstats$Fs[1,], breaks = 50, main = "log(CS)", xlab = "F")
abline(v = Fstats$Fs[1, 1])
hist(Fstats$Fs[2,], breaks = 50, main = "Sex", xlab = "F")
abline(v = Fstats$Fs[2, 1])
hist(Fstats$Fs[3,], breaks = 50, main = "Pop", xlab = "F")
abline(v = Fstats$Fs[3, 1])
hist(Fstats$Fs[4,], breaks = 50, main = "Sex:Pop", xlab = "F")
abline(v = Fstats$Fs[4, 1])
}
}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/measurement.error.r
\name{measurement.error}
\alias{measurement.error}
\title{Evaluation of measurement error for two or more multivariate measurements, 
for common research subjects}
\usage{
measurement.error(
  data,
  Y,
  subjects,
  replicates,
  groups = NULL,
  groups.first = FALSE,
  iter = 999,
  seed = NULL,
  multivariate = FALSE,
  use.PCs = TRUE,
  tol = 0.001,
  Parallel = FALSE,
  turbo = TRUE,
  print.progress = FALSE,
  verbose = FALSE
)
}
\arguments{
\item{data}{A required data frame, either of class \code{\link{data.frame}} or
class \code{\link{rrpp.data.frame}}.  This function cannot be used without a
data frame.  All arguments for data and variables are names that must exist in the data frame.}

\item{Y}{A name for a matrix (n x p) of data for n observations and p variables that can be found
in the data frame.  For example, Y = "morphData".}

\item{subjects}{A name for a vector or factor of research subjects, found within the data frame
(each subject should occur twice or more).  The length of the vector in the data frame must equal the 
number of observations and will be coerced into a factor.  For example, subjects = "ID".}

\item{replicates}{A name for a vector or factor for replicate measurements for research subjects, found
within the data frame.  The length of the vector  in the data frame must equal the number of observations and will 
be coerced into a factor.  For example, replicates = "Rep".}

\item{groups}{An optional name for a vector in the data frame, coercible to factor, to be included 
in the linear model (as an interaction with replicates).  This would be of interest if one 
were concerned with systematic ME occurring perhaps differently among certain strata within the data.  
For example, systematic ME because of an observer bias might only be observed with females or males,
in which case the argument might be: groups = "Sex".}

\item{groups.first}{A logical value for whether to use groups as a term before subjects, so that it can be included
in an ANOVA table (if TRUE).  Otherwise, a group effect is likely subsumed by a subject effect, since subjects are 
unique to groups.}

\item{iter}{Number of iterations for significance testing}

\item{seed}{An optional argument for setting the seed for random 
permutations of the resampling procedure.
If left NULL (the default), the exact same P-values will be found 
for repeated runs of the analysis (with the same number of iterations).
If seed = "random", a random seed will be used, and P-values will vary.  
One can also specify an integer for specific seed values,
which might be of interest for advanced users.}

\item{multivariate}{Logical value for whether to include multivariate analyses.  Intraclass correlation 
matrices and relative eigenanalysis are based on products of sums of squares and cross-products (SSCP)
matrices, some of which must be inverted and potentially require
 significant computation time.  If FALSE, only statistics based on dispersion of values are calculated.}

\item{use.PCs}{A logical argument for whether to use the principal components of the data.
This might be helpful for relative eigenanalysis, and if p > n, 
in which case inverting singular covariance matrices would not be possible.}

\item{tol}{A value indicating the magnitude below which 
components should be omitted, if use.PCs is TRUE. (Components are omitted if their 
standard deviations are less than or equal to tol times the 
standard deviation of the first component.)  See \code{\link{ordinate}} for more details.}

\item{Parallel}{The same argument as in \code{\link{lm.rrpp}} to govern parallel processing (
either a logical value -- TRUE or FALSE -- or the number of threaded cores to use).  See \code{\link{lm.rrpp}} 
for additional details.}

\item{turbo}{Logical value for whether to suppress coefficient estimation in RRPP iteration,
thus turbo-charging RRPP.}

\item{print.progress}{A logical value to indicate whether a progress 
bar should be printed to the screen.}

\item{verbose}{A logical value to indicate if all the output from an
\code{\link{lm.rrpp}} analysis should be retained.  If FALSE, only the needed
output for summaries and plotting is retained.}
}
\value{
Objects of class "measurement.error" return the same objects
as a \code{\link{lm.rrpp}} fit, plus a list of the following:
 \item{AOV}{Analysis of variance to test for systematic error, based on dispersion of values.}
 \item{mAOV}{Multivariate AOV based on product of the inverse of the random component (SSCP) of ME
 times the systematic component of ME.}
 \item{SSCP}{The sums of squares and cross-products matrices for model effects.}
 \item{SSCP.ME.product}{The products of the inverse of the random ME SSCP and the SSCP matrices
 for systematic ME,.  These are the same matrix products used for eigenanalysis.  
 This is the observed matrix.}
 \item{SSCP.ME.product.std}{A list of the symmetric forms of standardized SSCP.ME.products 
 that yield orthogonal eigenvectors.}
}
\description{
Function performs analyses concerned with the repeatability (reliability) of multivariate data 
(measurements) collected from the same research subjects.  Although there is no 
requirement for repeated measurements on all research subjects, the analysis assumes
that multiple observations are made.
}
\details{
This function performs analyses as described in Collyer and Adams (2024)
 to assess systematic and random components of 
measurement error (ME).  It basically performs ANOVA with RRPP,
but with different restricted randomization strategies.  The reliability of research subject variation 
can be considered by restricting randomization within replicates; the consistency of replicate measures
can be considered by restricting randomization within subjects.
Inter-subject variation remains constant across all random permutations within subjects and 
inter-replicate variation remains constant across all random permutations within replicates.  Type II
sums of squares and cross-products (SSCP) are calculated to assure conditional estimation.

The results include univariate-like statistics based on dispersion of values and
eigenanalysis performed on a signal to noise matrix product of SSCP matrices 
(sensu Bookstein and Mitteroecker, 2014) 
including the inverse of the random component of ME and the systematic
component of ME.  The multivariate test is a form of multivariate ANOVA (MANOVA), using
RRPP to generate sampling distributions of the major eigenvalue (Roy's maximum root).
Likelihood-ratio tests can also be performed using \code{\link{lr_test}}.

Intraclass correlation coefficients (ICC) can also be 
calculated (using \code{\link{ICCstats}}), 
both based on dispersion of values and 
covariance matrices, as descriptive statistics.  
Details are provided in \code{\link{ICCstats}}.
}
\examples{
\dontrun{
# Measurement error analysis on simulated data of fish shapes

data(fishy)

# Example two digitization replicates of the same research subjects
rep1 <- matrix(fishy$coords[1,], 11, 2, byrow = TRUE)
rep2 <- matrix(fishy$coords[61,], 11, 2, byrow = TRUE)
plot(rep1, pch = 16, col = gray(0.5, alpha = 0.5), cex = 2, asp = 1)
points(rep2, pch = 16, col = gray(0.2, alpha = 0.5), cex = 2, asp = 1)

# Analysis unconcerned with groups 

ME1 <- measurement.error(
  Y = "coords",
  subjects = "subj",
  replicates = "reps",
  data = fishy)

anova(ME1)
ICCstats(ME1, subjects = "Subjects", with_in = "Systematic ME")
plot(ME1)

# Analysis concerned with groups 

ME2 <- measurement.error(
  Y = "coords",
  subjects = "subj",
  replicates = "reps",
  groups = "groups",
  data = fishy)

anova(ME2)
ICCstats(ME2, subjects = "Subjects", 
  with_in = "Systematic ME", groups = "groups")
P <- plot(ME2)
focusMEonSubjects(P, subjects = 18:20, shadow = TRUE)
}

}
\references{
Collyer, M.L. and D.C. Adams.  2024. Interrogating Random and Systematic Measurement Error 
in Morphometric Data. Evolutionary Biology, 51, 179–20.

Bookstein, F.L., & Mitteroecker, P. (2014). Comparing covariance matrices by relative eigenanalysis, 
with applications to organismal biology. Evolutionary Biology, 41(2), 336-350.
}
\seealso{
\code{\link{lm.rrpp.ws}}, \code{\link{manova.update}}, 
\code{\link{lr_test}}
}
\author{
Michael Collyer
}
\keyword{analysis}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{summary.kcomp}
\alias{summary.kcomp}
\title{Print/Summary Function for RRPP}
\usage{
\method{summary}{kcomp}(object, ...)
}
\arguments{
\item{object}{Object from \code{\link{kcomp}}}

\item{...}{Other arguments passed onto print.kcomp}
}
\description{
Print/Summary Function for RRPP
}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{summary.pairwise.model.Z}
\alias{summary.pairwise.model.Z}
\title{Print/Summary Function for RRPP}
\usage{
\method{summary}{pairwise.model.Z}(object, ...)
}
\arguments{
\item{object}{print/summary object (from \code{\link{pairwise.model.Z}})}

\item{...}{other arguments passed to print/summary}
}
\description{
Print/Summary Function for RRPP
}
\author{
Dean Adams and Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/model.comparison.r
\name{model.comparison}
\alias{model.comparison}
\title{Model Comparisons, in terms of the log-likelihood, covariance trace,
or Z-score.}
\usage{
model.comparison(
  ...,
  type = c("cov.trace", "logLik", "Z"),
  predictor = NULL,
  tol = NULL,
  pc.no = NULL,
  gls.null = FALSE,
  mod.names = TRUE,
  verbose = FALSE
)
}
\arguments{
\item{...}{Any number of lm.rrpp class objects for model fits 
to be compared.}

\item{type}{An argument to choose between log-likelihood, 
covariance trace, or Z results.  If Z is chosen, Z-scores are calculated, the same log-likelihoods
are calculated as with the log-likelihood type, but also in every RRPP permutation,
as describe for the initial mode fits, with choice of null model (below).}

\item{predictor}{An optional vector that can be used to profile the results based on type
across a range of numerical values described by the predictor.  A spline will also be fit, which
will reveal estimated values of the predictor that yield maximum and minimum values of model
comparison metric.}

\item{tol}{If type = logLik or Z, tol is a tolerance value between 0 and 1, 
indicating the magnitude below which 
components should be omitted (if standard deviations of 
components are less than the eigenvalue of the first 
component times the tolerance), for calculating the log-likelihood.}

\item{pc.no}{If type = logLik or Z, an optional value to indicate the 
number of principal components (maximum rank) to use 
for calculating the log-likelihood.}

\item{gls.null}{A logical value indicating whether GLS estimation should be
used with the null (intercept) model, for calculating Z scores via RRPP of 
log-likelihoods.  This should be FALSE if comparing different GLS estimations
of covariance matrices.  It should be TRUE if comparing different model fits
with the same GLS-estimated covariance matrix.}

\item{mod.names}{A logical value for whether to label models by their object names (TRUE)
or formulas (FALSE).}

\item{verbose}{A logical value for whether to include distributions of
random log-likelihoods, if applicable.}
}
\value{
An object of class \code{model.comparison} is a data 
frame with either log-likelihoods,
covariance traces, or, Z-scores, plus parameter penalties.  AIC scores 
might be included, if applicable. If verbose results are returned,
random log-likelihoods are also included.
}
\description{
Function calculates either log-likelihoods or traces of covariance 
matrices for comparison with 
respect to parameter penalties, or calculates Z-scores from RRPP, which
can be profiled across a gradient (predictor).
}
\details{
The function calculates either log-likelihoods or traces of (residual) 
covariance matrices, plus parameter 
penalties, to assist in comparative model evaluation or selection.  
Because high-dimensional data often
produce singular or ill-conditioned residual covariance matrices, 
this function does one of two things: 1) uses 
the trace of a covariance matrix rather than its determinant; or 2) 
provides a ridge-regularization (Warton, 2008)
of the covariance matrix, only if it is determined that it is 
ill-conditioned.  Regardless of implementation,
covariance matrices are projected into a principal component (PC) 
space of appropriate dimensions.

The parameter penalty is based on that proposed by Bedrick and Tsai 
(1994), equal to 2(pk + p(p + 1)/2), where 
p is the appropriate dimension (not number of variables) of the 
covariance matrix.  The parameter, k,
 is the rank of the model design matrix.
 
In the case that "logLik" is chosen for the argument, type, 
AIC scores are calculated.  These scores
may not perfectly match other packages or software that 
calculate AIC for multivariate data, if ridge regularization
was used (and if other packages require p = the number of 
data variables).  When choosing logLik as the type of comparison,
it might be a good idea to adjust the tolerance or number of data 
principal components.  The default (NULL) values will
use all data dimensions to calculate log-likelihoods, which 
might cause problems if the number of variables exceeds the number 
of observations (producing singular residual covariance matrices).  
However, one should not reduce data dimensions haphazardly,
as this can lead to poor estimates of log-likelihood.  Furthermore, 
using the tolerance argument could result in different
numbers of principal components used for each model to calculate 
log-likelihoods, which might be a concern for comparing models.  
If both tol and pc.no arguments are used, the solution will use 
the fewest PCs produced by either argument.  Because the trace
of a covariance matrix is not sensitive to matrix singularity, 
no PC adjustment is used for the cov.trace argument.

This function can also calculate Z-scores from RRPP on model log-likelihoods, 
which can be compared directly or profiled along a gradient (predictor).  
This might be useful for for comparing generalized least-squares (GLS) models, 
for example, along a gradient of a parameter used to scale the covariance 
matrix for GLS estimation. See Collyer et al. 2022 for an example of using
RRPP on log-likelihoods with different covariance matrices.

Users can construct their own tables 
from the results but this function does not attempt to 
summarize results, as interpreting results requires 
some arbitrary decisions.  The \code{\link{anova}} function 
explicitly tests multiple models and can be used for nested 
model comparisons.

Results can also be plotted using the generic \code{\link{plot}} 
function.
}
\examples{
\dontrun{
data(Pupfish)
Pupfish$logSize <- log(Pupfish$CS)
fit1 <- lm.rrpp(coords ~ logSize, data = Pupfish, iter = 0, 
print.progress = FALSE)
fit2 <- lm.rrpp(coords ~ Pop, data = Pupfish, iter = 0, 
print.progress = FALSE)
fit3 <- lm.rrpp(coords ~ Sex, data = Pupfish, iter = 0, 
print.progress = FALSE)
fit4 <- lm.rrpp(coords ~ logSize + Sex, data = Pupfish, iter = 0, 
print.progress = FALSE)
fit5 <- lm.rrpp(coords ~ logSize + Pop, data = Pupfish, iter = 0, 
print.progress = FALSE)
fit6 <- lm.rrpp(coords ~ logSize + Sex * Pop, data = Pupfish, iter = 0, 
print.progress = FALSE)

modComp1 <- model.comparison(fit1, fit2, fit3, fit4, fit5, 
fit6, type = "cov.trace")
modComp2 <- model.comparison(fit1, fit2, fit3, fit4, fit5, 
fit6, type = "logLik", tol = 0.01)

summary(modComp1)
summary(modComp2)

par(mfcol = c(1,2))
plot(modComp1)
plot(modComp2)

# Comparing fits with covariance matrices
# an example for scaling a phylogenetic covariance matrix with
# the scaling parameter, lambda

data("PlethMorph")
Cov <- PlethMorph$PhyCov
lambda <- seq(0, 1, 0.1)

Cov1 <- scaleCov(Cov, scale. = lambda[1])
Cov2 <- scaleCov(Cov, scale. = lambda[2])
Cov3 <- scaleCov(Cov, scale. = lambda[3])
Cov4 <- scaleCov(Cov, scale. = lambda[4])
Cov5 <- scaleCov(Cov, scale. = lambda[5])
Cov6 <- scaleCov(Cov, scale. = lambda[6])
Cov7 <- scaleCov(Cov, scale. = lambda[7])
Cov8 <- scaleCov(Cov, scale. = lambda[8])
Cov9 <- scaleCov(Cov, scale. = lambda[9])
Cov10 <- scaleCov(Cov, scale. = lambda[10])
Cov11 <- scaleCov(Cov, scale. = lambda[11])


fit1 <- lm.rrpp(SVL ~ 1, data = PlethMorph, Cov = Cov1)
fit2 <- lm.rrpp(SVL ~ 1, data = PlethMorph, Cov = Cov2)
fit3 <- lm.rrpp(SVL ~ 1, data = PlethMorph, Cov = Cov3)
fit4 <- lm.rrpp(SVL ~ 1, data = PlethMorph, Cov = Cov4)
fit5 <- lm.rrpp(SVL ~ 1, data = PlethMorph, Cov = Cov5)
fit6 <- lm.rrpp(SVL ~ 1, data = PlethMorph, Cov = Cov6)
fit7 <- lm.rrpp(SVL ~ 1, data = PlethMorph, Cov = Cov7)
fit8 <- lm.rrpp(SVL ~ 1, data = PlethMorph, Cov = Cov8)
fit9 <- lm.rrpp(SVL ~ 1, data = PlethMorph, Cov = Cov9)
fit10 <- lm.rrpp(SVL ~ 1, data = PlethMorph, Cov = Cov10)
fit11 <- lm.rrpp(SVL ~ 1, data = PlethMorph, Cov = Cov11)

par(mfrow = c(1,1))

MC1 <- model.comparison(fit1, fit2, fit3, fit4, fit5, fit6,
fit7, fit8, fit9, fit10, fit11,
type = "logLik")
MC1
plot(MC1)

MC2 <- model.comparison(fit1, fit2, fit3, fit4, fit5, fit6,
fit7, fit8, fit9, fit10, fit11,
type = "logLik", predictor = lambda)
MC2
plot(MC2)


MC3 <- model.comparison(fit1, fit2, fit3, fit4, fit5, fit6,
fit7, fit8, fit9, fit10, fit11,
type = "Z", predictor = lambda)
MC3
plot(MC3)
}
}
\references{
Bedrick, E.J., and C.L. Tsai. 1994. Model selection 
for multivariate regression in small samples. 
Biometrics, 226-231.

Warton, D.I., 2008. Penalized normal likelihood and 
ridge regularization of correlation and covariance matrices. 
Journal of the American Statistical Association. 103: 340-349.

Collyer,  M.L., E.K. Baken, & D.C. Adams.  A standardized 
effect size for evaluating and comparing the strength of phylogenetic 
signal. Methods in Ecology and Evolution. 13: 367–382.
}
\author{
Michael Collyer
}
\keyword{analysis}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{getModelCov}
\alias{getModelCov}
\title{Utility Function for RRPP}
\usage{
getModelCov(
  fit,
  type = c("Cov", "Pcov"),
  format = c("S3", "S4"),
  forceSym = TRUE
)
}
\arguments{
\item{fit}{Object from \code{\link{lm.rrpp}}}

\item{type}{Whether the Cov or Pcov matrix is returned}

\item{format}{Whether an S3 or S4 format is returned}

\item{forceSym}{Logical value for whether a symmetric matrix should 
be returned for Pcov, even if Pcov was triangular as a solution.}
}
\description{
A function mostly for internal processing but can be used to extract either
the model covariance matrix (Cov) or the projection matrix for transformations made 
from the covariance matrix (Pcov), which is basically the square-root of 
the covariance matrix.  This matrix is the model covariance used for estimation,
not the residual covariance matrix (see \code{\link{getResCov}}). 
There are also options for S3 or S4 format versions, or
a forcing of symmetry for Pcov.
}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.data.r
\docType{data}
\name{PupfishHeads}
\alias{PupfishHeads}
\title{Landmarks on pupfish heads}
\description{
Landmark data from Cyprinodon pecosensis head shapes, with 
variables for 
sex, month and year sampled, locality, head size, and coordinates of 
landmarks for head shape,
per specimen.  These data are a subset of a larger data set.
}
\details{
The variable, "coords", are data that were previously aligned
with GPA.  The variable, "headSize", is the Centroid size of each vector 
of coordinates.
See the \pkg{geomorph} package for details.
}
\references{
Gilbert, M.C. 2016. Impacts of habitat fragmentation on the 
cranial morphology of a 
threatened desert fish (Cyprinodon pecosensis). Masters Thesis, 
Western Kentucky University.
}
\author{
Michael Collyer
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{print.trajectory.analysis}
\alias{print.trajectory.analysis}
\title{Print/Summary Function for RRPP}
\usage{
\method{print}{trajectory.analysis}(x, ...)
}
\arguments{
\item{x}{Object from \code{\link{trajectory.analysis}}}

\item{...}{Other arguments passed onto}
}
\description{
Print/Summary Function for RRPP
}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{summary.measurement.error}
\alias{summary.measurement.error}
\title{Print/Summary Function for RRPP}
\usage{
\method{summary}{measurement.error}(object, ...)
}
\arguments{
\item{object}{Object from \code{\link{measurement.error}}}

\item{...}{Other arguments passed onto measurement.error}
}
\description{
Print/Summary Function for RRPP
}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{print.summary.pairwise}
\alias{print.summary.pairwise}
\title{Print/Summary Function for RRPP}
\usage{
\method{print}{summary.pairwise}(x, ...)
}
\arguments{
\item{x}{Object from \code{\link{summary.pairwise}}}

\item{...}{Other arguments passed onto summary.pairwise}
}
\description{
Print/Summary Function for RRPP
}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprecated.defunct.R
\name{classify}
\alias{classify}
\title{Deprecated functions in RRPP}
\usage{
classify()
}
\description{
The following function has been deprecated in RRPP
}
\details{
This function has been deprecated. Use \code{\link{prep.lda}} instead.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{summary.manova.lm.rrpp}
\alias{summary.manova.lm.rrpp}
\title{Print/Summary Function for RRPP}
\usage{
\method{summary}{manova.lm.rrpp}(object, test = c("Roy", "Pillai", "Hotelling-Lawley", "Wilks"), ...)
}
\arguments{
\item{object}{Object from \code{\link{lm.rrpp}}, updated with 
\code{\link{manova.update}}}

\item{test}{Type of multivariate test statistic to use.}

\item{...}{Other arguments passed onto manova.lm.rrpp}
}
\description{
Print/Summary Function for RRPP
}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.data.r
\docType{data}
\name{mammalHRphy}
\alias{mammalHRphy}
\title{Phylogenetic Data for the Mammal Home Range Data}
\description{
Phylogenetic tree data that can be used along with 
\code{\link{mammalHR}}.
}
\details{
A class \code{phylo} object containing data for the phylogenetic tree
that pertains to the \code{\link{mammalHR}} data set.  Although these data are
not required for the generalized least squares (GLS)
estimation example, used in the book, 
Permutational Biometry (Collyer & Adams, 2025), they provide additional 
graphical support for the example.
}
\references{
Collyer, M. L., & Adams D. C. (2025). Permutational Biometry: Volume
1, Univariate Data. Iowa State University Digital Press.  NOT YET PUBLISHED.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{add.trajectories}
\alias{add.trajectories}
\title{Plot Function for RRPP}
\usage{
add.trajectories(
  TP,
  traj.pch = 21,
  traj.col = 1,
  traj.lty = 1,
  traj.lwd = 1,
  traj.cex = 1.5,
  traj.bg = 1,
  start.bg = 3,
  end.bg = 2
)
}
\arguments{
\item{TP}{plot object (from \code{\link{plot.trajectory.analysis}})}

\item{traj.pch}{Plotting "character" for trajectory points.
Can be a single value or vector 
of length equal to the number of trajectories.  
See \code{\link{par}} and its description 
for pch.}

\item{traj.col}{The color of trajectory lines.
Can be a single value or vector 
of length equal to the number of trajectories.  
See \code{\link{par}} and its description 
for col.}

\item{traj.lty}{Trajectory line type.  Can be a single value or vector 
of length equal to the number of trajectories.  
See \code{\link{par}} and its description 
for lty.}

\item{traj.lwd}{Trajectory line width.  Can be a single value or vector 
of length equal to the number of trajectories.  
See \code{\link{par}} and its description 
for lwd.}

\item{traj.cex}{Trajectory point character expansion.  Can be a single value or vector 
of length equal to the number of trajectories.  
See \code{\link{par}} and its description 
for cex.}

\item{traj.bg}{Trajectory point background.  Can be a single value or vector 
of length equal to the number of trajectories.  
See \code{\link{par}} and its description 
for bg.}

\item{start.bg}{Trajectory point background, just the start points.
Can be a single value or vector 
of length equal to the number of trajectories.  
See \code{\link{par}} and its description 
for bg.  Green start points are the default.}

\item{end.bg}{Trajectory point background, just the end points.
Can be a single value or vector 
of length equal to the number of trajectories.  
See \code{\link{par}} and its description 
for bg.  Red end points are the default.}
}
\description{
Function adds trajectories to a principal component plot
}
\details{
The function adds trajectories to a plot made by 
 \code{\link{plot.trajectory.analysis}}.
 This function has a restricted set of plot parameters 
 based on the number of trajectories
 to be added to the plot.
}
\references{
Adams, D. C., and M. M. Cerney. 2007. 
Quantifying biomechanical motion using Procrustes 
  motion analysis. J. Biomech. 40:437-444.

Adams, D. C., and M. L. Collyer. 2007. 
The analysis of character divergence along environmental 
  gradients and other covariates. Evolution 61:510-515.

Adams, D. C., and M. L. Collyer. 2009. 
A general framework for the analysis of phenotypic 
  trajectories in evolutionary studies. Evolution 63:1143-1154.

Collyer, M. L., and D. C. Adams. 2007. 
Analysis of two-state multivariate phenotypic change 
  in ecological studies. Ecology 88:683-692.

Collyer, M. L., and D. C. Adams. 2013. 
Phenotypic trajectory analysis: comparison of shape change patterns 
in evolution and ecology. Hystrix 24: 75-83.

Collyer, M.L., D.J. Sekora, and D.C. Adams. 2015. 
A method for analysis of phenotypic change for phenotypes described 
by high-dimensional data. Heredity. 115:357-365.
}
\seealso{
\code{\link{plot.default}} and \code{\link{par}}
}
\author{
Michael Collyer
}
\keyword{utilities}
\keyword{visualization}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{summary.pairwise}
\alias{summary.pairwise}
\title{Print/Summary Function for RRPP}
\usage{
\method{summary}{pairwise}(
  object,
  stat.table = TRUE,
  test.type = c("dist", "stdist", "mdist", "VC", "DL", "var"),
  angle.type = c("rad", "deg"),
  confidence = 0.95,
  show.vectors = FALSE,
  ...
)
}
\arguments{
\item{object}{Object from \code{\link{pairwise}}}

\item{stat.table}{Logical argument for whether results should be 
returned in one table 
(if TRUE) or separate pairwise tables (if FALSE)}

\item{test.type}{the type of statistic to test.  See below
should be used in the test.}

\item{angle.type}{If test.type = "VC", whether angle results are 
expressed in radians or degrees.}

\item{confidence}{Confidence level to use for upper confidence 
limit; default = 0.95 (alpha = 0.05)}

\item{show.vectors}{Logical value to indicate whether vectors 
should be printed.}

\item{...}{Other arguments passed onto pairwise}
}
\description{
See \code{\link{pairwise}} for further description.
}
\details{
The following summarize the test that can be performed: 

\describe{
\item{\bold{Distance between vectors, "dist"}}{ Vectors for LS means or 
slopes originate at the origin and point to some location, having both a 
magnitude
and direction.  A distance between two vectors is the inner-product of of 
the vector difference, i.e., the distance between their endpoints.  For
LS means, this distance is the difference between means.  For multivariate 
slope vectors, this is the difference in location between estimated change 
for the dependent variables, per one-unit change of the covariate considered.
For univariate slopes, this is the absolute difference between slopes.}
\item{\bold{Standardized distance between vectors, "stdist"}}{Same as the 
distance between vectors, but distances are divided by the model standard error 
(square-root of the trace of the residual covariance matrix). Pairwise tests
with this statistic should be consistent with ANOVA results.}
\item{\bold{Mahalanobis distance between vectors, "mdist"}}{Similar to the
standardized distance between vectors but the inverse of the 
residual covariance matrix is used in calculation of the distance, rather than
dividing the Euclidean distance between means and dividing by the standard error. 
If the residual covariance matrix is singular, Mahalanobis distances will not be 
calculated.  Pairwise tests with this statistic should be consistent with 
MANOVA results.}
\item{\bold{Vector correlation, "VC"}}{ If LS mean or slope vectors are 
scaled to unit size, the vector correlation is the inner-product of the 
scaled vectors.
The arccosine (acos) of this value is the angle between vectors, which 
can be expressed in radians or degrees.  Vector correlation indicates 
the similarity of 
vector orientation, independent of vector length.}
\item{\bold{Difference between vector lengths, "DL"}}{  If the length of a vector 
is an important attribute -- e.g., the amount of multivariate change per 
one-unit
change in a covariate -- then the absolute value of the difference in 
vector lengths is a practical statistic to compare vector lengths, rather
than the estimates the vectors make.  Let 
d1 and
d2 be the distances (length) of vectors.  Then |d1 - d2| is a statistic 
that compares their lengths.  For slope vectors, this is a comparison of rates.
For comparison, if vectors are rates, "dist" finds the difference between estimates per unit
change of, e.g., time, size, etc., which could be large, even for small rates of change, if 
vectors point in dissimilar directions.  "DL" is a comparison of rates, irrespective of direction.}
\item{\bold{Variance, "var"}}{  Vectors of residuals from a linear 
model indicate can express the distances of observed values from 
fitted values.  Mean
squared distances of values (variance), by group, can be used to 
measure the amount of dispersion around estimated values for groups.  
Absolute
differences between variances are used as test statistics to compare 
mean dispersion of values among groups.  Variance degrees of freedom 
equal n, 
the group size, rather than n-1, as the purpose is to compare mean 
dispersion 
in the sample.  (Additionally, tests with one subject in a group 
are possible, or at least not a hindrance to the analysis.)}
}

The argument, \code{test.type} is used to select one of the tests 
above.  See \code{\link{pairwise}} for examples.

 \subsection{Notes for RRPP 0.6.2 and subsequent versions}{ 
 In previous versions of pairwise, \code{\link{summary.pairwise}} had three 
 test types: "dist", "VC", and "var".  When one chose "dist", for LS mean 
 vectors, the statistic was the inner-product of the vector difference.  
 For slope vectors, "dist" returned the absolute value  of the difference 
 between vector lengths, which is "DL" in 0.6.2 and subsequent versions.  This
 update uses the same calculation, irrespective of vector types.  Generally,
 "DL" is the same as a contrast in rates for slope vectors, but might not have
 much meaning for LS means.  Likewise, "dist" is the distance between vector
 endpoints, which might make more sense for LS means than slope vectors.  
 Nevertheless, the user has more control over these decisions with version 0.6.2
 and subsequent versions.
}

 \subsection{Notes for RRPP 2.0.4 and subsequent versions}{ 
 The test types, standardized distance between vectors, "stdist", and Mahalanobis distances
 between vectors were added.  The former
 simply divides the distance between vectors by model standard error (square-root 
 of the trace of the residual covariance matrix).  This is a multivariate generalization
 of a t-statistic for the difference between means.  It is not the same as Hotelling 
 squared-T-statistic, which requires incorporating the inverse of the residual covariance 
 matrix in the calculation of the distance, a calculation that also requires a non-singular
 covariance matrix.  However, the Mahalanobis distances are similar (and proportional) 
 to the Hotelling squared-T-statistic. Pairwise tests using Mahalanobis distances represent
 a non-parametric analog to the parametric Hotelling squared-T test.  Both tests should be 
 better for GLS model fits compared
 to Euclidean distances between means, as the total sums of squares are more likely to vary
 across random permutations. In general, if ANOVA is 
 performed a pairwise test with "stdist" is a good association; if MANOVA is performed,
 a pairwise test with "mdist" is a good association.
}
}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.data.r
\docType{data}
\name{PlethMorph}
\alias{PlethMorph}
\title{Plethodon comparative morphological data}
\description{
Data for 37 species of plethodontid salamanders.  
Variables include snout to vent length
(SVL) as species size, tail length, head length, snout to eye length, 
body width, forelimb length,
and hind limb length, all measured in mm.  A grouping variable is also 
included for functional guild size.  A variable for species names is also 
included.
The data set also includes a phylogenetic covariance matrix based on a 
Brownian model of evolution, to assist in 
generalized least squares (GLS) estimation.
}
\details{
The covariance matrix was estimated with the vcv.phylo function 
of the R package, ape, based on the tree
described in Adams and Collyer (2018).
}
\references{
Adams, D.C and Collyer, M.L. 2018. Multivariate phylogenetic 
anova: group-clade aggregation, biological 
challenges, and a refined permutation procedure. Evolution, 72: 1204-1215.
}
\author{
Michael Collyer and Dean Adams
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/effects.r
\name{pval}
\alias{pval}
\title{Obtain P-value from a vector of statistics}
\usage{
pval(s, target = NULL, greater = TRUE)
}
\arguments{
\item{s}{The sampling distribution vector to use.}

\item{target}{The value to target in the distribution.  (If null, the first value
in the vector is used.).  If the target exists outside the range of s,
a probability of 0 or 1 is certain.}

\item{greater}{Logical value for whether the probability should be "greater than 
or equal to".  Change to greater = FALSE for "less than or equal to".}
}
\description{
A function to find the probability of values greater or lesser than target,
from a vector of statistics presumably obtained in random permutations.
}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{print.summary.trajectory.analysis}
\alias{print.summary.trajectory.analysis}
\title{Print/Summary Function for RRPP}
\usage{
\method{print}{summary.trajectory.analysis}(x, ...)
}
\arguments{
\item{x}{Object from \code{\link{summary.trajectory.analysis}}}

\item{...}{Other arguments passed onto summary.trajectory.analysis}
}
\description{
Print/Summary Function for RRPP
}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/QRforX.r
\name{QRforX}
\alias{QRforX}
\title{QR decomposition of linear model design matrices}
\usage{
QRforX(X, returnQ = TRUE, ...)
}
\arguments{
\item{X}{A linear model design matrix, but can be any object coercible to matrix.}

\item{returnQ}{A logical value whether to return the Q matrix.  Generating a
Q matrix can be computationally intense for large matrices.  If it is not
explicitly needed, this argument can be FALSE.}

\item{...}{Further arguments passed to base::qr.}
}
\value{
An object of class \code{QR} is a list containing the 
following:
\item{Q}{The Q matrix, if requested.}
\item{R}{The R matrix.}
\item{X}{The X matrix, which could be changed from dense to sparse,
or vice versa, and redundant columns removed.}
\item{dimnames}{The dimnames associated with Q, R, or X}
\item{rank}{The rank of the X matrix.}
\item{S4}{Logical value for whether Q, R, and X are S4 class objects.}
}
\description{
This function performs a QR decomposition (factorization) on a linear 
model design matrix (X) and returns useful results for subsequent analysis.
This is intended as an internal function but can be used externally.  Because
base::qr and Matrix::qr have different options for QR algorithms, this
function assures that results are consistent for other RRPP function use, 
whether X is a dense or sparse matrix.
}
\examples{
## Simple Example
data(Pupfish)
fit <- lm.rrpp(coords ~ Pop, data = Pupfish, print.progress = FALSE)
QR <- QRforX(model.matrix(fit))
QR$Q
QR$R
QR$rank
QR$S4

## Not run, but one could get base::qr and Matrix::qr results as

# base::qr(as.matrix(QR$X))
# Matrix::qr(QR$X)

## Complex example

data("PupfishHeads")
fit <- suppressWarnings(lm.rrpp(headSize ~ sex + 
locality/year, data = PupfishHeads))
X <- model.matrix(fit)
dim(X) # Already reduced
colnames(X)
X <- model.matrix(terms(fit), fit$LM$data)
dim(X) # Retains redundant parameters
colnames(X)
QR <- QRforX(X)
dim(QR$X)
dim(QR$Q)
dim(QR$R)
QR$rank

}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/effects.r
\name{powerTrans}
\alias{powerTrans}
\title{Power Transformation}
\usage{
powerTrans(
  y,
  eps = 0.001,
  target = NULL,
  interp = FALSE,
  forceYJ = FALSE,
  useStDev = FALSE
)
}
\arguments{
\item{y}{A vector of values to use.}

\item{eps}{Tolerance for lambda = 0.}

\item{target}{The value to target in the distribution.  (If null, the first value
in the vector is used.)  This will be moved to the first value in results.}

\item{interp}{Logical value for whether to spline interpolate lambda 
(ideal for vectors of small length).  This will default if the number
of values is less than 100, as in \link[MASS]{boxcox}.}

\item{forceYJ}{A logical value whether to force a Yeo-Johnson transformation for non-negative
values.  Negative values trigger
the transformation, but positive values can be transformed similarly.}

\item{useStDev}{A logical value whether to initially use standard deviates of y.
This might be useful if the range of y is restricted, say between 0 and 1.}
}
\value{
A list containing the following:
\item{opt.lambda}{The optimized lambda parameter.}
\item{transformed}{The power-transformed values.}
\item{lambda}{If spline interpolation is used, the values of lambda used.}
\item{loglik}{If spline interpolation is used, the log-liklihoods calculated for lambda.}
}
\description{
A function that transforms a vector of statistics or data to be 
approximately normally distributed.
}
\details{
This function uses a Box-Cox power transformation for a vector of values,
(presumably statistics from RRPP) with the following conditions:

(1) If any values are 0 or negative, a Yeo-Johnson transformation is performed
to optimize lambda with only positive values.

(2) The lambda parameter is optimized over a range of -2 to +2,
as is typical for most applications.

(3) The first (or targeted) value (assumed to be the observed value in a 
distribution of random values) is removed from the vector to 
perform optimization of lambda, but is returned to the vector for
transformation.  This assures that large, aberrant values, such
as might be found via RRPP for large effects, do not have 
leverage for the optimization.

A Yeo-Johnson transformation can be forced, which for positive values
merely shifts the distribution by adding 1.  For most RRPP statistics
(SS. MS, Rsq, F, d), this is not needed for effect sizes, as values are positive.  
However, for log-likelihoods, this might be a good idea for comparisons, 
because values might be strongly positive or strongly negative.

Using standard deviates (useStDev) is a good idea for statistics with
aberrant ranges (like 0 to 1, or 1,000 to 2,000).  If used, a Yeo-Johnson
transformation is implicit, since standardizing introduces negative values.
}
\references{
Box, G. E. P. and Cox, D. R. (1964) An analysis of transformations 
(with discussion). Journal of the Royal Statistical Society B, 26, 211–252.

Yeo, I. and Johnson, R. (2000) A new family of power transformations 
to improve normality or symmetry. Biometrika, 87, 954-959.
}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kcomp.r
\name{kcomp}
\alias{kcomp}
\title{K-component analysis}
\usage{
kcomp(Y, Cov, transform. = TRUE, scale. = FALSE, tol = NULL, rank. = NULL)
}
\arguments{
\item{Y}{An n x p data matrix.}

\item{Cov}{An required n x n covariance matrix to describe 
the non-independence among
observations in Y, and provide a GLS-centering of data.}

\item{transform.}{An optional argument to transform GLS-centered residuals, 
if TRUE.  If FALSE, only GLS-centering is performed.}

\item{scale.}{A logical value indicating whether the variables 
should be scaled to have unit variance before the analysis 
takes place. The default is FALSE.}

\item{tol}{A value indicating the magnitude below which 
components should be omitted. (Components are omitted if their 
standard deviations are less than or equal to tol times the 
standard deviation of the first component.)}

\item{rank.}{Optionally, a number specifying the maximal rank, 
i.e., maximal number of K components to be used. 
This argument can be set as alternative or in addition to tol, 
useful notably when the desired rank is considerably 
smaller than the dimensions of the matrix of the K matrix.}
}
\value{
An object of class \code{kcomp} is a list containing 
the following
\item{values}{The eigenvalues of the K matrix,}
\item{vectors}{The eigenvectors of the K matrix.}
\item{scores}{The projected scores of data onto the eigenvectors.}
}
\description{
Function performs a K-comonent analysis, which is an eigen decomposition
of a ratio of ordinary least-squares (OLS) to generalized least squares (GLS)
covariance matrices.
}
\details{
The function performs a K-component analysis (KCA), as described in Mitteroecker et al. (2025).  
The analysis is similar to many that can be performed with  the \code{\link{ordinate}} function,
but whereas that function finds ordinate scores for a rotation or shear of a data space, KCA performs 
a relative eigen decomposition of a matrix product that represents a ratio between two covariance
matrices (Mitteroecker and Bookstein, 2014).  The eigenvalues are helpful for understanding the importance
of the GLS estimation of a covariance matrix.  For example, if the GLS estimation is made with
respect to a matrix of phylogenetic covariances, the eigenvalues express the distribution of 
phylogenetic signal across components of the data space.  

The matrix product (K matrix) is the inverse of the OLS covariance matrix times 
the GLS covariance matrix.  
Redundancies between these matrices will mean the K matrix is likely not full rank.  Therefore,
an algorithm is used to compute scores in appropriate dimensions.  First, data are aligned to the 
square-root matrix of the GLS
covariance matrix (Collyer and Adams, 2021), using the \code{\link{ordinate}} function.  Second, relative eigen decomposition is 
performed, and the number of real eigenvectors that can be computed are retained.  Lastly, the same number of 
aligned components are used for projection of data onto relative eigenvectors (the K components).
}
\examples{
 data(PlethMorph)
 PCA <- ordinate(as.data.frame(PlethMorph[c("TailLength",
 "HeadLength", "Snout.eye", "BodyWidth","Forelimb", "Hindlimb")],
 scale. = TRUE))
 Y <- PCA$x
 KCA <- kcomp(Y, Cov = PlethMorph$PhyCov, tol = 0.001)
 
}
\references{
Collyer, M.L. and D.C. Adams.  2021. 
Phylogenetically-aligned Component Analysis. Methods 
in Ecology and evolution. In press.

Bookstein, F. L., & Mitteroecker, P. (2014). Comparing 
covariance matrices by relative eigenanalysis, with applications to 
organismal biology. Evolutionary biology, 41, 336-350.

Mitteroecker, P., Collyer, M. L., & Adams, D. C. (2024). 
Exploring Phylogenetic Signal in Multivariate Phenotypes by Maximizing 
Blomberg’s K. Systematic Biology, syae035.
}
\seealso{
\code{\link{plot.kcomp}}, \code{\link{ordinate}}, 
\code{\link{plot.default}}, \code{\link{rrpp.data.frame}}
}
\author{
Michael Collyer
}
\keyword{analysis}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{summary.trajectory.analysis}
\alias{summary.trajectory.analysis}
\title{Print/Summary Function for RRPP}
\usage{
\method{summary}{trajectory.analysis}(
  object,
  stat.table = TRUE,
  attribute = c("MD", "TC", "SD"),
  angle.type = c("rad", "deg"),
  confidence = 0.95,
  show.trajectories = FALSE,
  ...
)
}
\arguments{
\item{object}{Object from \code{\link{trajectory.analysis}}}

\item{stat.table}{Logical argument for whether results should be 
returned in one table 
(if TRUE) or separate pairwise tables (if FALSE)}

\item{attribute}{Whether magnitude differences (MD, absolute difference 
in trajectory path lengths), 
trajectory correlations (TC), or trajectory shape differences (SD) are 
summarized.}

\item{angle.type}{If attribute = "TC", whether angle results are 
expressed in radians or degrees.}

\item{confidence}{Confidence level to use for upper confidence limit; 
default = 0.95 (alpha = 0.05)}

\item{show.trajectories}{Logical value to indicate whether trajectories 
should be printed.}

\item{...}{Other arguments passed onto trajectory.analysis}
}
\description{
Print/Summary Function for RRPP
}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP-package.R, R/RRPP.support.code.r
\docType{package}
\name{RRPP-package}
\alias{RRPP-package}
\alias{RRPP}
\title{RRPP: Linear Model Evaluation with Randomized Residuals in a Permutation Procedure}
\value{
Key functions for this package:
\item{\code{\link{lm.rrpp}}}{Fits linear models, using RRPP.
plus model comparisons.}
\item{\code{\link{coef.lm.rrpp}}}{Extract coefficients or perform test 
on coefficients, using RRPP.}
\item{\code{\link{predict.lm.rrpp}}}{Predict values from lm.rrpp fits 
and generate bootstrapped confidence intervals.}
\item{\code{\link{pairwise}}}{Perform pairwise tests, based on lm.rrpp 
model fits.}
}
\description{
Linear model calculations are made for many random versions of data. Using residual randomization in a permutation procedure, sums of squares are calculated over many permutations to generate empirical probability distributions for evaluating model effects. Additionally, coefficients, statistics, fitted values, and residuals generated over many permutations can be used for various procedures including pairwise tests, prediction, classification, and model comparison. This package should provide most tools one could need for the analysis of high-dimensional data, especially in ecology and evolutionary biology, but certainly other fields, as well.

Functions in this package allow one to evaluate linear models 
with residual randomization.
The name, "RRPP", is an acronym for, "Randomization of Residuals in a 
Permutation Procedure."  Through
the various functions in this package, one can use randomization of 
residuals to generate empirical probability
distributions for linear model effects, for high-dimensional data or 
distance matrices.

An especially useful option of this package is to fit models with 
either ordinary or generalized
least squares estimation (OLS or GLS, respectively), using theoretic 
covariance matrices.  Mixed linear
effects can also be evaluated.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/mlcollyer/RRPP}
}

}
\author{
\strong{Maintainer}: Michael Collyer \email{mlcollyer@gmail.com} (\href{https://orcid.org/0000-0003-0238-2201}{ORCID})

Authors:
\itemize{
  \item Michael Collyer \email{mlcollyer@gmail.com} (\href{https://orcid.org/0000-0003-0238-2201}{ORCID})
  \item Dean Adams (\href{https://orcid.org/0000-0001-9172-7894}{ORCID})
}


Michael Collyer and Dean Adams
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{focusMEonSubjects}
\alias{focusMEonSubjects}
\title{Plot Function for RRPP}
\usage{
focusMEonSubjects(x, subjects = NULL, shadow = TRUE, ...)
}
\arguments{
\item{x}{Plot from \code{\link{plot.measurement.error}}}

\item{subjects}{The specific subject to plot}

\item{shadow}{A logical value for whether to show other subject values as
shadows of their locations.}

\item{...}{Other arguments passed onto plot}
}
\description{
Reduces a plot.measurement.error to a single research subject.  This can be 
for cases when many overlapping subjects in a plot obscure interpretation 
for specific subjects.
}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{print.summary.manova.lm.rrpp}
\alias{print.summary.manova.lm.rrpp}
\title{Print/Summary Function for RRPP}
\usage{
\method{print}{summary.manova.lm.rrpp}(x, ...)
}
\arguments{
\item{x}{Object from \code{\link{summary.manova.lm.rrpp}}}

\item{...}{Other arguments passed onto summary.manova.lm.rrpp}
}
\description{
Print/Summary Function for RRPP
}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{plot.lm.rrpp}
\alias{plot.lm.rrpp}
\title{Plot Function for RRPP}
\usage{
\method{plot}{lm.rrpp}(
  x,
  type = c("diagnostics", "regression", "PC"),
  resid.type = c("p", "n"),
  fitted.type = c("o", "t"),
  predictor = NULL,
  reg.type = c("PredLine", "RegScore"),
  ...
)
}
\arguments{
\item{x}{plot object (from \code{\link{lm.rrpp}})}

\item{type}{Indicates which type of plot, choosing among diagnostics,
regression, or principal component plots.  Diagnostic plots are similar to 
\code{\link{lm}} diagnostic plots, but for multivariate data.  Regression plots
plot multivariate dispersion in some fashion against predictor values. PC plots
project data onto the eigenvectors of the covariance matrix for fitted values.}

\item{resid.type}{If type = "diagnostics", an optional argument for whether Pearson ("p") or normalized ("n")
residuals should be used.  These residuals are the same for ordinary least-squares (OLS) estimation
but differ for generalized least-squares (GLS) estimation.  For the latter, normalizing residuals
requires multiplying them by the transformation matrix obtained for GLS estimation.}

\item{fitted.type}{As with resid.type, whether fitted values use observed ("o") or transformed ("t")
values.}

\item{predictor}{An optional vector if "regression" plot type is chosen, 
and is a variable likely used in \code{\link{lm.rrpp}}.
This vector is a vector of covariate values equal to the number of observations.}

\item{reg.type}{If "regression" is chosen for plot type, this argument
indicates whether prediction line 
(PredLine) or regression score (RegScore) plotting is performed.  
For explanation of prediction line,
see Adams and Nistri (2010).  For explanation of regression score, see 
Drake and Klingenberg (2008).}

\item{...}{other arguments passed to plot (helpful to employ
different colors or symbols for different groups).  See
\code{\link{plot.default}} and \code{\link{par}}}
}
\description{
Plot Function for RRPP
}
\examples{
\dontrun{
# Univariate example
data(PlethMorph)
fitGLS <- lm.rrpp(TailLength ~ SVL, data = PlethMorph, Cov = PlethMorph$PhyCov, 
 print.progress = FALSE, iter = 0)
 
 par(mfrow = c(2, 2))
 plot(fitGLS)
 plot(fitGLS, resid.type = "n") # use normalized (transformed) residuals
 plot(fitGLS, resid.type = "n", fitted.type = "t") # use also transformed fitted values
 
 # Multivariate example
 
Y <- as.matrix(cbind(PlethMorph$TailLength,
PlethMorph$HeadLength,
PlethMorph$Snout.eye,
PlethMorph$BodyWidth,
PlethMorph$Forelimb,
PlethMorph$Hindlimb))
PlethMorph$Y <- Y
fitGLSm <- lm.rrpp(Y ~ SVL, data = PlethMorph, 
Cov = PlethMorph$PhyCov,
print.progress = FALSE, iter = 0)

par(mfrow = c(2, 2))
 plot(fitGLSm)
 plot(fitGLSm, resid.type = "n") # use normalized (transformed) residuals
 plot(fitGLSm, resid.type = "n", fitted.type = "t") # use also transformed fitted values
 par(mfrow = c(1, 1))
 }
}
\references{
Drake, A. G., and C. P. Klingenberg. 2008. 
The pace of morphological change: Historical 
transformation of skull shape in St Bernard dogs. Proc. R. Soc. B. 275:71-76.

Adams, D. C., and A. Nistri. 2010. Ontogenetic convergence 
and evolution of foot morphology 
in European cave salamanders (Family: Plethodontidae). BMC Evol. Biol. 10:1-10.
}
\author{
Michael Collyer
}
\keyword{utilities}
\keyword{visualization}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/looCV.r
\name{looCV_table}
\alias{looCV_table}
\title{Table for looCV comparisons}
\usage{
looCV_table(...)
}
\arguments{
\item{...}{Any number of \code{\link{looCV}} objects}
}
\description{
Function simply produces a table of MSE and cross-validated
MSE for any number of compared models.  The table is sorted
from lowest to highest cross-validated MSE.  The looCV objects
must use method = 'fit' or a table cannot be generated.
}
\examples{

# Example with snakeHS data

data(snakeHS)
fitS <- lm.rrpp(HS ~ SVL, data = snakeHS,
                iter = 0)
fitSR <- lm.rrpp(HS ~ SVL + Region, data = snakeHS,
                 iter = 0)
fitSxR <- lm.rrpp(HS ~ SVL * Region, data = snakeHS,
                  iter = 0)

CVS <- looCV(fitS)
CVSR <- looCV(fitSR)
CVSxR <- looCV(fitSxR)

summary(CVS)
summary(CVSR)
summary(CVSxR)

looCV_table(CVS, CVSR, CVSxR)

}
\seealso{
\code{\link{looCV}}, \code{\link{model.comparison}}
}
\author{
Michael Collyer
}
\keyword{analysis}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{getModels}
\alias{getModels}
\title{Utility Function for RRPP}
\usage{
getModels(fit, attribute = c("terms", "X", "qr", "all"))
}
\arguments{
\item{fit}{Object from \code{\link{lm.rrpp}}}

\item{attribute}{The various attributes that are used to extract RRPP
Model information, including "terms", "X" (design matrices), "qr" (QR
decompositions), or "all".}
}
\description{
A function mostly for internal processing but can be used to obtain terms,
design matrices, or QR decompositions used for each reduced or full
model that is fitted in an \code{\link{lm.rrpp}} fit.
}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{plot.looCV}
\alias{plot.looCV}
\title{Plot Function for RRPP}
\usage{
\method{plot}{looCV}(x, axis1 = 1, axis2 = 2, flip = NULL, ...)
}
\arguments{
\item{x}{An object of class \code{\link{looCV}}}

\item{axis1}{A value indicating which component should be 
displayed as the X-axis (default = C1)}

\item{axis2}{A value indicating which component should be 
displayed as the Y-axis (default = C2)}

\item{flip}{An argument that if not NULL can be used to flip 
components in the plot.  
The values need to match axis1 or axis2.  For example, if axis1 = 3 
and axis2 = 4, flip = 1 will not
change either axis; flip = 3 will flip only the horizontal axis; 
flip = c(3, 4) will flip both axes.  Axis will only be flipped in first
plot.}

\item{...}{other arguments passed to plot (helpful to employ
different colors or symbols for different groups).  See}
}
\description{
Plot Function for RRPP
}
\author{
Michael Collyer
}
\keyword{utilities}
\keyword{visualization}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.data.r
\docType{data}
\name{PlethCinHoff}
\alias{PlethCinHoff}
\title{Plethodon comparative SVL data}
\description{
Snout-to-vent-length (SVL) data for two species of salamanders
}
\details{
The data set is a subset of data from two species of salamanders 
(Plethodon cinereus and 
P. hoffmani) in eastern North America (based on Adams and Rohlf, 2000; 
Adams, 2000), collected from the Smithsonian Museum (Washington DC, USA).  Data 
include species, locality (whether in sympatry or allopatry), and SVL in mm.
These data are suitable data for demonstrating single factor and
factorial analysis of variance (ANOVA).
(ANCOVA), used in the book, Permutational Biometry (Collyer & Adams, 2025).
}
\references{
Adams, D. C. (2000). Divergence of trophic morphology and resource use
among populations of Plethodon cinereus and P. hoffmani in Pennsylvania.
In Bruce, R. C., Jaeger, R. G., and Houck, L. D., editors, The Biology of
Plethodontid Salamanders, pages 383–394. Springer US.

Adams, D. C. and Rohlf, F. J. (2000). Ecological character displacement in
Plethodon: Biomechanical differences found from a geometric morphometric
study. Proceedings of the National Academy of Sciences, 97:4106–4111.

Collyer, M. L., & Adams D. C. (2025). Permutational Biometry: Volume
1, Univariate Data. Iowa State University Digital Press.  NOT YET PUBLISHED.
}
\author{
Dean Adams
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.data.r
\docType{data}
\name{podarcis}
\alias{podarcis}
\title{Podarcis Wall Lizards from the Mediterranean Basin}
\description{
A data set of body size for males and females form different lizard 
species in different geographic locations.
}
\details{
These data are a subset of the data collected originally by 
Kaliontzopoulou et al.(2015).  Variables include snout-to-vent-length (SVL),
sex, species, and habitat preference.  These 
data are suitable for comparing effect sizes in the book,
Permutational Biometry (Collyer & Adams, 2025).
}
\references{
Kaliontzopoulou, A., Carretero, M. A., and Adams, D. C. (2015). Ecomorphological
variation in male and female wall lizards and the macroevolution
of sexual dimorphism in relation to habitat use. Journal of Evolutionary
Biology, 28:80–94.

Collyer, M. L., & Adams D. C. (2025). Permutational Biometry: Volume
1, Univariate Data. Iowa State University Digital Press.  NOT YET PUBLISHED.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{summary.predict.lm.rrpp}
\alias{summary.predict.lm.rrpp}
\title{Print/Summary Function for RRPP}
\usage{
\method{summary}{predict.lm.rrpp}(object, ...)
}
\arguments{
\item{object}{Object from \code{\link{predict.lm.rrpp}}}

\item{...}{Other arguments passed onto predict.lm.rrpp}
}
\description{
Print/Summary Function for RRPP
}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{summary.lm.rrpp}
\alias{summary.lm.rrpp}
\title{Print/Summary Function for RRPP}
\usage{
\method{summary}{lm.rrpp}(object, formula = TRUE, ...)
}
\arguments{
\item{object}{print/summary object (from \code{\link{lm.rrpp}})}

\item{formula}{Logical argument for whether to include formula in summary table}

\item{...}{other arguments passed to print/summary}
}
\description{
Print/Summary Function for RRPP
}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/anova.lm.rrpp.r
\name{anova.lm.rrpp}
\alias{anova.lm.rrpp}
\title{ANOVA for lm.rrpp model fits}
\usage{
\method{anova}{lm.rrpp}(
  object,
  ...,
  effect.type = c("F", "cohenf", "SS", "MS", "Rsq", "LR"),
  error = NULL,
  print.progress = TRUE
)
}
\arguments{
\item{object}{Object from \code{\link{lm.rrpp}}}

\item{...}{Additional lm.rrpp model fits or other arguments passed to anova.}

\item{effect.type}{One of "F", "cohenf", "SS", "MS", "Rsq" to choose from 
which distribution of statistics to calculate effect sizes (Z).  "LR" can
be chosen to include a log-likelihood ratio, for multi-model comparisons.  If
"LR" is used, the effect type is automatically "F".
See \code{\link{lm.rrpp}}.}

\item{error}{An optional character string to define MS error term for 
calculation of F values. See \code{\link{lm.rrpp}} for examples.}

\item{print.progress}{A logical argument if multiple models are used and 
one wishes to view progress for sums of squares (SS) calculations.}
}
\description{
Computes an analysis of variance (ANOVA) table using 
distributions of random statistics from \code{\link{lm.rrpp}}.  
ANOVA can be performed on one model or multiple models.  
If the latter, the first model is considered a null model for 
comparison to other models.  The ANOVA is functionally similar to a 
non-parametric likelihood ratio test for all null-full model comparisons
Residuals from the null model will be used to generate random pseudo-values 
via RRPP for evaluation of subsequent models. The permutation schedule from 
the null model will be used for random permutations.
This function does not correct for improper null models.  One must assure 
that the null model is nested within the other models.  Illogical results 
can be generated if this is not the case.
}
\examples{
\dontrun{
# See examples for lm.rrpp to see how anova.lm.rrpp works in conjunction
# with other functions

data(Pupfish)
names(Pupfish)
Pupfish$logSize <- log(Pupfish$CS) # better to not have functions in formulas

# Single-Model ANOVA

fit <- lm.rrpp(coords ~ logSize + Sex*Pop, SS.type = "I", 
data = Pupfish, print.progress = FALSE, iter = 999) 
anova(fit)
anova(fit, effect.type = "MS")
anova(fit, effect.type = "Rsq")
anova(fit, effect.type = "cohenf")

# Multi-Model ANOVA (like a Likelihood Ratio Test)
fit.size <- lm.rrpp(coords ~ logSize, SS.type = "I", data = Pupfish, 
print.progress = FALSE, iter = 999) 
fit.sex <- lm.rrpp(coords ~ logSize + Sex, SS.type = "I", data = Pupfish, 
print.progress = FALSE, iter = 999) 
fit.pop <- lm.rrpp(coords ~ logSize + Pop, SS.type = "I", data = Pupfish, 
print.progress = FALSE, iter = 999) 
anova(fit.size, fit.sex, fit.pop, 
print.progress = FALSE) # compares two models to the first

# see lm.rrpp examples for mixed model ANOVA example and how to vary SS type
}

}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{plot.trajectory.analysis}
\alias{plot.trajectory.analysis}
\title{Plot Function for RRPP}
\usage{
\method{plot}{trajectory.analysis}(x, ...)
}
\arguments{
\item{x}{plot object (from \code{\link{trajectory.analysis}})}

\item{...}{other arguments passed to plot (helpful to employ
different colors or symbols for different groups).  See
\code{\link{plot.default}} and \code{\link{par}}}
}
\value{
If an object is assigned, it will return:
 \item{pca}{Principal component analysis performed using \code{\link{prcomp}}.}
 \item{pc.points}{Principal component scores for all data.}
 \item{trajectory.analysis}{Trajectory analysis passed on.}
 \item{trajectories}{pca Observed trajectories projected onto principal 
 components.}
}
\description{
Function generates a principal component plot for trajectories
}
\details{
The function calculates and plots principal components of fitted values from 
 \code{\link{lm.rrpp}} that are passed onto \code{\link{trajectory.analysis}}, 
 and projects
 data onto them.  This function is a set.up, and \code{\link{add.trajectories}} 
 is needed to 
 add trajectories to the plot.  By having two stages of control, the plotting 
 functions are more 
 flexible.  This function also returns plotting information that can be 
 valuable for making
 individualized plots, if \code{\link{add.trajectories}} is not preferred.
}
\examples{
# See trajectory.analysis help file for examples
}
\references{
Adams, D. C., and M. M. Cerney. 2007. 
Quantifying biomechanical motion using Procrustes 
  motion analysis. J. Biomech. 40:437-444.

Adams, D. C., and M. L. Collyer. 2007. 
The analysis of character divergence along environmental 
  gradients and other covariates. Evolution 61:510-515.

Adams, D. C., and M. L. Collyer. 2009. 
A general framework for the analysis of phenotypic 
  trajectories in evolutionary studies. Evolution 63:1143-1154.

Collyer, M. L., and D. C. Adams. 2007. 
Analysis of two-state multivariate phenotypic change 
  in ecological studies. Ecology 88:683-692.

Collyer, M. L., and D. C. Adams. 2013. 
Phenotypic trajectory analysis: comparison of shape change patterns 
in evolution and ecology. Hystrix 24: 75-83.

Collyer, M.L., D.J. Sekora, and D.C. Adams. 2015. 
A method for analysis of phenotypic change for phenotypes described 
by high-dimensional data. Heredity. 115:357-365.
}
\seealso{
\code{\link{plot.default}} and \code{\link{par}}
}
\author{
Michael Collyer
}
\keyword{utilities}
\keyword{visualization}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{print.summary.ordinate}
\alias{print.summary.ordinate}
\title{Print/Summary Function for RRPP}
\usage{
\method{print}{summary.ordinate}(x, ...)
}
\arguments{
\item{x}{Object from \code{\link{summary.ordinate}}}

\item{...}{Other arguments passed onto print.ordinate}
}
\description{
Print/Summary Function for RRPP
}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/manova.update.r
\name{manova.update}
\alias{manova.update}
\title{MANOVA update for lm.rrpp model fits}
\usage{
manova.update(
  fit,
  error = NULL,
  tol = 1e-07,
  PC.no = NULL,
  print.progress = TRUE,
  verbose = NULL
)
}
\arguments{
\item{fit}{Linear model fit from \code{\link{lm.rrpp}}}

\item{error}{An optional character string to define R matrices 
used to calculate invR.H.
(Currently only Residuals can be used and this argument defaults 
to NULL.  Future versions
will update this argument.)}

\item{tol}{A tolerance value for culling data dimensions to 
prevent spurious results.  The distribution
of eigenvalues for the data will be examined and if the decay 
becomes less than the tolerance,
the data will be truncated to principal components ahead of this 
point.  This will possibly prevent spurious results
calculated from eigenvalues near 0.  If tol = 0, all possible 
PC axes are used, which is likely
not a problem if observations outnumber variables.  If tol = 0 
and the number of variables exceeds the number of observations,
the value of tol will be made slightly positive to prevent problems 
with eigen-analysis.}

\item{PC.no}{A value that, if not NULL, can override the tolerance 
argument, and forces a desired number of 
data PCs to use for analysis.  If a value larger than the possible 
number of PCs is chosen, the full compliment of PCs
(the full data space) will be used.  If a number larger than tol 
would permit is chosen, the minimum number of PCs between the tol
argument and PC.no argument is returned.}

\item{print.progress}{A logical value to indicate whether a 
progress bar should be printed to the screen.
This is helpful for long-running analyses.}

\item{verbose}{Either a NULL or logical value for whether to retain
all MANOVA result (if TRUE).  If NULL, the verbose argument used for
the \code{\link{lm.rrpp}} is retained for MANOVA update.  Essentially,
verbose indicates whether to retain all SSCP matrices and all invR.H matrices,
for every model effect, in every RRPP iteration.}
}
\value{
An object of class \code{lm.rrpp} is updated to include 
class \code{manova.lm.rrpp}, and the object,
$MANOVA, which includes
\item{SSCP}{Terms and Model SSCP matrices.}
\item{invR.H}{The inverse of the residuals SSCP times the H SSCP.}
\item{eigs}{The eigenvalues of invR.H.}
\item{e.rank}{Rank of the error (residuals) covariance matrix.  
Currently NULL only.}
\item{PCA}{Principal component analysis of data, using either tol 
or PC.no.}
\item{manova.pc.dims}{Resulting number of PC vectors in the 
analysis.}
\item{e.rank}{Rank of the residual (error) covariance matrix, 
irrespective of the number of dimensions 
 used for analysis.}
}
\description{
Function updates a lm.rrpp fit to add $MANOVA, which like 
$ANOVA, provides statistics
or matrices typically associated with multivariate analysis 
of variance (MANOVA).

MANOVA statistics or sums of squares and cross-products (SSCP) matrices
are calculated over the random permutations of a \code{\link{lm.rrpp}} 
fit.  SSCP matrices are 
computed, as are the inverse of R times H (invR.H), where R is a SSCP 
for the residuals or random effects and H is
the difference between SSCP matrices of full and reduced models 
(see below).   From invR.H,
MANOVA statistics are calculated, including Roy's maximum root 
(eigenvalue), Pillai trace, Hotelling-Lawley trace,
and Wilks lambda (via \code{\link{summary.manova.lm.rrpp}}).

The manova.update to add $MANOVA to \code{\link{lm.rrpp}} fits 
requires more computation time than the $ANOVA
statistics that are computed automatically in \code{\link{lm.rrpp}}.  
Generally, the same inferential conclusions will
be found with either approach, when observations outnumber response 
variables.  For high-dimensional data (more variables
than observations) data are projected into a Euclidean space of 
appropriate dimensions (rank of residual covariance matrix).  
One can vary the tolerance for eigenvalue decay or specify the number 
of PCs, if a smaller set of PCs than the maximum is desired.  
This is advised if there is strong correlation among variables 
(the data space could be simplified to fewer dimensions), as spurious
results are possible.  Because distributions of MANOVA stats 
can be generated from the random permutations,
there is no need to approximate F-values, like with parametric MANOVA. 
By restricting analysis to the real, positive eigenvalues calculated,
all statistics can be calculated (but Wilks lambda, as a product but 
not a trace, might be less reliable as variable number approaches
the number of observations).

 \subsection{ANOVA vs. MANOVA}{ 
 Two SSCP matrices are calculated for each linear model effect, for 
 every random permutation: R (Residuals or Random effects) and
 H, the difference between SSCPs for "full" and "reduced" models. 
 (Full models contain and reduced models lack
 the effect tested; SSCPs are hypothesized to be the same under 
 a null hypothesis, if there is no effect.  The 
 difference, H, would have a trace of 0 if the null hypothesis 
 were true.)  In RRPP, ANOVA and MANOVA correspond to
 two different ways to calculate statistics from R and H matrices.
 
 ANOVA statistics are those that find the trace of R and H SSCP 
 matrices before calculating subsequent statistics,
 including sums of squares (SS), mean squares (MS), and F-values.  
 These statistics can be calculated with univariate data
 and provide univariate-like statistics for multivariate data.  
 These statistics are dispersion measures only (covariances
 among variables do not contribute) and are the same as "distance-based" 
 stats proposed by Goodall (1991) and Anderson (2001).
 MANOVA stats require multivariate data and are implicitly 
 affected by variable covariances.  For MANOVA, the inverse of R times
 H (invR.H) is first calculated for each effect, then eigen-analysis 
 is performed on these matrix products.  Multivariate
 statistics are calculated from the positive, real eigenvalues.  
 In general, inferential
 conclusions will be similar with either approach, but effect 
 sizes might differ.
 
 Two important differences between manova.update and 
 \code{\link{summary.manova}} (for \code{\link{lm}} objects) 
 are that manova.update
 does not attempt to normalize residual SSCP matrices (unneeded 
 for non-parametric statistical solutions) and (2) uses a generalized
 inverse of the residual SSCP, if needed, when the number of 
 variables could render eigen-analysis problematic.  This approach 
 is consistent
 with covariance regularization methods that attempt to make covariance 
 matrices positive-definite for calculating model likelihoods or
 multivariate statistics.  If the number of observations far exceeds 
 the number of response variables, observed statistics from 
 manova.update and 
 \code{\link{summary.manova}} will be quite similar.  If the 
 number of response variables approaches or exceeds the number 
 of observations, manova.update
 statistics will be much more reliable.
 
 ANOVA tables are generated by \code{\link{anova.lm.rrpp}} on 
 lm.rrpp fits and MANOVA tables are generated
 by \code{\link{summary.manova.lm.rrpp}}, after running 
 manova.update on lm.rrpp fits.
 
 Currently, mixed model effects are only possible with 
 $ANOVA statistics, not $MANOVA.
 
 More detail is found in the vignette, ANOVA versus MANOVA.  
}
}
\examples{
\dontrun{
# Body Shape Analysis (Multivariate) ----------------

data(Pupfish)

# Although not recommended as a practice, this example will use only
# three principal components of body shape for demonstration.  
# A larger number of random permutations should also be used.

Pupfish$shape <- ordinate(Pupfish$coords)$x[, 1:3]

Pupfish$logSize <- log(Pupfish$CS) 

fit <- lm.rrpp(shape ~ logSize + Sex, SS.type = "I", 
data = Pupfish, print.progress = FALSE, iter = 499) 
summary(fit, formula = FALSE)
anova(fit) # ANOVA table

# MANOVA

fit.m <- manova.update(fit, print.progress = FALSE, tol = 0.001)
summary(fit.m, test = "Roy")
summary(fit.m, test = "Pillai")

fit.m$MANOVA$eigs$obs # observed eigenvalues
fit.m$MANOVA$SSCP$obs # observed SSCP
fit.m$MANOVA$invR.H$obs # observed invR.H 

# Distributions of test statistics

summ.roy <- summary(fit.m, test = "Roy")
dens <- apply(summ.roy$rand.stats, 1, density)
par(mfcol = c(1, length(dens)))
for(i in 1:length(dens)) {
     plot(dens[[i]], xlab = "Roy max root", ylab = "Density",
     type = "l", main = names(dens)[[i]])
     abline(v = summ.roy$rand.stats[1, i], col = "red")
}
par(mfcol = c(1,1))
}
}
\references{
Goodall, C.R. 1991. Procrustes methods in the 
statistical analysis of shape. Journal of the Royal Statistical 
Society B 53:285-339.

Anderson MJ. 2001. A new method for non-parametric 
multivariate analysis of variance.
   Austral Ecology 26: 32-46.
}
\author{
Michael Collyer
}
\keyword{analysis}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.data.r
\docType{data}
\name{flower}
\alias{flower}
\title{Flower Production and Temperature}
\description{
A data set containing variables for flower production and
temperature, averaged yearly for 30 years, from a dry tropical forest.
}
\details{
The data in this data set are samples from a larger study 
that compared flower production between forest types over 30 years, 
based on climate variables.  The data in this set are from a dry tropical forest,
include variables for annual flower production, temperature, and year. These 
data are suitable for considering the temporal non-independence of observations in 
generalized least squares (GLS) estimation, for the example in the book,
Permutational Biometry (Collyer & Adams, 2025).
}
\references{
Pau, S., Wolkovich, E. M., Cook, B. I., Nytch, C. J., Regetz, J., Zimmerman,
J. K., and Joseph Wright, S. (2013). Clouds and temperature drive dynamic
changes in tropical flower production. Nature Climate Change, 3:838–842.

Collyer, M. L., & Adams D. C. (2025). Permutational Biometry: Volume
1, Univariate Data. Iowa State University Digital Press.  NOT YET PUBLISHED.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{summary.betaTest}
\alias{summary.betaTest}
\title{Print/Summary Function for RRPP}
\usage{
\method{summary}{betaTest}(object, confidence = 0.95, ...)
}
\arguments{
\item{object}{Object from \code{\link{betaTest}}}

\item{confidence}{The desired confidence limit to print with a table of 
summary statistics.  Because distances are directionless, confidence limits 
are one-tailed.}

\item{...}{Other arguments passed onto betaTest}
}
\description{
Print/Summary Function for RRPP
}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{plot.kcomp}
\alias{plot.kcomp}
\title{Plot Function for RRPP}
\usage{
\method{plot}{kcomp}(x, axis1 = 1, axis2 = 2, flip = NULL, include.axes = TRUE, ...)
}
\arguments{
\item{x}{An object of class \code{\link{kcomp}}}

\item{axis1}{A value indicating which component should be 
displayed as the X-axis (default = K1)}

\item{axis2}{A value indicating which component should be 
displayed as the Y-axis (default = K2)}

\item{flip}{An argument that if not NULL can be used to flip 
components in the plot.  
The values need to match axis1 or axis2.  For example, if axis1 = 3 
and axis2 = 4, flip = 1 will not
change either axis; flip = 3 will flip only the horizontal axis; 
flip = c(3, 4) will flip both axes.}

\item{include.axes}{A logical argument for whether axes should be shown at x = 0 and y = 0.
This is different than the axes argument in the generic \code{\link{plot.default}} function, which
controls the edges of the plot (providing a box effect or not).  Using include.axes = TRUE does not 
allow aesthetic control of the axes.  If desired, it is better to use include.axes = FALSE and augment
the plot object with \code{\link{abline}} (choosing h = 0 and v = 0 in separate applications).}

\item{...}{other arguments passed to plot (helpful to employ
different colors or symbols for different groups).  See}
}
\value{
An object of class "plot.kcomp" is a list with components
 that can be used in other plot functions, such as the type of plot, points, 
 a group factor, and other information depending on the plot parameters used.
}
\description{
Plot Function for RRPP
}
\author{
Michael Collyer
}
\keyword{utilities}
\keyword{visualization}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logLik.lm.rrpp.r
\name{logLik.lm.rrpp}
\alias{logLik.lm.rrpp}
\title{Calculate the log-likelihood of a lm.rrpp fit}
\usage{
\method{logLik}{lm.rrpp}(
  object,
  tol = NULL,
  pc.no = NULL,
  Z = TRUE,
  verbose = FALSE,
  gls.null = FALSE,
  ...
)
}
\arguments{
\item{object}{Object from \code{\link{lm.rrpp}}}

\item{tol}{A value indicating the magnitude below which 
components should be omitted, following projection.  See \code{\link{ordinate}} 
for details.}

\item{pc.no}{Optionally, a number specifying the maximal number of 
principal components, passed onto \code{\link{ordinate}}, as argument, rank.}

\item{Z}{A logical value for whether to calculate Z scores based on RRPP.}

\item{verbose}{A logical value for whether to return random log-likelihood values,
if Z-scores are calculated.}

\item{gls.null}{A logical value for if a fit has a GLS estimation, should
the null model (intercept) also have a GLS estimation, for estimating Z.  
Should be FALSE if the log-likelihood is measured to compare different GLS
estimations for a covariance matrices}

\item{...}{further arguments passed to or from other methods}
}
\description{
\code{logLik.lm.rrpp} returns the log-likelihood of
an \code{lm.rrpp} object or an \code{lmm.rrpp} object.  
Ridge regularization will be performed for 
ill-conditioned or singular residual covariance matrices, but dimension
reduction could be augmented via projection, using the arguments, tol
and pc.no.  See \code{\link{ordinate}} for details.
}
\details{
Note that options are limited for \code{lmm.rrpp} objects.  It is currently
not possible to use RRPP to find Z scores.
}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{print.model.comparison}
\alias{print.model.comparison}
\title{Print/Summary Function for RRPP}
\usage{
\method{print}{model.comparison}(x, ...)
}
\arguments{
\item{x}{Object from \code{\link{model.comparison}}}

\item{...}{Other arguments passed onto model.comparison}
}
\description{
Print/Summary Function for RRPP
}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{print.pairwise}
\alias{print.pairwise}
\title{Print/Summary Function for RRPP}
\usage{
\method{print}{pairwise}(x, ...)
}
\arguments{
\item{x}{Object from \code{\link{pairwise}}}

\item{...}{Other arguments passed onto pairwise}
}
\description{
Print/Summary Function for RRPP
}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{print.predict.lm.rrpp}
\alias{print.predict.lm.rrpp}
\title{Print/Summary Function for RRPP}
\usage{
\method{print}{predict.lm.rrpp}(x, PC = FALSE, ...)
}
\arguments{
\item{x}{Object from \code{\link{predict.lm.rrpp}}}

\item{PC}{Logical argument for whether to use predicted values 
rotated to their PCs}

\item{...}{Other arguments passed onto predict.lm.rrpp}
}
\description{
Print/Summary Function for RRPP
}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{print.pairwise.model.Z}
\alias{print.pairwise.model.Z}
\title{Print/Summary Function for RRPP}
\usage{
\method{print}{pairwise.model.Z}(x, stats.table = TRUE, ...)
}
\arguments{
\item{x}{print/summary object (from \code{\link{pairwise.model.Z}})}

\item{stats.table}{A logical value for whether to condense results into
a single table of multiple statistics.}

\item{...}{other arguments passed to print/summary}
}
\description{
Print/Summary Function for RRPP
}
\author{
Dean Adams and Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pairwise.r
\name{pairwise}
\alias{pairwise}
\title{Pairwise comparisons of lm.rrpp fits}
\usage{
pairwise(
  fit,
  fit.null = NULL,
  groups,
  covariate = NULL,
  verbose = FALSE,
  print.progress = FALSE
)
}
\arguments{
\item{fit}{A linear model fit using \code{\link{lm.rrpp}}.}

\item{fit.null}{An alternative linear model fit to use as a null 
model for RRPP, if the null model
of the fit is not desired.  Note, for FRPP this argument should 
remain NULL and FRPP
must be established in the lm.rrpp fit (RRPP = FALSE).  If the 
null model is uncertain, 
using \code{\link{reveal.model.designs}} will help elucidate the 
inherent null model used.}

\item{groups}{A factor or vector that is coercible into a factor, 
describing the levels of
the groups for which to find LS means or slopes.  Normally this 
factor would be part of the 
model fit, but it is not necessary for that to be the case in order 
to obtain results.}

\item{covariate}{A numeric vector for which to calculate slopes for 
comparison  If NULL, 
LS means will be calculated instead of slopes.  Normally this variable 
would be part of the 
model fit, but it is not necessary for that to be the case in order 
to obtain results.}

\item{verbose}{A logical value for whether to include extra results, specifically 
the Mahalanobis distances among means, which requires the calculation of residual 
covariance matrices for each permutation.  This should be generally
FALSE, unless Mahalanobis distances are desired, in which case it must be TRUE.  
Verbose computations can require much additional time.}

\item{print.progress}{If a null model fit is provided, a logical 
value to indicate whether analytical 
results progress should be printed on screen.  Unless large data 
sets are analyzed, this argument 
is probably not helpful.}
}
\value{
An object of class \code{pairwise} is a list containing the following
\item{LS.means}{LS means for groups, across permutations.}
\item{slopes}{Slopes for groups, across permutations.}
\item{means.dist}{Pairwise distances between means, across permutations.}
\item{std.means.dist}{Pairwise distances between means, across permutations, standardized.}
\item{mah.means.dist}{Pairwise Mahalanobis distances between means, across permutations.}
\item{means.vec.cor}{Pairwise vector correlations between 
means, across permutations.}
\item{means.lengths}{LS means vector lengths, by group, across permutations.}
\item{means.diff.length}{Pairwise absolute differences between 
mean vector lengths, across permutations.}
\item{slopes.dist}{Pairwise distances between slopes (end-points), 
across permutations.}
\item{slopes.vec.cor}{Pairwise vector correlations between slope 
vectors, across permutations.}
\item{slopes.lengths}{Slope vector lengths, by group, across permutations.}
\item{slopes.diff.length}{Pairwise absolute differences between 
slope vector lengths, across permutations.}
\item{n}{Sample size}
\item{p}{Data dimensions; i.e., variable number}
\item{PermInfo}{Information for random permutations, passed on 
from lm.rrpp fit and possibly
modified if an alternative null model was used.}
}
\description{
Function generates distributions of pairwise statistics for a lm.rrpp fit and
returns important statistics for hypothesis tests.
}
\details{
Based on an lm.rrpp fit, this function will find fitted values over all 
permutations and based 
on a grouping factor, calculate either least squares (LS) means or 
slopes, and pairwise statistics among
them.  Pairwise statistics have multiple flavors, related to vector attributes: 

\describe{
\item{\bold{Distance between vectors, "dist"}}{ Vectors for LS means or 
slopes originate at the origin and point to some location, having both a 
magnitude
and direction.  A distance between two vectors is the inner-product of of 
the vector difference, i.e., the distance between their endpoints.  For
LS means, this distance is the difference between means.  For multivariate 
slope vectors, this is the difference in location between estimated change 
for the dependent variables, per one-unit change of the covariate considered.
For univariate slopes, this is the absolute difference between slopes.}
\item{\bold{Standardized distance between vectors, "stdist"}}{Same as the 
distance between vectors, but distances are divided by the model standard error 
(square-root of the trace of the residual covariance matrix, adjusted by sample size). 
Pairwise tests with this statistic should be consistent with ANOVA results.}
\item{\bold{Mahalanobis distance between vectors, "mdist"}}{Similar to the
standardized distance between vectors but the inverse of the 
residual covariance matrix is used in calculation of the distance, rather than
dividing the Euclidean distance between means and dividing by the standard error. 
If the residual covariance matrix is singular, Mahalanobis distances will not be 
calculated.  Pairwise tests with this statistic should be consistent with 
MANOVA results.}
\item{\bold{Vector correlation, "VC"}}{ If LS mean or slope vectors are 
scaled to unit size, the vector correlation is the inner-product of the 
scaled vectors.
The arccosine (acos) of this value is the angle between vectors, which can 
be expressed in radians or degrees.  Vector correlation indicates the 
similarity of 
vector orientation, independent of vector length.}
\item{\bold{Difference between vector lengths, "DL"}}{  If the length of a vector 
is an important attribute -- e.g., the amount of multivariate change per 
one-unit
change in a covariate -- then the absolute value of the difference in 
vector lengths is a practical statistic to compare vector lengths, rather
than the estimates the vectors make.  Let 
d1 and
d2 be the distances (length) of vectors.  Then |d1 - d2| is a statistic 
that compares their lengths.  For slope vectors, this is a comparison of rates.
For comparison, if vectors are rates, "dist" finds the difference between estimates per unit
change of, e.g., time, size, etc., which could be large, even for small rates of change, if 
vectors point in dissimilar directions.  "DL" is a comparison of rates, irrespective of direction.}
\item{\bold{Variance, "var"}}{  Vectors of residuals from a linear model 
indicate can express the distances of observed values from fitted values.  
Mean
squared distances of values (variance), by group, can be used to measure 
the amount of dispersion around estimated values for groups.  Absolute
differences between variances are used as test statistics to compare mean 
dispersion of values among groups.  Variance degrees of freedom equal n, 
the group size, rather than n-1, as the purpose is to compare mean dispersion 
in the sample.  (Additionally, tests with one subject in a group are 
possible, or at least not a hindrance to the analysis.)}
}

The \code{\link{summary.pairwise}} function is used to select a test 
statistic for the statistics described above, as
"dist", "VC", "DL", and "var", respectively.  If vector correlation is tested, 
the \code{angle.type} argument can be used to choose between radians and
degrees.

The null model is defined via \code{\link{lm.rrpp}}, but one can 
also use an alternative null model as an optional argument.
In this case, residual randomization in the permutation procedure 
(RRPP) will be performed using the alternative null model 
to generate fitted values.  If full randomization of values (FRPP) 
is preferred,
it must be established in the lm.rrpp fit and an alternative model 
should not be chosen. If one is unsure about the inherent
null model used if an alternative is not specified as an argument, 
the function \code{\link{reveal.model.designs}} can be used.

Observed statistics, effect sizes, P-values, and one-tailed confidence 
limits based on the confidence requested will
be summarized with the \code{\link{summary.pairwise}} function.  
Confidence limits are inherently one-tailed as
the statistics are similar to absolute values.  For example, a 
distance is analogous to an absolute difference.  Therefore,
the one-tailed confidence limits are more akin to two-tailed 
hypothesis tests.  (A comparable example is to use the absolute 
value of a t-statistic, in which case the distribution has a lower 
bound of 0.)  

 \subsection{Notes for RRPP 0.6.2 and subsequent versions}{ 
 In previous versions of pairwise, \code{\link{summary.pairwise}} had three 
 test types: "dist", "VC", and "var".  When one chose "dist", for LS mean 
 vectors, the statistic was the inner-product of the vector difference.  
 For slope vectors, "dist" returned the absolute value of the difference 
 between vector lengths, which is "DL" in 0.6.2 and subsequent versions.  This
 update uses the same calculation, irrespective of vector types.  Generally,
 "DL" is the same as a contrast in rates for slope vectors, but might not have
 much meaning for LS means.  Likewise, "dist" is the distance between vector
 endpoints, which might make more sense for LS means than slope vectors.  
 Nevertheless, the user has more control over these decisions with version 0.6.2
 and subsequent versions.
}

 \subsection{Notes for RRPP 2.0.4 and subsequent versions}{ 
 The test types, standardized distance between vectors, "stdist", and Mahalanobis distances
 between vectors were added.  The former
 simply divides the distance between vectors by model standard error (square-root 
 of the trace of the residual covariance matrix, adjusted by sample size).  
 This is a multivariate generalization
 of a t-statistic for the difference between means.  It is not the same as Hotelling 
 squared-T-statistic, which requires incorporating the inverse of the residual covariance 
 matrix in the calculation of the distance, a calculation that also requires a non-singular
 covariance matrix.  However, the Mahalanobis distances are similar (and proportional) 
 to the Hotelling squared-T-statistic. Pairwise tests using Mahalanobis distances represent
 a non-parametric analog to the parametric Hotelling squared-T test.  Both tests should be 
 better for GLS model fits compared
 to Euclidean distances between means, as the total sums of squares are more likely to vary
 across random permutations. In general, if ANOVA is 
 performed a pairwise test with "stdist" is a good association; if MANOVA is performed,
 a pairwise test with "mdist" is a good association.
}
}
\examples{
\dontrun{
# Examples use geometric morphometric data on pupfishes
# See the package, geomorph, for details about obtaining such data

# Body Shape Analysis (Multivariate) --------------

data("Pupfish")

# Note:

dim(Pupfish$coords) # highly multivariate!

Pupfish$logSize <- log(Pupfish$CS) 

# Note: one should use all dimensions of the data but with this 
# example, there are many.  Thus, only three principal components 
# will be used for demonstration purposes.

Pupfish$Y <- ordinate(Pupfish$coords)$x[, 1:3]

## Pairwise comparisons among LS means

# Note: one should increase RRPP iterations but a 
# smaller number is used here for demonstration 
# efficiency.  Generally, iter = 999 will take less
# than 1s for these examples with a modern computer.

fit1 <- lm.rrpp(Y ~ logSize + Sex * Pop, SS.type = "I", 
data = Pupfish, print.progress = FALSE, iter = 199) 
summary(fit1, formula = FALSE)
anova(fit1) 

pup.group <- interaction(Pupfish$Sex, Pupfish$Pop)
pup.group
PW1 <- pairwise(fit1, groups = pup.group)
PW1

# distances among means
summary(PW1, confidence = 0.95, test.type = "dist") 
summary(PW1, confidence = 0.95, test.type = "dist", stat.table = FALSE)

# standardized distances among means
summary(PW1, confidence = 0.95, test.type = "stdist") 

# Mahalanobis (generalized) distances among means
summary(PW1, confidence = 0.95, test.type = "mdist") 

# absolute difference between mean vector lengths
summary(PW1, confidence = 0.95, test.type = "DL") 

# correlation between mean vectors (angles in degrees)
summary(PW1, confidence = 0.95, test.type = "VC", 
   angle.type = "deg") 

# Can also compare the dispersion around means
summary(PW1, confidence = 0.95, test.type = "var")

## Pairwise comparisons of slopes

fit2 <- lm.rrpp(Y ~ logSize * Sex * Pop, SS.type = "I", 
data = Pupfish, print.progress = FALSE, iter = 199) 
summary(fit2, formula = FALSE)
anova(fit1, fit2)

# Using a null fit that excludes all factor-covariate 
# interactions, not just the last one  

PW2 <- pairwise(fit2, fit.null = fit1, groups = pup.group, 
covariate = Pupfish$logSize, print.progress = FALSE) 
PW2

# distances between slope vectors (end-points)
summary(PW2, confidence = 0.95, test.type = "dist") 
summary(PW2, confidence = 0.95, test.type = "dist", stat.table = FALSE)

# absolute difference between slope vector lengths
summary(PW2, confidence = 0.95, test.type = "DL") 

# correlation between slope vectors (and angles)
summary(PW2, confidence = 0.95, test.type = "VC",
   angle.type = "deg") 
   
# Can also compare the dispersion around group slopes
summary(PW2, confidence = 0.95, test.type = "var")
}

}
\references{
Collyer, M.L., D.J. Sekora, and D.C. Adams. 2015. A 
method for analysis of phenotypic change for phenotypes described
by high-dimensional data. Heredity. 115:357-365.

Adams, D.C and M.L. Collyer. 2018. Multivariate 
phylogenetic ANOVA: group-clade aggregation, biological 
challenges, and a refined permutation procedure. Evolution. In press.
}
\seealso{
\code{\link{lm.rrpp}}
}
\author{
Michael Collyer
}
\keyword{analysis}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{fixef.lmm.rrpp}
\alias{fixef.lmm.rrpp}
\title{fixef for lmm.rrpp model fits}
\usage{
\method{fixef}{lmm.rrpp}(object, ...)
}
\arguments{
\item{object}{Object from \code{\link{lmm.rrpp}}}

\item{...}{Other arguments passed.}
}
\description{
Returns fixed effects from a lmm.rrpp object.
}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{residuals.lm.rrpp}
\alias{residuals.lm.rrpp}
\title{Extract residuals}
\usage{
\method{residuals}{lm.rrpp}(object, ...)
}
\arguments{
\item{object}{plot object (from \code{\link{lm.rrpp}})}

\item{...}{Arguments passed to other functions}
}
\description{
Extract residuals
}
\examples{
# See examples for lm.rrpp
}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{model.matrix.lm.rrpp}
\alias{model.matrix.lm.rrpp}
\title{Extract the model design matrix from an lm.rrpp object}
\usage{
\method{model.matrix}{lm.rrpp}(object, ...)
}
\arguments{
\item{object}{Object from \code{\link{lm.rrpp}}}

\item{...}{further arguments passed to or from other methods}
}
\description{
\code{model.matrix.lm.rrpp} returns the design matrix constructed for 
an \code{lm.rrpp} object.
}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/effects.r
\name{effect.size}
\alias{effect.size}
\title{Obtain Effect-size from a vector of values}
\usage{
effect.size(x, center = TRUE, ...)
}
\arguments{
\item{x}{The vector of data to use.}

\item{center}{Logical value for whether to center x.}

\item{...}{Arguments passed to \link{powerTrans}.}
}
\description{
A function to find the effect size (Z-score) of a target,
from a vector of values presumably obtained in random permutations.
}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{summary.lr_test}
\alias{summary.lr_test}
\title{Print/Summary Function for RRPP}
\usage{
\method{summary}{lr_test}(object, ...)
}
\arguments{
\item{object}{Object from \code{\link{lr_test}}}

\item{...}{Other arguments passed onto print.lr_test}
}
\description{
Print/Summary Function for RRPP
}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/trajectory.analysis.r
\name{trajectory.analysis}
\alias{trajectory.analysis}
\title{Quantify and compare shape change trajectories}
\usage{
trajectory.analysis(
  fit,
  fit.null = NULL,
  groups,
  traj.pts,
  pca = TRUE,
  print.progress = FALSE
)
}
\arguments{
\item{fit}{A linear model fit using \code{\link{lm.rrpp}}.}

\item{fit.null}{An alternative linear model fit to use as a null model for 
RRPP, if the null model
of the fit is not desired.  Note, if RRPP = FALSE (FRPP rather than RRPP), 
then the null model has only an intercept.
If the null model is uncertain, using \code{\link{reveal.model.designs}} 
will help elucidate the inherent null model used.}

\item{groups}{A factor or vector coercible to factor that defines trajectories.}

\item{traj.pts}{Either a single value or a vector coercible to factor to 
define trajectory points.  If only a single value, 
it is assumed that the data are already in the form, 
y1p1, y2p1, y3p1, ...., y2p2, y2p2, y3p2, ..., yjp1, yjp2, yjp3, ..., yjpk, 
for j variables comprising k trajectory points;
i.e., traj.pts = k.  If a factor, then a group * traj.pt factorial model 
is assumed, where traj.pts defines the levels for points within groups.}

\item{pca}{A logical value to optionally project group:point means onto 
principal components (perform PCA on a covariance matrix of the means)
This option only applies to factorial designs (traj.pts is a factor).}

\item{print.progress}{A logical value to indicate whether a progress bar 
should be printed to the screen.  
This is helpful for long-running analyses.}
}
\value{
An object of class "trajectory.analysis" returns a list of the 
following:
  \item{LS.means}{LS.means from pairwise function.}
  \item{trajectories}{Trajectories from every permutation.}
  \item{PD}{Path distances of trajectories from every permutation.}
  \item{MD}{Magnitude differences between trajectories from every permutation.}
  \item{TC}{Trajectory correlations from every permutation.}
  \item{SD}{Trajectory shape differences from every permutation.}
}
\description{
Function estimates attributes of multivariate trajectories
}
\details{
The function quantifies multivariate trajectories from a set 
 of observations, and assesses variation 
 in attributes of the trajectories via RRPP. A trajectory is defined 
 by a sequence 
 of points in the data space. These trajectories can be quantified for 
 various attributes (their size, orientation, 
 and shape), and comparisons of these attribute enable the statistical 
 comparison of shape change 
 trajectories (Collyer and Adams 2007; Adams and Collyer 2007; 
 Adams and Collyer 2009; Turner et al. 2010; Collyer and Adams 2013). 
 
 This function is a modified version of \code{\link{pairwise}}, retaining 
 the least squares (LS) means as trajectory points.
 Analysis starts with a \code{\link{lm.rrpp}} fit (but a procD.lm fit from 
 geomorph can also be used).  LS means are calculated using a grouping
 variable.  Data can be trajectories, as a start(sensu Adams and Cerney 2007), 
 or trajectories can be calculated from data using a factorial model 
 (in which case
 trajectory points are defined by factor levels).  

 This function produces statistics that can be summarized with the 
 \code{\link{summary.trajectory.analysis}} function.  The summaries
 are consistent with those in the \code{\link{summary.pairwise}} 
 function, pertaining to trajectory attributes including,
 magnitude difference (MD), the difference in path lengths of trajectories; 
 trajectory correlations (TC), better
 thought of as angular differences between trajectory principal axes; and if 
 trajectories have three or more points,
 shape difference (SD), the square root of summed squared point differences, 
 after scaling, centering, and rotating trajectories.  The SD is
 the "Procrustes" distance between trajectories (Adams and Collyer 2009), 
 much the same way as the shape difference between anatomical landmark
 configurations in geometric morphometrics.  If attribute = "TC" is chosen 
 for the summary, then the angle type ("rad" or "deg",
 can be chosen for either radians and degrees, respectively, to return 
 angles between principal axes.)
 
 Plotting can be performed with \code{\link{plot.trajectory.analysis}} and 
 \code{\link{add.trajectories}}.  The former
 plots all principal component scores for the data, and allows point-by-point 
 control of plot parameters.  The later
 adds trajectories points and lines, with constrained control.  By saving the 
 plot.trajectory.analysis
 object, plotting information can be retained and advanced plotting can be 
 performed.  See examples below.
}
\examples{
\dontrun{
### Analysis of sexual dimorphism vectors (factorial approach)
data(Pupfish)
fit <- lm.rrpp(coords ~ Pop * Sex, data = Pupfish, iter = 199)
reveal.model.designs(fit)
TA <- trajectory.analysis(fit, groups = Pupfish$Pop, 
traj.pts = Pupfish$Sex, print.progress = FALSE)

# Magnitude difference (absolute difference between path distances)
summary(TA, attribute = "MD") 

# Correlations (angles) between trajectories
summary(TA, attribute = "TC", angle.type = "deg") 

# No shape differences between vectors
summary(TA, attribute = "SD") 

# Retain results
TA.summary <- summary(TA, attribute = "MD")
TA.summary$summary.table

# Plot results
TP <- plot(TA, pch = as.numeric(Pupfish$Pop) + 20, bg = as.numeric(Pupfish$Sex),
cex = 0.7, col = "gray")
add.trajectories(TP, traj.pch = c(21, 22), start.bg = 1, end.bg = 2)
legend("topright", levels(Pupfish$Pop), pch =  c(21, 22), pt.bg = 1)

### Analysis when data are already trajectories (motion paths)

# data are planar Cartesian coordinates (x, y) across 5 points (10 variables)
data(motionpaths)
fit <- lm.rrpp(trajectories ~ groups, data = motionpaths, iter = 199)
TA <- trajectory.analysis(fit, groups = motionpaths$groups, traj.pts = 5)

# Magnitude difference (absolute difference between path distances)
summary(TA, attribute = "MD") 

# Correlations (angles) between trajectories
summary(TA, attribute = "TC", angle.type = "deg") 

# Shape differences between trajectories 
summary(TA, attribute = "SD") 

TP <- plot(TA, pch = 21, bg = as.numeric(motionpaths$groups),
cex = 0.7, col = "gray")
add.trajectories(TP, traj.pch = 21, traj.bg = 1:4)
}
}
\references{
Adams, D. C., and M. M. Cerney. 2007. Quantifying biomechanical 
motion using Procrustes 
  motion analysis. J. Biomech. 40:437-444.

Adams, D. C., and M. L. Collyer. 2007. The analysis of 
character divergence along environmental 
  gradients and other covariates. Evolution 61:510-515.

Adams, D. C., and M. L. Collyer. 2009. A general framework 
for the analysis of phenotypic 
  trajectories in evolutionary studies. Evolution 63:1143-1154.

Collyer, M. L., and D. C. Adams. 2007. Analysis of two-state 
multivariate phenotypic change 
  in ecological studies. Ecology 88:683-692.

Collyer, M. L., and D. C. Adams. 2013. Phenotypic trajectory 
analysis: comparison of shape change patterns 
in evolution and ecology. Hystrix 24: 75-83.

Collyer, M.L., D.J. Sekora, and D.C. Adams. 2015. A method for 
analysis of phenotypic change for phenotypes described 
by high-dimensional data. Heredity. 115:357-365.
}
\author{
Dean Adams and Michael Collyer
}
\keyword{analysis}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reveal.model.designs.r
\name{reveal.model.designs}
\alias{reveal.model.designs}
\title{Reveal model designs used in lm.rrpp fit}
\usage{
reveal.model.designs(fit)
}
\arguments{
\item{fit}{A linear model fit from \code{\link{lm.rrpp}}.}
}
\description{
Function returns every full and reduced model for model terms used in 
lm.rrpp fits.  This function is useful for revealing 
the null and full model that would be used in the pairwise function, 
if a specific null model is not declared as an argument
(fit.null in the \code{\link{pairwise}} function).
It also helps to demonstrate how sums of squares and cross-products 
(SSCP) are calculated in lm.rrpp permutations (iterations),
from the difference between fitted values for null and full designs.
}
\examples{

data(Pupfish)
fit1 <- lm.rrpp(coords~ Pop*Sex, data = Pupfish, 
SS.type = "I", print.progress = FALSE, iter = 0)
fit2 <- lm.rrpp(coords~ Pop*Sex, data = Pupfish, 
SS.type = "II", print.progress = FALSE, iter = 0)
fit3 <- lm.rrpp(coords~ Pop*Sex, data = Pupfish, 
SS.type = "III", print.progress = FALSE, iter = 0)

reveal.model.designs(fit1)
reveal.model.designs(fit2)
reveal.model.designs(fit3)

}
\author{
Michael Collyer
}
\keyword{analysis}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{print.measurement.error}
\alias{print.measurement.error}
\title{Print/Summary Function for RRPP}
\usage{
\method{print}{measurement.error}(x, ...)
}
\arguments{
\item{x}{Object from \code{\link{measurement.error}}}

\item{...}{Other arguments passed onto measurement.error}
}
\description{
Print/Summary Function for RRPP
}
\examples{
\dontrun{

# Measurement error analysis on simulated data of fish shapes

data(fishy)

# Analysis unconcerned with groups 

ME1 <- measurement.error(
  Y = "coords",
  subjects = "subj",
  replicates = "reps",
  data = fishy)

anova(ME1)
ICCstats(ME1, subjects = "Subjects", with_in = "Systematic ME")
plot(ME1)

# Analysis concerned with groups 

ME2 <- measurement.error(
  Y = "coords",
  subjects = "subj",
  replicates = "reps",
  groups = "groups",
  data = fishy)

anova(ME2)
ICCstats(ME2, subjects = "Subjects", 
  with_in = "Systematic ME", groups = "groups")
P <- plot(ME2)
focusMEonSubjects(P, subjects = 18:20, shadow = TRUE)
}
}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{print.ordinate}
\alias{print.ordinate}
\title{Print/Summary Function for RRPP}
\usage{
\method{print}{ordinate}(x, ...)
}
\arguments{
\item{x}{Object from \code{\link{ordinate}}}

\item{...}{Other arguments passed onto print.ordinate}
}
\description{
Print/Summary Function for RRPP
}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{summary.ordinate}
\alias{summary.ordinate}
\title{Print/Summary Function for RRPP}
\usage{
\method{summary}{ordinate}(object, ...)
}
\arguments{
\item{object}{Object from \code{\link{ordinate}}}

\item{...}{Other arguments passed onto print.ordinate}
}
\description{
Print/Summary Function for RRPP
}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{plot.interSubVar}
\alias{plot.interSubVar}
\title{Plot Function for RRPP}
\usage{
\method{plot}{interSubVar}(x, ...)
}
\arguments{
\item{x}{Object from \code{\link{interSubVar}}}

\item{...}{Arguments passed onto \code{\link{image}} and \code{\link{plot}}.}
}
\description{
This function produces a heat map for inter-subject variability, based on 
results from a \code{\link{measurement.error}} object.  The function, 
\code{\link{interSubVar}}, must first be used on the \code{\link{measurement.error}} 
object to obtain variability statistics.  This function use the \code{\link{image}}
function to produce plots.  It does little to manipulate such plots, but any
argument for \code{\link{image}} can be manipulated here, as well as the graphical
parameters that can be adjusted within \code{\link{image}}.
}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lmm.rrpp.r
\name{lmm.rrpp}
\alias{lmm.rrpp}
\title{Mixed Linear Model Evaluation with RRPP performed within subjects}
\usage{
lmm.rrpp(
  fixed,
  subjects,
  type = c("intercepts", "slopes"),
  slopeTerm = NULL,
  estimation = c("REML", "ML", "LS"),
  iter = 999,
  data,
  ...
)
}
\arguments{
\item{fixed}{A formula for the fixed effects of linear model (e.g., y~x1+x2).}

\item{subjects}{A variable that can be found in the data frame indicating the research subjects
for the analysis.  This variable must be in the data frame.  Is can be either numeric 
(if its slot in the data frame is known) or a character, e.g., "sub_id".  It is imperative that
it is ordered the same as the data but that the data do not have row names the same as subjects.
For example, the subjects variable in the data frame might be sub_id: sub1, sub1, sub1, sub2,
sub2, sub2, ... and the row names of the data might be obs1, obs2, obs3, obs4, obs5, obs6, ...  
The data do not need to have row names but the subjects variable has to be provided.}

\item{type}{An indication whether to use random intercepts ("intercepts") or random intercepts 
and slopes ("slopes") for random effects associated with subjects.}

\item{slopeTerm}{An optional character value (e.g., "mass") indicating which term in the data frame is to be
used in the estimation of random slopes.  This term should also be in the fixed formula.}

\item{estimation}{One of "LS", "ML", or "REML", for guiding how random effects should be estimated.}

\item{iter}{Number of iterations for significance testing}

\item{data}{A data frame for the function environment, see 
\code{\link{rrpp.data.frame}}.  A data frame is required for this analysis.}

\item{...}{Arguments typically used in \code{\link{lm}}, \code{\link{lm.rrpp}},
or \code{\link{lm.rrpp.ws}} such as weights or offset, seed for permutations, etc., passed on to
\code{lm.rrpp.ws} for estimation of coefficients.}
}
\value{
An object of class \code{lmm.rrpp} is a list containing the 
following
\item{call}{The matched call.}
\item{LM}{Linear Model objects, including data (Y), coefficients, 
design matrix (X), sample size
(n), number of dependent variables (p), dimension of data space (p.prime),
QR decomposition of the design matrix, fitted values, residuals,
weights, offset, model terms, data (model) frame, random coefficients 
(through permutations),
random vector distances for coefficients (through permutations), 
whether OLS or GLS was performed, 
and the mean for OLS and/or GLS methods. Note that the data returned 
resemble a model frame rather than 
a data frame; i.e., it contains the values used in analysis, which 
might have been transformed according to 
the formula.  The response variables are always labeled Y.1, Y.2, ..., 
in this frame.  Most important objects for mixed models are also found here. }
\item{ANOVA}{Analysis of variance objects, including the SS type, 
random SS outcomes, random MS outcomes,
random R-squared outcomes, random F outcomes, random Cohen's f-squared 
outcomes, P-values based on random F
outcomes, effect sizes for random outcomes, sample size (n), number of 
variables (p), and degrees of freedom for
model terms (df).  These objects are used to construct ANOVA tables.}
\item{PermInfo}{Permutation procedure information, including the number 
of permutations (perms), The method
of residual randomization (perm.method), and each permutation's sampling 
frame (perm.schedule), which
is a list of reordered sequences of 1:n, for how residuals were 
randomized.}
\item{devcomp}{Deviance components, a list of components used
in deviance and likelihood calculations.}
}
\description{
Function performs a mixed linear model fit over many random permutations of 
data, using a randomized residual permutation procedure restricted to
subjects.  This function is likely to evolve greater flexibility in the future.
}
\details{
The function fits a mixed linear model using ordinary least squares (OLS) or 
generalized least squares (GLS) estimation of coefficients for fixed effects, and 
maximum likelihood (ML) or restricted ML (REML) for random effects, over any 
number of random permutations of the data, but the permutations are mostly 
restricted to occur with subject blocks for any model terms other than subjects.  
Most functionality should resemble that of \code{\link{lm.rrpp}} and \code{\link{lm.rrpp.ws}},  
with some caveats.  First, estimation of random effects uses the methodology of the 
\code{\link[lme4]{lmer}} function (Bates et al., 2015), 
for only one random term based on a subjects designation.  
Second, this means that a "random slopes" model involves an interaction between subjects and a numeric 
variable (not a categorical variable).  Third, although any number of fixed effects can be used,
only one fixed effect can interaction with subjects. Finally, the function is intended to allow
analysis of variance (ANOVA), like with \code{\link{lm.rrpp.ws}}, but not treated random 
effects as fixed.  
There is not a coefficients test associated with this function, as with other \code{\link{lm.rrpp}}
that include only fixed effects.

Although this function allows covariance matrices to be used for generalized least squares (GLS)
estimation of fixed effects, it does not currently 
expand the covariance mnatrix based only on subjects, as does 
\code{\link{lm.rrpp.ws}}.  However, that function can be used to obtain a covariance matrix for this 
function.  The covariance structure (Lambda matrix) among subjects is estimated via ML or REML (Bates et al., 2015) and 
is not impacted by the expected covariances among fixed effects, however, the expected covariances
will influence interactions between fixed and random effects.  The Lambda matrix is estimated as a single matrix,
even for multivariate data.  This matrix is obtained by projecting data onto the first component of
a singular value decomposition of the product between the data and a hat matrix of the the fixed effects 
(basically projection following a principal component analysis of model fitted values).  This is a computationally
proficient step that is similar to averaging covariance parameters (theta) over all variables.
If independent Lambda matrices are desired, then this function can be used on, e.g., each principal
component of data, followed by combining the coefficients.  For ANOVA, the precise estimation of Lambda
is not as important for tests of fixed effects, as it means a linear transformation of coefficients
that would be consistent across random permutations of RRPP.  It might be more important for model 
comparison and selection, in which case whether to use LS, ML, REML, and whether to use separate Lambda matrices
is a much deeper consideration.

A data frame (preferably an \code{\link{rrpp.data.frame}} object) must be
used for this function.  Not all "downstream" functions (e.g., logLik, predict) are guaranteed 
to work as expected, as this function is still very much under development.

The \code{\link{lm.rrpp}} arguments not available for this function include: 
full.resid, block, and SS.type.  These arguments are fixed because of
the within-subject blocking for tests, plus the requirement for type II SS
for within-subject effects.
}
\examples{
\dontrun{
# Random intercepts models

data(ReptAbund)
fit_ls <- lmm.rrpp(fixed = reptile_abundance ~ Year, 
subjects = "Site",      
type = "intercepts", 
estimation = "LS", 
data = ReptAbund)

fit_ml <- lmm.rrpp(fixed = reptile_abundance ~ Year, 
subjects = "Site",      
type = "intercepts", 
estimation = "ML", 
data = ReptAbund)

fit_reml <- lmm.rrpp(fixed = reptile_abundance ~ Year, 
subjects = "Site",      
type = "intercepts", 
estimation = "REML", 
data = ReptAbund) 

anova(fit_ls, error = c("Site", "Residuals"))
anova(fit_ml, error = c("Site", "Residuals"))
anova(fit_reml, error = c("Site", "Residuals"))

coef(fit_ls, type = "list")
coef(fit_ml, type = "list")
coef(fit_reml, type = "list")

# Random slopes models
# none should not work!

fit_ls <- lmm.rrpp(fixed = reptile_abundance ~ Year, 
subjects = "Site",      
type = "slopes",
slopeTerm = "Year", 
estimation = "LS", 
data = ReptAbund)

fit_ml <- lmm.rrpp(fixed = reptile_abundance ~ Year, 
subjects = "Site",      
type = "slopes",
slopeTerm = "Year", 
estimation = "ML", 
data = ReptAbund)


fit_reml <- lmm.rrpp(fixed = reptile_abundance ~ Year, 
subjects = "Site",      
type = "slopes",
slopeTerm = "Year",  
estimation = "REML", 
data = ReptAbund) 

anova(fit_ls, error = c("Year:Site", "Year:Site", "Residuals"))
anova(fit_ml, error = c("Year:Site", "Year:Site", "Residuals"))

coef(fit_ls, type = "list")
coef(fit_ml, type = "list")
 }
}
\references{
Douglas Bates, Martin Maechler, Ben Bolker, Steve Walker (2015). Fitting Linear
Mixed-Effects Models Using lme4. Journal of Statistical Software, 67(1), 1-48.
doi:10.18637/jss.v067.i01.

\subsection{Limitations}{
This function currently only works for "Random Intercepts" and "Random slopes" models,
where the intercepts or slopes of a single subject term (random effect) are free to vary.  
If a Random Slopes model is used, a single covariate (numeric variable) must be specified for slopes.  
Generally, this is term that is also among the fixed effects.  These limitations are imposed
to allow ML or REML estimation.  More complex models can be entertained with \code{lmm.rrpp.ws},
but using only "LS" estimation.  If "LS" estimation is used in this function, results should be the same
with a comparable formula, using \code{lmm.rrpp.ws}

If a ML or REML fit is near singular (one or more covariance parameters are 0), the analysis will be arrested
rather than continued with some random effects rendered as 0.  This should be an indication that the model
is over-fit with the type of estimation used. 

These limitations might be addressed with future versions of this function, as theoretical advances are made.
 }
}
\seealso{
\code{\link{lm.rrpp}}; \code{\link{lm.rrpp.ws}}; \code{\link{measurement.error}}
}
\author{
Michael Collyer
}
\keyword{analysis}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{getPermInfo}
\alias{getPermInfo}
\title{Utility Function for RRPP}
\usage{
getPermInfo(
  fit,
  attribute = c("perms", "perm.method", "block", "perm.seed", "perm.schedule", "all")
)
}
\arguments{
\item{fit}{Object from \code{\link{lm.rrpp}}}

\item{attribute}{The various attributes that are used to generate RRPP
permutation schedules.  If there are n observations, each iteration has
some randomization of 1:n, restricted by the arguments that match attributes
provided by this function.}
}
\description{
A function mostly for internal processing but can be used to extract
RRPP permutation information for other reasons.
}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{getTerms}
\alias{getTerms}
\title{Utility Function for RRPP}
\usage{
getTerms(fit)
}
\arguments{
\item{fit}{Object from \code{\link{lm.rrpp}}}
}
\description{
A function mostly for internal processing but can be used to extract
The terms for each reduced and full model used in an \code{\link{lm.rrpp}} fit.
}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.data.r
\docType{data}
\name{fishy}
\alias{fishy}
\title{Simulated Fish Data for Measurement Error Analysis}
\description{
Simulated Fish Data for Measurement Error Analysis
}
\details{
Data as simulated in Collyer and Adams (2024),
resembling a fish shape, comprising Procrustes coordinates
for 11 anatomical landmarks.  Data represent 120 
configurations for 60 subjects, each with two replicates of
measurement.  The 60 subjects represent 20 subjects each 
from three groups.
}
\references{
Collyer, M.L, and D.C. Adams. 2024. 
Interrogating random and systematic measurement 
error in morphometric data. Evolutionary Biology. 
In press.
}
\author{
Michael Collyer
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.data.r
\docType{data}
\name{phytoplankton}
\alias{phytoplankton}
\title{Phytoplankton Biomass and Eutrophication}
\description{
A data set containing variables for phytoplanton biomass,
phosphorous levels, trophic status, and locations (coordinates) of 70 temperate lakes.
}
\details{
The data in this data set are samples from a larger study 
that compared phytoplankton biomass
from temperate lakes in Alberta, Canada, with different levels of eutrophication
(Loewen et al., 2020).  These 
data are suitable for considering the spatial non-independence of observations in 
generalized least squares (GLS) estimation, for the example in the book,
Permutational Biometry (Collyer & Adams, 2025).
}
\references{
Loewen, C. J. G., Wyatt, F. R., Mortimer, C. A., Vinebrooke, R. D., and
Zurawell, R. W. (2020). Multiscale drivers of phytoplankton communities
in north‐temperate lakes. Ecological Applications, 30(5):e02102.

Collyer, M. L., & Adams D. C. (2025). Permutational Biometry: Volume
1, Univariate Data. Iowa State University Digital Press.  NOT YET PUBLISHED.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coef.lm.rrpp.r
\name{coef.lmm.rrpp}
\alias{coef.lmm.rrpp}
\title{coef for lmm.rrpp model fits}
\usage{
\method{coef}{lmm.rrpp}(object, type = c("matrix", "list"), ...)
}
\arguments{
\item{object}{Object from \code{\link{lmm.rrpp}}}

\item{type}{Whether to combine coefficients in a matrix or a list by variable.}

\item{...}{Other arguments (currently none)}
}
\description{
The coef.lmm.rrpp function computes 
combined fixed and random effects by subject for 
\code{\link{lmm.rrpp}} model fits. This function does not operate the same as
\code{\link{coef.lm.rrpp}}, for which coefficients and fixed effects are the same,
and tests can be performed on fixed effects.  It is merely a wrapper to combine 
the results of \code{\link{ranef.lmm.rrpp}} and \code{\link{fixef.lmm.rrpp}}.
}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coef.lm.rrpp.r
\name{coef.lm.rrpp}
\alias{coef.lm.rrpp}
\title{coef for lm.rrpp model fits}
\usage{
\method{coef}{lm.rrpp}(object, SE = FALSE, test = FALSE, confidence = 0.95, ...)
}
\arguments{
\item{object}{Object from \code{\link{lm.rrpp}}}

\item{SE}{Whether to include standard errors of coefficients.  Standard
errors are muted if test = TRUE.}

\item{test}{Logical argument that if TRUE, performs hypothesis tests 
(Null hypothesis is vector distance = 0)
for the observed coefficients.  If FALSE, only the observed coefficients 
are returned.}

\item{confidence}{The desired confidence limit to print with a table of 
summary statistics,
if test = TRUE.  Because distances are directionless, confidence limits 
are one-tailed.}

\item{...}{Other arguments (currently none)}
}
\description{
Computes ordinary or generalized least squares coefficients
over the permutations of an \code{\link{lm.rrpp}} model fit with predefined 
random permutations.
For each coefficient vector, the Euclidean distance is calculated as an 
estimate of
the amount of change in Y, the n x p matrix of dependent variables; larger 
distances mean more change 
in location in the data space associated with a one unit change in the model 
design, for the parameter
described.  Random coefficients are based on either RRPP or FRPP, as defined 
by the 
\code{\link{lm.rrpp}} model fit.  

 \subsection{The test argument will be deprecated}{ 
 
The following details will also be removed:
 
This function can be used to test the specific coefficients of an 
lm.rrpp fit.  The test
statistics are the distances (d), which are also standardized (Z-scores).  
The Z-scores might be easier to compare,
as the expected values for random distances can vary among coefficient 
vectors.

If RRPP is used, all distributions of coefficient vector distances are 
based on appropriate null models, as defined by SS type.  Please be aware that this 
can result in two seemingly strange but reasonable phenomena.  First, if type II or
type III SS is used, the intercept will not appear in test results (because the function
seeks model parameter differences to know for which coefficients to calculate Euclidean 
distances).  Even if it appears for type I SS, this is merely an artifact of sequential
model building and there really is no meaningful test of intercept = 0.  Second, 
Euclidean distances might not always be logical, especially when viewing univariate
coefficients, in which case the expected d is |b|.  Coefficients without a test are
based on the full model; tests are based on the estimates of coefficients (b), 
given a null model.  For example, for a model, y ~ b1 + b2 + b3, with type I SS,
b2 will be estimated and tested, using a null model, y ~ b1 and a full model, 
y ~ b1 + b2.  The estimate for b2 might not be the same in the test as when estimated 
from the model, y ~ b1 + b2 + b3.  Therefore, the d statistic might not reflect what one
would expect from the full model (like when using type III SS).  
 }

 \subsection{Difference between coef.lm.rrpp test and betaTest}{ 
 The test for coef.lm.rrpp uses the square-root of inner-products of vectors (d) as a 
 test statistic and only tests the null hypothesis that the length of the vector is 0.  
 The significance of the test is based on random values produced by RRPP, based on the
 matrices of coefficients that are produced in all permutations.  The null models for generating
 RRPP distributions are consistent with those used for ANOVA, as specified in the 
 \code{\link{lm.rrpp}} fit by choice of SS type.  Therefore, the random coefficients are
 consistent with those produced by RRPP for generating random estimates used in ANOVA.
 
 The betaTest analysis allows different null hypotheses to be used (vector length is not necessarily 0) 
 and unless otherwise specified, uses a null model that lacks one vector of parameters and a full
 model that contains all vectors of parameters, for the parameter for which coefficients are estimated.
 This is closest to a type III SS method of estimation, but each parameter is dropped from the model,
 rather than terms potentially comprising several parameters.  Additionally, betaTest calculates
 Mahalanobis distance, in addition to Euclidean distance, for vectors of coefficients.  
 This statistic is probably 
 better for more types of models (like generalized least squares fits).
}
\subsection{coef.lmm.rrpp}{
The coef.lmm.rrpp function computes 
combined fixed and random effects by subject for 
\code{\link{lmm.rrpp}} model fits. This function does not operate the same as
\code{\link{coef.lm.rrpp}}, for which coefficients and fixed effects are the same,
and tests can be performed on fixed effects.  It is merely a wrapper to combine 
the results of \code{\link{ranef.lmm.rrpp}} and \code{\link{fixef.lmm.rrpp}}.  
}
}
\examples{
\dontrun{
# See examples for lm.rrpp to see how anova.lm.rrpp works in conjunction
# with other functions

data(Pupfish)
names(Pupfish)
Pupfish$logSize <- log(Pupfish$CS)

fit <- lm.rrpp(coords ~ logSize + Sex*Pop, 
SS.type = "I", data = Pupfish, verbose = TRUE) 

coef(fit) # Just coefficients
coef(fit, SE = TRUE) # Coefficients with SE
coef(fit, test = TRUE, 
confidence = 0.99) # Test of coefficients
}
}
\seealso{
\code{\link{betaTest}}
}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{terms.lm.rrpp}
\alias{terms.lm.rrpp}
\title{Extract the terms from an lm.rrpp object}
\usage{
\method{terms}{lm.rrpp}(x, ...)
}
\arguments{
\item{x}{Object from \code{\link{lm.rrpp}}}

\item{...}{further arguments passed to or from other methods}
}
\description{
\code{terms.lm.rrpp} returns the terms constructed for an \code{lm.rrpp} object.
}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{model.frame.lm.rrpp}
\alias{model.frame.lm.rrpp}
\title{Extract model frame from a lm.rrpp object}
\usage{
\method{model.frame}{lm.rrpp}(formula, ...)
}
\arguments{
\item{formula}{Object from \code{\link{lm.rrpp}}}

\item{...}{further arguments passed to or from other methods}
}
\description{
\code{model.frame.lm.rrpp} returns the model frame constructed for 
an \code{lm.rrpp} object.
}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convert2ggplot.r
\name{convert2ggplot}
\alias{convert2ggplot}
\title{Convert RRPP plots to ggplot objects}
\usage{
convert2ggplot(object)
}
\arguments{
\item{object}{A plot object produced from \code{\link{plot.lm.rrpp}} and type 
equals either "PC" or "regression, \code{\link{plot.predict.lm.rrpp}}, or 
\code{\link{plot.ordinate}}.  Essentially, any RRPP plot except a series of diagnostic
plots should work.}
}
\description{
Function attempts to coerce plot information from an RRPP plot object to an 
amenable ggplot object.
}
\details{
This function will attempt to use the plot arguments from an RRPP plot object 
to make a ggplot that can be additionally updated, as desired.  Not all plot 
characteristics can be converted.  For example, text arguments are not currently
passed to \code{\link[ggplot2]{ggplot}}, as the \code{\link{text}} function and geom_text
arguments do not easily align.  However, one can use text arguments produced by
a RRPP plot object and geom_text to augment a ggplot object the way they like.

This function assumes no responsibility for arguments made by \code{\link[ggplot2]{ggplot}}.
It merely produces a ggplot object that should resemble an RRPP plot default.  Any 
augmentation of ggplot objects can be done either by direct intervention of the ggplot 
produced or reformatting the initial RRPP plot produced.  One should not expect direct
correspondence between R base plot parameters and ggplot parameters.  For example,
error bars will generally appear as different widths, without an easy way to control them,
changing from one format to the other.
}
\examples{
\dontrun{
### Linear Model Example

data(Pupfish)
fit <- lm.rrpp(coords ~ log(CS) + Sex*Pop, SS.type = "I", 
data = Pupfish, print.progress = FALSE) 

# Predictions (holding alternative effects constant)

shapeDF <- expand.grid(Sex = levels(Pupfish$Sex), 
Pop = levels(Pupfish$Pop))
rownames(shapeDF) <- paste(shapeDF$Sex, shapeDF$Pop, sep = ".")

shapePreds <- predict(fit, shapeDF)
summary(shapePreds, PC = TRUE)

# Plot prediction

P <- plot(shapePreds, PC = TRUE, ellipse = TRUE)
convert2ggplot(P)

### Ordination Example

data("PlethMorph")

Y <- as.data.frame(PlethMorph[c("TailLength", "HeadLength", 
                               "Snout.eye", "BodyWidth", 
                               "Forelimb", "Hindlimb")])
Y <- as.matrix(Y)
R <- lm.rrpp(Y ~ SVL, data = PlethMorph, 
             print.progress = FALSE)$LM$residuals

# PCA (on correlation matrix)


PCA.ols <- ordinate(R, scale. = TRUE)
PCA.ols$rot
prcomp(R, scale. = TRUE)$rotation # should be the same


PCA.gls <- ordinate(R, scale. = TRUE, 
                   transform. = FALSE, 
                   Cov = PlethMorph$PhyCov)
               
P <- plot(PCA.gls)
convert2ggplot(P)                   
}
}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{plot.predict.lm.rrpp}
\alias{plot.predict.lm.rrpp}
\title{Plot Function for RRPP}
\usage{
\method{plot}{predict.lm.rrpp}(x, PC = FALSE, ellipse = FALSE, abscissa = NULL, label = TRUE, ...)
}
\arguments{
\item{x}{plot object (from \code{\link{predict.lm.rrpp}})}

\item{PC}{A logical argument for whether the data space should be 
rotated to its 
principal components}

\item{ellipse}{A logical argument to change error bars to ellipses 
in multivariate plots.  
It has no function for univariate plots or is abscissa is not NULL.}

\item{abscissa}{An optional vector (numeric of factor) equal in length 
to predictions to use for 
plotting as the abscissa (x-axis), in which case predictions are the 
ordinate (y-axis).  This might be 
helpful if predictions are made for a continuous independent variable.  
The abscissa would be the
same variable used to make predictions (and can be the data.frame used 
for 
newdata in \code{\link{predict.lm.rrpp}}).}

\item{label}{A logical argument for whether points should be labeled 
(in multivariate plots).}

\item{...}{other arguments passed to plot, arrows, points, or text (helpful 
to employ
different colors or symbols for different groups).  See
\code{\link{plot.default}},  \code{\link{arrows}}, \code{\link{points}},
\code{\link{par}}, and \code{\link{text}}}
}
\description{
Plot Function for RRPP
}
\examples{
# See lm.rrpp help file for examples.
}
\author{
Michael Collyer
}
\keyword{utilities}
\keyword{visualization}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{print.ICCstats}
\alias{print.ICCstats}
\title{Print/Summary Function for RRPP}
\usage{
\method{print}{ICCstats}(x, ...)
}
\arguments{
\item{x}{print/summary object (from \code{\link{ICCstats}})}

\item{...}{other arguments passed to print/summary}
}
\description{
Print/Summary Function for RRPP
}
\author{
Michael Collyer
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ICCstats.r
\name{ICCstats}
\alias{ICCstats}
\title{Intraclass correlation statistics from an lm.rrpp model fits}
\usage{
ICCstats(
  fit,
  subjects = NULL,
  with_in = NULL,
  groups = NULL,
  multivariate = FALSE,
  print.AOV = TRUE
)
}
\arguments{
\item{fit}{The \code{\link{lm.rrpp}}, previously evaluated.}

\item{subjects}{A single character value indicating which term in an ANOVA table
corresponds to research subjects.}

\item{with_in}{One or more character values indicating which terms in an ANOVA table
are measured within subjects (replications, plus maybe interactions).  If NULL,
the only replication within-subject will be considered as residuals.}

\item{groups}{An optional character value to indicate if a factor in the 
model frame of the \code{\link{lm.rrpp}} fit that could account for subject variation.
Using this argument might minimize the importance of subject variation, if subjects
have disparate values that could inflate ICC.  Note that this name could be different
than what is shown in the ANOVA table, if \code{\link{measurement.error}}
was used.  Use names(fit$LM$data), substituting fit with the name assigned
to the \code{\link{measurement.error}} object, to know the groups factor, 
if used.}

\item{multivariate}{Logical value for whether to include to calculate ICC matrix 
generalizations and perform eigenanalysis.}

\item{print.AOV}{Logical value for whether to include ANOVA table as screen 
output, when calculating ISS statistics.

Note that this function can return ICC statistics, even if they do not make sense.
It is possible to generate ICC stats with any ANOVA table, with at least one
term.}
}
\value{
Objects of class "ICCstats" return the following:
 \item{ICC_disp}{The intraclass correlation coefficient (ICC) based on the dispersion of values.}
 \item{ICC_mult}{The eigenvalues of ICC matrices}
}
\description{
Function performs analyses concerned with the repeatability (reliability) of multivariate data 
(measurements) collected from the same research subjects.  Although there is no
requirement for repeated measurements on all research subjects, the analysis assumes
that multiple observations are made.
}
\details{
Function uses ANOVA statistics or SSCP matrices to find the ratio of among-subject
to within-subject variance.  The former is a dispersion-based approach and the latter
is a multivariate generalization of the ICC statistic (as a matrix product).  The
multivariate generalizations of the statistics
described by Liljequist et al. (2019) are used to find matrix products, 
from which eigenanalysis is performed, providing ICC statistics by eigenvectors.  

Three statistics describe the ICC for the population,
agreement of measurements among subjects, and consistency between measurements.  
The last statistic does not necessarily measure the sameness 
between measurements but the consistency of change between measurements,
which might be indicative of a systematic measurement error.  
If groups are used, these three statistics are 
repeated, using the SSCP for groups-adjusted data.  
This approach accounts for group differences,
which would avoid large subject variation compared to measurement error 
inflating ICC values.  If there are 
inherently disparate groups from which subjects are sampled, 
this approach can elucidate better agreement and 
consistency in light of group differences.

This function is most useful for analyses performed with 
\code{\link{measurement.error}}, but any \code{\link{lm.rrpp}} fit can be used,
so long as research subjects can be defined. 

It is essential that all arguments are terms that can be found in the model frame
of the model fit, as provoke by ANOVA.  Using anova(fit) will elucidate the row
names of the ANOVA that could be used.
}
\examples{
\dontrun{
# Measurement error analysis on simulated data of fish shapes

data(fishy)

# Analysis unconcerned with groups 

ME1 <- measurement.error(
  Y = "coords",
  subjects = "subj",
  replicates = "reps",
  data = fishy)

anova(ME1)
ICCstats(ME1, subjects = "Subjects", with_in = "Systematic ME")

# Analysis concerned with groups 

ME2 <- measurement.error(
  Y = "coords",
  subjects = "subj",
  replicates = "reps",
  groups = "groups",
  data = fishy)

anova(ME2)
ICCstats(ME2, subjects = "Subjects", 
  with_in = "Systematic ME", groups = "groups")
ICCstats(ME2, subjects = "Subjects", 
  with_in = c("Systematic ME", "Systematic ME:Groups"), 
  groups = "groups")
  }
  
}
\references{
Liljequist, D., Elfving, B., & Skavberg Roaldsen, K. (2019). Intraclass correlation–A discussion 
and demonstration of basic features. PloS one, 14(7), e0219854.
}
\author{
Michael Collyer
}
\keyword{analysis}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{print.lr_test}
\alias{print.lr_test}
\title{Print/Summary Function for RRPP}
\usage{
\method{print}{lr_test}(x, ...)
}
\arguments{
\item{x}{Object from \code{\link{lr_test}}}

\item{...}{Other arguments passed onto print.lr_test}
}
\description{
Print/Summary Function for RRPP
}
\author{
Michael Collyer
}
\keyword{utilities}
